#!/bin/sh
# ---------------------------------------------------------------------------
# JOnAS: Java(TM) Open Application Server
# Copyright (C) 1999 Bull S.A.
# Contact: jonas-team@objectweb.org
# 
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): ____________________________________.
# Contributor(s): ______________________________________.
#
# ---------------------------------------------------------------------------
# $Id$
# ---------------------------------------------------------------------------

TEMPLATE=$JONAS_ROOT/bin/template

# --------------------------------------------------------
# Get args
# no arg => interactive mode.
# --------------------------------------------------------
BNAME=
TYPE=
PACKAGE=
MYJAR=
PK=
if [ $# -gt 0 ]
then
    BNAME=$1
fi
if [ $# -gt 1 ]
then
    TYPE=$2
fi
if [ $# -gt 2 ]
then
    PACKAGE=$3
fi
if [ $# -gt 3 ]
then
    MYJAR=$4
fi
if [ $# -gt 4 ]
then
    PK=$5
fi

# --------------------------------------------------------
# Get bean caracteristics
# --------------------------------------------------------
if [ -z "$BNAME" ]
then
    echo "Choose the bean name, starting with a capital letter."
    echo "Avoid the reserved names : Home, EJB, Session, Entity."
    echo "bean name : \c"
    read BNAME
fi

if [ -z "$TYPE" ]
then
    echo "Type of bean ?"
    echo "EC = Entity CMP"
    echo "EB = Entity BMP"
    echo "SL = Session Stateless"
    echo "SF = Session StateFul"
    echo "MD = Message Driven"
    echo "Your choice: \c"
    read TYPE
fi
case $TYPE in
ec)
    TYPE=EC
    ;;
eb)
    TYPE=EB
    ;;
sl)
    TYPE=SL
    ;;
sf)
    TYPE=SF
    ;;
md)
    TYPE=MD
    ;;
esac

if [ -z "$PACKAGE" ]
then
    echo "Enter the dot separated package name, usually matching"
    echo "the current directory, where the beans will be compiled."
    echo "Package Name: (dot separated name: x.y.z) \c"
    read PACKAGE
fi

if [ -z "$MYJAR" ]
then
    echo "Enter a name for the jar file, in small letters, without the .jar extension: \c"
    read MYJAR
fi

case $TYPE in
E*)
    if [ -z "$PK" ]
    then
	case $TYPE in
	E*)
	    echo "Choose the PK Class name."
	    echo "s = java.lang.String"
	    echo "i = java.lang.Integer"
	    echo "o = deferred to deployment time"
	    read PK
	    ;;
	esac
    fi
    ;;
*)
    PK=none
    ;;
esac

# --------------------------------------------------------
# Build PK Class Name
# --------------------------------------------------------
case $PK in
s)
    PKCLASS=java.lang.String
    ;;
i)
    PKCLASS=java.lang.Integer
    ;;
o)
    PKCLASS=java.lang.Object
    ;;
*)
    PKCLASS= 
    ;;
esac

export BNAME TYPE PACKAGE PKCLASS

echo "Creating bean $BNAME (type $TYPE) in package $PACKAGE"

# --------------------------------------------------------
# Build Standard Deployment Descriptor file
# --------------------------------------------------------
FILE=${MYJAR}.xml
[ -r $FILE ] && mv $FILE ${FILE}.old
>$FILE
case $TYPE in
SL)
	cat $TEMPLATE/Stateless.xml | 
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
SF)
	cat $TEMPLATE/Stateful.xml |  
	sed "s;@TYPE@;$TYPE;g" |  
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
EC)
	cat $TEMPLATE/CMP.xml |  
	sed "s;@TYPE@;$TYPE;g" |  
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PKCLASS@;$PKCLASS;" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
EB)
	cat $TEMPLATE/BMP.xml |  
	sed "s;@TYPE@;$TYPE;g" |  
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@PKCLASS@;$PKCLASS;" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
MD)
	cat $TEMPLATE/MDB.xml |  
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
esac

# --------------------------------------------------------
# Build JOnAS specific  Deployment Descriptor file
# --------------------------------------------------------
FILE=jonas-${MYJAR}.xml

case $TYPE in
S*)
	cat $TEMPLATE/jonas-Session.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" | 
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" >> $FILE
	;;
EC)
	cat $TEMPLATE/jonas-CMP.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@BNAME@;$BNAME;g" |  
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
EB)
	cat $TEMPLATE/jonas-BMP.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@BNAME@;$BNAME;g" |  
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@MYJAR@;$MYJAR;" | 
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
MD)
	cat $TEMPLATE/jonas-MDB.xml | 
	sed "s;@JONAS_ROOT@;$JONAS_ROOT;" |
	sed "s;@BNAME@;$BNAME;g" |  
	sed "s;@TYPE@;$TYPE;g" | 
	sed "s;@PACKAGE@;$PACKAGE;" >> $FILE
	;;
esac

# --------------------------------------------------------
# Build remote interface file
# --------------------------------------------------------

if [ "$TYPE" != "MD" ]
then
    FILE=${BNAME}.java
    >$FILE

    cat $TEMPLATE/Remote.java |
    sed "s;@BNAME@;$BNAME;g" | 
    sed "s;@TYPE@;$TYPE;g" | 
    sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
fi

# --------------------------------------------------------
# Build implementation file
# --------------------------------------------------------
FILE=${BNAME}${TYPE}.java
>$FILE

case $TYPE in
SL)
	cat $TEMPLATE/Stateless.java |
        sed "s;@TYPE@;$TYPE;g" | 	    
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Session.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	cat $TEMPLATE/Bean.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	;;
SF)
	cat $TEMPLATE/Stateful.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Session.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	cat $TEMPLATE/Bean.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	;;
EB)
	cat $TEMPLATE/Entity.java |
	sed "s;@BNAME@;$BNAME;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PKCLASS@;$PKCLASS;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Bmp.java |
	sed "s;@PKCLASS@;$PKCLASS;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
        sed "s;@MYJAR@;$MYJAR;" | 	
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	cat $TEMPLATE/Bean.java |
	sed "s;@BNAME@;$BNAME;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	;;
EC)
	cat $TEMPLATE/Entity.java |
	sed "s;@BNAME@;$BNAME;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PKCLASS@;$PKCLASS;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	cat $TEMPLATE/Cmp.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PKCLASS@;$PKCLASS;" |
	sed "s;@BNAME@;$BNAME;g" >>$FILE
	cat $TEMPLATE/Bean.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	;;
MD)
	cat $TEMPLATE/MessageDriven.java |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@BNAME@;$BNAME;g" |
	sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
	;;
    
esac

	

# --------------------------------------------------------
# Build Home Interface file
# --------------------------------------------------------
if [ "$TYPE" != "MD" ]
then
    FILE=${BNAME}Home.java
    >$FILE

    cat $TEMPLATE/Home.java |
    sed "s;@BNAME@;$BNAME;g" |
    sed "s;@TYPE@;$TYPE;g" | 	
    sed "s;@PACKAGE@;$PACKAGE;" >>$FILE

    case $TYPE in
    E*)
	cat $TEMPLATE/EHome.java |
	sed "s;@BNAME@;$BNAME;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PKCLASS@;$PKCLASS;g" >>$FILE
	;;
    S*)
	cat $TEMPLATE/SHome.java |
	sed "s;@BNAME@;$BNAME;g" |
        sed "s;@TYPE@;$TYPE;g" | 	
	sed "s;@PKCLASS@;$PKCLASS;g" >>$FILE
	;;
    esac

    echo "}" >>$FILE
fi

# --------------------------------------------------------
# Build Makefile
# --------------------------------------------------------

FILE=${BNAME}.mk
>$FILE
if [ "$TYPE" != "MD" ]
then
    cat $TEMPLATE/M1 | 
    sed "s;@PACKAGE@;$PACKAGE;" | 
    sed "s;@TYPE@;$TYPE;g" | 	
    sed "s;@MYJAR@;$MYJAR;" |
    sed "s;@BNAME@;$BNAME;g" >>$FILE
else
    cat $TEMPLATE/M2 | 
    sed "s;@PACKAGE@;$PACKAGE;" | 
    sed "s;@TYPE@;$TYPE;g" | 	
    sed "s;@MYJAR@;$MYJAR;" |	
    sed "s;@BNAME@;$BNAME;g" >>$FILE
fi  

# --------------------------------------------------------
# Build a jonas.properties file
# --------------------------------------------------------
FILE=jonas.properties
echo "# List of deployment descriptors. " >>$FILE
echo "jonas.service.ejb.descriptors	myjar.jar" >>$FILE

# --------------------------------------------------------
# Build a mini client for this bean
# --------------------------------------------------------
FILE=${BNAME}Client.java
>$FILE
if [ "$TYPE" != "MD" ]
then
cat $TEMPLATE/Client.java |
    sed "s;@BNAME@;$BNAME;g" |
    sed "s;@TYPE@;$TYPE;g" | 
    sed "s;@MYJAR@;$MYJAR;" | 
    sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
else
cat $TEMPLATE/ClientJms.java |
    sed "s;@BNAME@;$BNAME;g" |
    sed "s;@TYPE@;$TYPE;g" | 
    sed "s;@MYJAR@;$MYJAR;" | 
    sed "s;@PACKAGE@;$PACKAGE;" >>$FILE
fi
# --------------------------------------------------------
# ok
# --------------------------------------------------------
echo "Your bean files have been created. You can now customize them."
