    
    static final String tableName = "@MYJAR@@BNAME@@TYPE@";
    
    /**
     * A container invokes this method before it removes the EJB object
     * that is currently associated with the instance. This method is
     * invoked when a client invokes a remove operation on the enterprise Bean's
     * home interface or the EJB object's remote interface. This method
     * transitions the instance from the ready state to the pool of available
     * instances. 
     *
     * This method is called in the transaction context of the remove operation.
     * @throws RemoveException  The enterprise Bean does not allow destruction of the object.
     * @throws EJBException - Thrown by the method to indicate a failure caused by a system-level
     * error.
     */
    public void ejbRemove() throws RemoveException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbRemove");

	// Access database to remove bean in table
	// -> This part does not exist if Container Managed Persistence
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    @PKCLASS@ pk = (@PKCLASS@)(ejbContext.getEJBObject().getPrimaryKey());
	    conn = getConnection();
	    stmt = conn.prepareStatement("delete from "+tableName+" where c_field3=?");
	    stmt.setString(1, pk); // <- change this if pk is not a String
            stmt.executeUpdate();
        } catch (Exception e) {
            throw new RemoveException(e.toString());
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by loading it state from the underlying database. 
     * This method always executes in the proper transaction context.
     *
     * @throws EJBException Thrown by the method to indicate a failure caused by
     * a system-level error.
     */
    public void ejbLoad() {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbLoad");

	// Access database to load bean state from table
	// -> This part does not exist if Container Managed Persistence
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    @PKCLASS@ pk = (@PKCLASS@)(ejbContext.getEJBObject().getPrimaryKey());
	    conn = getConnection();
	    stmt = conn.prepareStatement("select c_field1, c_field2, c_field3 from "+tableName+" where c_field3=?");
	    stmt.setString(1, pk); // <- change this if pk is not a String
            ResultSet rs = stmt.executeQuery();
            if (rs.next() == false) {
                throw new EJBException("Failed to load bean from database");
            }
            field1 = rs.getInt("c_field1");
            field2 = rs.getInt("c_field2");
            field3 = rs.getString("c_field3");
        } catch (Exception e) {
            throw new EJBException(e);
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by storing it to the underlying database. 
     * This method always executes in the proper transaction context.
     *
     * @throws EJBException Thrown by the method to indicate a failure caused by
     * a system-level error.
     */
    public void ejbStore() {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbStore");

	// Access database to store bean state in table
	// -> This part does not exist if Container Managed Persistence
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    @PKCLASS@ pk = (@PKCLASS@)(ejbContext.getEJBObject().getPrimaryKey());
	    conn = getConnection();
	    stmt = conn.prepareStatement("update "+tableName+" set c_field1=?, c_field2=? where c_field3=?");
	    stmt.setInt(1, field1);
	    stmt.setInt(2, field2);
	    stmt.setString(3, pk); // <- change this if pk is not a String
            stmt.executeUpdate();
        } catch (Exception e) {
            throw new EJBException(e);
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

    }
	
    /**
     * There must be an ejbPostCreate par ejbCreate method
     *
     * @throws CreateException Failure to create an entity EJB object.
     */
    public void ejbPostCreate(int f1, int f2, String f3) throws CreateException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbPostCreate");
    }
	
    /**
     * The Entity bean can define 0 or more ejbCreate methods.
     *
     * @throws CreateException Failure to create an entity EJB object.
     * @throws DuplicateKeyException An object with the same key already exists. 
     */
    public @PKCLASS@ ejbCreate(int f1, int f2, String f3) throws CreateException, DuplicateKeyException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbCreate");

	// Init here the bean fields
	field1 = f1;
	field2 = f2;
	field3 = f3;

	// Access database to create entry in table
	// -> This part does not exist if Container Managed Persistence
        @PKCLASS@ pk = new @PKCLASS@();
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    conn = getConnection();
	    stmt = conn.prepareStatement("insert into "+tableName+" (c_field1, c_field2, c_field3) values (?, ?, ?)");
	    stmt.setInt(1, field1);
	    stmt.setInt(2, field2);
	    stmt.setString(3, field3); // <- change this if pk is not a String
            stmt.executeUpdate();
        } catch (Exception e) {
	    Trace.errln("Failed to create bean in database:"+e);
            throw new CreateException(e.toString());
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

	return pk;
    }

    /**
     * Each Entity must define at least this finder method.
     *
     * @throws FinderException failure to find the requested EJB object
     */
    public @PKCLASS@ ejbFindByPrimaryKey(@PKCLASS@ pk) throws FinderException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbFindByPrimaryKey");

	// Access database to find entry in table
	// -> This part does not exist if Container Managed Persistence
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    conn = getConnection();
	    stmt = conn.prepareStatement("select c_field3 from "+tableName+" where c_field3=?");
	    stmt.setString(1, pk); // <- change this if pk is not a String
            ResultSet rs = stmt.executeQuery();
            if (rs.next() == false) {
                throw new ObjectNotFoundException("Object not found in database (findByPrimaryKey)");
            }
        } catch (SQLException e) {
            throw new FinderException("Failed to find bean in database (findByPrimaryKey)");
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

	return pk;
    }

    /**
     * Find by field1: Another possible finder method.
     *
     * @throws FinderException failure to find the requested EJB object
     */
    public @PKCLASS@ ejbFindByField1(int f1) throws FinderException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbFindByField1");
	java.lang.String pk;

	// Access database to find entry in table
	// -> This part does not exist if Container Managed Persistence
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    conn = getConnection();
	    stmt = conn.prepareStatement("select c_field3 from "+tableName+" where c_field1=?");
	    stmt.setInt(1, f1);
            ResultSet rs = stmt.executeQuery();
            if (rs.next() == false) {
                throw new ObjectNotFoundException("Object not found in database (findByField1)");
            }
	    pk = rs.getString("c_field3"); // <- change this if pk is not a String
        } catch (SQLException e) {
            throw new FinderException("Failed to find bean in database (findByField1)");
	} finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}

	return pk;
    }

    /**
     * Find all beans
     */
    public Collection ejbFindAll() throws FinderException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbFindAll");

	Vector vect = new Vector();
	Connection conn = null;
	PreparedStatement stmt = null;
	try {
	    conn = getConnection();
	    stmt = conn.prepareStatement("select c_field3 from "+tableName);
            ResultSet rs = stmt.executeQuery();
            while (rs.next()) {
                @PKCLASS@ pk = rs.getString("c_field3"); // <- change this if pk is not a String
                vect.addElement((Object)pk);
            }
        } catch (SQLException e) {
            throw new FinderException("Failed to find bean from database (findAll)");
        } finally {
	    try {
		if (stmt != null) stmt.close();
		if (conn != null) conn.close();
	    } catch (Exception ignore) {
	    }
	}
	return vect;
    }

    private Connection getConnection() throws SQLException {
	if (dataSource == null) {
	    // Finds DataSource from JNDI
	    try {
		initialContext = new InitialContext();
		dataSource = (DataSource)initialContext.lookup("java:comp/env/jdbc/mydb");
	    } catch (Exception e) {
		Trace.errln("Pb with naming context");
		throw new javax.ejb.EJBException("Pb with naming context ");
	    }
	}
	return dataSource.getConnection();
    }

    private DataSource dataSource = null;
    private Context initialContext = null;
