    
    /**
     * A container invokes this method before it removes the EJB object
     * that is currently associated with the instance. This method is
     * invoked when a client invokes a remove operation on the enterprise Bean's
     * home interface or the EJB object's remote interface. This method
     * transitions the instance from the ready state to the pool of available
     * instances. 
     *
     * This method is called in the transaction context of the remove operation.
     * @throws RemoveException  The enterprise Bean does not allow destruction of the object.
     * @throws EJBException - Thrown by the method to indicate a failure caused by a system-level
     * error.
     */
    public void ejbRemove() throws RemoveException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbRemove");
    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by loading it state from the underlying database. 
     * This method always executes in the proper transaction context.
     *
     * @throws EJBException Thrown by the method to indicate a failure caused by
     * a system-level error.
     */
    public void ejbLoad() {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbLoad");
    }

    /**
     * A container invokes this method to instruct the instance to synchronize
     * its state by storing it to the underlying database. 
     * This method always executes in the proper transaction context.
     *
     * @throws EJBException Thrown by the method to indicate a failure caused by
     * a system-level error.
     */
    public void ejbStore() {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbStore");
    }
	
    /**
     * There must be an ejbPostCreate par ejbCreate method
     *
     * @throws CreateException Failure to create an entity EJB object.
     */
    public void ejbPostCreate(int f1, int f2, String f3) throws CreateException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbPostCreate");
    }
	
    /**
     * The Entity bean can define 0 or more ejbCreate methods.
     *
     * @throws CreateException Failure to create an entity EJB object.
     * @throws DuplicateKeyException An object with the same key already exists. 
     */
    public @PKCLASS@ ejbCreate(int f1, int f2, String f3) throws CreateException, DuplicateKeyException {
	Trace.outln(trace, "@BNAME@@TYPE@ ejbCreate");

	// Init here the bean fields
	field1 = f1;
	field2 = f2;
	field3 = f3;

	// In CMP, should return null.
	return null;
    }
