/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * Christophe Germain
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.common;
    

/**
 * This class allows to manipulate jar files by providing
 * add, remove, replace, extract functionalities.
 */

import java.io.File;
import java.io.FileReader;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;

import java.util.Enumeration;
import java.util.Hashtable;

import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import java.util.zip.ZipEntry;

public class Jjar {

    // Name of the jar
    private String jFile;
    // Hastable of the ZipEntries of the jar (ZipEntry-Name, ZipEntry)
    private Hashtable zip;

    /**
     * Constructor method of Jjar class.
     * @param String jarFile
     */
    public Jjar(String jarFile) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar("+jarFile+")");

	this.zip = new Hashtable();
	this.jFile = jarFile;

	File fTemp= new File(this.jFile);
	if (fTemp.exists()) {
	    this.read();
	}
    }


   /**
     * Construtor method of Jjar class.
     * @param String jarFile
     * @param String[] file initial state
     */
    public Jjar(String jarFile, String[] file)
	throws java.io.FileNotFoundException, java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar("+jarFile+", "+file+")");

	//si le fichier jarfile existe, alors open et remplissage de zipentry
	//sinon create()

	this.zip = new Hashtable();
	this.jFile = jarFile;
	try{
	    if (TraceCore.isDebug) TraceCore.debug("Writing in "+jarFile);
	    FileOutputStream jarFOS = new FileOutputStream(this.jFile);
	    ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(jarFOS));
	    for (int i=0; i<file.length; i++){
		if (TraceCore.isDebug) TraceCore.debug("Writing file "+file[i]);
		BufferedReader in = new BufferedReader(new FileReader(file[i]));
		out.putNextEntry(new ZipEntry(file[i]));
		this.add(file[i]);
		int c;
		while ((c = in.read()) != -1)
		    out.write(c);
		in.close();
	    }
	    out.close();
	    jarFOS.close();
	} catch (java.io.FileNotFoundException fnfe) {
	    TraceCore.error("Jjar("+jarFile+", "+file+") ERROR: ", fnfe);
	    throw fnfe;
	} catch (java.io.IOException ioe) {
	    TraceCore.error("Jjar("+jarFile+", "+file+") ERROR: ", ioe);
	    throw ioe;
	} 

     }


    /**
     * Read a Jar file.
     *
     */
    private void read() throws java.io.IOException {
	ZipEntry ze;
	if (TraceCore.isDebug) TraceCore.debug("Jjar.read(): jFile = "+this.jFile);
	try {
	    FileInputStream f = new FileInputStream(this.jFile);
	    ZipInputStream in = new ZipInputStream(new BufferedInputStream(f));
	    while ((ze = in.getNextEntry()) != null){
		this.zip.put(ze.toString(),ze);
	    }
	    in.close();
	    f.close();
	} catch (java.io.IOException ioe) {
	    TraceCore.error("Jjar.read() ERROR: ", ioe);
	    throw ioe;
	}
    }


    /**
     * List  a Jar file content.
     * @return table of String
     */ 

    public String[] list() {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.list(): jFile = "+this.jFile);
	String [] temp = new String[zip.size()];
	int i = 0;
	for (Enumeration e = this.zip.keys(); e.hasMoreElements();){
	    temp[i] = new String((String)e.nextElement());
	    i++;
	}
	return(temp);
    }


    /**
     * Add a file in a Jar file.
     * @param String file
     */
    public void add(String file) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.add("+file+")");
	String[] files = new String[1];
	files[0] = file;
	this.add(files);
    }


    /**
     * Add files in a Jar file.
     * @param String file
     */
    public void add(String[] files) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.add("+files+")");

	try {

	    FileOutputStream jarFOS = new FileOutputStream(this.jFile);
	    ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(jarFOS));
	    for (Enumeration e = this.zip.keys(); e.hasMoreElements();){
		String key= (String)e.nextElement();
		BufferedReader in = new BufferedReader(new FileReader(key));
		out.putNextEntry((ZipEntry)this.zip.get(key));
		int c;
		while ((c = in.read()) != -1)
		    out.write(c);
		in.close();
	    }
	    for (int nf=0; nf<files.length; nf++) {
		String file = files[nf];
		BufferedReader in = new BufferedReader(new FileReader(file));	
		ZipEntry zt = new ZipEntry(file);
		out.putNextEntry(zt);
		this.zip.put(zt.toString(),zt);
		int c;
		while ((c = in.read()) != -1){
		    out.write(c);
		}
		in.close();
	    }
	    out.close();
	    jarFOS.close();

	} catch (java.io.FileNotFoundException fnfe) {
	    TraceCore.error("Jjar.add("+files.toString()+") ERROR: ", fnfe);
	    throw fnfe;
	} catch (java.io.IOException ioe) {
	    TraceCore.error("Jjar.add("+files.toString()+") ERROR: ", ioe);
	    throw ioe;
	}
    }
    

    /**
     * remove a file in a Jar file.
     * @param String file
     */
    public void remove(String file) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.remove("+file+")");

	try {
	    FileOutputStream jarFOS = new FileOutputStream(this.jFile);
	    ZipOutputStream out = new ZipOutputStream(new BufferedOutputStream(jarFOS));
	    for (Enumeration e = this.zip.keys(); e.hasMoreElements();){
		String key= (String)e.nextElement();
		if (!key.equals(file)) {
		    BufferedReader in = new BufferedReader(new FileReader(key));
		    out.putNextEntry((ZipEntry)this.zip.get(key));
		    int c;
		    while ((c = in.read()) != -1)
			out.write(c);
		    in.close();
		}
		else {
		    this.zip.remove(file);
		}
	    }
	    //out.close();
	    //jarFOS.close();
	} catch (java.io.FileNotFoundException fnfe) {
	    TraceCore.error("Jjar.remove("+file+"): "+fnfe);
	    throw fnfe;
	} catch (java.io.IOException ioe) {
	    TraceCore.error("Jjar.remove("+file+"): "+ioe);
	    throw ioe;
	}
    }	


    /**
     * Replace a file by another in a Jar file.
     * @param String oldfile
     * @param String newfile
     */
    public void replace(String oldFile, String newFile) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.replace("+oldFile+", "+newFile+")");
	this.remove(oldFile);
	this.add(newFile);
    }


    /**
     * extract a Jar file under a directory.
     * @param String directory
     */
    public void extract(String directory) throws java.io.IOException {
	if (TraceCore.isDebug) TraceCore.debug("Jjar.extract("+directory+")");
	
	try {

	    // check the true directory
	    if (!directory.endsWith(File.separator)) {
		directory = directory.concat(File.separator);
	    }

	    // check if the directory exist
	    // if it doesn't creation of this directory
	    File currentDir = new File(directory);
	    if (!currentDir.exists()){
		currentDir.mkdir();
	    }	
	    
	    //read the jar file
	    FileInputStream fi = new FileInputStream(this.jFile);
	    ZipInputStream in = new ZipInputStream(new BufferedInputStream(fi));
	    if (TraceCore.isDebug) TraceCore.debug("Reading file :"+this.jFile);
	    
	    // write files of the jar file on the disk
	    ZipEntry ze;
	    while ((ze = in.getNextEntry()) != null) {
		String dest = directory.concat(ze.getName());
		File filetemp = new File(dest);
		if (TraceCore.isDebug) TraceCore.debug("destination "+dest);

		if (filetemp.exists()){
		    if (TraceCore.isDebug) TraceCore.debug(filetemp.toString()+" already exists");
		}

		if (!ze.isDirectory()) {
		    if (filetemp.getPath()!=null){
			File dir = new File(filetemp.getParent());
			if (!dir.exists()) {
			    if (TraceCore.isDebug) TraceCore.debug("make directory "+dir.toString());
			    dir.mkdirs();
			}
		    }
		    filetemp.createNewFile();
		    FileOutputStream out = new FileOutputStream(new String(dest));
		    int c;
		    while ((c = in.read()) != -1){
			out.write(c);
		    }
		    out.flush();
		    out.close();
		    if (TraceCore.isDebug) TraceCore.debug("writing file "+dest);
		}
		else {
		    if (TraceCore.isDebug) TraceCore.debug("Writing a directory "+filetemp);
		    filetemp.mkdirs();
		}
	    }	
	    in.close();
	    fi.close();

	} catch (java.io.FileNotFoundException e) {
	    TraceCore.error("Jjar.extract("+directory+"): ", e);
	    throw e;
	} catch (java.io.IOException ioe) {
	    TraceCore.error("Jjar.extract("+directory+"): ", ioe);
	    throw ioe;
	}
    }

}
