/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   CheckBoxList                     */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.GridBagConstraints;

import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import javax.swing.JCheckBox;
import javax.swing.JComponent;

/**
 * Component to display a label with a list of check boxes.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class CheckBoxList extends DeployerComponent implements ItemListener {

    static private final int trace = Trace.DB_19;

    private Vector buttonListNames;
    private Vector buttonList;

    /**
     * Build the component.
     * @param labelName is the name of the label.
     * @param buttonListNames is a list of button's name inside a vector.
     * @param obligatory is true if the field is obligatory,
     *        and false if it is optional.
     */
    public CheckBoxList(String labelName, Vector buttonListNames, boolean obligatory) {
	super(labelName,obligatory);
	this.buttonListNames = buttonListNames;
	init();
    }

    /**
     * Add a label and a list of check boxes.
     */
    private void init() {
	addLabel();
	addCheckBoxes();
    }

    /**
     * Add a JCheckBox component in the panel and in a vector of buttons' names.
     */
    private void addCheckBoxes() {
       
	String currentButtonName;
	JCheckBox currentButton;

	buttonList=new Vector();
	int i = 0;
	for (Enumeration enum = buttonListNames.elements() ; enum.hasMoreElements() ;) {
	    currentButtonName = (String)enum.nextElement();
	    currentButton = new JCheckBox(currentButtonName);

	    constraints.gridx = Consts.GRIDX_FIRST+1;
	    constraints.gridy = Consts.GRIDY_FIRST+i;
	    constraints.gridwidth  = Consts.GRIDWIDTH;
	    constraints.gridheight = Consts.GRIDHEIGHT;
	    constraints.weightx = Consts.WEIGHTX;
	    constraints.weighty = Consts.WEIGHTY;
	    constraints.anchor = GridBagConstraints.WEST;
	    panelLayout.setConstraints(currentButton,constraints);

	    currentButton.addItemListener(this);
	    buttonList.add(currentButton);

	    this.add(currentButton);
	    i++;
	}
    }

    /**
     * Obligatory method to use Item Listener.
     * @param e is the ItemEvent.
     */
    public void itemStateChanged(ItemEvent e){}

    /**
     * Permits to know if the JCheckBox is selected
     * @param string it's the names's button to select
     * @param boolean selected
     */
    public void setSelected(String elem,boolean selected){
	int i = this.buttonListNames.indexOf(elem);
	((JCheckBox)this.buttonList.elementAt(i)).setSelected(selected);
    }
    
    /**
     * Permits to selected an button if the JCheckBox is selected
     */
    public boolean isSelectedAt(int elemNumber){
	return ((JCheckBox)this.buttonList.elementAt(elemNumber)).isSelected();
    }

    /**
     * Get the list of the buttons' names.
     */
    public Vector getJCheckBoxes(){
	return  buttonList;
    }

    /**
     * Get the specific JComponent of this panel.
     * Not used for CheckBoxList.
     */
    JComponent getSpecificJComponent() {
	throw new Error("CheckBoxList.getSpecificJComponent : there are a lot of JCheckBox");
    }
    
    /**
     * Get a JCheckBox with its number.
     * @param elemNumber is the place of the  in the panel
     *        
     */
    public JCheckBox getJCheckBoxAt(int elemNumber) {
	return (JCheckBox)this.buttonList.elementAt(elemNumber);
    }

    /**
     * Print all selected buttons.
     * Method used only for testing the component.
     */
    public void print(){
	Trace.outln(trace, "CheckBoxList.print(): selected buttons are:");
	for (int i=0;i<buttonList.size();i++){
	    // TO DO: Add the button 'names'
	    Trace.outln(trace,
			(new Boolean(((JCheckBox)buttonList.elementAt(i)).isSelected())).toString());
	}	 
    }

}
