/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 DeployerComponent                  */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JComponent;
import javax.swing.SwingConstants;

/**
 * Mother abstract class for each component.
 * <p>
 * A component is indeed a JPanel with
 * <li> a JLabel
 * <li> at least one JComponent
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public abstract class DeployerComponent extends JPanel {

    /** GridBagLayout of the component. **/
    GridBagLayout panelLayout;

    /** GridBagConstraints of the component. **/
    GridBagConstraints constraints;

    /** Label of the component. **/
    JLabel label;

    /** Label's name of the component. **/
    String labelName;

    /** Boolean to know if the component is obligatory for the DTD. **/
    boolean obligatory;

    /**
     * Initialise the label's name and the obligatory attribute.
     * @param labelName is the label's name of the component.
     * @param obligatory is the obligatory attribute of the component.
     */
    protected DeployerComponent(String labelName, boolean obligatory) {
	this.labelName = labelName;
	this.obligatory = obligatory;
	if (obligatory)
	    this.labelName = this.labelName.concat(Consts.OBLIGATORY_STAR);
	this.panelLayout = new GridBagLayout();
	this.setLayout(this.panelLayout);
	this.constraints = new GridBagConstraints();
    }

    /**
     * Permits to check if the field is obligatory given the boolean.
     * @return true if the component is obligatory, and false if it is optional.
     */
    public boolean checkObligatory(){
	return this.obligatory;
    }

    /**
     * Add a label above left of the component.
     */
    public void addLabel() {
	this.constraints.gridx = Consts.GRIDX_LABEL;
	this.constraints.gridy = Consts.GRIDY_LABEL;
	this.constraints.insets = new Insets(Consts.TOP_INSETS_LABEL,
					     Consts.LEFT_INSETS_LABEL,
					     Consts.BOTTOM_INSETS_LABEL,
					     Consts.RIGHT_INSETS_LABEL);
	this.constraints.gridwidth  = Consts.GRIDWIDTH;
	this.constraints.gridheight = Consts.GRIDHEIGHT;
	this.constraints.weightx = Consts.WEIGHTX;
	this.constraints.weighty = Consts.WEIGHTY;
	this.constraints.anchor = GridBagConstraints.WEST;

	this.label = new JLabel();
	this.label.setHorizontalTextPosition(SwingConstants.LEFT);
	this.label.setText(this.labelName);
	this.panelLayout.setConstraints(this.label,this.constraints);
	this.add(this.label);
    }

    /**
     * Set the specific JComponent of the panel enable or not.
     * @param flag is true is the JComponent must be enabled.
     */
    public void setEnabled(boolean flag) {
	getSpecificJComponent().setEnabled(flag);
    }

    /**
     * Get the specific JComponent of the panel.
     * @return the specific JComponent of the panel.
     */
    abstract JComponent getSpecificJComponent();

    /**
     * Method used only for testing the component.
     */
    public abstract void print();
}
