/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   FileChooserElement               */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.io.File;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JComponent;
import javax.swing.filechooser.FileFilter;


/**
 * Component to display a TextFieldElement and a bouton for display filechooser.
 * This component is composed by :
 * <li> a JLabel,
 * <li> a JTextField,
 * <li> a JButton.
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class FileChooserElement extends DeployerComponent implements ActionListener, FocusListener {

    static private final int trace = Trace.DB_19;

    private JPanel             intermediaryPanel;
    private GridBagLayout      intermediaryPanelLayout;
    private GridBagConstraints intermediaryConstraints;

    private JTextField textField;
    private JButton    buttonFile;

    private JFileChooser fc;
    private boolean selectionMultiple = false;
    private File         file;
    private File[]       files;
    /**
     * Builds the component with a TextFieldElement and a TextAreaDescription.
     * @param labelNameTextField  the label's name of the TextFieldElement.
     * @param obligatoryTextField true if the filling of the TextFieldElement is obligatory,
     *                            and false if it is optional.
     */
    public FileChooserElement(String labelName, boolean obligatory) {
	super(labelName,obligatory);
	init();
    }
    
    /**
     * Builds the component with a TextFieldElement and a TextAreaDescription.
     * @param labelNameTextField  the label's name of the TextFieldElement.
     * @param obligatoryTextField true if the filling of the TextFieldElement is obligatory,
     *                            and false if it is optional.
     * @param selectionMultiple is the filechoose uses the multiple selection
     */
    public FileChooserElement(String labelName, boolean obligatory, boolean selectionMultiple) {
	this(labelName,obligatory);
	this.selectionMultiple = selectionMultiple; 
	this.fc.setMultiSelectionEnabled(selectionMultiple);

    }

    /**
     * Initialize a panel with a GridBagLayout and its constraints.
     * Add a TextFieldElement and a TextAreaDescription.
     */
    private void init() {
	// Intermediary panel for a good display of the component in 
	// comparison with other components in the same panel.
	intermediaryPanel = new JPanel();
	intermediaryPanelLayout = new GridBagLayout();
	intermediaryPanel.setLayout(intermediaryPanelLayout);
	intermediaryConstraints = new GridBagConstraints();

	// Add a JLabel
	addLabel();

	// Add a JTextField
	addTextField();
	// Add a Button intitulated "..."
	addButton();

	// Add the intermediary panel in the component's panel.	
	constraints.gridx = Consts.GRIDX_FIRST+1;
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.gridwidth  = Consts.GRIDWIDTH;
	constraints.gridheight = Consts.GRIDHEIGHT;
	constraints.weightx = Consts.WEIGHTX;
	constraints.weighty = Consts.WEIGHTY;
	constraints.anchor = GridBagConstraints.EAST;
	panelLayout.setConstraints(intermediaryPanel,constraints);
	this.add(intermediaryPanel);
	
        //Create a file chooser
         fc = new JFileChooser();
    }

    /**
     * Add a JTextField component in the panel.
     */
    private void addTextField() {
	intermediaryConstraints.gridx = Consts.GRIDX_FIRST;
	intermediaryConstraints.gridy = Consts.GRIDY_FIRST;
	intermediaryConstraints.gridwidth  = Consts.GRIDWIDTH;
	intermediaryConstraints.gridheight = Consts.GRIDHEIGHT;
	intermediaryConstraints.weightx = Consts.WEIGHTX;
	intermediaryConstraints.weighty = Consts.WEIGHTY;
	intermediaryConstraints.anchor = GridBagConstraints.EAST;
	
	this.textField = new JTextField();
	this.textField.setPreferredSize(new Dimension(Consts.WITDH_TEXT_FIELD_FILE_CHOOSER,
						      Consts.HEIGHT_TEXT_FIELD_FILE_CHOOSER));
	this.textField.setHorizontalAlignment(JTextField.LEFT);
	this.textField.addFocusListener(this);

	intermediaryPanelLayout.setConstraints(textField,intermediaryConstraints);
	intermediaryPanel.add(this.textField);
    }

    /**
     * Add a JButton.
     */
    private void addButton() {
	intermediaryConstraints.gridx = Consts.GRIDX_FIRST+1;
	intermediaryConstraints.gridy = Consts.GRIDY_FIRST;
	intermediaryConstraints.gridwidth  = Consts.GRIDWIDTH;
	intermediaryConstraints.gridheight = Consts.GRIDHEIGHT;
	intermediaryConstraints.weightx = Consts.WEIGHTX;
	intermediaryConstraints.weighty = Consts.WEIGHTY;
	intermediaryConstraints.anchor = GridBagConstraints.EAST;
	
	buttonFile = new JButton(Consts.SUSPENSION_LABEL);
	buttonFile.setPreferredSize(new Dimension(Consts.WITDH_SUSPENSION_BUTTON,
						  Consts.HEIGHT_SUSPENSION_BUTTON));
	buttonFile.setVerticalTextPosition(AbstractButton.CENTER);
        buttonFile.setHorizontalTextPosition(AbstractButton.CENTER);
	buttonFile.addActionListener(this);

	intermediaryPanelLayout.setConstraints(buttonFile,intermediaryConstraints);
	intermediaryPanel.add(this.buttonFile);
    }

    /**
     * Obligatory method to use Focus Listener.
     * @param fe is the FocusEvent.
     */
    public void focusGained(FocusEvent fe) {}
    
    /**
     * Obligatory method to use Focus Listener.
     * @param fe is the FocusEvent.
     */
    public void focusLost(FocusEvent fe) {}

    /**
     * Obligatory method to use Action Listener.
     * @param e is the ActionEvent.
     */
    public void actionPerformed(ActionEvent e) {
	this.fc.rescanCurrentDirectory();
	if (selectionMultiple) {
	    actionFiles();
	} else {
	    actionOneFile();
	}
    }
    
    private void actionOneFile(){
	if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
	    file = fc.getSelectedFile();
	    //this is where a real application would open the file.
	    textField.setText(file.getName());
	    Trace.outln(trace,
			"FileChooserElement.actionOneFile(): Opening '" +
			file.getName() + "'");
	} else {
	    Trace.outln(trace,
			"FileChooserElement.actionOneFile(): Open command cancelled by user.");
	}
    }

    private void actionFiles(){
	Trace.outln(trace,
		    "FileChooserElement.actionFiles(): fc.isMultiSelectionEnabled()="+
		    fc.isMultiSelectionEnabled());
	if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
	    files = fc.getSelectedFiles();
	    // To work around a knowned bug in javax.swing.JFileChooser in case of multi-selection
	    // on some platforms !!!
	    if (files.length == 0) {
		files = new File[1];
		files[0] =  fc.getSelectedFile();
		Trace.outln("BeanDeployer Warning: Work around a knowned bug in javax.swing.JFileChooser in case of multi-selection, take only the first selected file !!!");
	    }
	    String s = displayTextField(files);
	    textField.setText(s);
	    Trace.outln(trace,
			"FileChooserElement.actionFiles(): NB selected files="+files.length);
	    Trace.outln(trace,
			"FileChooserElement.actionFiles(): Opening '" + s);
	} else {
	    Trace.outln(trace,
			"FileChooserElement.actionFiles(): Open command cancelled by user.");
	}
    }

    private String displayTextField(File[] files) {
	String text = new String("");
	if (files.length > 0) {
	    text=text.concat(files[0].getName());
	    for (int i = 1 ; i < files.length ; i++){
		text=text.concat(";"+files[i].getName());
	    }
	}
	return text;
    }
    
    /**
     * Get the JFileChooser of the component.
     * @return the specific component of the panel.
     */
    JComponent getSpecificJComponent() {
	return (JComponent)this.fc;
    }
    
    /**
     * Set the component enable or not.
     * @param bool true if the FileChooserElement must be enabled.
     */
    public void setEnabled(boolean bool){
	this.textField.setEnabled(bool);
	this.buttonFile.setEnabled(bool);
    }
    /**
     * Sets the specified boolean to indicate whether or not this TextComponent should be editable. A
     * PropertyChange event ("editable") is fired when the state is changed.
     **/
    public void setEditable(boolean bool){
	this.textField.setEditable(bool);

    }
    /**
     * Get the text field value.
     * @return the value of the JTextField.
     */
    public String getText() {
	return this.textField.getText();
    }
    
    /**
     * Set the text field value.
     * @param String value of value
     */
    public void setText(String text) {
	this.textField.setText(text);
    }

    /**
     * Get the file.
     * @return the file selected via the file chooser.
     */
    public File getFile() {
	return this.file;
    }

    public File[] getFiles() {
	return this.files;
    }
    /**
     * Adds a filter to the list of user choosable file filters.
     * @param filter the FileFilter to add to the choosable file filter list.
     */
    public void addChoosableFileFilter(FileFilter filter) {
	this.fc.addChoosableFileFilter(filter);
    }

    /**
     * Print all elments of the component.
     * Method used only for testing component.
     */
    public void print() {
	String textField = "The name of the label of the TextField is "
	    +labelName+"and its value is "+getText();

	Trace.outln(trace, "FileChooserList.print(): "+textField+".");
    }
}
