/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 RadioButtonList                    */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JComponent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.GridBagConstraints;

/**
 * Component to display a list of radio button in the same group,
 * and the associated label.
 * @author Christophe Germain and Magali Morael.
 */
public class RadioButtonList extends DeployerComponent implements ActionListener{

    static private final int trace = Trace.DB_19;

    private String itemSelected;
    private Vector buttonListNames;
    private Hashtable buttonListButton;

    private ButtonGroup group;

    /**
     * Build the radio button list.
     * @param labelName is the name of the label.
     * @param buttonListNames is the list of all buttons of the group.
     *        This information is represented in a Vector
     * @param obligatory is true if the filling of the component is obligatory,
     *        and false if it is optional.
     */
    public RadioButtonList(String labelName, Vector buttonListNames, boolean obligatory) {
	super(labelName,obligatory);
	this.buttonListNames = buttonListNames;
	buttonListButton = new Hashtable();
	init();
    }

    /**
     * Add a label and the list of radio buttons.
     */
    private void init() {
	addLabel();
	addRadioButtons();
	itemSelected = new String();
    }
    
    /**
     * Add all JRadioButton component in the panel and in a ButtonGroup Component.
     */
    private void addRadioButtons() {
	JRadioButton currentButton;
	String currentButtonName;
    
	group = new ButtonGroup();
	int i = 0;
	for (Enumeration enum = buttonListNames.elements() ; enum.hasMoreElements() ;) {
	    currentButtonName = (String)enum.nextElement();
	    currentButton = new JRadioButton(currentButtonName);
	    currentButton.setActionCommand(currentButtonName);
	    
	    constraints.gridx = Consts.GRIDX_FIRST+1+i;
	    constraints.gridy = Consts.GRIDY_FIRST;
	    constraints.gridwidth  = Consts.GRIDWIDTH;
	    constraints.gridheight = Consts.GRIDHEIGHT;
	    constraints.weightx = Consts.WEIGHTX;
	    constraints.weighty = Consts.WEIGHTY;
	    panelLayout.setConstraints(currentButton,constraints);

	    buttonListButton.put(currentButtonName,currentButton);
	    currentButton.addActionListener(this);
	    this.add(currentButton);
	    group.add(currentButton);
	    i++;
	}
    }

    /**
     * Obligatory method to use Action Listener.
     * @param e is the ActionEvent.
     */
    public void actionPerformed(ActionEvent e) {
	String temp =  e.getActionCommand();
	if (temp == itemSelected){
	    itemSelected = "";
	    group.setSelected(null,true);

	}
	else 
	    itemSelected = temp;
    }

    /**
     * Get the ButtonGroup component.
     */
    public ButtonGroup getButtonGroup(){
	return this.group;
    }  

    /**
     * Get the specific JComponent of this panel.
     * Not used for RadioButtonList.
     */
    JComponent getSpecificJComponent() {
	throw new Error("RadioButtonList.getSpecificJComponent : there are a lot of JCheckBox");
    }
    
    /**
     * Get the selected radioButton.
     */
    public String getText() {
	return itemSelected;
    }

    /**
     * Set the selected radioButton.
     * @param text is the name of the new selected button.
     */
    public void setText(String text) {
	try{
	    ((JRadioButton)buttonListButton.get(text)).setSelected(true);
	    this.itemSelected = text;
	} catch (java.lang.NullPointerException e){
	    this.itemSelected = null;
	
	}
    }

    /**
     * Set the selected radioButton.
     * @param text is the name of the new selected button.
     * @return if one button has beem choiced or not
     */
    public boolean setTextAndCheck(String text) {
	setText(text);
	Trace.outln(trace,
		    "RadioButtonList.setTextAndCheck("+text+"): buttonListNames.contains() = "+
		    buttonListNames.contains(text));
	return buttonListNames.contains(text);
    }


    /**
     * Print the selected button.
     * Method used only for testing the component.
     */
    public void print(){
	Trace.outln(trace,
		    "RadioButtonList.print(): Selected Button is "+ itemSelected);
    }
}
