/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 TableComponent                     */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Dimension;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JComponent;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.ChangeEvent;

/**
 * Component to display a label with a JTable element.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class TableComponent extends DeployerComponent implements TableModelListener {

    static private final int trace = Trace.DB_19;

    private JScrollPane scrollPane;
    private JTable   table;
    private Vector   data;
    private String[] columnNames;
    
    private MyTableModel myTable;
    //    private MyTableModel model;

    /**
     * Construct a TableComponent.
     * @param labelName  the label's name.
     * @param data       a vector of existing data in the table.
     * @param obligatory true if the table is obligatory,
     *                   else it is false.
     */
    public TableComponent(String labelName, String[] columnNames, Vector data, boolean obligatory) {
	super(labelName,obligatory);
	this.data = data;
	this.columnNames = columnNames;
	init();
    }
    
    /**
     * Add a label and an associated JTable.
     */
    private void init() {
	addLabel();
	addTable();
    }
       
    /**
     * Addition of the JTable composent in the panel
     */
    private void addTable() {
	
	//add a model table
	myTable = new MyTableModel(this.columnNames, this.data);
	myTable.addTableModelListener(this);

	//set contraint of the layout
	constraints.gridx = Consts.GRIDX_SCROLLPANE;
	constraints.gridy = Consts.GRIDY_SCROLLPANE;
	constraints.insets = new Insets(Consts.TOP_INSETS_SCROLLPANE,
					Consts.LEFT_INSETS_SCROLLPANE,
					Consts.BOTTOM_INSETS_SCROLLPANE,
					Consts.RIGHT_INSETS_SCROLLPANE);

	constraints.gridwidth  = Consts.GRIDWIDTH;
	constraints.gridheight = Consts.GRIDHEIGHT;
	constraints.weightx = Consts.WEIGHTX;
	constraints.weighty = Consts.WEIGHTY;
	constraints.anchor = GridBagConstraints.EAST;

	//add a JScrollPane
	scrollPane = new JScrollPane();
	scrollPane.setPreferredSize(new Dimension(Consts.WIDTH_SCROLLPANE,
						  Consts.HEIGHT_SCROLLPANE));
	panelLayout.setConstraints(scrollPane, constraints);
	this.add(scrollPane);
	
	//add a JTable in the JScrollPane
	table = new JTable(myTable);
	
	//myTable.addTableModelListener(this);
	scrollPane.getViewport().add(table);
    }
    
    /**
     * Obligatory method used for Table Model Listener.
     * @param e is the TabelModelEvent.
     */
    public void tableChanged(TableModelEvent e){
	int row = e.getFirstRow();
	int column = e.getColumn();
	int type = e.getType();

	if (type == e.INSERT){
	    int i = 0;
	    Object[] obj =(Object[]) data.elementAt(0);
	    String[] temp = new String[obj.length];
	    while (i <myTable.getColumnCount()) {
		temp[i]="";
		i++;
		}	    
	    data.addElement(temp);
	    this.repaint();
	    }	

	if (type == e.DELETE){
	    data.removeElementAt(row);
	    this.repaint();
	    }

	if (type == e.UPDATE){
	    String val = (String)myTable.getValueAt(row,column);
	    Object[] tempObj =(Object[]) data.elementAt(row);
	    tempObj[column] = val;
	} 
    }

    /**
     * Reading data.
     * Permit to take changing of node into account.
     **/
    public void tableReading() {
	if (table.isEditing()) {
	    int row = table.getEditingRow() ;
	    int col = table.getEditingColumn() ;
	    String data = ((JTextField)table.getEditorComponent()).getText() ;
	    ChangeEvent editEvent = new ChangeEvent(table.getEditorComponent()) ;
	   table.editingStopped(editEvent) ;
	}
    }
 
    /**
     * Get the JTable component. Here it is the JTable.
     * @return the specfic component of the panel.
     */
    JComponent getSpecificJComponent() {
	return (JComponent)this.table;
    }

    /**
     * Get the vector of Jtable's data.
     * @return the vector of data.
     */
    public Vector getData(){
	return this.data;
    }  

    /**
     * Get a data of Jtable's cell.
     * @param numRow number of the row.
     * @param numCol number of the column.
     * @return the object contained in this cell.
     */
    public Object getData(int numRow, int numCol){
	return myTable.getValueAt(numRow, numCol);
    }  

    /**
     * Set the value of JTable.
     * @param value is the value of the table.
     */    
    public void setValue(Vector value){   
       	this.data = value;
	myTable.setValue(value);
    }

    /**
     * Get the number of the Jtable's row.
     * @return the number of rows for the table.
     */
    public int getRowsCount() {
	return table.getRowCount();
    }

    /**
     * Get the number of the table's columns.
     * @return the number of columns for the table.
     */
    public int getColumnCount() {
	return table.getColumnCount();
    }

    /**
     * Set the component enable or not.
     * @param b is true if the component must be enabled.
     */
    public void setEnabled(boolean b){
	table.setCellSelectionEnabled(b);
	table.setEnabled(b);
    }

    /**
     * The method displays Jtable's data.
     */
    public void print() {
        int numRows = table.getRowCount();
        int numCols = table.getColumnCount();

	Trace.outln(trace, "TableComponent.print(): Begin");
	Trace.outln(trace, "  Value of data: ");
        for (int i=0; i < numRows; i++) {
            Trace.out(trace,"    row " + i + ":");
            for (int j=0; j < numCols; j++) {
                Trace.out(trace,"  " + myTable.getValueAt(i, j));
            }
           Trace.outln(trace, " ");
        }
	Trace.outln(trace, "TableComponent.print(): End");
    }
 
/*************************************************************/
    
    /**
     * This class takes care of the management of listners and provides
     * some conveniences for generating TableModelEvents and dispatching
     * them to the listeners.
     */
    class MyTableModel extends AbstractTableModel {
	private int numRows = 0;
	private int numColumns = 0;
	private String[] columnNames; 
	private Vector data;

	/**
	 * Constructor of the AbstractTableModel.
	 * @param columnNames table of titles of the columns.
	 * @param data        vector of data of the table.
	 */
	public MyTableModel(String[] columnNames, Vector data){
	    this.columnNames = columnNames;
	    this.numRows = data.size();
	    this.data = data;
	    Object[] temp =((Object[])data.elementAt(0));
	    this.numColumns = temp.length;
	}
	
	/**
	 * Get the column's name via its index.
	 * @param col index of column.
	 * @return the name of the column.
	 */
	public String getColumnName(int col) { 
	    return columnNames[col].toString(); 
	}

	/**
	 * Get the number of columns.
	 * @return the number of columns.
	 */
	public int getColumnCount() {
	    return this.numColumns; 
	}

	/**
	 * Get the number of rows.
	 * @return the number of rows.
	 */
	public int getRowCount() { 
	    return this.data.size();
	}
        
	/**
	 * Get the value of a cell via its row's index and its column's index.
	 * @param row the index of the row.
	 * @param col the index of the column.
	 * @return the object contained in the cell.
	 */
	public Object getValueAt(int row, int col) { 
	    Object[] tempData = (Object[])data.elementAt(row);
	    Object value = tempData[col];
	    return value;
	}
	
	/*
	 * Don't need to implement this method unless your table's
	 * editable.
	 * @param row the index of the row.
	 * @param col the index of the column.
	 * @return true if the cell is editable.
	 */
	public boolean isCellEditable(int row, int col) {
	    return true;
	}
	
	/*
	 * Invokes when you change all data in the table.
	 * @param value the vector of values to put in the table.
	 */
       	public void setValue(Vector value) {
	    int row =  value.size();
	    this.numRows = value.size();
	    this.data = value;

	    for (int j=0 ; j < row  ;  j++ )
		{
		    String[] temp= (String[])data.elementAt(j);
		    Object[] tempObj =(Object[]) value.elementAt(j);
		    for (int i=0 ; i < temp.length ; i++) {
			tempObj[i] = temp[i];
			setValueAt(tempObj[i],j,i);
		    }
		}
	}

	/*
	 * Permits to change one data's table and manage the representation.
	 * @param value the object to insere.
	 * @param row   the index of the row.
	 * @param col   the index of the column.
	 */
	public void setValueAt(Object value, int row, int col) {
	    boolean emptyRow=true;
	    
	    
	    Object[] tempObj =(Object[]) this.data.elementAt(row);
	    tempObj[col] = value;
	    if (row == (this.numRows-1)){
		fireTableRowsInserted(0,row);
		this.numRows++;
	    }

	    int i = 0;
	    while ( emptyRow && (i <this.numColumns)) {
		if (!((String)tempObj[i]).equals("")){
		    emptyRow = false;
		}
		i++;
	    }
	    
	    if ( emptyRow ){
		fireTableRowsDeleted(row,row);
		this.numRows--;	
	    }
	    fireTableCellUpdated(row,col);
	}
    }
} 
