/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*              TextAreaDescription                   */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Insets;

import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JComponent;

/**
 * Component to display a label "description" with a text area,
 * except if another label is specified.
 * @author Magali MORAEL
 */
public class TextAreaDescription extends DeployerComponent implements FocusListener {

    static private final int trace = Trace.DB_19;

    private JScrollPane scrollPane;
    private JTextArea textArea;

    /**
     * Build the component.
     */
    public TextAreaDescription() {
	super(Consts.LABEL_DESCRIPTION,false);
	init();
    }

    /**
     * Build the component with another label.
     * @param label is the name of the associated label
     * @param obligatory is true if the filling of this textArea is obligatory,
     *        and false if it is optional.
     */
    public TextAreaDescription(String labelName, boolean obligatory) {
	super(labelName,obligatory);
	init();
    }

    /**
     * Initialize the layout and its constraints.
     * Add a label and a textArea.
     */
    private void init() {
	addLabel();
	addTextArea();
    }
    
    /**
     * Add the text area at the right of the component.
     */
    private void addTextArea() {
	constraints.gridx = Consts.GRIDX_SCROLLPANE;
	constraints.gridy = Consts.GRIDY_SCROLLPANE;
	constraints.insets = new Insets(Consts.TOP_INSETS_SCROLLPANE,
					Consts.LEFT_INSETS_SCROLLPANE,
					Consts.BOTTOM_INSETS_SCROLLPANE,
					Consts.RIGHT_INSETS_SCROLLPANE);

	constraints.gridwidth  = Consts.GRIDWIDTH;
	constraints.gridheight = Consts.GRIDHEIGHT;
	constraints.weightx = Consts.WEIGHTX;
	constraints.weighty = Consts.WEIGHTY;
	constraints.anchor = GridBagConstraints.EAST;

	//add a JScrollPane
	scrollPane = new JScrollPane();
	scrollPane.setPreferredSize(new Dimension(Consts.WIDTH_SCROLLPANE,
						  Consts.HEIGHT_SCROLLPANE));

	panelLayout.setConstraints(scrollPane, constraints);
	this.add(scrollPane);
	//add a JTextArea in the JScrollPane
	textArea = new JTextArea();
	textArea.setLineWrap(true);
	textArea.setWrapStyleWord(true);
	textArea.addFocusListener(this);
	scrollPane.getViewport().add(textArea);
    }

    /**
     * Obligatory method to use Focus Listener.
     * @param fe is the FocusEvent.
     */
    public void focusGained(FocusEvent fe) {}
    
    /**
     * Obligatory method to use Focus Listener.
     * @param fe is the FocusEvent.
     */
    public void focusLost(FocusEvent fe) {}

    /**
     * Get the textArea of the component.
     */
    JComponent getSpecificJComponent() {
	return (JComponent)this.textArea;
    }
    
    /**
     * Set the text in the textArea of the component.
     * @param description is the text to insere.
     */
    public void setText(String description){
	this.textArea.setText(description);
    }

    /**
     * Get the value of the textArea.
     */
    public String getText() {
	return this.textArea.getText();
    }

    /**
     * Print the text in the textArea.
     * Method used only for testing component.
     */
    public void print(){
	String temp = textArea.getText();  
	Trace.outln(trace,
		    "TextAreaDescription.print(): For the textArea "+labelName+
		    " the value is "+temp);
    }
}
