/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 TextFieldElement                   */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.components;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Insets;

import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import javax.swing.JTextField;
import javax.swing.JComponent;

/**
 * Component to display a label with a textField element.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class TextFieldElement extends DeployerComponent implements FocusListener {

    static private final int trace = Trace.DB_19;

    private JTextField textField;

    /**
     * Construct a TextFieldElement.
     * @param labelName is the label's name.
     * @param obligatory is true if the textfield is obligatory,
     *        else it is false.
     */
    public TextFieldElement(String labelName, boolean obligatory) {
	super(labelName,obligatory);
	init();		
    }
    
    /**
     * Initialize a panel with a GridBagLayout and its constraints.
     * Add a label and an associated textfield.    
     */
    private void init(){
	// a label
	addLabel();
	
	// a textfield
	addTextField();
    }
    
    /**
     * Add a textfield to the panel.
     */ 
    private void addTextField(){
	constraints.gridx = Consts.GRIDX_FIRST+1;
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.insets = new Insets(Consts.TOP_INSETS_LABEL,
					Consts.LEFT_INSETS_LABEL,
					Consts.BOTTOM_INSETS_LABEL,
					Consts.RIGHT_INSETS_LABEL);
	constraints.gridwidth  = Consts.GRIDWIDTH;
	constraints.gridheight = Consts.GRIDHEIGHT;
	constraints.weightx = Consts.WEIGHTX;
	constraints.weighty = Consts.WEIGHTY;
	constraints.anchor = GridBagConstraints.EAST;

	//	this.textField = new JTextField(Consts.MAX_COLUMNS);
	this.textField = new JTextField();
	this.textField.setPreferredSize(new Dimension(Consts.WITDH_TEXT_FIELD,
						      Consts.HEIGHT_TEXT_FIELD));
	this.textField.setHorizontalAlignment(JTextField.LEFT);
	this.textField.addFocusListener(this);
	this.add(this.textField);
    }

    /**
     * Obligatory method for using Focus Listener.
     */
    public void focusGained(FocusEvent fe) {}
    
    /**
     * Obligatory method for using Focus Listener.
     */
    public void focusLost(FocusEvent fe) {}
    
    /**
     * Get the JTextField component.
     */
    JComponent getSpecificJComponent() {
	return (JComponent)this.textField;
    }
    
    /**
     * Set the value of the textfield.
     * @param fieldString is the new String in the textField.
     */    
    public void setText(String fieldString){
	this.textField.setText(fieldString);
    }
    
    /**
     * Get the value of the textfield.
     */
    public String getText(){
	return this.textField.getText();
    }

    /**
     * Print the value of the textField.
     * Method used only for testing the component.
     */
    public void print() {
	Trace.outln(trace,
		    "TextFieldElement.print(): For the textField "+labelName+
		    " the value is "+ this.getText());
    }    
}
