/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*           BeanDeployer                             */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.gui;


import java.awt.Dimension;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.guicomponents.DeployerTree;
import org.objectweb.jonas.beandeployer.guicomponents.DeployerMenuBar;
import org.objectweb.jonas.beandeployer.guicomponents.PopupListener;
import org.objectweb.jonas.common.Trace;


/**
 * BeanDeployer is the main class of the application BeanDeployer.
 * It is a <I><B>Deployment tool of EJB on the JOnAS platform</B></I>.
 */
public class BeanDeployer extends JFrame {
    // private static boolean DEBUG = false;

    static private final int trace = Trace.DB_19;

    private static JFrame frame;

    private DeployerTree tree;
    private DeployerMenuBar menu;

    private JScrollPane treeView;
    private JPanel      principalRightPanel; 
    private JScrollPane rightPanelView;      
    private JSplitPane  splitPane;

    /**
     * Creates a new frame of the application.
     */
    public BeanDeployer() {
	super(Consts.TITLE_FRAME);

	init();
	menu = DeployerMenuBar.create(this);
    }

    /**
     * Initialisation of the application.
     */
    private void init() {

	// a voir
	principalRightPanel = new JPanel();
        //Create the scroll pane and add the tree to it. 
	tree = DeployerTree.create(principalRightPanel);
        treeView = new JScrollPane(tree);
	
	//Create the rigth viewing pane.
        //principalRightPanel = new JPanel();
        rightPanelView = new JScrollPane(principalRightPanel);
	
	//Add the scroll panes to a split pane.
        splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setLeftComponent(treeView);
        splitPane.setRightComponent(rightPanelView);
	
        Dimension minimumSize = new Dimension(Consts.MINIMUM_SIZE_WIDTH, 
					      Consts.MINIMUM_SIZE_HEIGHT);
        rightPanelView.setMinimumSize(minimumSize);
        treeView.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(Consts.DIVIDER_LOCATION);
	
        getContentPane().add(splitPane);

	//Add listener to components that can bring up popup menus.
        MouseListener popupListener = new PopupListener(this);
        tree.addMouseListener(popupListener);

    }

    /**
     * Main method of the class.
     */
    public static void main(String[] args) {

	// Init Trace system
	Trace.configure();

	frame = new BeanDeployer();
	
	frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    DeployerMenuBar.getInstance().exit();
		}
	    });  
	
	frame.pack(); //minimum frame (adapted for each resolution)
	frame.setVisible(true);

    }    

}
