/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s):
 * Markus Karg: Novell port
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 DeployerMenuBar                    */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.guicomponents;

import org.objectweb.jonas.beandeployer.Consts;

import org.objectweb.common.Jjar;

import org.objectweb.jonas.beandeployer.panels.DeployerPanel;

import org.objectweb.jonas.beandeployer.panels.PRootEjbJar;
import org.objectweb.jonas.beandeployer.panels.PAssemblyDescriptor;
import org.objectweb.jonas.beandeployer.panels.PCmpField;
import org.objectweb.jonas.beandeployer.panels.PContainerTransaction;
import org.objectweb.jonas.beandeployer.panels.PEjbJar;
import org.objectweb.jonas.beandeployer.panels.PEjbRef;
import org.objectweb.jonas.beandeployer.panels.PEnterpriseBeans;
import org.objectweb.jonas.beandeployer.panels.PEntity;
import org.objectweb.jonas.beandeployer.panels.PEnvEntry;
import org.objectweb.jonas.beandeployer.panels.PFinderMethodJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PJonasEjbJar;
import org.objectweb.jonas.beandeployer.panels.PJonasEntity;
import org.objectweb.jonas.beandeployer.panels.PJonasSession;
import org.objectweb.jonas.beandeployer.panels.PJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PMethod;
import org.objectweb.jonas.beandeployer.panels.PMethodPermission;
import org.objectweb.jonas.beandeployer.panels.PResourceRef;
import org.objectweb.jonas.beandeployer.panels.PSecurityRole;
import org.objectweb.jonas.beandeployer.panels.PSecurityRoleRef;
import org.objectweb.jonas.beandeployer.panels.PSession;
import org.objectweb.jonas_ejb.lib.Version;

import java.io.File;
import java.util.zip.ZipFile;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileOutputStream;
import java.io.Writer;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;

import java.io.IOException;
import java.io.FileNotFoundException;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;

import javax.swing.tree.DefaultMutableTreeNode;

import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;

import java.net.URL;


import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.CmpField;
import org.objectweb.jonas_ejb.deployment.xml.ContainerTransaction;
import org.objectweb.jonas_ejb.deployment.xml.EjbJar;
import org.objectweb.jonas_ejb.deployment.xml.EjbRef;
import org.objectweb.jonas_ejb.deployment.xml.EnterpriseBeans;
import org.objectweb.jonas_ejb.deployment.xml.EnvEntry;
import org.objectweb.jonas_ejb.deployment.xml.Entity;
import org.objectweb.jonas_ejb.deployment.xml.FinderMethodJdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.JdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbJar;
import org.objectweb.jonas_ejb.deployment.xml.JonasEntity;
import org.objectweb.jonas_ejb.deployment.xml.JonasSession;
import org.objectweb.jonas_ejb.deployment.xml.Method;
import org.objectweb.jonas_ejb.deployment.xml.MethodPermission;
import org.objectweb.jonas_ejb.deployment.xml.ResourceRef;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRole;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRoleRef;
import org.objectweb.jonas_ejb.deployment.xml.Session;

import org.xml.sax.SAXParseException;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;
import org.objectweb.jonas_ejb.deployment.api.DeploymentDescException;

/**
 * This object represents the menu bar of Bean Deployer
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class DeployerMenuBar extends JMenuBar implements ActionListener{

    static private final int trace = Trace.DB_19;

    private static DeployerMenuBar uniqueMenuBar;
    
    private JMenu menu, menuEdit, menuInsert, menuBean;
    private JMenuItem menuItem, menuSave, menuSaveAs, menuClose, menuItemBean, menuDelete;
    private static JFrame frame;
    private DeployerTree tree;
    private JFileChooser fc = new JFileChooser();
    private String fileLocation;
    private boolean saveRequired = true;

    // oneEjbJar is true if there is yet an Ejb Jar.
    private boolean oneEjbJar = false;

    // all files located in this vector will delest at the end of application
    Vector delFile = new Vector();

    /**
     * Construct a DeployerMenuBar.
     * @param frame is the BeanDeployer's frame.
     */
    private DeployerMenuBar(JFrame frame) {
	URL image;
	this.frame = frame;
	this.tree = DeployerTree.getInstance();
	if (this.tree == null) {
	    Trace.outln(trace, "DeployerMenuBar.DeployerMenuBar(): this.tree == null");
	}
	
	fc.addChoosableFileFilter(new JarFilter());

	// get the classloader
	ClassLoader loader = DeployerMenuBar.class.getClassLoader();
	if (loader == null) {
	    loader = ClassLoader.getSystemClassLoader();
	}
	
	// Create the menu bar.
        frame.setJMenuBar(this);
	
	menu = new JMenu(Consts.FILE);
        menu.setMnemonic(Consts.FILE_MNEMONIC);
	this.add(menu);
	
	//a group of JMenuItems
	image = loader.getResource(Consts.GIF_NEW);
	if (image != null) {
	    menuItem = new JMenuItem(Consts.NEW,
				     new ImageIcon(image));
	} else {
	    menuItem = new JMenuItem(Consts.NEW);
	}
 
	menuItem.setMnemonic(KeyEvent.VK_N); //used constructor instead
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
						       ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);
	
        image = loader.getResource(Consts.GIF_OPEN);
	if (image != null) {
	    menuItem = new JMenuItem(Consts.OPEN, new ImageIcon(image));
	} else {
	    menuItem = new JMenuItem(Consts.OPEN);
	}
	menuItem.setMnemonic(KeyEvent.VK_O);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
						       ActionEvent.CTRL_MASK));
	menuItem.addActionListener(this);
	menu.add(menuItem);

	menu.addSeparator();

	image = loader.getResource(Consts.GIF_SAVE);
        if (image != null) {
	    menuSave = new JMenuItem(Consts.SAVE, new ImageIcon(image));
	} else {
	    menuSave = new JMenuItem(Consts.SAVE);
	}
        menuSave.setMnemonic(Consts.SAVE_MNEMONIC);
	menuSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
						       ActionEvent.CTRL_MASK));
	menuSave.setEnabled(false);
        menuSave.addActionListener(this);
        menu.add(menuSave);

	menuSaveAs = new JMenuItem(Consts.SAVE_AS);
        menuSaveAs.setMnemonic(KeyEvent.VK_A);
	menuSaveAs.setEnabled(false);
        menuSaveAs.addActionListener(this);
        menu.add(menuSaveAs);

	menu.addSeparator();
	
	menuClose = new JMenuItem(Consts.CLOSE);
        menuClose.setMnemonic(KeyEvent.VK_C);
	menuClose.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
							ActionEvent.CTRL_MASK));
	menuClose.setEnabled(false);
        menuClose.addActionListener(this);
	menu.add(menuClose);

	menuItem = new JMenuItem(Consts.EXIT);
        menuItem.setMnemonic(KeyEvent.VK_X);
	menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
						       ActionEvent.CTRL_MASK));
        menuItem.addActionListener(this);
        menu.add(menuItem);

        //  Edit Menu
        menuEdit = new JMenu(Consts.EDIT);
        menuEdit.setMnemonic(Consts.EDIT_MNEMONIC);
	menuEdit.setEnabled(false);
        this.add(menuEdit);

	image = loader.getResource(Consts.GIF_CUT);
	if (image != null) {
	    menuItem = new JMenuItem(Consts.CUT, new ImageIcon(image));
	} else {
	    menuItem = new JMenuItem(Consts.CUT);
	}
	menuItem.setMnemonic(KeyEvent.VK_C);
	menuItem.setEnabled(false);
        menuItem.addActionListener(this);
        menuEdit.add(menuItem);

	image = loader.getResource(Consts.GIF_COPY);	
	if (image != null) {
	    menuItem = new JMenuItem(Consts.COPY, new ImageIcon(image));
	} else {
	    menuItem = new JMenuItem(Consts.COPY);
	}
        menuItem.setMnemonic(KeyEvent.VK_O);
	menuItem.setEnabled(false);
	menuItem.addActionListener(this);
        menuEdit.add(menuItem);

	image = loader.getResource(Consts.GIF_PASTE);	
	if (image != null) {
	    menuItem = new JMenuItem(Consts.PASTE, new ImageIcon(image));
	} else {
	    menuItem = new JMenuItem(Consts.PASTE);
	}
        menuItem.setMnemonic(KeyEvent.VK_P);
	menuItem.setEnabled(false);
	menuItem.addActionListener(this);
        menuEdit.add(menuItem);

	menuEdit.addSeparator();
	
	menuDelete = new JMenuItem(Consts.DELETE);
	menuDelete.setMnemonic(KeyEvent.VK_D);
	menuDelete.addActionListener(this);
        menuEdit.add(menuDelete);
	
	///  Bean
	menuBean = new JMenu(Consts.BEAN);
        menuBean.setMnemonic(Consts.BEAN_MNEMONIC);
	menuBean.setEnabled(false);
	this.add(menuBean);
	
	menuItemBean = new JMenuItem(Consts.NEW_BEAN);
	menuItemBean.setMnemonic(KeyEvent.VK_B);
  	menuItemBean.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,
							   ActionEvent.CTRL_MASK));
	menuItemBean.addActionListener(this);
	menuItemBean.setEnabled(false);
        menuBean.add(menuItemBean);

	///  Insert
	menuInsert = new JMenu(Consts.INSERT);
        menuInsert.setMnemonic(KeyEvent.VK_I);
	// apply our own menulistener beacause this menu is dynamic
	MenuListener menuListener = new InsertMenuListener(this,frame);
	menuInsert.addMenuListener(menuListener);
	menuInsert.setEnabled(false);
        this.add(menuInsert);

	/// Tools
	menu = new JMenu(Consts.TOOLS);
        menu.setMnemonic(Consts.TOOLS_MNEMONIC);
	menu.setEnabled(false);
        this.add(menu);
	
	menuItem = new JMenuItem(Consts.CHECK);
	menuItem.setMnemonic(KeyEvent.VK_C);
	menuItem.setEnabled(false);
        menuItem.addActionListener(this);
        menu.add(menuItem);

	menuItem = new JMenuItem(Consts.INTERPOSITION);
	menuItem.setMnemonic(KeyEvent.VK_I);
	menuItem.setEnabled(false);
        menuItem.addActionListener(this);
        menu.add(menuItem);	
	
	///  Help
	this.add(Box.createHorizontalGlue());
	menu = new JMenu(Consts.HELP);
	menu.setMnemonic(Consts.HELP_MNEMONIC);
        this.add(menu);

	menuItem = new JMenuItem(Consts.ABOUT);
	menuItem.setMnemonic(KeyEvent.VK_A);
	menuItem.addActionListener(this);
	menuItem.setEnabled(true);
        menu.add(menuItem);

    }
     
    /**
     * Creation of the DeployerMenuBar.
     * Invoked by BeanDeployer.
     * @param frame is the frame of the application BeanDeployer.
     * @return the instance of DeployerMenuBar.
     */
    public static DeployerMenuBar create(JFrame frame) {
	Trace.outln(trace, "DeployerMenuBar.create()");
	if (uniqueMenuBar == null) {
	    uniqueMenuBar = new DeployerMenuBar(frame);
	}
	return uniqueMenuBar;
    }
    
    /**
     * Get the instance of the DeployerMenuBar.
     * @return the only instance of DeployerMenuBar
     */
    public static DeployerMenuBar getInstance() {
	if (uniqueMenuBar == null) {
	    Trace.outln(trace, "DeployerMenuBar.getInstance(): uniqueMenuBar == null");
	}
	return uniqueMenuBar;
    }
    
    /**
     * This method permits to have the insert menu (only used by InsertMenuListener).
     * @return the reference of insert item.
     */
     public JMenu getMenuInsert() {
	 return menuInsert;
     }

    /**
     * Refresh the insert menu and Edit menu according to the selected node.
     * @param the selected node in tree
     */
    public void refreshMenu(DefaultMutableTreeNode node) {
	// for insert menu 
	if ((tree.getSelectedNodeType().equals(Consts.STRING_METHOD)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_CMPF)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_EJB_REF)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_ENV)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_RES_REF)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_ROLE)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_ROLE_REF)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_JONAS_SESSION)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_FINDER_METHOD_JM)) ||
	    (node.toString().equals(Consts.STRING_META_INF_EJB_JAR_XML)
	     && this.findAssemblyDescriptor(node) 
	     && this.findEntrepriseBeans(node)) || 
	    (node.toString().equals(Consts.STRING_JONAS_ENTITY)
	     && this.findJdbcMapping(node)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_FILE)) ||
	    (! this.getOneEjbJar())
	    ) {
	    this.selectMenuInsert(false);
	} else {
	    this.selectMenuInsert(true);
	}
	
	// for edit menu
	if ((tree.getSelectedNodeType().equals(Consts.STRING_META_INF_EJB_JAR_XML)) ||
	    (tree.getSelectedNodeType().equals(Consts.STRING_META_INF_JONAS_EJB_JAR_XML)) ||
	    (! this.getOneEjbJar())
	    ) {
	    menuEdit.setEnabled(false);
	} else {
	    menuEdit.setEnabled(true);
	}
    }


    /**
     * Selected the insert menu.
     * @param selected true if the insert menu is enabled.
     */
    public void selectMenuInsert(boolean selected) {
	menuInsert.removeAll();
	menuInsert.setEnabled(selected);
    }  
  
    /**
     * Invoked when a JMenuItems occurs.
     * @param e the ActionEvent.
     */
    public void actionPerformed(ActionEvent e) {
        JMenuItem source = (JMenuItem)(e.getSource());
        String s = "Action event detected: "
                   + " Event source: " + source.getText();
	Trace.outln(trace, "DeployerMenuBar.actionPerformed(): "+s);
	String selectedItem = source.getText();

	if (selectedItem == Consts.NEW) {
	    newEjb();
	} else if (selectedItem == Consts.OPEN) {
	    try {
		open();
	    } catch (DeploymentDescException de) {
		JOptionPane.showMessageDialog(frame,
					      "Error when reading XML Deployment Descriptors: "+de.toString(),
					      Consts.TITLE_ERROR_MESSAGE_BOX,
					      JOptionPane.ERROR_MESSAGE);
		saveRequired = false;
		fileLocation = null;
		this.setOneEjbJar(false);
	    }
	} else if (selectedItem == Consts.SAVE) {
	    save();
	} else if (selectedItem == Consts.SAVE_AS) {
	    saveAs();
	} else if (selectedItem == Consts.CLOSE) {
	    close();
	} else if (selectedItem == Consts.EXIT) {
	    exit();
	} else if (selectedItem == Consts.CUT) {
	    cut();
	} else if (selectedItem == Consts.COPY) {
	    copy();
	} else if (selectedItem == Consts.PASTE) {
	    paste();
	} else if (selectedItem == Consts.DELETE) {
	    delete();
	} else if (selectedItem == Consts.NEW_BEAN) {
	    newBean();
	} else if (selectedItem == Consts.CHECK) {
	    check();
	} else if (selectedItem == Consts.INTERPOSITION) {
	    interpositionClasses();
	} else if (selectedItem == Consts.ABOUT) {
	    about();
	}
    }

    /**
     * Set the boolean to know if there is yet an Ejb Jar.
     * Update the different items of menu bar (Save,SaveAs,Close,Edit and Bean).
     * @param value the new value of the boolean oneEjbJar.
     */
    public void setOneEjbJar(boolean value) {
	oneEjbJar = value;
	menuSave.setEnabled(value);
	menuSaveAs.setEnabled(value);
	menuClose.setEnabled(value);
	menuEdit.setEnabled(value);
	menuBean.setEnabled(value);
	menuItemBean.setEnabled(value);
    }
    
    /**
     * Get the boolean to know if there is yet an Ejb Jar.
     * @return true is there is yet an EjbJar.
     */
    public boolean getOneEjbJar() {
	return oneEjbJar;
    }

    /**
     * Creation of a new EjbJar.
     * Creation of 4 nodes :
     * <li> EjbJar
     *   <DL><li> META-INF/ejb-jar.xml</DL>
     *   <DL><li> assembly-descriptor </DL>
     *   <DL><li> enterprise-beans    </DL>
     * <li> META-INF/jonas-ejb-jar.xml
     * If there is yet an EjbJar, 'new' must close (and save it)
     * the existing EjbJar before create this new EjbJar.
     */
    private void newEjb() {
	boolean newEjb = true;

	if (getOneEjbJar()) {
	    Trace.outln(trace, "DeployerMenuBar.newEjb() : It is not possible to have several Ejb Jar opened.");
	    String message = "It is not possible to have several Ejb Jar opened.\nDo you want to close the current Ejb Jar ?";
	    int option = JOptionPane.showConfirmDialog(frame, 
						       message,
						       Consts.CLOSE,
						       JOptionPane.YES_NO_OPTION);
	    if (option == JOptionPane.YES_OPTION) {
		this.close();
	    } else if (option == JOptionPane.NO_OPTION) {
		newEjb = false;
	    }
	}
	if (newEjb) {
	    saveRequired = true;
	    fileLocation = null;
	    this.setOneEjbJar(true);

	    DefaultMutableTreeNode parent = tree.addPanelInTree(tree.getTop(),
								new PRootEjbJar());
	    tree.setSelectedNode(parent);
	    DefaultMutableTreeNode ejbJar = tree.addPanelInTree(parent,
								new PEjbJar());
	    tree.addPanelInTree(ejbJar,new PAssemblyDescriptor());
	    tree.addPanelInTree(ejbJar,new PEnterpriseBeans());
	    DefaultMutableTreeNode jonasEjbJar = tree.addPanelInTree(parent,
								     new PJonasEjbJar());
	}
    }

    /**
     * Open an EjbJar from a jar file, it contains :
     * <LI> META-INF/ejb-jar.xml
     * <LI> META-INF/jonas-ejb-jar.xml
     * <LI> Business classes
     * <LI> eventually helper classes
     */
    private void open()	throws DeploymentDescException {
	boolean newEjb = true;
	String fileLocationTemp = null;

	if (oneEjbJar) {
	    // There is yet an Ejb Jar
	    String message = "It is not possible to have several Ejb Jar opened.\nDo you want to close the current Ejb Jar ?";
	    int option = JOptionPane.showConfirmDialog(frame, 
						       message,
						       Consts.CLOSE,
						       JOptionPane.YES_NO_OPTION);
	    if (option == JOptionPane.YES_OPTION) {
		this.close();
	    } else if (option == JOptionPane.NO_OPTION) {
		newEjb = false;
	    }
	}
	if (newEjb) {
	    if (fileLocation == null) {
		String fileName = dialogFileChooser(Consts.OPEN);
		if (fileName != null) { 
		    fileLocationTemp = fileName;
		}
	    }

	    if (fileLocationTemp != null) {
		File file = new File(fileLocationTemp);
		if (!file.exists()) {
		    JOptionPane.showMessageDialog(frame,
						  "File '"+file.getName()+"' not exist",
						  Consts.TITLE_ERROR_MESSAGE_BOX,
						  JOptionPane.ERROR_MESSAGE);
		    return;
		}
		
		//this is where a real application would open the file.
		Trace.outln(trace, "DeployerMenuBar.open(): "+
			    "Opening: " + file.getName() + ".");
		InputStream inEjb = null;
		InputStream inJonasEjb = null;
		ZipFile jar = null;
		
		EjbJar ejbJar = null;
		JonasEjbJar jonasEjbJar = null; 
		ZipEntry zipEjbJar = null;
		ZipEntry zipJonasEjbJar = null;

		try {
		    jar = new ZipFile(fileLocationTemp);
		    zipEjbJar = jar.getEntry(Consts.STRING_META_INF_EJB_JAR_XML);
		    if (zipEjbJar != null) {
			Trace.outln(trace, "DeployerMenuBar.open(): "+
				    "DeployerMenu load zipEjbJar not null");
			inEjb = jar.getInputStream(zipEjbJar);
			ejbJar = loadEjbJar(inEjb,Consts.STRING_META_INF_EJB_JAR_XML);
		    }
		    zipJonasEjbJar = jar.getEntry(Consts.STRING_META_INF_JONAS_EJB_JAR_XML);
		    if (zipJonasEjbJar != null) {
			Trace.outln(trace, "DeployerMenuBar.open(): "+
				    "DeployerMenu load zipJonasEjbJar not null");
			inJonasEjb = jar.getInputStream(zipJonasEjbJar);
			jonasEjbJar = loadJonasEjbJar(inJonasEjb,Consts.STRING_META_INF_JONAS_EJB_JAR_XML);
		    }
		} catch (java.io.FileNotFoundException e) {
		    fileLocationTemp = null;
		    JOptionPane.showMessageDialog(frame,
						  "Error when opening: "+e.toString(),
						  Consts.TITLE_ERROR_MESSAGE_BOX,
						  JOptionPane.ERROR_MESSAGE);
		} catch (java.io.IOException e) {
		    JOptionPane.showMessageDialog(frame,
						  "Error when opening: "+e.toString(),
						  Consts.TITLE_ERROR_MESSAGE_BOX,
						  JOptionPane.ERROR_MESSAGE);
		}

		createNodeLoad(ejbJar,jonasEjbJar);
		
		extract(fileLocationTemp,jar);

		try {
		    if (zipEjbJar != null) {
			inEjb.close();
		    }
		    if (zipJonasEjbJar != null) {
			inJonasEjb.close();
		    }
		} catch(IOException e) {
		    JOptionPane.showMessageDialog(frame,
						  "Error when closing the XML files: "+e.toString(),
						  Consts.TITLE_ERROR_MESSAGE_BOX,
						  JOptionPane.ERROR_MESSAGE);
		}
		fileLocation = fileLocationTemp;
		frame.setTitle(Consts.TITLE_FRAME.concat(" : "+fileLocation));
		setSaveRequired(false);
	    } else {
		Trace.outln(trace, "DeployerMenuBar.Open(): Open command cancelled by user.");
	    }
	}
    }
    
    /**
     * Load the ejb-jar.xml file to return associated EjbJar class.
     * @param is   the InputStream.
     * @param name the name of the EjbJar to load.
     * @return an EjbJar castor object.
     */
    private static EjbJar loadEjbJar(InputStream is,String name)
	throws DeploymentDescException {

	if (is == null) {
	    return null;
	}

	Reader reader = new InputStreamReader(is);
	try {
	    return(EjbJar.unmarshal(reader));
	} catch (ValidationException e) {
	    JOptionPane.showMessageDialog(frame,
					  "XML Validation Error in "+name,
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	    throw new DeploymentDescException("XML Validation Error in "+name, e);
	} catch (MarshalException e) {
	    String detailMsg = "Invalid XML in " + name;
	    if (e.getException() instanceof SAXParseException) {
		detailMsg=getSAXMsg(name,((SAXParseException)e.getException()),"Invalid XML");
	    }
	    throw new DeploymentDescException(detailMsg,e);
	    
	} finally {
	    try {
		reader.close();
	    } catch (IOException e) {
	    }
	}
    }

    /**
     * Load the jonas-ejb-jar.xml file to return associated JonasEjbJar class.
     * @param is   the InputStream.
     * @param name the name of the JonasEjbJar to load.
     * @return a JonasEjbJar castor object.
     */
    private static JonasEjbJar loadJonasEjbJar(InputStream is,String name)
	throws DeploymentDescException {

	if (is == null) {
	    return null;
	}

	Reader reader = new InputStreamReader(is);
	try {
	    return JonasEjbJar.unmarshal(reader);
	} catch (ValidationException e) {
	    JOptionPane.showMessageDialog(frame,
					  "XML Validation Error in "+name,
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	    throw new DeploymentDescException("XML Validation Error in "+name,e);
	} catch (MarshalException e) {
	    String detailMsg = "Invalid XML in " + name;
	    if (e.getException() instanceof SAXParseException) {
		detailMsg=getSAXMsg(name,((SAXParseException)e.getException()),"Invalid XML");
	    }
	    throw new DeploymentDescException(detailMsg,e);
	} finally {
	    try {
		reader.close();
	    } catch (IOException e) {
	    }
	}
    }
    
    /**
     * This method is invoked when the user opens an ejb-jar.
     * At first it extracts the Jar in the current directory,
     *     then it inserts in the tree nodes associated helper files 
     *      and it memorizes the temporary file
     *@param ejbjar the name of ejb-jar.
     *@param jar    the jar file of ejb-jar.
     **/
    private void extract(String ejbjar, ZipFile jar) {
	try {
	    Jjar jarIn = new Jjar(ejbjar);
	    jarIn.extract(".");
	    Enumeration enumFile = jar.entries();
	    while (enumFile.hasMoreElements()) {
		ZipEntry jarEntry = (ZipEntry)enumFile.nextElement();
		String name = new String(jarEntry.getName());
		if (!(name.equals(Consts.STRING_META_INF_EJB_JAR_XML)) &&
		    !(name.equals(Consts.STRING_META_INF_JONAS_EJB_JAR_XML))) {
		    Trace.outln(trace, "DeployerMenuBar.extract(): jarEntry.getName() = "+name);
		    File file = new File(name);
		    String fileName = file.getName();
		    // Don't insert a directory file as a node in the tree
		    if (!file.isDirectory()) {
			insertClass(new FilePaths(file.getParent(),fileName,file.getAbsolutePath()));
		    }
		    delFile.add(file.getAbsoluteFile());
		}
	    }
	} catch (java.io.IOException e) {
	    Trace.outln("DeployerMenuBar.extract() ERROR: get IOException: "+e.toString());
	}
    }
    
    /**
     * Creation of the <i>root</i> node of the Ejb Jar.
     * Invoke the two method into graphical creation the ejb-jar tree, and the JOnAS tree
     * after the ejb tree add in panel manager.
     * @param ejbjar      the EjbJar castor object.
     * @param jonasEjbjar the JonasEjbJar castor object.
     */
    private void createNodeLoad(EjbJar ejbJar, JonasEjbJar jonasEjbJar) {
	if (!oneEjbJar) {
	    this.setOneEjbJar(true);
	    // create one root node (ejb-jar)
	    DefaultMutableTreeNode parent 
		= tree.addLoadingPanelInTree(tree.getTop(),new PRootEjbJar());
	    tree.setSelectedNode(parent);
	    if (ejbJar != null) {
		createNodeLoadEjb(parent,ejbJar);
	    } else {
		Trace.outln(trace, "DeployerMenuBar.createNodeLoad(): ejb-jar.xml is null");
		tree.addPanelInTree(parent,new PEjbJar());
	    }

	    if (jonasEjbJar != null) {
		createNodeLoadJonasEjb(parent,jonasEjbJar);
	    } else {
		tree.addPanelInTree(parent,new PJonasEjbJar());
		Trace.outln(trace, "DeployerMenuBar.createNodeLoad():  Jonas-ejb-jar.xml is null");
	    }
	}
    }

    /**
     * Creation of one ejb-jar node.
     * @param parent the rood node.
     * @param ejbjar the EjbJar castor object.
     * @return the node associated to ejb-jar.xml (with all children)
     */
    private DefaultMutableTreeNode createNodeLoadEjb(DefaultMutableTreeNode parent,
						     EjbJar ejbJar) {
	Enumeration e;
	Enumeration enumTx;
	Enumeration enumCmpField;
	Enumeration enumEjbRef ;
	Enumeration enumEnvEntry ;
	Enumeration enumResourceRef;
	Enumeration enumSecurityRoleRef;
	Enumeration enumMethod;
	Enumeration enumMethodPermission;
	Enumeration enumRole;
	Enumeration enumSession;
	Enumeration enumEntity;

	DefaultMutableTreeNode ejbJarNode =
	    tree.addLoadingPanelInTree(parent, new PEjbJar(ejbJar));
	// add one assembly-descriptor node 
	AssemblyDescriptor assemplyDescriptor = ejbJar.getAssemblyDescriptor();
	if (assemplyDescriptor != null) {
	    DefaultMutableTreeNode assemplyDescriptorNode =
		tree.addLoadingPanelInTree(ejbJarNode,
					   new PAssemblyDescriptor(assemplyDescriptor));
	    enumTx = assemplyDescriptor.enumerateContainerTransaction();
	    while (enumTx.hasMoreElements()) {
		ContainerTransaction tx = (ContainerTransaction)enumTx.nextElement();
		DefaultMutableTreeNode txNode =
		    tree.addLoadingPanelInTree(assemplyDescriptorNode,
					       new PContainerTransaction(tx));
		enumMethod = tx.enumerateMethod();
		while (enumMethod.hasMoreElements()) {
		    tree.addLoadingPanelInTree(txNode,
					       new PMethod((Method)enumMethod.nextElement()));
		}
	    }
	    enumMethodPermission = assemplyDescriptor.enumerateMethodPermission();
	    while (enumMethodPermission.hasMoreElements()) {
		MethodPermission methodP = (MethodPermission)enumMethodPermission.nextElement();
		DefaultMutableTreeNode methodePNode =
		    tree.addLoadingPanelInTree(assemplyDescriptorNode,
					       new PMethodPermission(methodP));
		enumMethod = methodP.enumerateMethod();
		while (enumMethod.hasMoreElements()) {
		    tree.addLoadingPanelInTree(methodePNode,
					       new PMethod((Method)enumMethod.nextElement()));
		}
	    }
	    enumRole = assemplyDescriptor.enumerateSecurityRole();
	    while (enumRole.hasMoreElements()) {   
		tree.addLoadingPanelInTree(assemplyDescriptorNode, 
					   new PSecurityRole((SecurityRole)enumRole.nextElement()));
	    }  
	}	    
	
	EnterpriseBeans entrepriseBeans= ejbJar.getEnterpriseBeans();
	DefaultMutableTreeNode entrepriseBeansNode 
	    = tree.addLoadingPanelInTree(ejbJarNode,new PEnterpriseBeans(entrepriseBeans));

	// add all session node
	enumSession = entrepriseBeans.enumerateSession();
	while (enumSession.hasMoreElements()) {
	    Session session = (Session)enumSession.nextElement();
	    PSession panelSession = new PSession(session);
	    DefaultMutableTreeNode sessionNode = 
		tree.addLoadingPanelInTree(entrepriseBeansNode,panelSession);
	    tree.addEjbName(panelSession.getEjbName());
	    // add all ejb ref node concerning the current session node
	    enumEjbRef = session.enumerateEjbRef();
	    while (enumEjbRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(sessionNode,
					   new PEjbRef((EjbRef)enumEjbRef.nextElement()));
	    }
	    // add all env entry node concerning the current session node
	    enumEnvEntry = session.enumerateEnvEntry();
	    while (enumEnvEntry.hasMoreElements()) {
		tree.addLoadingPanelInTree(sessionNode,
					   new PEnvEntry((EnvEntry)enumEnvEntry.nextElement()));
	    }
	    // add all resource ref node concerning the current session node
	    enumResourceRef = session.enumerateResourceRef();
	    while (enumResourceRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(sessionNode,
					    new PResourceRef((ResourceRef)enumResourceRef.nextElement()));
	    }
	    // add all security role ref node concerning the current session node
	    enumSecurityRoleRef = session.enumerateSecurityRoleRef();
	    while (enumSecurityRoleRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(sessionNode,
					   new PSecurityRoleRef((SecurityRoleRef)enumSecurityRoleRef.nextElement()));
	    }   
	}
	
	// add all entity node
	enumEntity = entrepriseBeans.enumerateEntity();
	while (enumEntity.hasMoreElements()) {
	    Entity entity = (Entity)enumEntity.nextElement();
	    PEntity panelEntity = new PEntity(entity);
	    DefaultMutableTreeNode entityNode =
		tree.addLoadingPanelInTree(entrepriseBeansNode,
					   panelEntity);
	    tree.addEjbName(panelEntity.getEjbName());
	    // add all cmpfield node concerning the current entity node
	    enumCmpField= entity.enumerateCmpField();
	    while (enumCmpField.hasMoreElements()) {
		tree.addLoadingPanelInTree(entityNode,
					   new PCmpField((CmpField)enumCmpField.nextElement()));
	    }
	    // add all ejb-ref node concerning the current entity node
	    enumEjbRef = entity.enumerateEjbRef();
	    while (enumEjbRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(entityNode,new PEjbRef((EjbRef)enumEjbRef.nextElement()));	
	    }
	    // add all env-entry node concerning the current entity node
	    enumEnvEntry = entity.enumerateEnvEntry();
	    while (enumEnvEntry.hasMoreElements()) {
		tree.addLoadingPanelInTree(entityNode,
					   new PEnvEntry((EnvEntry)enumEnvEntry.nextElement()));
	    }
	    // add resource-ref node concerning the current entity node
	    enumResourceRef = entity.enumerateResourceRef();
	    while (enumResourceRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(entityNode,
					   new PResourceRef((ResourceRef)enumResourceRef.nextElement()));
	    }
	    // add security role node concerning the current entity node
	    enumSecurityRoleRef = entity.enumerateSecurityRoleRef();
	    while (enumSecurityRoleRef.hasMoreElements()) {
		tree.addLoadingPanelInTree(entityNode,
					   new PSecurityRoleRef((SecurityRoleRef)enumSecurityRoleRef.nextElement()));
	    }
	}
	return ejbJarNode;
    }
    
    /**
     * Creation of one jonas-ejb-jar node.
     * @param parent      the root node.
     * @param jonasEjbjar the JonasEjbJar castor object.
     * @return the node associated to jonas-ejb-jar.xml (with all children)
     */
    private DefaultMutableTreeNode createNodeLoadJonasEjb(DefaultMutableTreeNode parent,
						       JonasEjbJar jonasEjbJar) {
	Enumeration enumEntity;
	Enumeration enumSession;
	Enumeration enumFinder;

	DefaultMutableTreeNode jonasEjbJarNode =
	    tree.addLoadingPanelInTree(parent, new PJonasEjbJar(jonasEjbJar));

	enumEntity = jonasEjbJar.enumerateJonasEntity();
	while (enumEntity.hasMoreElements()) {
	    JonasEntity entity = (JonasEntity)enumEntity.nextElement();
	    DefaultMutableTreeNode jonasEntityNode =
		tree.addLoadingPanelInTree(jonasEjbJarNode,
					   new PJonasEntity(entity));

	    if (entity.getJdbcMapping() != null) {
		JdbcMapping jdbcMapping = entity.getJdbcMapping();
		DefaultMutableTreeNode jdbcMappingNode =
		    tree.addLoadingPanelInTree(jonasEntityNode,
					       new PJdbcMapping(jdbcMapping));

		enumFinder = jdbcMapping.enumerateFinderMethodJdbcMapping();
		while (enumFinder.hasMoreElements()) {
		    tree.addLoadingPanelInTree(jdbcMappingNode,
					       new PFinderMethodJdbcMapping((FinderMethodJdbcMapping)enumFinder.nextElement()));
		}
	    }
	}
	
	enumSession = jonasEjbJar.enumerateJonasSession();
	while (enumSession.hasMoreElements()) {
	    tree.addLoadingPanelInTree(jonasEjbJarNode,
				       new PJonasSession((JonasSession)enumSession.nextElement()));
	}
	return jonasEjbJarNode;
    }

    /**
     * Display a message to inform user that its deployment descriptor contains some fields not filled.
     * The node selected after this message is the first node of the tree witch miss fields.
     * @param firstNode the node to get the obligatoryTable whitch contain the list of fields not filled.
     */
    private void displayWarningBeforeSave(DefaultMutableTreeNode firstNode) {
	Hashtable obligatoryTable = ((DeployerPanel)firstNode.getUserObject()).getObligatoryTable();

	String message = "Some obligatory fields are not filled :";
	DeployerPanel temporaryDP;
	Vector temporaryFields;
	for (Enumeration enumKey=obligatoryTable.keys(); enumKey.hasMoreElements();) {
	    temporaryDP = (DeployerPanel)enumKey.nextElement();
	    temporaryFields = (Vector)obligatoryTable.get(temporaryDP);
	    message = message.concat("\n"+Consts.FIRST_TABULATION_SPACE
				     +"Panel : "+temporaryDP.toString());
	    for (Enumeration enumVector = temporaryFields.elements(); 
		 enumVector.hasMoreElements();) {
		message = message.concat("\n"+Consts.SECOND_TABULATION_SPACE
					 +(String)enumVector.nextElement());
	    }
	}
	message = message.concat("\nYou will cannot save this Ejb Jar as long as there are obligatory fields not filled.");
	JScrollPane scrollPane = new JScrollPane();
	scrollPane.setPreferredSize(new Dimension(Consts.WIDTH_SCROLLPANE,
						  Consts.HEIGHT_SCROLLPANE*2));
	JTextArea textArea = new JTextArea();
	textArea.setLineWrap(true);
	textArea.setWrapStyleWord(true);
	textArea.setText(message);
	textArea.setEditable(false);
	scrollPane.getViewport().add(textArea);

	JOptionPane.showMessageDialog(frame,scrollPane);
	
	// The selected node is the first node where there are obligatory fields
	// not filled.
	tree.setSelectedNode(courseTree(firstNode,obligatoryTable));
    }
    
    /**
     * Display a dialog box to inform under JAR file the user wants save this work.
     * @param title the title of the dialog box.
     * @return the absolute path of the file if there is an approve option.
     **/
    private String dialogFileChooser(String title) {
	int returnVal = JFileChooser.CANCEL_OPTION ;
	
	fc.rescanCurrentDirectory();
	fc.setSelectedFile(null);
	if (title.equals(Consts.OPEN)) {
	    fc.setDialogTitle(Consts.OPEN);
	    returnVal = fc.showOpenDialog(frame);
	} else if (title.equals(Consts.SAVE)) {
	    fc.setDialogTitle(Consts.SAVE);
	    returnVal = fc.showSaveDialog(frame);
	} else if (title.equals(Consts.SAVE_AS)) {
	    fc.setDialogTitle(Consts.SAVE_AS);
	    returnVal = fc.showSaveDialog(frame);
	}

	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File file = fc.getSelectedFile();
	    //this is where a real application would open the file.
	    Trace.outln(trace,
			"DeployerMenuBar.dialogFileChooser("+title+"): "+
			"file = "+ file.getName() + ".");
	    return fc.getSelectedFile().getAbsolutePath();
	} else {
	    return null;
	}
    }

    /**
     * Save the EjbJar in the file that its name is fileLocation
     * @return true if the save operation has been right finished.
     */
    private boolean save() {
	DefaultMutableTreeNode ejbJar;
	DefaultMutableTreeNode jonasEjbJar;

	// This boolean is true if the list of obligatory fields not filled has been displayed.
	boolean obligatoryFieldsNotFilled = false;

	Trace.outln(trace, "DeployMenuBar.save(): filelocation = "+fileLocation);
	if (fileLocation == null) {
	    String fileName = dialogFileChooser(Consts.SAVE);
	    if (fileName != null) {
		fileLocation = fileName;
	    } else {
		return false;
	    }
	}	
	// Get concerned xmlEjbJar and xmlJonasEjbJar.
	DefaultMutableTreeNode parentEjb = (DefaultMutableTreeNode)tree.getCurrentEjbJar();
	
	// save information of current node
	Trace.outln(trace, "DeployMenuBar.save(): save all information of current node");
	tree.setSelectedNode(parentEjb);
	
	Hashtable nodes = this.getEjbNodes();
	ejbJar = (DefaultMutableTreeNode)nodes.get(Consts.STRING_META_INF_EJB_JAR_XML);
	jonasEjbJar = (DefaultMutableTreeNode)nodes.get(Consts.STRING_META_INF_JONAS_EJB_JAR_XML);
	
	// For the first version of this tool, there is only one EjbJar open.
	EjbJar xmlEjbJar = (EjbJar)(((DeployerPanel)ejbJar.getUserObject()).getCastorObject());
	JonasEjbJar xmlJonasEjbJar = (JonasEjbJar)(((DeployerPanel)jonasEjbJar.
						    getUserObject()).getCastorObject());
	
	OutputStream outEjbJar = null;
	OutputStream outJonasEjbJar = null;
	try {
	    File metaInf = new File("META-INF");
	    if (!metaInf.exists()) {
		metaInf.mkdirs();
	    }
	    // create one file ejb-jar.xml in the sub-directory META-INF
	    outEjbJar = new FileOutputStream(new String(Consts.STRING_META_INF_EJB_JAR_XML));
	    // create one file jonas-ejb-jar.xml in the sub-directory META-INF
	    outJonasEjbJar = new FileOutputStream(new String(Consts.STRING_META_INF_JONAS_EJB_JAR_XML));
	} catch (FileNotFoundException e) {
	    JOptionPane.showMessageDialog(frame,
					  "Error when saving: File not found ("+e.toString()+").",
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	    Trace.outln(trace, "DeployMenuBar.save() ERROR: "+e.toString());
	    return false;
	}
	
	Writer writerEjbJar;
	Writer writerJonasEjbJar;
	if (outEjbJar != null) {
	    // open in writing the file META-INF/ejb-jar.xml 
	    writerEjbJar = new OutputStreamWriter(outEjbJar);    
	    try {
		//writing the ejb-jar.xml 
		xmlEjbJar.marshal(writerEjbJar);
	    } catch (ValidationException ve) {
		Trace.outln(trace,
			    "DeployMenuBar.save() ERROR: "+
			    "ejbJar -> catch ValidationException : "+
			    "You must fill all obligatory fields : "+ve.toString());
		// Prevent the user about the obligatory fields no filled.
		obligatoryFieldsNotFilled = true;
		displayWarningBeforeSave(ejbJar);
		return false;
	    } catch (MarshalException me) {
		Trace.outln(trace,
			    "DeployMenuBar.save() ERROR: "+me.toString());
		return false;
	    } finally {
		try {
		    writerEjbJar.close();
		} catch (IOException ioe) {
		    Trace.outln(trace,
				"DeployMenuBar.save() ERROR: "+ioe.toString());
		    return false;
		}
	    }
	}
	if (outJonasEjbJar != null) {
	    // open in writing the file META-INF/jonas-ejb-jar.xml
	    writerJonasEjbJar = new OutputStreamWriter(outJonasEjbJar);    
	    try {
		//writing the jonas-ejb-jar.xml 
		xmlJonasEjbJar.marshal(writerJonasEjbJar);
	    } catch (ValidationException ve) {
		Trace.outln(trace,
			    "DeployMenuBar.save() ERROR: "+
			    "jonasEjbJar -> catch ValidationException : "+
			    "You must fill all obligatory fields : "+ve.toString());
		if (!obligatoryFieldsNotFilled)
		    displayWarningBeforeSave(jonasEjbJar);
		return false;
	    } catch (MarshalException me) {
		Trace.outln(trace,
			    "DeployMenuBar.save() ERROR: "+me.toString());
		return false;
	    } finally {
		try {
		    writerJonasEjbJar.close();
		} catch (IOException ioe) {
		    Trace.outln(trace,
				"DeployMenuBar.save() ERROR: "+ioe.toString());
		    return false;
		}
	    }
	}
	
	try {
	    //delete the existing jar file
	    if (fileLocation != null) {
		deleteFile(new File(fileLocation));
	    }

	    Jjar jJarFile = new Jjar(fileLocation);
	    
	    // get class file of beans or images
	    File tempFile=null;
	    Vector files = tree.getFiles();
	    Enumeration enumFile = files.elements();

	    while (enumFile.hasMoreElements()) {
		FilePaths infoFile = (FilePaths)enumFile.nextElement();
		boolean del = copyFile(infoFile);
		jJarFile.add(infoFile.toString());
	    }

	    // Save xml files in a .jar file
	    jJarFile.add(Consts.STRING_META_INF_EJB_JAR_XML);
	    jJarFile.add(Consts.STRING_META_INF_JONAS_EJB_JAR_XML);
	    // delete ejb-jar.xml and jonas-ejb-jar.xml
	    deleteFile(new File(Consts.STRING_META_INF_EJB_JAR_XML));
	    deleteFile(new File(Consts.STRING_META_INF_JONAS_EJB_JAR_XML));
	
	} catch (IOException ioe) { 
	    JOptionPane.showMessageDialog(frame,
					  "DeployerMenuBar.save():"+ioe.toString(),
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	    Trace.outln(trace, "DeployerMenuBar.save() ERROR: "+ioe.toString());
	    return false;
	}
	frame.setTitle(Consts.TITLE_FRAME.concat(" : "+fileLocation));
	setSaveRequired(false);

	return true;
    }

    /**
     * Permits to delete all tempory files written by open.
     **/
    private void deleteFiles() {
	Enumeration enumFile = delFile.elements();
	while (enumFile.hasMoreElements()) {
	    deleteFile((File)enumFile.nextElement());
	}
    }

    /**
     * Permits to course the tree under a root node.
     * This method stops when the first node contained in the obligatoryTable is found.
     * @param topNode         the root of the tree to course.
     * @param obligatoryTable the table of registered fields not filled.
     * @return the first node contained in the obligatoryTable.
    */
    private DefaultMutableTreeNode courseTree(DefaultMutableTreeNode topNode,
					      Hashtable obligatoryTable) {
	DefaultMutableTreeNode nextElement = null;
	DefaultMutableTreeNode result = null;
	DeployerPanel deployerPanelElement = null;
	Enumeration enumChildren = topNode.children();
	boolean found = false;
	while ((enumChildren.hasMoreElements()) && !found) {
	    nextElement = (DefaultMutableTreeNode)enumChildren.nextElement();
	    if (nextElement.getUserObject() instanceof DeployerPanel) {
		deployerPanelElement = (DeployerPanel)nextElement.getUserObject();
	    }
	    // Keys of the obligatoryTable are DeployerPanel
	    if (obligatoryTable.containsKey(deployerPanelElement)) {
		result = nextElement;
		found = true;
	    } else if (!nextElement.isLeaf()) {
		    result = courseTree(nextElement,obligatoryTable);
		    if (result != null) {
			found = true;
		    }
	    }
	}
	return result;
    }

    /**
     * This method is only invoked when the user save his work,
     * it permits to copy a file into Jar file.
     * @param infoFile the path of file to copy.
     * @return true if the operation is completed.
     **/
    private boolean copyFile(FilePaths infoFile) {
	try {
	    if (infoFile.getPackage() != null) {
		File dir=new File(infoFile.getPackage());
		dir.mkdirs();
	    }
	    File file=new File(infoFile.toString());
	    if (file.exists()) {
		return false;
	    }
	    BufferedReader in =
		new BufferedReader(new FileReader(infoFile.getAbsolutePath()));
	    BufferedWriter out =
		new BufferedWriter(new FileWriter(infoFile.toString()));
	    int c;
	    while ((c = in.read()) != -1) {
		out.write(c);
	    }
	    out.close();
	    in.close();
	} catch (java.io.FileNotFoundException fnfe) {
	    Trace.outln(trace, "DeployerMenuBar.copyFile() ERROR :"+ fnfe.toString());
	    return false;
	} catch (java.io.IOException ioe) {
	    Trace.outln(trace, "DeployerMenuBar.copyFile() ERROR :"+ ioe.toString());
	    return false;
	}
	
	return true;
    }

    /**
     * Method to delete a file.
     * @param file the file to delete.
     */    
    private void deleteFile(File file) {
	if (file.isDirectory()) {
	    if ((file.listFiles()).length == 0) {
		file.delete();
	    }
	} else {
	    file.delete();
	    deleteFile(new File(file.getParent()));
	}
    }

    /**
     * Save an EjbJar for the first time.
     */
    private void saveAs() {
	String fileName = dialogFileChooser(Consts.SAVE_AS);
	if (fileName != null) {
	    fileLocation = fileName;
	} else {
	    return;
	}
	if (fileLocation != null) {
	    deleteFile(new File(fileLocation));
	    if (save()) { 
		setSaveRequired(false);
	    }
	} else {
	    JOptionPane.showMessageDialog(frame,
					  "The save operation failed.",
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	}
    }

    /**
     * Close an EjbJar.
     * A dialog box is opened to ask if the save is needed.
     */
    private void close() {
	boolean saveSuccessful = true;

	tree.setSelectedNode((DefaultMutableTreeNode)tree.getCurrentEjbJar());

	if (getSaveRequired()) {
	    int n = JOptionPane.showConfirmDialog(frame, 
						  "Do you want to save the current Ejb Jar before closing ?",
						  Consts.CLOSE,
						  JOptionPane.YES_NO_OPTION);
	    if (n == JOptionPane.YES_OPTION) {
		if (!save()) {
		    saveSuccessful = false;
		    Trace.outln(trace,
				"DeployerMenuBar.close(): The save operation fell through.");
		}
	    } else if (n == JOptionPane.NO_OPTION) {
		fileLocation = null;
		setSaveRequired(true);
		this.setOneEjbJar(false);
	    }
	}

	if (saveSuccessful) {
	    // Close the currentEjbJar.
	    fileLocation = null;
	    setSaveRequired(false);
	    this.setOneEjbJar(false);
	    DefaultMutableTreeNode ejbJar =
		(DefaultMutableTreeNode)tree.getCurrentEjbJar();
	    // Remove the root node of the Ejb Jar.
	    tree.removeNode(ejbJar);
	}

	frame.setTitle(Consts.TITLE_FRAME);
	    
    }

    /**
     * Exit BeanDeployer.
     * A dialog box is opened to ask if the save is needed.
     */
    public void exit() {
	if (oneEjbJar && getSaveRequired()) {
	    int n = JOptionPane.showConfirmDialog(frame,
						  "Do you want to save before exiting BeanDeployer ?",
						  Consts.EXIT,
						  JOptionPane.YES_NO_CANCEL_OPTION);
	    if (n == JOptionPane.YES_OPTION) {
		save();
		deleteFiles();
		System.exit(0);
	    } else if (n == JOptionPane.NO_OPTION) {
		deleteFiles();
		System.exit(0);
	    } else if (n == JOptionPane.CANCEL_OPTION) {}
	} else {
	    deleteFiles();
	    System.exit(0);
	}
    }
    

    /**
     * Cut a node of the tree.
     */
    private void cut() {}

    /**
     * Copy a node of the tree.
     */
    private void copy() {}

    /**
     * Paste a node in the tree.
     */
    private void paste() {}

    /**
     * Delete a node of the tree.
     */
    protected void delete() {
	tree.removeCurrentNode();
	
    }

    /**
     * Create a new bean.
     * A dialog box appears to ask ejb-name and bean type (entity or session).
     */
    private void newBean() {
	setSaveRequired(true);
	NewBean newBean = new NewBean(frame);
	newBean.setLocationRelativeTo(frame);
	newBean.setVisible(true);
    }

    /**
     * Get nodes to insert bean :
     * <LI> enterprise-beans to insert entity or session
     * <LI> META-INF/jonas-ejb-jar.xml to insert jonas-entity or jonas-session
     * @return the hashtable containing enterprise-beans and META-INF/jonas-ejb-jar.xml nodes.
     */
    public Hashtable getEjbNodes() {
	boolean foundEnterpriseBeans = false;
	DefaultMutableTreeNode ejbJar = null;
	DefaultMutableTreeNode enterpriseBeans = null;
	DefaultMutableTreeNode jonasEjbJar = null;

	DefaultMutableTreeNode nextElement,littleChildren;
	Hashtable nodes = new Hashtable(); // contains enterpriseBeans and jonasEjbJar

	DefaultMutableTreeNode parentEjb = (DefaultMutableTreeNode)tree.getCurrentEjbJar();

	// There is only one META-INF/ejb-jar.xml 
	// and it is obligatory the first child of the Ejb Jar node.
	// There is only one META-INF/jonas-ejb-jar.xml 
	// and it is obligatory the second child of the Ejb Jar node.
	// In addition these two nodes are present in an Ejb Jar, they cannot be removed.
	ejbJar = (DefaultMutableTreeNode)parentEjb.getChildAt(0);
	// Research of enterprise-beans
	for (Enumeration enumEjbJarChildren = ejbJar.children(); enumEjbJarChildren.hasMoreElements();) {
	    nextElement = (DefaultMutableTreeNode)enumEjbJarChildren.nextElement();
	    if (((DeployerPanel)nextElement.getUserObject()).getType()
		.equals(Consts.STRING_ENTERPRISE_BEANS)) {
		enterpriseBeans = nextElement;
		foundEnterpriseBeans = true;
	    }
	}
	jonasEjbJar = (DefaultMutableTreeNode)parentEjb.getChildAt(1);

	if (!foundEnterpriseBeans)
	    enterpriseBeans = tree.addPanelInTree(ejbJar,new PEnterpriseBeans());

	nodes.put(((DeployerPanel)ejbJar.getUserObject()).getType(),ejbJar);
	nodes.put(((DeployerPanel)enterpriseBeans.getUserObject()).getType(),enterpriseBeans);
	nodes.put(((DeployerPanel)jonasEjbJar.getUserObject()).getType(),jonasEjbJar);
	return nodes;
    }

    /**
     * Creates a new session bean.
     * @param ejbName the ejb-name of the bean.
     */
    public void newSessionBean(String ejbName) {
	setSaveRequired(true);
	Hashtable nodes = this.getEjbNodes();

	DefaultMutableTreeNode enterpriseBeans 
	    = (DefaultMutableTreeNode)nodes.get(Consts.STRING_ENTERPRISE_BEANS);
	DefaultMutableTreeNode jonasEjbJar 
	    = (DefaultMutableTreeNode)nodes.get(Consts.STRING_META_INF_JONAS_EJB_JAR_XML);

	PSession pSession = new PSession();
	pSession.setEjbName(ejbName);
	DefaultMutableTreeNode sessionNode = tree.addPanelInTree(enterpriseBeans,pSession);
	tree.setSelectedNode(sessionNode);
	PJonasSession pJonasSession = new PJonasSession();
	pJonasSession.setEjbName(ejbName);
	tree.addPanelInTree(jonasEjbJar,pJonasSession);
    }

    /**
     * Creates a new entity bean.
     * @param ejbName the ejb-name of the bean.
     */
    public void newEntityBean(String ejbName) {
	setSaveRequired(true);
	Hashtable nodes = getEjbNodes();

	DefaultMutableTreeNode enterpriseBeans 
	    = (DefaultMutableTreeNode)nodes.get(Consts.STRING_ENTERPRISE_BEANS);
	DefaultMutableTreeNode jonasEjbJar 
	    = (DefaultMutableTreeNode)nodes.get(Consts.STRING_META_INF_JONAS_EJB_JAR_XML);

	PEntity pEntity = new PEntity();
	pEntity.setEjbName(ejbName);
	DefaultMutableTreeNode entityNode = tree.addPanelInTree(enterpriseBeans,pEntity);
	tree.setSelectedNode(entityNode);
	PJonasEntity pJonasEntity = new PJonasEntity();
	pJonasEntity.setEjbName(ejbName);
	tree.addPanelInTree(jonasEjbJar,pJonasEntity);
    }    

    /**
     * Inserts a class in the tree.
     * @param filePaths a couple of (path in the jar file, absolute path of the file).
     */
    public void insertClass(FilePaths filePaths) {
	File f = new File(filePaths.getAbsolutePath());
	if (f.exists()) {
	    setSaveRequired(true);
	    //  parentEjb is obligary an ejb-jar
	    DefaultMutableTreeNode parentEjb
		= (DefaultMutableTreeNode)tree.getCurrentEjbJar();
	    tree.addPanelInTree(parentEjb,filePaths);
	} else {
	    JOptionPane.showMessageDialog(frame,
					  "File '"+filePaths.getAbsolutePath()+"' not exist",
					  Consts.TITLE_ERROR_MESSAGE_BOX,
					  JOptionPane.ERROR_MESSAGE);
	}
    }

    /**
     * Check of an Ejb Jar.
     */
    private void check() {}
 
    /**
     * Generate interposition classes.
     */
    private void interpositionClasses() {}

    /**
     * Open a message box.
     */
    private void about() {
	JOptionPane.showMessageDialog(frame,
				      "ejb-jar file editor for JOnAS "+Version.NUMBER,
				      Consts.ABOUT,
				      JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Build a message from SAX Exception in a consistent style, consistent
     * with emacs compile mode (same as grep, cc, javac, etc).
     * @param fileName  the name of the file.
     * @param exception the SAX Exception.
     * @param msg       the message.
     * @return the message from SAX Exception.
     */
    private static String getSAXMsg(String fileName,
				   SAXParseException exception,
				   String msg) {
	String ret
	    = fileName
	    + ":" + exception.getLineNumber()
	    + ":"+ exception.getColumnNumber()
	    + ": ";
	ret+=msg;
	return ret;
    }
   
    /**
     * This method tests if the node of META-INF/ejb-jar.xml has one assembly descriptor
     * @param currentNode the selected node (META-INF/ejb-jar.xml)
     * @return true if there is yet a node assembly-descriptor.
     */
    public boolean findAssemblyDescriptor(DefaultMutableTreeNode currentNode) {
	Enumeration df = currentNode.depthFirstEnumeration();
	while (df.hasMoreElements()) {
	    DefaultMutableTreeNode temp = (DefaultMutableTreeNode)df.nextElement();
	    if (temp.toString().equals(Consts.STRING_ASSEMBLY_DESCRIPTOR)) {
		return true;
	    }
	}
	return false;
    }
    
    /**
     * This method tests if the node of META-INF/ejb-jar.xml has one entreprise bean's node
     * @param currentNode the selected node (META-INF/ejb-jar.xml)
     * @return true if there is yet a node enterprise-beans.
     */
    public boolean findEntrepriseBeans(DefaultMutableTreeNode currentNode) {
	Enumeration df = currentNode.depthFirstEnumeration();
	while (df.hasMoreElements()) {
	    DefaultMutableTreeNode temp = (DefaultMutableTreeNode)df.nextElement();
	    if (temp.toString().equals(Consts.STRING_ENTERPRISE_BEANS)) {
		return true;
	    }
	}
	return false;
    }
    
    /**
     * This method tests if the node of jonas-entity has one jdbc-mapping
     * @param currentNode the selected node (jonas-entity)
     * @return true if there is yet a node jdbc-mapping.
   */
    public boolean findJdbcMapping(DefaultMutableTreeNode currentNode) {
	Enumeration df = currentNode.depthFirstEnumeration();
	while (df.hasMoreElements()) {
	    DefaultMutableTreeNode temp = (DefaultMutableTreeNode)df.nextElement();
	    if (temp.toString().equals(Consts.STRING_JDBC_MAPPING)) {
		return true;
	    }
	}
	return false;
    } 

    /**
     * This method permits to put a value saveRequired (invoked by deployerTree).
     * @param bool true for invoked save if it is need.
     **/
    protected void setSaveRequired(boolean bool) {
	saveRequired = bool;
	Trace.outln(trace,
		    "DeployerMenuBar.setSaveRequired(): new value is "+
		    (new Boolean(bool)).toString());
    }
    
    /**
     * This method permits to know a value saveRequired.
     * @return true if the save operation is required.
     **/
    protected boolean getSaveRequired() {
	return saveRequired;
    }

}
