/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                  DeployerTree                      */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.guicomponents;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.panels.DeployerPanel;
import org.objectweb.jonas.beandeployer.panels.DeployerPanelWithEjbName;
import org.objectweb.jonas.beandeployer.panels.PRootEjbJar;
import org.objectweb.jonas.beandeployer.panels.PMethod;
import org.objectweb.jonas.beandeployer.panels.PAssemblyDescriptor;
import org.objectweb.jonas.beandeployer.panels.PCmpField;
import org.objectweb.jonas.beandeployer.panels.PContainerTransaction;
import org.objectweb.jonas.beandeployer.panels.PEjbJar;
import org.objectweb.jonas.beandeployer.panels.PEjbRef;
import org.objectweb.jonas.beandeployer.panels.PEnterpriseBeans;
import org.objectweb.jonas.beandeployer.panels.PEntity;
import org.objectweb.jonas.beandeployer.panels.PEnvEntry;
import org.objectweb.jonas.beandeployer.panels.PFinderMethodJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PJonasEjbJar;
import org.objectweb.jonas.beandeployer.panels.PJonasEntity;
import org.objectweb.jonas.beandeployer.panels.PJonasSession;
import org.objectweb.jonas.beandeployer.panels.PJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PMethod;
import org.objectweb.jonas.beandeployer.panels.PMethodPermission;
import org.objectweb.jonas.beandeployer.panels.PResourceRef;
import org.objectweb.jonas.beandeployer.panels.PSecurityRole;
import org.objectweb.jonas.beandeployer.panels.PSecurityRoleRef;
import org.objectweb.jonas.beandeployer.panels.PSession;

import org.objectweb.jonas.common.Trace;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTree;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.DefaultTreeModel;

import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

// For MyTreeModelListener class
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;

import java.util.Enumeration;
import java.util.Vector;
 

/**
 * Singleton class to manage the tree.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class DeployerTree extends JTree implements TreeSelectionListener {

    static private final int trace = Trace.DB_19;

    private static JPanel        principalRightPanel;
    private static DeployerPanel deployerPanel, previousPanel;
    private static Object        nodeObject;

    private static DefaultMutableTreeNode top;
    private static DefaultMutableTreeNode node;

    private static boolean playWithLineStyle = false;
    private static String  lineStyle         = "Angled";

    private static DefaultTreeModel treeModel;

    private static DeployerTree uniqueTree = null; 

    private DefaultMutableTreeNode lastNodeSelected = null;

   
    /**
     * Build the manager of the tree.
     * @param principalRightPanel is the right panel of the application.
     */
    private DeployerTree(JPanel principalRightPanel) {
	this.principalRightPanel = principalRightPanel;
	
	top = new DefaultMutableTreeNode(Consts.ROOT_NAME);

	treeModel = new DefaultTreeModel(top);
	treeModel.addTreeModelListener(new MyTreeModelListener());

	// Creation of a tree that allows one selection at a time.
	this.setModel(treeModel);
	this.setRootVisible(false);
	this.setShowsRootHandles(true);
	this.setAutoscrolls(true);
	this.getSelectionModel().setSelectionMode
	    (TreeSelectionModel.SINGLE_TREE_SELECTION);

	//Listen for when the selection changes.
	this.addTreeSelectionListener(this);
	
        if (playWithLineStyle) {
            this.putClientProperty("JTree.lineStyle", lineStyle);
        }
    }

    /**
     * Called whenever the value of the selection changes.
     * @param e  the event that characterizes the change.
     **/
    public void valueChanged(TreeSelectionEvent e) {
	node = (DefaultMutableTreeNode)this.getLastSelectedPathComponent();
	if (node == null) return;
	
	DeployerMenuBar.getInstance().refreshMenu(node);
	nodeObject = node.getUserObject(); // A String if there is no associated panel.
	previousPanel = deployerPanel;
	
	// previousPanel is an instance of DeployerPanel.
	if ( previousPanel != null){
	    previousPanel.setVisible(false);
	    if (previousPanel instanceof DeployerPanel) {
		if ( previousPanel.getChangedPanel() ) {
		    previousPanel.setChangedPanel(false);
		    // take the last castor class
		    Object lastCastorObject = previousPanel.getCastorObject();

		    // if setValues is true, there is a change.
		    if (previousPanel.setValues()) {
 			DeployerMenuBar.getInstance().setSaveRequired(true);
			Trace.outln(trace, "DeployerTree.valueChanged(): The previous panel has values changed.");
		    }

		    if (previousPanel instanceof DeployerPanelWithEjbName) {
			addEjbName((DeployerPanelWithEjbName)previousPanel);
		    }
		   
		    previousPanel.getValues(); //For traces
		    // Refresh previous node.
		    treeModel.nodeChanged(lastNodeSelected);
		}
	    } 
	}
	if (! node.isRoot()) {
	    if (nodeObject instanceof DeployerPanel) {
		deployerPanel = (DeployerPanel)nodeObject;
		if (nodeObject instanceof PMethod){
		    updateVectorEjbName();
		}
		displayPanel(deployerPanel);
		lastNodeSelected = node;
	    }
	    
	    if (nodeObject instanceof PRootEjbJar) {
		principalRightPanel.removeAll();
		principalRightPanel.validate();
		principalRightPanel.repaint();
	    }
	} else {
	    Trace.outln(trace, "DeployerTree.valueChanged(): Node is root, there is no associated panel.");
	}
	
    }

    /**
     * Create an instance of the DeployerTree.
     * @param principalRightPanel the right panel of the BeanDeployer application.
     * @return the new instance of DeployerTree
     */
    public static DeployerTree create(JPanel principalRightPanel) {
	if (uniqueTree == null) {
	    uniqueTree = new DeployerTree(principalRightPanel);
	}
	return uniqueTree;
    }
    
    /**
     * Get the unique instance of the DeployerTree.
     * @return the only instance of DeployerTree
     */
    public static DeployerTree getInstance(){
	if (uniqueTree == null) {
	    Trace.outln(trace, "Deployer.getInstance(): uniqueTree == null");
	}
	return uniqueTree;
    }

    /**
     * Display the panel in the tree and the right panel.
     * @param tempAssPanel is the associated panel to display.
     */
    private void displayPanel(DeployerPanel panel) {
	panel.setVisible(true);
	principalRightPanel.add(panel);
	principalRightPanel.validate();
	principalRightPanel.repaint();
    }

    /**
     * This method allows udpate all items combobox for method panel.
     **/
    public void updateVectorEjbName(){
	PRootEjbJar rootEjbJar = ((PRootEjbJar)((DefaultMutableTreeNode)getCurrentEjbJar()).getUserObject());
	Trace.outln(trace,
		    "DeployerTree.updateVectorEjbName(): udpateListEjbName with "+
		    rootEjbJar.getVectorEjbName());
	((PMethod)deployerPanel).udpateListEjbName(rootEjbJar.getVectorEjbName());
    }
    
    /**
     * This method allows add an item in PRootEjbJar when Session or Entity is informed.
     * @param panel the panel where the information is filled.
     **/
    public void addEjbName(DeployerPanelWithEjbName panel) {
	Trace.outln(trace,
		    "DeployerTree.addEjbName(): addEjbName with "+
		    panel.getEjbName());
	PRootEjbJar rootEjbJar = ((PRootEjbJar)((DefaultMutableTreeNode)getCurrentEjbJar()).getUserObject());
	rootEjbJar.addEjbName(panel.getEjbName());
    }

    /**
     * This method allows add an item in PRootEjbJar 
     * when Session or Entity is informed during ejb-jar reading.
     * @param ejbName the ejb-name of the to add in the list.
     **/
    public void addEjbName(String ejbName){
	Trace.outln(trace,
		    "DeployerTree.addEjbName(): addEjbName with "+
		    ejbName);
	PRootEjbJar rootEjbJar = ((PRootEjbJar)((DefaultMutableTreeNode)getCurrentEjbJar()).getUserObject());
	rootEjbJar.addEjbName(ejbName);
    }
    
    /**
     * Add a new panel in the tree.
     * @param parent the parent's node of the new panel.
     * @param type   the type of the new panel.
     * @return the inserted node
     */
    public DefaultMutableTreeNode addPanelInTree(DefaultMutableTreeNode parent, Object panel) {
	DefaultMutableTreeNode child  = null;

	child = addObject(parent,panel);
	Trace.outln(trace,
		    "DeployerTree.addPanelInTree(): parent = "+parent+
		    " and child = "+child);
	if (! parent.isRoot()) 
	    if ((panel instanceof DeployerPanel) && (parent.getUserObject() instanceof DeployerPanel)) {
		((DeployerPanel)parent.getUserObject()).
		    addInSpecificList(((DeployerPanel)panel).getCastorObject());
	    }
	return child;
    }

    /**
     * Add a new panel in the tree.
     * This method add a node under the current node.
     * @param object the object to add in Tree (generally a panel).
     * @return the inserted node
     */    
    public DefaultMutableTreeNode addPanelInTree(Object object) {
	TreePath currentSelection = this.getSelectionPath();
	DefaultMutableTreeNode currentNode = 
	    (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());

	DefaultMutableTreeNode newChild = addPanelInTree(currentNode,object);

	setSelectedNode(newChild);
	return newChild;
    }

    /**
     * Select an existing node of the tree.
     * @param node the node to select.
     **/
    public void setSelectedNode(DefaultMutableTreeNode node) {
	TreePath nextSelection;

	nextSelection = new TreePath(node.getPath());
	this.setSelectionPath(nextSelection);
    }

    /**
     * Get the current selected node.
     * @return the selected node.
     */
    public DefaultMutableTreeNode getSelectedNode() {
	TreePath currentPath = this.getSelectionPath();
	return (DefaultMutableTreeNode)(currentPath.getLastPathComponent());
    }

    /**
     * Get the type of the associated panel of the selected node.
     * @return the type of associated panel of the selected node.
     */
    public String getSelectedNodeType() {
	String nodeType;

	DefaultMutableTreeNode currentNode = getSelectedNode();
	Object currentPanel = currentNode.getUserObject();
	if (currentPanel instanceof DeployerPanel) 
	    nodeType = (String)(((DeployerPanel)currentPanel).getType());
	else if (currentPanel instanceof PRootEjbJar)
	    nodeType = (String)(((PRootEjbJar)currentPanel).toString());
	else if (currentPanel instanceof String)
	    nodeType = "String";
	else if (currentPanel instanceof FilePaths)
	    nodeType = Consts.STRING_FILE;
	else throw new Error("The associated object of the current node is not a DeployerPanel, not a String, not a PRootEjbJar and not a FilePaths.");
	return nodeType;
    }
    
    /**
     * Add node in the tree.
     * @param parent the parent of the node to add.
     * @param child  the associated object of the node to add.
     * @return the created node.
     */
    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Object child) {
	TreePath childPath;

	DeployerMenuBar.getInstance().setSaveRequired(true);

	DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
	if (parent == null) {
	    parent = top;
	}
	treeModel.insertNodeInto(childNode, parent, parent.getChildCount());
	childPath = new TreePath(childNode.getPath());
	this.collapsePath(childPath);
	return childNode;
    }

    /**
     * Add new panel build from an xml file in the tree.
     * @param parent the parent of this new node.
     * @param child  the associated object of the node.
     * @return the created node.
     */
    public DefaultMutableTreeNode addLoadingPanelInTree(DefaultMutableTreeNode parent,
							Object child) {
	TreePath childPath;

	DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
	treeModel.insertNodeInto(childNode, parent, parent.getChildCount());
	childPath = new TreePath(childNode.getPath());
	this.collapsePath(childPath);
	return childNode;
    }

    /**
     * Get the current Ejb Jar.
     * Currently this method is correct if there is only one Ejb Jar in the tree.
     * @return the current Ejb Jar.
     */
    public Object getCurrentEjbJar() {
	TreePath currentSelection = this.getSelectionPath();
	if (currentSelection == null) {
	    Trace.outln(trace, "DeployerTree.getCurrentEjbJar(): You must select a node.");
	}
	return currentSelection.getPathComponent(1);
    }

    /**
     * Remove a node of the tree (without taking account of Castor objects).
     * @param node the node to remove of the tree.
     */
    public void removeNode(DefaultMutableTreeNode node) {
	if (node.toString().equals(Consts.STRING_EJB_JAR)) {
	    DefaultMutableTreeNode metaInfEjbJarNode = (DefaultMutableTreeNode)node.getChildAt(0);
	    ((DeployerPanel)metaInfEjbJarNode.getUserObject()).clearObligatoryTable();
	    DeployerMenuBar.getInstance().setOneEjbJar(false);
	}
	DefaultMutableTreeNode parent = (DefaultMutableTreeNode)(node.getParent());
	if ((node.getUserObject() instanceof DeployerPanel) 
	    && (parent.getUserObject() instanceof DeployerPanel)) {
	    ((DeployerPanel)parent.getUserObject()).
		removeInSpecificList(((DeployerPanel)node.getUserObject()).getCastorObject());
	}
	DeployerMenuBar.getInstance().setSaveRequired(true);
	if (parent != null) {
	    treeModel.removeNodeFromParent(node);
	    setSelectedNode(parent);
	}
    }

    /** 
     * Remove the currently selected node.
     */
    public void removeCurrentNode() {
	DefaultMutableTreeNode currentNode = new DefaultMutableTreeNode();
	TreePath currentSelection = this.getSelectionPath();
	if (currentSelection != null) {
	    currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
	    removeNode(currentNode);
	} else {
	    Trace.outln(trace, "DeployerTree.removeCurrentNode(): There is no node selected !");
	}
    }

    /**
     * Get the root node of the tree.
     * @return the root node of the tree.
     */
    public DefaultMutableTreeNode getTop() {
	return top;
    }    

    /**
     * Get business and helper classes.
     * @return a list of classes.
     */
    public Vector getFiles(){
	DefaultMutableTreeNode classFile = null;
	DefaultMutableTreeNode parentEjb = (DefaultMutableTreeNode)this.getCurrentEjbJar();
	DefaultMutableTreeNode nextElement;

	Vector files = new Vector();
	//all files including in the ejb jar is represented from the third node of ejb-jar's tree
	for (int i=2; i<parentEjb.getChildCount();i++) {
	    classFile = (DefaultMutableTreeNode)parentEjb.getChildAt(i);
	    if ((classFile.getUserObject()) instanceof FilePaths) {
		FilePaths file = (FilePaths)classFile.getUserObject();
		files.add(file);
	    }
	}
	return files;
    }
    
    /**
     * Invoked by PopupListener and InsertMenuListener to realize the action correspond to user choice
     * @param item the selected item
     * @param frame the frame of BeanDeployer
     */

    protected void insertContextuelNode (String item,JFrame frame) {
	if (item.equals(Consts.TITLE_INSERT_FILE))
	    new InsertFile(frame);
	else if (item.equals(Consts.TITLE_INSERT_ASSEMBLY_DESCRIPTOR))
	    //add a empty assembly-descriptor node in tree
	    this.addPanelInTree(new PAssemblyDescriptor());
	
	else if (item.equals(Consts.TITLE_INSERT_ENTERPRISE_BEANS))
	    // add a empty method-permission node in tree
	    this.addPanelInTree(new PEnterpriseBeans());
	
	else if (item.equals(Consts.TITLE_INSERT_METHOD_PERMISSION))
	    insertMethodPermission();
	
	else if (item.equals(Consts.TITLE_INSERT_SESSION_BEAN))
	    // add a empty session node in tree
	    this.addPanelInTree(new PSession());
	
	else if (item.equals(Consts.TITLE_INSERT_ENTITY_BEAN))
	    //add a empty entity node in tree 
	    this.addPanelInTree(new PEntity());
   
	else if (item.equals(Consts.TITLE_INSERT_METHOD))
	    // add a empty method node in tree
	    this.addPanelInTree(new PMethod());
	   
	else if (item.equals(Consts.TITLE_INSERT_CONTAINER_TRANSACTION))
	    insertContainerTransaction(); 

	else if (item.equals(Consts.TITLE_INSERT_CMP_FIELD))
	    // add a cmp-field node in tree
	    this.addPanelInTree(new PCmpField());

	else if (item.equals(Consts.TITLE_INSERT_EJB_REF))
	    // add a empty ejb-ref node in tree
	    this.addPanelInTree(new PEjbRef());

	else if (item.equals(Consts.TITLE_INSERT_ENV_ENTRY))
	    //add a empty env-entry node in tree
	    this.addPanelInTree(new PEnvEntry());
	
	else if (item.equals(Consts.TITLE_INSERT_RESOURCE_REF))
	    //add a empty resource-ref node in tree
	    this.addPanelInTree(new PResourceRef());
	
	else if (item.equals(Consts.TITLE_INSERT_JONAS_ENTITY))
	    //add a empty jonas-entity node in tree
	    this.addPanelInTree(new PJonasEntity());

	else if (item.equals(Consts.TITLE_INSERT_JONAS_SESSION))
	    //add a empty jonas-session node in tree
	    this.addPanelInTree(new PJonasSession());
	
	else if (item.equals(Consts.TITLE_INSERT_JDBC_MAPPING))
	    //add a empty jonas-jdbc-mapping node in tree
	    this.addPanelInTree(new PJdbcMapping());

	else if (item.equals(Consts.TITLE_INSERT_FINDER_METHOD_JM))
	    //add a empty jonas-finder-jdbc-mapping node in tree
	    this.addPanelInTree(new PFinderMethodJdbcMapping());
	    
	else if (item.equals(Consts.TITLE_INSERT_SECURITY))
	    //add a empty security node in tree
	    this.addPanelInTree(new PSecurityRole());
	
	else if (item.equals(Consts.TITLE_INSERT_SECURITY_REF))
	    //add a empty security-ref node in tree
	    this.addPanelInTree(new PSecurityRoleRef());
	else if (item.equals(Consts.DELETE))
	    DeployerMenuBar.getInstance().delete();
    }

    /**
     * permits to add a empty method-permission node in tree
     **/
    private void insertMethodPermission() {
	DefaultMutableTreeNode methodPermission
	    = this.addPanelInTree(new PMethodPermission());
	// For each method-permission, it is obligatory to have a least one method.
	PMethod temporaryMethod = new PMethod();
	temporaryMethod.setValues(); // This is a stratagem to fill obligatoryTable.
	this.addPanelInTree(methodPermission,temporaryMethod);
    }
    
    /**
     * permits to add a empty container transaction node in tree
     **/
    private void insertContainerTransaction() {
	DefaultMutableTreeNode containerTransaction
	    = this.addPanelInTree(new PContainerTransaction());
	// For each container-transaction, it is obligatory to have a least one method.
	PMethod temporaryMethod = new PMethod();
	temporaryMethod.setValues(); // This is a stratagem to fill obligatoryTable.
	this.addPanelInTree(containerTransaction,temporaryMethod);
    }
	
}


/*********************************************************/
/**
 * MyTreeModelListener extends TreeChangeListener interface.
 * TreeChangeListener defines the interface for an object that listens to changes in a TreeModel.
 **/ 
class MyTreeModelListener implements TreeModelListener {

    static private final int trace = Trace.DB_19;

    /**
     * Invoked after a node (or a set of siblings) has changed in some way.
     * @param e the TreeModelEvent.
     */
    public void treeNodesChanged(TreeModelEvent e) {
	
	DefaultMutableTreeNode node;
	node = (DefaultMutableTreeNode)(e.getTreePath().getLastPathComponent());
	
	/*
	 * If the event lists children, then the changed
	 * node is the child of the node we've already
	 * gotten.  Otherwise, the changed node and the
	 * specified node are the same.
	 */
	try {
	    int index = e.getChildIndices()[0];
	    node = (DefaultMutableTreeNode)(node.getChildAt(index));
	} catch (NullPointerException exc) {
	    Trace.outln(trace, "MyTreeModelListener.treeNodesChanged() ERROR: "+exc.getMessage());
	}

	Trace.outln(trace, "MyTreeModelListener.treeNodesChanged(): The user has finished editing the node.");
	Trace.outln(trace,
		    "MyTreeModelListener.treeNodesChanged(): "+
		    "New value = " + node.getUserObject());
    }

    public void treeNodesInserted(TreeModelEvent e) {
    }

    public void treeNodesRemoved(TreeModelEvent e) {
    }
    
    public void treeStructureChanged(TreeModelEvent e) {
    }	

}
