/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                     FilePaths                      */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.guicomponents;

import org.objectweb.jonas.beandeployer.Consts;
import java.io.File;
import java.util.StringTokenizer;


/**
 * This class permits to know all information about a class file :
 *  <li> its file's name,
 *  <li> its associated package,
 *  <li> its absolute path.
 */ 
public class FilePaths {
    private String file;
    private String jarPath;
    private String absolutePath;
    private String packageName;
    
    /**
     * Build this File path with associated information.
     * @param packageName  the package's name of the class file.
     * @param file         the file's name.
     * @param absolutePath the absolute path where are located the class file.
     **/
    public FilePaths(String packageName,String file, String absolutePath) {
	this.file = file;
	if ( packageName != null) {
	    this.packageName = ChangePointToSlash(packageName);
	    this.jarPath = this.packageName.concat(file);
	}
	else
	    this.jarPath = file;
	this.absolutePath = absolutePath;
    }

    /**
     * Obligatory method for an object associated to a node.
     * @return the path of the class.
     **/
    public String toString() {
	return this.jarPath;
    }
    
    /**
     * Get the type of the object.
     * @return the type of this object = file.
     **/    
    public String getType() {
	return Consts.STRING_FILE;
    }

    /**
     * Get the package's name of this file.
     * @return the package's name of the class file
     **/    
    public String getPackage() {
	return this.packageName;
    }

    /**
     * Get the file of this object.
     * @return the file is the file's name
     **/    
    public String getFile() {
	return this.file;
    }

    /**
     * Get the absolute path of the file.
     * @return the absolute path where are located the class file 
     **/  
    public String getAbsolutePath() {
	return this.absolutePath;
    }
    
    /**
     * Permits to change all point in a slash (ie :javax.swing to javax/swing)
     * used technique String.
     * @param packageName the package's name.
     * @return the package name like a path.
     **/
    private String ChangePointToSlash(String packageName){
	String result = new String();
	StringTokenizer st = new StringTokenizer(packageName,".");
	while (st.hasMoreTokens()){
	    String token = st.nextToken();
	    result = result.concat(token.concat(File.separator));
	}
	return result;
    }    
}
