/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.beandeployer.guicomponents;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.JFrame;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.event.ActionEvent;
import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;
import javax.swing.JMenuItem;

import org.objectweb.jonas.beandeployer.panels.PAssemblyDescriptor;
import org.objectweb.jonas.beandeployer.panels.PCmpField;
import org.objectweb.jonas.beandeployer.panels.PContainerTransaction;
import org.objectweb.jonas.beandeployer.panels.PEjbJar;
import org.objectweb.jonas.beandeployer.panels.PEjbRef;
import org.objectweb.jonas.beandeployer.panels.PEnterpriseBeans;
import org.objectweb.jonas.beandeployer.panels.PEntity;
import org.objectweb.jonas.beandeployer.panels.PEnvEntry;
import org.objectweb.jonas.beandeployer.panels.PFinderMethodJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PJonasEjbJar;
import org.objectweb.jonas.beandeployer.panels.PJonasEntity;
import org.objectweb.jonas.beandeployer.panels.PJonasSession;
import org.objectweb.jonas.beandeployer.panels.PJdbcMapping;
import org.objectweb.jonas.beandeployer.panels.PMethod;
import org.objectweb.jonas.beandeployer.panels.PMethodPermission;
import org.objectweb.jonas.beandeployer.panels.PResourceRef;
import org.objectweb.jonas.beandeployer.panels.PSecurityRole;
import org.objectweb.jonas.beandeployer.panels.PSecurityRoleRef;
import org.objectweb.jonas.beandeployer.panels.PSession;

/**
 * Defines a listener for insert menu event.  
 **/
public class InsertMenuListener implements MenuListener, java.awt.event.ActionListener{

    static private final int trace = Trace.DB_19;

    Hashtable diffChoice;
    private String[] choice;
    private DeployerTree tree;
    private DeployerMenuBar menu;
    private JFrame frame;
    private JMenuItem menuItem;

    /**
     * Construct the InsertMenuListener.
     * It is a dynamic menu : the items changes according to the selected tree's node.
     * @param menu is the menubar of BeanDeployer
     * @param frame is the frame of BeanDeployer (used to display dialogbox)
     **/
    public InsertMenuListener(DeployerMenuBar menu,JFrame frame) {
	this.frame = frame;
	diffChoice = new Hashtable();
	tree = DeployerTree.getInstance();
	//	menu = DeployerMenuBar.getInstance();
	this.menu = menu;
	// for ejb jar node
	diffChoice.put(Consts.STRING_EJB_JAR,Consts.INSERT_EJBJAR);
	
	// for Standard DD node
	diffChoice.put(Consts.STRING_META_INF_EJB_JAR_XML,Consts.INSERT_STANDARDD);
	
	// For assembly-descriptor
	diffChoice.put(Consts.STRING_ASSEMBLY_DESCRIPTOR,Consts.INSERT_ASSENBLY_DESCRIPTOR);
	
	// For enterprise-beans
	diffChoice.put(Consts.STRING_ENTERPRISE_BEANS,Consts.INSERT_ENTERPRISE_BEANS);
	
	// For container transaction node
	diffChoice.put(Consts.STRING_TX,Consts.INSERT_CONTAINER);
	
	// For method-permission
	diffChoice.put(Consts.STRING_METHOD_PERMISSION,Consts.INSERT_METHOD_PERMISSION);
	
	// for entity node
	diffChoice.put(Consts.STRING_ENTITY,Consts.INSERT_ENTITY);
	
	// for session node
	diffChoice.put(Consts.STRING_SESSION,Consts.INSERT_SESSION);
	
	// for JONAS specific node
	diffChoice.put(Consts.STRING_META_INF_JONAS_EJB_JAR_XML,Consts.INSERT_JSPECIFIC);
	
	// for JONAS Entity node
	diffChoice.put(Consts.STRING_JONAS_ENTITY,Consts.INSERT_JENTITY);
	
	// for jdbc-mapping node
	diffChoice.put(Consts.STRING_JDBC_MAPPING,Consts.INSERT_JJDBC_MAPPING);
    }

    /**
     * Invoked when the menu selection changes. 
     * @param e the MenuEvent.
     **/
    public void menuDeselected(MenuEvent e) {}
    
    /**
     * Invoked when the menu selection is canceled.
     * @param e the MenuEvent.
     **/
    public void menuCanceled(MenuEvent e) {}
    
    /**
     *  Invoked when a menu item is selected.
     *  and updated to insert menu which is a dynamic tree.
     * @param e the MenuEvent.
     */
    public void menuSelected(MenuEvent e) {
	boolean assemblyExist = false;
	boolean entrepriseBeansExist = false;
	boolean jdbcMappingExist = false;
	boolean notInsert = false;
	
	DefaultMutableTreeNode currentNode = tree.getSelectedNode();
	
	String selectedNodeType = tree.getSelectedNodeType();
	Trace.outln(trace,
		    "InsertMenuListener.menuSelected(): "+
		    "selected node type = "+selectedNodeType);
	
	if (selectedNodeType.equals(Consts.STRING_META_INF_EJB_JAR_XML)) {
	    assemblyExist = menu.findAssemblyDescriptor(currentNode);
	    entrepriseBeansExist = menu.findEntrepriseBeans(currentNode);
	}
	
	if (selectedNodeType.equals(Consts.STRING_JONAS_ENTITY))
	    jdbcMappingExist = menu.findJdbcMapping(currentNode);

	if (menu.getMenuInsert() ==  null ) {
	    Trace.outln(trace,
			"InsertMenuListener.menuSelected(): menu.getMenuInsert() is null.");
	}
	menu.getMenuInsert().removeAll();
	choice = (String[]) diffChoice.get(selectedNodeType);
	for (int i = 0; i < choice.length; i++){
	    Trace.outln(trace, "InsertMenuListener.menuSelected(): choice = "+choice[i]);
	    if (choice[i].equals(Consts.TITLE_INSERT_ASSEMBLY_DESCRIPTOR)) {
		if ( !assemblyExist ) {
		    addItem(choice[i]);
		} else {
		    notInsert = true;
		    
		}
	    } else if (choice[i].equals(Consts.TITLE_INSERT_ENTERPRISE_BEANS)) {
		if (!entrepriseBeansExist) {
		    addItem(choice[i]);
		} else {
		    notInsert = true;
		}
	    } else if ( choice[i].equals(Consts.TITLE_INSERT_JDBC_MAPPING)) {
		if (!jdbcMappingExist) {
		    addItem(choice[i]);
		} else {
		    notInsert = true;
		}
	    } else if ( notInsert != true  ){
		Trace.outln(trace, "InsertMenuListener.menuSelected(): add item "+choice[i]);
		addItem(choice[i]);
	    } else {
		notInsert = false;
	    }
	}
	assemblyExist = false; 
	entrepriseBeansExist = false;
	jdbcMappingExist = false; 
    }
    
    /**
     * Add an item in Insert menu.
     * @param item is the name which will display in insert menu.
     */
    private void addItem(String item) {
	menuItem = new JMenuItem(item);
	menuItem.addActionListener(this);
	menu.getMenuInsert().add(menuItem); 
    }
    
    /**
     * Invoked when an action occurs.
     * @param e the ActionEvent.
     */
    public void actionPerformed(ActionEvent e) {
	JMenuItem source = (JMenuItem)(e.getSource());
	String s = "Action event detected.\n"
	    + "    Event source: " + source.getText();
	Trace.outln(trace, "InsertMenuListener.actionPerformed(): "+s+"\n");
	String selectedItem = source.getText();
	tree.insertContextuelNode(selectedItem,frame);
    }

}
