/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                      JarFilter                     */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.guicomponents;

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * This class permits to have a filter when user use file chooser.
 */
public class JarFilter extends FileFilter {
    final static String jar = "jar";
    final static String classes = "class";
  
    /**
     * Whether the given file is accepted by this filter.  
     * Accept all directories and all jar, class, or other files.
     **/
    public boolean accept(File f) {
	
        if (f.isDirectory()) {
            return true;
        }
	
        String s = f.getName();
        int i = s.lastIndexOf('.');
	
        if (i > 0 &&  i < s.length() - 1) {
            String extension = s.substring(i+1).toLowerCase();
            if (jar.equals(extension) ||
                classes.equals(extension))
		return true;
	    else
                return false;
	}
	return false;
    }

    /**   
     * The description of this filter
     * for this filter is "java class (*.jar, *.class)"
     * @return the description of this filter.
     **/
    public String getDescription() {
        return "java class (*.jar, *.class)";
    }
}
