/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                    NewBean                         */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.guicomponents;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.common.Trace;

import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

public class NewBean extends JDialog implements PropertyChangeListener {

    static private final int trace = Trace.DB_19;

    private TextFieldElement    displayName;
    private RadioButtonList     beanType;

    private JOptionPane         optionPane;

    final String btnOk = Consts.OK;
    final String btnCancel = Consts.CANCEL;

    private DeployerMenuBar menu;

    /**
     * Create an optionPane when the user clicks on the new bean item.
     * @param JFrame is the frame of the dialog box.
     **/
    public NewBean(JFrame frame){
	super(frame,Consts.NEW_BEAN_TITLE,true);
	init();
    }
    
   
    /**
     * Add graphic elements and get an instance of the DeployerMenuBar.
     */ 
    private void init() {
	Trace.outln(trace, "NewBean.init():");
	this.addGraphicElements();
	menu = DeployerMenuBar.getInstance();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {

	displayName = new TextFieldElement(Consts.LABEL_EJB_NAME, true);

	Vector beanList = new Vector();
	beanList.add(Consts.STRING_SESSION);
	beanList.add(Consts.STRING_ENTITY);
	beanType = new RadioButtonList(Consts.LABEL_BEAN_TYPE,beanList,true);
	
	Object[] array={displayName,beanType};

        Object[] options = {btnOk, btnCancel};
	
        optionPane = new JOptionPane(array, 
                                    JOptionPane.QUESTION_MESSAGE,
                                    JOptionPane.YES_NO_OPTION,
                                    null,
                                    options,
                                    options[0]);

	setContentPane(optionPane);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	pack();
	//	setLocation(xOffset,yOffset);
	optionPane.addPropertyChangeListener(this);
    }

    /**
     * This method gets called when a bound property is changed.
     * @param e is the PropertyChangeEvent.
     **/
    public void propertyChange(PropertyChangeEvent e) {
	String prop = e.getPropertyName();
	
	if (isVisible() 
	    && (e.getSource() == optionPane)
	    && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
		prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
	    Object value = optionPane.getValue();
	    
	    if (value == JOptionPane.UNINITIALIZED_VALUE) {
		//ignore reset
		return;
	    }
	    
	    // Reset the JOptionPane's value.
	    // If you don't do this, then if the user
	    // presses the same button next time, no
	    // property change event will be fired.
	    optionPane.setValue(JOptionPane.UNINITIALIZED_VALUE);
	    //if enter
	    if (value.equals(btnOk)) {
		String ejbName = displayName.getText();
		String bean = beanType.getText();

		if (bean.equals(Consts.STRING_SESSION))
		    menu.newSessionBean(ejbName);
		else // bean.equals(Consts.STRING_ENTITY)
		    menu.newEntityBean(ejbName);
		setVisible(false);
		
	    } else { // user closed dialog or clicked cancel
		Trace.outln(trace, "NewBean.propertyChange(): It's canceled. ");
		setVisible(false);
	    }
	}
    }

}
