/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.beandeployer.guicomponents;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;

import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.JFrame;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import java.awt.event.MouseAdapter;
import javax.swing.tree.TreePath;


/**
 * Defines a listener for insert menu event.  
 **/
public class PopupListener extends MouseAdapter implements java.awt.event.ActionListener {

    static private final int trace = Trace.DB_19;

    Hashtable diffChoice;
    private String[] choice;
    private DeployerTree tree;
    private DeployerMenuBar menu;
    private JFrame frame;
    private JMenuItem menuItem;
    private JPopupMenu popup;

    /**
     * Construct the PopupListener
     * it's a dynamic menu : the items changes according to the selected tree's node 
     * @param menu is the menubar of BeanDeployer
     * @param frame is the frame of BeanDeployer (used to display dialogbox)
     **/
    public PopupListener(JFrame frame) {
	this.frame = frame;
	diffChoice = new Hashtable();
	tree = DeployerTree.getInstance();
	menu = DeployerMenuBar.getInstance();
		// for ejb jar node
	diffChoice.put(Consts.STRING_EJB_JAR,Consts.INSERT_EJBJAR);
	
	// for Standard DD node
	diffChoice.put(Consts.STRING_META_INF_EJB_JAR_XML,Consts.INSERT_STANDARDD);
	
	// For assembly-descriptor
	diffChoice.put(Consts.STRING_ASSEMBLY_DESCRIPTOR,Consts.INSERT_ASSENBLY_DESCRIPTOR);
	
	// For enterprise-beans
	diffChoice.put(Consts.STRING_ENTERPRISE_BEANS,Consts.INSERT_ENTERPRISE_BEANS);
	
	// For container transaction node
	diffChoice.put(Consts.STRING_TX,Consts.INSERT_CONTAINER);
	
	// For method-permission
	diffChoice.put(Consts.STRING_METHOD_PERMISSION,Consts.INSERT_METHOD_PERMISSION);
	
	// for entity node
	diffChoice.put(Consts.STRING_ENTITY,Consts.INSERT_ENTITY);
	
	// for session node
	diffChoice.put(Consts.STRING_SESSION,Consts.INSERT_SESSION);
	
	// for JONAS specific node
	diffChoice.put(Consts.STRING_META_INF_JONAS_EJB_JAR_XML,Consts.INSERT_JSPECIFIC);
	
	// for JONAS Entity node
	diffChoice.put(Consts.STRING_JONAS_ENTITY,Consts.INSERT_JENTITY);
	
	// for jdbc-mapping node
	diffChoice.put(Consts.STRING_JDBC_MAPPING,Consts.INSERT_JJDBC_MAPPING);

	popup = new JPopupMenu();
    }

    /**
     * Manages the event mouse pressed.
     * @param e The mouse event
     */
    public void mousePressed(MouseEvent e) {
	maybeShowPopup(e);
    }

    /**
     * Manages the event mouse released.
     * @param e The mouse event
     */
    public void mouseReleased(MouseEvent e) {
    }
    
    
    /**
     *  Invoked when a menu item is selected.
     *  and updated to insert menu which is a dynamic tree
     */
    public void maybeShowPopup(MouseEvent e) {
	boolean assemblyExist = false;
	boolean entrepriseBeansExist = false;
	boolean jdbcMappingExist = false;
	boolean notInsert = false;
	if (e.isPopupTrigger()) {
	    popup.removeAll();
	    if ( menu == null ) {
		menu = DeployerMenuBar.getInstance();
	    }
	  
	    // Finds the selected node
	    TreePath path = tree.getPathForLocation(e.getX(), e.getY());
	    if (path == null) return;
	    DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
	    if (node == null) return;

	    // Select the node
	    tree.setSelectionPath(path);
	    DefaultMutableTreeNode currentNode = tree.getSelectedNode();
	    
	    if (((tree.getSelectedNodeType().equals(Consts.STRING_METHOD)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_CMPF)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_EJB_REF)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_ENV)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_RES_REF)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_ROLE)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_ROLE_REF)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_JONAS_SESSION)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_FINDER_METHOD_JM)) ||
		 (tree.getSelectedNodeType().equals(Consts.STRING_FILE))) ) {
		addItemDelete();
		popup.show(e.getComponent(),e.getX(), e.getY());
		return;
	    }
	    
	    String selectedNodeType = tree.getSelectedNodeType();
	    
	    if ( selectedNodeType.equals(Consts.STRING_META_INF_EJB_JAR_XML) ) {
		assemblyExist = menu.findAssemblyDescriptor(currentNode);
		entrepriseBeansExist = menu.findEntrepriseBeans(currentNode);
	    }
	    
	    if ( selectedNodeType.equals(Consts.STRING_JONAS_ENTITY) )
		jdbcMappingExist = menu.findJdbcMapping(currentNode);
	    
	    
	    choice = (String[]) diffChoice.get(selectedNodeType);
	    for (int i = 0; i < choice.length; i++){
		if ( choice[i].equals(Consts.TITLE_INSERT_ASSEMBLY_DESCRIPTOR)) {
		    if ( !assemblyExist ) {
			addItem(choice[i]);
		    }
		    else {
			notInsert = true;
			
		    }
		}
		else if ( choice[i].equals(Consts.TITLE_INSERT_ENTERPRISE_BEANS)) {
		    if ( !entrepriseBeansExist ) {
			addItem(choice[i]);
		    } 
		    else  {
			notInsert = true;
		    }
		
		}
		
		else if ( choice[i].equals(Consts.TITLE_INSERT_JDBC_MAPPING)) {
		    if (!jdbcMappingExist) {
			addItem(choice[i]);
		}
		    else  {
			notInsert = true;
		    }
		    
		}	    
	    
		else if ( notInsert != true  ){
		    addItem(choice[i]);
		}
		
		else 
		notInsert = false;
	    }
	    if (! ( tree.getSelectedNodeType().equals(Consts.STRING_META_INF_EJB_JAR_XML)|| 
		    tree.getSelectedNodeType().equals(Consts.STRING_META_INF_JONAS_EJB_JAR_XML)))
		addItemDelete();
	    assemblyExist = false; 
	    entrepriseBeansExist = false;
	    jdbcMappingExist = false; 
	    popup.show(e.getComponent(),e.getX(), e.getY());
	}
    }

    /**
     * add a item in Popup menu
     * @param item is the name which will display in insert menu
     */
    private void addItem(String item) {
	menuItem = new JMenuItem(item);
	menuItem.addActionListener(this);
	popup.add(menuItem); 
    }
    
    /**
     * add a item delete in Popup menu
     */
    private void addItemDelete() {
	popup.addSeparator();
	menuItem = new JMenuItem(Consts.DELETE);
	menuItem.addActionListener(this);
	popup.add(menuItem); 
    }

    /**
     *  Invoked when an action occurs
     */
    public void actionPerformed(ActionEvent e) {
	JMenuItem source = (JMenuItem)(e.getSource());
	String s = "Action event detected.\n"+ "    Event source: " + source.getText();
	Trace.outln(trace, "PopupListener.actionPerformed(): "+s +"\n");
	String selectedItem = source.getText();
	tree.insertContextuelNode(selectedItem,frame);
    }

}

