/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                  DeployerPanel                     */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import javax.swing.JPanel;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import org.objectweb.jonas.beandeployer.Consts;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;


/**
 * Base abstract class for each panel associated to a node for BeanDeployer.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public abstract class DeployerPanel extends JPanel implements MouseListener {
    /** Name of the panel. At the creation it is the type. **/
    protected String panelName;

    /** 
     * Boolean to know if the panel's name is complete or not.
     * Warning : this boolean is not used for panel which not change name.
     */
    protected boolean replacedName;

    /**
     * Key : Panel, Value : Vector containing all obligatory fields not filled.
     * This solution is used because there is only one Ejb Jar.
     */
    protected static Hashtable obligatoryTable = null;

    /** 
     * Boolean to know if the panel's is changed or not.
     */
    protected boolean changedPanel=true; 

    /**
     * Initialize if it is needed the hashtable obligatoryTable.
     */
    protected DeployerPanel() {
	if (this.obligatoryTable == null) obligatoryTable = new Hashtable();
	addMouseListener(this);
	this.initializeName();
    }

    /**
     * Get the table containing for each concerned panel, all obligatory fields not filled.
     * @return the obligatoryTable.
     */
    public Hashtable getObligatoryTable() {
	return obligatoryTable;
    }

    /**
     * Method called only if the node is META-INF/ejb-jar.xml
     * and when it is removed.
     */
    public void clearObligatoryTable() {
	this.obligatoryTable.clear();
    }

    /**
     * Method necessary for an object associated to a node to have a name.
     * @return the name of the panel.
     */
    public String toString() {
	return this.panelName;
    }

    /**
     * Initialisation of the panel's name and the replacedName boolean.
     */
    void initializeName() {
	this.panelName = this.getType();
	this.replacedName = false;
    }

    /**
     * Change the name of the current panel.
     * @param endOfName is the suffix of the new name.
     */
    void changeName(String endOfName) {
	this.replacedName = true;
	this.panelName = getType()+Consts.SEPARATOR+endOfName;
    }

    /**
     * Get the type of the panel.
     * @return the type of the panel.
     */
    public abstract String getType();

    /**
     * Get the associated Castor object.
     * @return the Castor object instanciate in the panel.
     */
    public abstract Object getCastorObject();

    /**
     * Add a child Castor object in the specific list in the associated Castor object of the panel.
     * @param xmlObject is the child Castor object to add.
     */
    public abstract void addInSpecificList(Object xmlObject);

    /**
     * Remove a child Castor object of the specific list in the associated Castor object of the panel.
     * @param xmlObject is the child Castor object to remove.
     */
    public abstract void removeInSpecificList(Object xmlObject);

    /** 
     * Set values in the associated Castor class.
     */
    public abstract boolean setValues();

    /*
     * Get Values of the associated Castor class.
     */
    public abstract void getValues();  

    /**
     * Permits to know if user has gone in the panel.
     * @return true if user has gone in the panel.
     */
    public boolean getChangedPanel(){
	return changedPanel;
    }

    /**
     * Set the value of the changedPanel attribute.
     * @param changedPanel true if the user has gone in the panel.
     */
    public void setChangedPanel(boolean changedPanel){
	this.changedPanel = changedPanel;
    }

    public void mousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {}

    /**
     * Event when the mouse has gone in the panel.
     * @param e the MouseEvent.
     */
    public void mouseEntered(MouseEvent e) {
	this.setChangedPanel(true);
    }

    public void mouseExited(MouseEvent e) {}

    public void mouseClicked(MouseEvent e) {}
   
}
