/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*             AssemblyDescriptor Panel               */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;

import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.ContainerTransaction;
import org.objectweb.jonas_ejb.deployment.xml.MethodPermission;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRole;

/**
 * Empty panel.
 * Contains a list of 
 * <LI>  ContainerTransaction reference,
 * <LI>  MethodPermission reference,
 * <LI>  ContainerTransaction reference.
 * <p>
 * The associated node is :
 *     Ejb Jar -> assembly-descriptor
 * @author Magali MORAEL and Christophe GERMAIN.
 **/
public class PAssemblyDescriptor extends DeployerPanel {
    private AssemblyDescriptor xmlAssemblyDescriptor; 

    /**
     * Build a new Assembly Descriptor.
     */
    public PAssemblyDescriptor() {
	super();
	this.xmlAssemblyDescriptor = new AssemblyDescriptor();
    }

    /**
     * Build an existing Assembly Descriptor.
     * @param xmlAssemblyDescriptor the existing assembly-descriptor Castor object.
     */
    public PAssemblyDescriptor(AssemblyDescriptor xmlAssemblyDescriptor) {
	super();
	this.xmlAssemblyDescriptor = xmlAssemblyDescriptor;
	this.xmlRead(xmlAssemblyDescriptor);
    }
    
    /**
     * Reading xml values.
     * @param xmlAssemblyDescriptor the existing Castor object
     *                              associated to this assembly-descriptor's panel.
     */
    private void xmlRead(AssemblyDescriptor xmlAssemblyDescriptor) {}

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_ASSEMBLY_DESCRIPTOR;
    }

    /**
     * Get the Castor object instanciated by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlAssemblyDescriptor;
    }

    /**
     * Add one of following Castor Object :
     * <LI> container-transaction 
     * <LI> method-permission     
     * <LI> security-role         
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof ContainerTransaction)
	    this.addContainerTransaction((ContainerTransaction)xmlObject);
	else if (xmlObject instanceof MethodPermission)
	    this.addMethodPermission((MethodPermission)xmlObject);
	else if (xmlObject instanceof SecurityRole)
	    this.addSecurityRole((SecurityRole)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor Object :
     * <LI> container-transaction 
     * <LI> method-permission    
     * <LI> security-role        
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof ContainerTransaction)
	    this.removeContainerTransaction((ContainerTransaction)xmlObject);
	else if (xmlObject instanceof MethodPermission)
	    this.removeMethodPermission((MethodPermission)xmlObject);
	else if (xmlObject instanceof SecurityRole)
	    this.removeSecurityRole((SecurityRole)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add a ContainerTransaction
     * @param xmlContainerTransaction the container-transaction to add.
    **/
    public void addContainerTransaction(ContainerTransaction xmlContainerTransaction) {
        this.xmlAssemblyDescriptor.addContainerTransaction(xmlContainerTransaction);
    }

    /**
     * Remove a ContainerTransaction
     * @param xmlContainerTransaction the container-transaction to remove.
     **/
    public void removeContainerTransaction(ContainerTransaction xmlContainerTransaction) {
	ContainerTransaction[] containerTransactionTable 
	    = this.xmlAssemblyDescriptor.getContainerTransaction();
	int i = 0;
	boolean found = false;
	while (i <= containerTransactionTable.length && !found) {
	    if (containerTransactionTable[i].equals(xmlContainerTransaction))
		found = true;
	    i++;
	}
        this.xmlAssemblyDescriptor.removeContainerTransaction(i-1);
    }

    /**
     * Add a MethodPermission
     * @param xmlMethodPermission the method-permission to add.
     **/
    public void addMethodPermission(MethodPermission xmlMethodPermission) {
        this.xmlAssemblyDescriptor.addMethodPermission(xmlMethodPermission);
    }

    /**
     * Remove a MethodPermission
     * @param xmlMethodPermission the method-permission to remove.
     **/
    public void removeMethodPermission(MethodPermission xmlMethodPermission) {
	MethodPermission[] methodPermissionTable
	    = xmlAssemblyDescriptor.getMethodPermission();
	int i = 0;
	boolean found = false;
	while (i <= methodPermissionTable.length && !found) {
	    if (methodPermissionTable[i].equals(xmlMethodPermission))
		found = true;
	    i++;
	}
        this.xmlAssemblyDescriptor.removeMethodPermission(i-1);
    }

    /**
     * Add a SecurityRole
     * @param xmlSecurityRole the security-role to add.
     **/
    public void addSecurityRole(SecurityRole xmlSecurityRole) {
        this.xmlAssemblyDescriptor.addSecurityRole(xmlSecurityRole);
    }
 
    /**
     * Remove a SecurityRole
     * @param xmlSecurityRole the security-role to remove.
     **/
    public void removeSecurityRole(SecurityRole xmlSecurityRole) {
	SecurityRole[] securityRoleTable
	    = xmlAssemblyDescriptor.getSecurityRole();
	int i = 0;
	boolean found = false;
	while (i <= securityRoleTable.length && !found) {
	    if (securityRoleTable[i].equals(xmlSecurityRole))
		found = true;
	    i++;
	}
        this.xmlAssemblyDescriptor.removeSecurityRole(i-1);
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public boolean setValues() {
	return true;
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public void getValues() {}
}
