/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                  CmpField Panel                    */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.common.Trace;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import org.objectweb.jonas_ejb.deployment.xml.CmpField;

import java.util.Vector;

/**
 * Panel CmpField.
 * There is one TextFieldDesc.
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> enterprise-beans -> entity -> CMPF
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PCmpField extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private CmpField xmlCmpField;

    private TextFieldElement    fieldName;
    private TextAreaDescription description;

    private Vector fieldsNoFilled;

    /**
     * Build a new CmpField panel.
     */
    public PCmpField() {
	super();
	this.init();
	this.xmlCmpField = new CmpField();
	this.xmlCreation();
    }

    /**
     * Build an existing CmpField panel.
     * @param xmlCmpField the existing cmp-field Castor object.
     */
    public PCmpField(CmpField xmlCmpField) {
	super();
	this.init();
	this.xmlCmpField = xmlCmpField;
	this.xmlCreation();
	this.xmlRead(xmlCmpField); 
    }

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }

    /**
     * Add a TextFieldDesc.
     */
    private void addGraphicElements() {
	// Add description : TextAreaDescription
	this.constraints.gridx = Consts.GRIDX_FIRST;
	this.constraints.gridy = Consts.GRIDY_FIRST;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.description = new TextAreaDescription();
	this.panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add field-name : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+1;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.fieldName = new TextFieldElement(Consts.LABEL_FIELD_NAME, true);
	this.panelLayout.setConstraints(fieldName,constraints);
	this.add(fieldName);
	fieldsNoFilled.add(Consts.LABEL_FIELD_NAME);

	obligatoryTable.put(this,fieldsNoFilled);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
   }

    /**
     * Reading xml values.
     * @param xmlCmpField the existing Castor object associated to this cmp-field's panel.
     */
    private void xmlRead(CmpField xmlCmpField) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlCmpField.getFieldName() != null ) {
	    String fieldNameString = xmlCmpField.getFieldName();
	    fieldName.setText(fieldNameString);
	    changeName(fieldNameString);
	} else {
	    fieldsNoFilled.add(Consts.LABEL_FIELD_NAME);
	}

	if ( xmlCmpField.getDescription() != null ) {
	    description.setText(xmlCmpField.getDescription());
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);	
	else obligatoryTable.remove(this);
    }
    
    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_CMPF;
    }

    /**
     * Get the Castor object instanciated by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlCmpField;
    }

    /**
     * This method is not used because CmpField is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because CmpField is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	fieldsNoFilled.removeAllElements(); // re-initialization

	// field-name
	tempString = this.fieldName.getText();
	if (tempString.length() != 0) {
	    this.xmlCmpField.setFieldName(tempString);
	    changeName(tempString);
	} else {
	    Trace.outln(trace, "PCmpField.setValues(): field-name is an obligatory field, it must be filled.");
	    this.xmlCmpField.setFieldName(null);
	    if (replacedName) {
		initializeName();
	    }
	    fieldsNoFilled.add(Consts.LABEL_FIELD_NAME);
	}

	// description
	tempString = this.description.getText();
	if (tempString.length() != 0) {
	    this.xmlCmpField.setDescription(tempString);
	}
	else {
	    this.xmlCmpField.setDescription(null);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);	
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if (this.xmlCmpField.getFieldName() != null) {
	    Trace.outln(trace,
			"PCmpField.getValues(): xmlCmpField.getFieldName() = "+
			this.xmlCmpField.getFieldName());
	}
	if (this.xmlCmpField.getDescription() != null) {
	    Trace.outln(trace,
			"PCmpField.getValues(): xmlCmpField.getDescription() = "+
			this.xmlCmpField.getDescription());
	}
    }
}
