/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*         Container transaction Panel                */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.common.Trace;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import org.objectweb.jonas_ejb.deployment.xml.ContainerTransaction;
import org.objectweb.jonas_ejb.deployment.xml.ResourceRef;
import org.objectweb.jonas_ejb.deployment.xml.Method;

import java.util.Vector;

/**
 * Panel Container-Transaction.
 * There are :
 * <LI> one TextAreaDescription
 * <LI> one ComboBoxList
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> assembly-descriptor -> TX
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PContainerTransaction extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private Vector transAttributeList;

    private TextAreaDescription description;
    private ComboBoxList        transAttribute;

    private ContainerTransaction xmlContainerTransaction;

    private Vector  fieldsNoFilled;
    private boolean oneMethod = false;

    /**
     * Build the Container Transaction panel.
     */
    public PContainerTransaction() {
	super();
	this.init();
	this.xmlContainerTransaction = new ContainerTransaction();
	this.xmlCreation();
    }

    /**
     * Build the Container Transaction panel.
     * @param xmlContainerTransaction is the associated Castor class.
     */
    public PContainerTransaction(ContainerTransaction xmlContainerTransaction) {
	super();
	this.init();
	this.xmlContainerTransaction = xmlContainerTransaction;
	this.xmlCreation();
	this.xmlRead(xmlContainerTransaction);
    }

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	this.panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	this.constraints = new GridBagConstraints();	
	
	this.addGraphicElements();
    }

    /**
     * Add a TextAreaDescription and a ComboBoxList.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	this.constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	this.constraints.gridy = Consts.GRIDY_FIRST;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.description = new TextAreaDescription();
	this.panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add trans-attribute :  ComboBoxList - obligatory field.
	this.transAttributeList = new Vector();
	this.transAttributeList.add(Consts.ATTRIBUTE_NOTSUPPORTED);
	this.transAttributeList.add(Consts.ATTRIBUTE_SUPPORTS);
	this.transAttributeList.add(Consts.ATTRIBUTE_REQUIRED);
	this.transAttributeList.add(Consts.ATTRIBUTE_REQUIREDNEW);
	this.transAttributeList.add(Consts.ATTRIBUTE_MANDATORY);
	this.transAttributeList.add(Consts.ATTRIBUTE_NEVER);

	this.constraints.gridy = Consts.GRIDY_FIRST+1;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.transAttribute = new ComboBoxList(Consts.LABEL_TRANS_ATTRIBUTE,
					transAttributeList,true);
	this.panelLayout.setConstraints(transAttribute,constraints);
	this.add(transAttribute);

	//There is no adding of method in the obligatory table because a node method is created.
	oneMethod = true;
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_TX;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlContainerTransaction;
    }

     /**
     * Reading xml values.
     * @param xmlContainerTransaction is the existing Castor object
     *        associated to this container-transaction's panel.
     */
    private void xmlRead(ContainerTransaction xmlContainerTransaction) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlContainerTransaction.getDescription() != null )
	    description.setText(xmlContainerTransaction.getDescription());

	if ( xmlContainerTransaction.getTransAttribute() != null ) {
	    String transAttributeString = xmlContainerTransaction.getTransAttribute();
	    transAttribute.setText(transAttributeString);
	    changeName(transAttributeString);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_TRANS_ATTRIBUTE);
	}

	if (!oneMethod)
	    fieldsNoFilled.add(Consts.STRING_METHOD);

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }

    /**
     * Add one Castor Object :
     * <LI> method.
     * @param xmlObject is the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Method)
	    this.addMethod((Method)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one Castor Object :
     * <LI> method.
     * @param xmlObject is the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Method)
	    this.removeMethod((Method)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }
   
    /**
     * Add a Method
     * @param xmlMethod the method to add.
     **/
    public void addMethod(Method xmlMethod) {
	this.xmlContainerTransaction.addMethod(xmlMethod);
	if (!oneMethod) {
	    oneMethod = true;
	    this.fieldsNoFilled.remove(Consts.STRING_METHOD);
 	    if (fieldsNoFilled.size() != 0)
		obligatoryTable.put(this,fieldsNoFilled);
	    else obligatoryTable.remove(this);
	}
    }

    /**
     * Remove a Method
     * @param xmlMethod the method to remove.
     **/
    public void removeMethod(Method xmlMethod) {
	Method[] methodTable = this.xmlContainerTransaction.getMethod();
	int i = 0;
	boolean found = false;
	while (i <= methodTable.length && !found) {
	    if (methodTable[i].equals(xmlMethod))
		found = true;
	    i++;
	}
	this.xmlContainerTransaction.removeMethod(i-1);
	// There are n methods in the list before the removing, there are n-1 method after.
	if (methodTable.length-1 == 0) {
	    oneMethod = false;
	    this.fieldsNoFilled.add(Consts.STRING_METHOD);
	    obligatoryTable.put(this,fieldsNoFilled);
	}
    }

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// Description
	tempString = this.description.getText();
	if (tempString.length() != 0) {
	    this.xmlContainerTransaction.setDescription(tempString);
	} else {
	    this.xmlContainerTransaction.setDescription(null);
	}
	
	// TransAttribute
	tempString = this.transAttribute.getText();
	if (tempString.length() != 0) {
	    this.xmlContainerTransaction.setTransAttribute(tempString);
	    changeName(tempString);
	} else { 
	    // Impossible here except for reading of an XML file which contains 
	    // the tag trans-attribute without value.
	    Trace.outln(trace,
			"PContainerTransaction.setValues(): trans-attribute is an obligatory field, it must be filled.");
	    this.xmlContainerTransaction.setTransAttribute(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_TRANS_ATTRIBUTE);
	}
	
	if (!oneMethod)
	    fieldsNoFilled.add(Consts.STRING_METHOD);

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( this.xmlContainerTransaction.getDescription() != null) {
	    Trace.outln(trace,
			"PContainerTransaction.getValues(): xmlContainerTransaction.getDescription() = "+
			this.xmlContainerTransaction.getDescription());
	}
	if ( this.xmlContainerTransaction.getTransAttribute() != null) {
	    Trace.outln(trace,
			"PContainerTransaction.getValues(): xmlContainerTransaction.getTransAttribute() = "+
			this.xmlContainerTransaction.getTransAttribute());
	}
    }

}
