/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   EjbJar Panel                     */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.FileChooserElement;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.AssemblyDescriptor;
import org.objectweb.jonas_ejb.deployment.xml.EjbJar;
// For lists
import org.objectweb.jonas_ejb.deployment.xml.EnterpriseBeans;
import org.objectweb.jonas_ejb.deployment.xml.Entity;
import org.objectweb.jonas_ejb.deployment.xml.Session;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import java.util.Vector;

/**
 * Panel EjbJar.
 * There are :
 * <LI> one TextFieldDesc
 * <LI> one TextFieldElement
 * <LI>  one FileChooserList
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml.
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PEjbJar extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    displayName;
    private TextAreaDescription description;
    private TextFieldElement    ejbClientJar;
    private FileChooserElement  smallIcon, largeIcon;

    private EjbJar          xmlEjbJar;
    
    private AssemblyDescriptor xmlAssemblyDescriptor;
    private EnterpriseBeans    xmlEnterpriseBeans;

    /**
     * Build a new panel.
     */
    public PEjbJar() {
	super();
	this.init();
	this.xmlEjbJar = new EjbJar();
	this.xmlCreation();
    }

    /**
     * Build an existing panel.
     * @param xmlEjbJar the existing ejb-jar Castor object.
     */
    public PEjbJar(EjbJar xmlEjbJar) {
	super();
	this.init();
	this.xmlEjbJar = xmlEjbJar;
	this.xmlCreation();
	this.xmlRead(xmlEjbJar);
    }

    /**
     * Initialize the layout and its constraints.
     * Add a TextFieldDesc, a TextFieldElement and a FileChooserList.
     */
    private void init() {
	this.panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	this.constraints = new GridBagConstraints();	
	this.addGraphicElements();
    }

    /*
     * Add a TextFieldDesc, a TextFieldElement and a FileChooserList.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	this.constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	this.constraints.gridy = Consts.GRIDY_FIRST;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.description = new TextAreaDescription();
	this.panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add display-name : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+1;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.displayName = new TextFieldElement(Consts.LABEL_DISPLAY_NAME, false);
	this.panelLayout.setConstraints(displayName,constraints);
	this.add(displayName);


	// Add smallIcon : FileChooserElement
	this.constraints.gridy = Consts.GRIDY_FIRST+2;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.smallIcon = new FileChooserElement(Consts.LABEL_SMALL_ICON,false);
	this.smallIcon.setEnabled(false);
	this.panelLayout.setConstraints(smallIcon,constraints);
	this.add(smallIcon);

	// Add largeIcon : FileChooserElement
	this.constraints.gridy = Consts.GRIDY_FIRST+3;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.largeIcon = new FileChooserElement(Consts.LABEL_LARGE_ICON,false);
	this.largeIcon.setEnabled(false);
	this.panelLayout.setConstraints(largeIcon,constraints);
	this.add(largeIcon);

	// Add ejbClientJar : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+4;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.ejbClientJar = new TextFieldElement(Consts.LABEL_EJB_CLIENT_JAR, false);
	this.panelLayout.setConstraints(ejbClientJar,constraints);
	this.add(ejbClientJar);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	this.xmlEnterpriseBeans = new EnterpriseBeans();
    }

    /**
     * Reading xml values.
     * @param xmlEjbJar the existing Castor object
     *        associated to this META-INF/ejb-jar.xml's panel.
     */
    private void xmlRead(EjbJar xmlEjbJar) {

	if ( xmlEjbJar.getDisplayName() != null ) {
	    this.displayName.setText(xmlEjbJar.getDisplayName());
	    Trace.outln(trace,
			"PEjbJar.xmlRead(): displayName = "+
			displayName.getText());
	}
	if ( xmlEjbJar.getDescription() != null ) {
	    this.description.setText(xmlEjbJar.getDescription());
	    Trace.outln(trace,
			"PEjbJar.xmlRead(): description = "+
			description.getText());
	}
	if (xmlEjbJar.getEjbClientJar() != null ) {
	    this.ejbClientJar.setText(xmlEjbJar.getEjbClientJar());
	    Trace.outln(trace,
			"PEjbJar.xmlRead(): ejbClientJar = "+
			ejbClientJar.getText());
	}
	if ( xmlEjbJar.getSmallIcon() != null ) {
	    this.smallIcon.setText(xmlEjbJar.getSmallIcon());
	    Trace.outln(trace,
			"PEjbJar.xmlRead(): smallIcon = "+
			smallIcon.getText());
	}
	if ( xmlEjbJar.getLargeIcon() != null ) {
	    this.largeIcon.setText(xmlEjbJar.getLargeIcon());
	    Trace.outln(trace,
			"PEjbJar.xmlRead(): largeIcon = "+
			largeIcon.getText());
	}

   }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_META_INF_EJB_JAR_XML;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlEjbJar;
    }


    /**
     * Add one of followinf Castor Object :
     * <li> assembly-descriptor
     * <li> enterprise-beans
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof AssemblyDescriptor) {
	    this.setAssemblyDescriptor((AssemblyDescriptor)xmlObject);
	} else if (xmlObject instanceof EnterpriseBeans){
	    Trace.outln(trace, "PEjbJar.addInSpecificList(): addInSpecificList one EnterpriseBeans");
	    this.setEnterpriseBeans((EnterpriseBeans)xmlObject);
	} else {
	    throw new Error("The parameter is not a Castor object.");
	}
    }

    /**
     * Remove one of followinf Castor Object :
     * <li> assembly-descriptor
     * <li> enterprise-beans
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof AssemblyDescriptor)
	    this.setAssemblyDescriptor(null);
	else if (xmlObject instanceof EnterpriseBeans)
	    this.setEnterpriseBeans(null);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Set the AssemblyDescriptor
     * @param xmlAssemblyDescriptor the assembly-descriptor to set.
     */
    public void setAssemblyDescriptor(AssemblyDescriptor xmlAssemblyDescriptor) {
	this.xmlEjbJar.setAssemblyDescriptor(xmlAssemblyDescriptor);
    }

    /**
     * Set the EnterpriseBeans
     * @param xmlEnterpriseBeans the enterprise-beans to set.
     */
    public void setEnterpriseBeans(EnterpriseBeans xmlEnterpriseBeans) {
	this.xmlEjbJar.setEnterpriseBeans(xmlEnterpriseBeans);
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	// DISPLAY NAME
	tempString = this.displayName.getText();
	if (tempString.length() != 0) {
	    this.xmlEjbJar.setDisplayName(tempString);
	} else {
	    this.xmlEjbJar.setDisplayName(null);
	}
	
	// DESCRIPTION
	tempString = this.description.getText();
	if (tempString.length() != 0) {
	    this.xmlEjbJar.setDescription(tempString);
	} else { 
	    this.xmlEjbJar.setDescription(null);
	}

	// EJB client jar
	tempString = this.ejbClientJar.getText();
	if (tempString.length() != 0) {
	    this.xmlEjbJar.setEjbClientJar(tempString);
	} else { 
	    this.xmlEjbJar.setEjbClientJar(null);
	}

	// Small-icon
	tempString = this.smallIcon.getText();
	if (tempString.length() != 0) {
	    this.xmlEjbJar.setSmallIcon(tempString);
	} else { 
	    this.xmlEjbJar.setSmallIcon(null);
	}

	// Large-icon
	tempString = this.largeIcon.getText();
	if (tempString.length() != 0) {
	    this.xmlEjbJar.setLargeIcon(tempString);
	} else { 
	   this.xmlEjbJar.setLargeIcon(null);
	}

	return result;
    }  

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if (this.xmlEjbJar.getDisplayName() != null) {
	    Trace.outln(trace,
			"PEjbJar.getValues(): xmlEjbJar.getDisplayName() = "+
			this.xmlEjbJar.getDisplayName());
	}
	if (this.xmlEjbJar.getDescription() != null) {
	    Trace.outln(trace,
			"PEjbJar.getValues(): xmlEjbJar.getDescription() = "+
			       this.xmlEjbJar.getDescription());
	}
	if (this.xmlEjbJar.getEjbClientJar() != null) {
 	    Trace.outln(trace,
			"PEjbJar.getValues(): xmlEjbJar.EjbClientJar() = "+
			       this.xmlEjbJar.getEjbClientJar());
	}
	if (this.xmlEjbJar.getSmallIcon() != null) {
	    Trace.outln(trace,
			"PEjbJar.getValues(): xmlEjbJar.SmallIcon() = "+
			       this.xmlEjbJar.getSmallIcon());
	}
	if (this.xmlEjbJar.getLargeIcon() != null) {
	    Trace.outln(trace,
			"PEjbJar.getValues(): xmlEjbJar.LargeIcon() = "+
			       this.xmlEjbJar.getLargeIcon());
	}
    }

}
