/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   Ejb-Ref Panel                    */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.FileChooserElement;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.EjbRef;

import java.util.Vector;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Ejb Ref Panel.
 * There are :
 * <li> one TextFieldDesc.
 * <li> four TextFieldElement,
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> enterprise-beans -> entity -> ejb-ref
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PEjbRef extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    ejbRefName;
    private TextAreaDescription description;
    private TextFieldElement    home, remote, ejbLink;
    private RadioButtonList     ejbRefType;

    private EjbRef         xmlEjbRef;

    private Vector fieldsNoFilled;

    /**
     * Build a new Ejb Ref panel.
     */
    public PEjbRef() {
	super();
	this.init();
	this.xmlEjbRef = new EjbRef();
	this.xmlCreation();
    }  
    /**
     * Build an existing Ejb Ref panel.
     * @param xmlEjbRef the existing ejb-ref Castor object.
     */
    public PEjbRef(EjbRef xmlEjbRef) {
	super();
	this.init();
	this.xmlEjbRef = xmlEjbRef;
	this.xmlCreation();
        this.xmlRead(xmlEjbRef);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	this.panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	this.constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	this.constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	this.constraints.gridy = Consts.GRIDY_FIRST;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.description = new TextAreaDescription();
	this.panelLayout.setConstraints(this.description,this.constraints);
	this.add(this.description);

	// Add ejb-ref-name : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+1;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.ejbRefName = new TextFieldElement(Consts.LABEL_EJB_REF_NAME, true);
	this.panelLayout.setConstraints(this.ejbRefName,this.constraints);
	this.add(this.ejbRefName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_NAME);

	// Add ejbRefType : RadioButtonList - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+2;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector buttonListNames = new Vector();
	buttonListNames.add(Consts.TYPE_ENTITY);
	buttonListNames.add(Consts.TYPE_SESSION);
	this.ejbRefType = new RadioButtonList(Consts.LABEL_EJB_REF_TYPE,buttonListNames,true);
	this.panelLayout.setConstraints(this.ejbRefType,this.constraints);
	this.add(this.ejbRefType);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_TYPE);

	// Add home : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+3;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.home = new TextFieldElement(Consts.LABEL_HOME,true);
	this.panelLayout.setConstraints(this.home,this.constraints);
	this.add(this.home);
	this.fieldsNoFilled.add(Consts.LABEL_HOME);

	// Add remote : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+4;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.remote = new TextFieldElement(Consts.LABEL_REMOTE,true);
	this.panelLayout.setConstraints(this.remote,this.constraints);
	this.add(this.remote);
	this.fieldsNoFilled.add(Consts.LABEL_REMOTE);

	// Add ejb-link : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+5;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.ejbLink = new TextFieldElement(Consts.LABEL_EJB_REF_LINK,false);
	this.panelLayout.setConstraints(this.ejbLink,this.constraints);
	this.add(this.ejbLink);

	obligatoryTable.put(this,fieldsNoFilled);
    }
    
    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlEjbRef the existing Castor object
     *        associated to this ejb-ref's panel.
     */   
    private void xmlRead(EjbRef xmlEjbRef) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlEjbRef.getDescription() != null )
	    description.setText(xmlEjbRef.getDescription());

	if (  xmlEjbRef.getEjbRefName() != null) {
	    String ejbRefNameString = xmlEjbRef.getEjbRefName();
	    ejbRefName.setText(ejbRefNameString);
	    changeName(ejbRefNameString);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_NAME);
	}

	if (  xmlEjbRef.getEjbRefType() != null) {
	    if (!(ejbRefType.setTextAndCheck(xmlEjbRef.getEjbRefType()))) {
		xmlEjbRef.setEjbRefType(null);
		this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_TYPE);
	    }
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_TYPE);
	}

	if (  xmlEjbRef.getHome() != null)
	    home.setText(xmlEjbRef.getHome());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);

	if (  xmlEjbRef.getRemote() != null)
	    remote.setText(xmlEjbRef.getRemote());
	else
	    this.fieldsNoFilled.remove(Consts.LABEL_REMOTE);

	if (  xmlEjbRef.getEjbLink() != null)
	    ejbLink.setText(xmlEjbRef.getEjbLink());

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);	
	else
	    obligatoryTable.remove(this);
    }
	
    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_EJB_REF;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlEjbRef;
    }

    /**
     * This method is not used because EjbRef is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because EjbRef is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization
	
	// EJB REJ NAME
	tempString = this.ejbRefName.getText();
	if (tempString.length() != 0) {
	    if (tempString.equals(this.xmlEjbRef.getEjbRefName())) {
		result = false; // There is no change.
	    } else {
		this.xmlEjbRef.setEjbRefName(tempString);
		changeName(tempString);
	    }
	} else {
	    Trace.outln(trace,
			"PEjbRef.setValues(): Ejb-ref-name is an obligatory field, it must be filled.");
	    this.xmlEjbRef.setEjbRefName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_NAME);
	} 
	 
	// DESCRIPTION
	tempString = this.description.getText();
	if (tempString.length() != 0) {
	    if (tempString.equals(this.xmlEjbRef.getDescription())) {
		result = false; // There is no change.
	    } else {
		this.xmlEjbRef.setDescription(tempString);
	    }
	} else { 
	    this.xmlEjbRef.setDescription(null);
	}
	
	// ejb-ref-type
	tempString = this.ejbRefType.getText();
	if ( tempString != null ) {
	    if (tempString.length() != 0) {
		if (tempString.equals(this.xmlEjbRef.getEjbRefType())) {
		    result = false; // There is no change.
		} else {
		    this.xmlEjbRef.setEjbRefType(tempString);
		}
	    } else {
	    Trace.outln(trace,
			"PEjbRef.setValues(): ejb-ref-type is an obligatory field, it must be filled.");
		this.xmlEjbRef.setEjbRefType(null);
		this.fieldsNoFilled.add(Consts.LABEL_EJB_REF_TYPE);
	    }
	}

	// Home class
	tempString = this.home.getText();
	if (tempString.length() != 0) {
	    if (tempString.equals(this.xmlEjbRef.getHome())) {
		result = false; // There is no change.
	    } else {
		this.xmlEjbRef.setHome(tempString);
	    }
	} else { 
	    Trace.outln(trace,
			"PEjbRef.setValues(): home is an obligatory field, it must be filled.");
	    this.xmlEjbRef.setHome(null);
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);
	}

	// Remote Class
	tempString = this.remote.getText();
	if (tempString.length() != 0) {
	    if (tempString.equals(this.xmlEjbRef.getRemote())) {
		result = false; // There is no change.
	    } else {
		this.xmlEjbRef.setRemote(tempString);
	    }
	} else { 
	    Trace.outln(trace,
			"PEjbRef.setValues(): remote is an obligatory field, it must be filled.");
	    this.xmlEjbRef.setRemote(null);
	    this.fieldsNoFilled.add(Consts.LABEL_REMOTE);
	}

	// ejb class
	tempString = this.ejbLink.getText(); 
	if (tempString.length() != 0) {
	    if (tempString.equals(this.xmlEjbRef.getEjbLink())) {
		result = false; // There is no change.
	    } else {
		this.xmlEjbRef.setEjbLink(tempString);
	    }
	} else {
	    this.xmlEjbRef.setEjbLink(null);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);	
	else
	    obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( this.xmlEjbRef.getEjbRefName() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.getEjbRefName() = "+
			this.xmlEjbRef.getEjbRefName());
	}
	if ( this.xmlEjbRef.getDescription() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.getDescription() = "+
			this.xmlEjbRef.getDescription());
	}
	if ( this.xmlEjbRef.getEjbRefType() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.getEjbRefType() = "+
			this.xmlEjbRef.getEjbRefType());
	}
	if ( this.xmlEjbRef.getHome() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.Home() = "+
			this.xmlEjbRef.getHome());
	}
   	if ( this.xmlEjbRef.getRemote() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.Remote() = "+
			this.xmlEjbRef.getRemote());
	}
   	if ( this.xmlEjbRef.getEjbLink() != null ) {
	    Trace.outln(trace,
			"PEjbRef.getValues(): xmlEjbRef.EjbLink() = "+
			this.xmlEjbRef.getEjbLink());
	}
    }

}
