/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*               EnterpriseBeans Panel                */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.EnterpriseBeans;
import org.objectweb.jonas_ejb.deployment.xml.Entity;
import org.objectweb.jonas_ejb.deployment.xml.Session;

/**
 * Empty panel.
 * Contains a list of 
 * <li> Entity reference,
 * <li> Session reference.
 * <p>
 * The associated node is Ejb Jar -> EnterpriseBeans
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PEnterpriseBeans extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private EnterpriseBeans xmlEnterpriseBeans;
    private Entity          xmlEntity;
    private Session         xmlSession;

    /**
     * Build a new EnterpriseBeans.
     */
    public PEnterpriseBeans() {
	super();
	this.xmlEnterpriseBeans = new EnterpriseBeans();
    }

    /**
     * Build an existing EnterpriseBeans.
     * @param xmlEnterpriseBeans the existing enterprise-beans Castor object.
     */
    public PEnterpriseBeans(EnterpriseBeans xmlEnterpriseBeans) {
	super();
	this.xmlEnterpriseBeans = xmlEnterpriseBeans;
	this.xmlRead(xmlEnterpriseBeans);
    }

    /**
     * Reading xml values.
     * @param xmlEnterpriseBeans is the existing Castor object
     *        associated to this enterprise-beans's panel.
     */
    private void xmlRead(EnterpriseBeans xmlEnterpriseBeans) {

    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_ENTERPRISE_BEANS;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlEnterpriseBeans;
    }

    /**
     * Add one of following Castor Object :
     * <li> entity
     * <li> session
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Entity) {
	    Trace.outln(trace,
			"PEnterpriseBeans.addInSpecificList(): addInSpecificList one Entity");
	    this.addEntity((Entity)xmlObject);
	} else if (xmlObject instanceof Session) {
	    Trace.outln(trace,
			"PEnterpriseBeans.addInSpecificList(): addInSpecificList one Session");
	    this.addSession((Session)xmlObject);
	} else {
	    throw new Error("The parameter is not a Castor object (Entity or Session).");
	}
    }  
  
    /**
     * Remove one of following Castor Object :
     * <li> entity
     * <li> session
     * @param xmlObject the Castor object to add.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Entity) {
	    this.removeEntity((Entity)xmlObject);
	} else if (xmlObject instanceof Session) {
	    this.removeSession((Session)xmlObject);
	} else {
	    throw new Error("The parameter is not a Castor object.");
	}
    }

    /**
     * Add an Entity.
     * @param xmlEntity the entity to add.
     */
    public void addEntity(Entity xmlEntity) {
	this.xmlEnterpriseBeans.addEntity(xmlEntity);
    }

    /**
     * Remove an Entity.
     * @param xmlEntity the entity to remove.
     */
    public void removeEntity(Entity xmlEntity) {
	Entity[] entityTable = this.xmlEnterpriseBeans.getEntity();
	int i = 0;
	boolean found = false;
	while (i <= entityTable.length && !found) {
	    if (entityTable[i].equals(xmlEntity))
		found = true;
	    i++;
	}
        this.xmlEnterpriseBeans.removeEntity(i-1);
    }
    /**
     * Add a Session.
     * @param xmlSession the session to add.
     */
    public void addSession(Session xmlSession) {
	this.xmlEnterpriseBeans.addSession(xmlSession);
    }

    /**
     * Remove a Session.
     * @param xmlSession the session to remove.
     */
    public void removeSession(Session xmlSession) {
	Session[] sessionTable = this.xmlEnterpriseBeans.getSession();
	int i = 0;
	boolean found = false;
	while (i <= sessionTable.length && !found) {
	    if (sessionTable[i].equals(xmlSession))
		found = true;
	    i++;
	}
        this.xmlEnterpriseBeans.removeSession(i-1);
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public boolean setValues() {
	return true;
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public void getValues() {}
}
