/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   Entity Panel                     */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;


import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.FileChooserElement;
import org.objectweb.jonas.beandeployer.components.CheckBoxList;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.Entity;

// For lists
import org.objectweb.jonas_ejb.deployment.xml.EnvEntry;
import org.objectweb.jonas_ejb.deployment.xml.CmpField;
import org.objectweb.jonas_ejb.deployment.xml.EjbRef;
import org.objectweb.jonas_ejb.deployment.xml.ResourceRef;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRoleRef;

import java.util.Enumeration;
import java.util.Vector;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

/**
 * Panel Entity.
 * There are :
 * <li> one TextFieldDesc.
 * <li> three TextFieldElement,
 * <li> one FileChooserMultiple,
 * <li> one TextFieldMultiple,
 * <li> one RadioButtonList,
 * <li> one CheckBoxList.
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> enterprise-beans -> entity -> CMPF
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PEntity extends DeployerPanelWithEjbName {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    displayName;
    private TextAreaDescription description;
    private TextFieldElement    ejbName;
    private FileChooserElement  smallIcon, largeIcon;
    private TextFieldElement    home, remote, ejbClass;
    private RadioButtonList     persistenceType;
    private ComboBoxList        primKeyClass;
    private TextFieldElement    primKeyField;
    private CheckBoxList        reentrant;

    private Entity          xmlEntity;
   
    private Vector fieldsNoFilled;

    /**
     * Build a new Entity panel.
     */
    public PEntity() {
	super();
	this.init();
	this.xmlEntity = new Entity();
	this.xmlCreation();
    }  

    /**
     * Build an existing Entity panel.
     * @param xmlEntity the existing entity Castor object.
     */
    public PEntity(Entity xmlEntity) {
	super();
	this.init();
	this.xmlEntity = xmlEntity;
	this.xmlCreation();
	this.xmlRead(xmlEntity);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	this.panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	this.constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	this.constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	this.constraints.gridy = Consts.GRIDY_FIRST;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.description = new TextAreaDescription();
	this.panelLayout.setConstraints(this.description,this.constraints);
	this.add(this.description);

	// Add display-name : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+1;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.displayName = new TextFieldElement(Consts.LABEL_DISPLAY_NAME, false);
	this.panelLayout.setConstraints(this.displayName,this.constraints);
	this.add(this.displayName);

	// Add smallIcon : FileChooserElement
	this.constraints.gridy = Consts.GRIDY_FIRST+2;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.smallIcon = new FileChooserElement(Consts.LABEL_SMALL_ICON,false);
	this.smallIcon.setEnabled(false);
	this.panelLayout.setConstraints(this.smallIcon,this.constraints);
	this.add(this.smallIcon);

	// Add largeIcon : FileChooserElement
	this.constraints.gridy = Consts.GRIDY_FIRST+3;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.largeIcon = new FileChooserElement(Consts.LABEL_LARGE_ICON,false);
	this.largeIcon.setEnabled(false);
	this.panelLayout.setConstraints(this.largeIcon,this.constraints);
	this.add(this.largeIcon);

	// Add ejbName : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+4;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.ejbName = new TextFieldElement(Consts.LABEL_EJB_NAME, true);
	this.panelLayout.setConstraints(this.ejbName,this.constraints);
	this.add(this.ejbName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	// Add home : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+5;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.home = new TextFieldElement(Consts.LABEL_HOME,true);
	this.panelLayout.setConstraints(this.home,this.constraints);
	this.add(this.home);
	this.fieldsNoFilled.add(Consts.LABEL_HOME);

	// Add remote : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+6;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.remote = new TextFieldElement(Consts.LABEL_REMOTE,true);
	this.panelLayout.setConstraints(this.remote,this.constraints);
	this.add(this.remote);
	this.fieldsNoFilled.add(Consts.LABEL_REMOTE);

	// Add ejb-class : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+7;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.ejbClass = new TextFieldElement(Consts.LABEL_EJB_CLASS,true);
	this.panelLayout.setConstraints(this.ejbClass,this.constraints);
	this.add(this.ejbClass);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);
	
	// Add persistence-type : RadioButtonList - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+8;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector persistenceList = new Vector();
	persistenceList.add(Consts.TYPE_CONTAINER_MANAGED_PERSISTENCE);
	persistenceList.add(Consts.TYPE_BEAN_MANAGED_PERSISTENCE);
	persistenceType = new RadioButtonList(Consts.LABEL_PERSISTENCE_TYPE,persistenceList,true);
	this.panelLayout.setConstraints(this.persistenceType,this.constraints);
	this.add(this.persistenceType);
	this.fieldsNoFilled.add(Consts.LABEL_PERSISTENCE_TYPE);

	// Add prim-key-class : TextFieldElement - obligatory field.
	this.constraints.gridy = Consts.GRIDY_FIRST+9;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;

	Vector primKeyClassList = new Vector();
	primKeyClassList.add(Consts.TYPE_BOOLEAN);
	primKeyClassList.add(Consts.TYPE_STRING);
	primKeyClassList.add(Consts.TYPE_INTEGER);
	primKeyClassList.add(Consts.TYPE_DOUBLE);
	primKeyClassList.add(Consts.TYPE_BYTE);
	primKeyClassList.add(Consts.TYPE_SHORT);
	primKeyClassList.add(Consts.TYPE_LONG);
	primKeyClassList.add(Consts.TYPE_FLOAT);
	this.primKeyClass = new ComboBoxList(Consts.LABEL_PRIM_KEY_CLASS, primKeyClassList, true);
	this.primKeyClass.setEditable(true);
	this.panelLayout.setConstraints(this.primKeyClass,this.constraints);
	this.add(this.primKeyClass);
	this.fieldsNoFilled.add(Consts.LABEL_PRIM_KEY_CLASS);

	// Add reentrant : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+10;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector reentrantList = new Vector();
	reentrantList.add(Consts.LABEL_REENTRANT);
	this.reentrant = new CheckBoxList(Consts.EMPTY_STRING,reentrantList,false);
	this.panelLayout.setConstraints(this.reentrant,this.constraints);
	this.add(this.reentrant);
	
	// Add prim-key-field : TextFieldElement
	this.constraints.gridy = Consts.GRIDY_FIRST+11;
	this.constraints.fill = GridBagConstraints.HORIZONTAL;
	this.primKeyField = new TextFieldElement(Consts.LABEL_PRIM_KEY_FIELD, false);
	this.panelLayout.setConstraints(this.primKeyField,this.constraints);
	this.add(this.primKeyField);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlEntity the existing Castor object
     *        associated to this entity's panel.
     */
    private void xmlRead(Entity xmlEntity) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlEntity.getDisplayName() != null )
	    displayName.setText(xmlEntity.getDisplayName());
    
	if ( xmlEntity.getDescription() != null )
	    description.setText(xmlEntity.getDescription());
	
	if ( xmlEntity.getEjbName() != null ) {
	    String ejbNameString = xmlEntity.getEjbName();
	    ejbName.setText(ejbNameString);
	    changeName(ejbNameString);
	}
	else
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	if ( xmlEntity.getSmallIcon() != null )
	    smallIcon.setText(xmlEntity.getSmallIcon());

	if ( xmlEntity.getLargeIcon() != null )
	    largeIcon.setText(xmlEntity.getLargeIcon());

	if ( xmlEntity.getHome() != null )
	    home.setText(xmlEntity.getHome());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);

	if ( xmlEntity.getRemote() != null )
	    remote.setText(xmlEntity.getRemote());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_REMOTE);

	if ( xmlEntity.getEjbClass() != null )
	    ejbClass.setText(xmlEntity.getEjbClass());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);

	if ( xmlEntity.getPersistenceType() != null ) {
	    if (!(persistenceType.setTextAndCheck(xmlEntity.getPersistenceType()))) {
		xmlEntity.setPersistenceType(null);
		this.fieldsNoFilled.add(Consts.LABEL_PERSISTENCE_TYPE);
	    }
	} else 
	    this.fieldsNoFilled.add(Consts.LABEL_PERSISTENCE_TYPE);

	if ( xmlEntity.getPrimKeyClass() != null )
	    primKeyClass.setText(xmlEntity.getPrimKeyClass());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_PRIM_KEY_CLASS);

	if ( xmlEntity.getPrimkeyField() != null )
	   primKeyField.setText(xmlEntity.getPrimkeyField());

	if ( xmlEntity.getReentrant() != null ){
	    if ( xmlEntity.getReentrant().equalsIgnoreCase("True")) {
		reentrant.setSelected(Consts.LABEL_REENTRANT,true);
	    } else {
		reentrant.setSelected(Consts.LABEL_REENTRANT,false);
	    }
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_ENTITY;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlEntity;
    }

    /**
     * Add one of following Castor Object :
     * <li> EjbRef
     * <li> CmpField
     * <li> EnvEntry
     * <li> ResourceRef
     * <li> SecurityRoleRef
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof EjbRef)
	    this.addEjbRef((EjbRef)xmlObject);
	else if (xmlObject instanceof CmpField)
	    this.addCmpField((CmpField)xmlObject);
	else if (xmlObject instanceof EnvEntry)
	    this.addEnvEntry((EnvEntry)xmlObject);
	else if (xmlObject instanceof ResourceRef)
	    this.addResourceRef((ResourceRef)xmlObject);
	else if (xmlObject instanceof SecurityRoleRef)
	    this.addSecurityRoleRef((SecurityRoleRef)xmlObject);
	else
	    throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor Object :
     * <li> EjbRef
     * <li> CmpField
     * <li> EnvEntry
     * <li> ResourceRef
     * <li> SecurityRoleRef
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof EjbRef)
	    this.removeEjbRef((EjbRef)xmlObject);
	else if (xmlObject instanceof CmpField)
	    this.removeCmpField((CmpField)xmlObject);
	else if (xmlObject instanceof EnvEntry)
	    this.removeEnvEntry((EnvEntry)xmlObject);
	else if (xmlObject instanceof ResourceRef)
	    this.removeResourceRef((ResourceRef)xmlObject);
	else if (xmlObject instanceof SecurityRoleRef)
	    this.removeSecurityRoleRef((SecurityRoleRef)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add an EjbRef
     * @param xmlEjbRef the ejbRef to add.
     */
    public void addEjbRef(EjbRef xmlEjbRef) {
	xmlEntity.addEjbRef(xmlEjbRef);
    }

    /**
     * Remove an EjbRef
     * @param xmlEjbRef the ejbRef to remove.
     */
    public void removeEjbRef(EjbRef xmlEjbRef) {
	EjbRef[] ejbRefTable = this.xmlEntity.getEjbRef();
	int i = 0;
	boolean found = false;
	while (i <= ejbRefTable.length && !found) {
	    if (ejbRefTable[i].equals(xmlEjbRef))
		found = true;
	    i++;
	}
	this.xmlEntity.removeEjbRef(i-1);
    }

    /**
     * Add a CmpField.
     * @param xmlCmpField the cmp-field to add.
     */
    public void addCmpField(CmpField xmlCmpField) {
	xmlEntity.addCmpField(xmlCmpField);
    }

    /**
     * Remove a CmpField.
     * @param xmlCmpField the cmp-field to remove.
     */
    public void removeCmpField(CmpField xmlCmpField) {
	CmpField[] cmpFieldTable = this.xmlEntity.getCmpField();
	int i = 0;
	boolean found = false;
	while (i <= cmpFieldTable.length && !found) {
	    if (cmpFieldTable[i].equals(xmlCmpField))
		found = true;
	    i++;
	}
	xmlEntity.removeCmpField(i-1);
    }

    /**
     * Add a EnvEntry.
     * @param xmlEnvEntry the env-entry to add.
     */
    public void addEnvEntry(EnvEntry xmlEnvEntry) {
	xmlEntity.addEnvEntry(xmlEnvEntry);
    }

    /**
     * Remove a EnvEntry.
     * @param xmlEnvEntry the env-entry to remove.
     */
    public void removeEnvEntry(EnvEntry xmlEnvEntry) {
	EnvEntry[] envEntryTable = this.xmlEntity.getEnvEntry();
	int i = 0;
	boolean found = false;
	while (i <= envEntryTable.length && !found) {
	    if (envEntryTable[i].equals(xmlEnvEntry))
		found = true;
	    i++;
	}
	xmlEntity.removeEnvEntry(i-1);
    }

    /**
     * Add a ResourceRef.
     * @param xmlResourceRef the resource-ref to add.
     */
    public void addResourceRef(ResourceRef xmlResourceRef) {
	xmlEntity.addResourceRef(xmlResourceRef);
    }

    /**
     * Remove a ResourceRef.
     * @param xmlResourceRef the resource-ref to remove.
     */
    public void removeResourceRef(ResourceRef xmlResourceRef) {
	ResourceRef[] resourceRefTable = this.xmlEntity.getResourceRef();
	int i = 0;
	boolean found = false;
	while (i <= resourceRefTable.length && !found) {
	    if (resourceRefTable[i].equals(xmlResourceRef))
		found = true;
	    i++;
	}
	this.xmlEntity.removeResourceRef(i-1);
    }

    /**
     * Add a SecurityRoleRef.
     * @param xmlSecurityRoleRef the security-role-ref to add.
     */
    public void addSecurityRoleRef(SecurityRoleRef xmlSecurityRoleRef) {
	xmlEntity.addSecurityRoleRef(xmlSecurityRoleRef);
    }

    /**
     * Remove a SecurityRoleRef.
     * @param xmlSecurityRoleRef the security-role-ref to remove.
     */
    public void removeSecurityRoleRef(SecurityRoleRef xmlSecurityRoleRef) {
	SecurityRoleRef[] securityRoleRefTable = this.xmlEntity.getSecurityRoleRef();
	int i = 0;
	boolean found = false;
	while (i <= securityRoleRefTable.length && !found) {
	    if (securityRoleRefTable[i].equals(xmlSecurityRoleRef))
		found = true;
	    i++;
	}
	xmlEntity.removeSecurityRoleRef(i-1);
    }

    /**
     * Set ejb-name in the panel.
     * This method is used in this class and invoked
     * by DeployerMenuBar when user create a new bean.
     * @param newEjbName the new ejb name.
     */
    public void setEjbName(String newEjbName) {
	if (newEjbName.length() != 0) {
	    xmlEntity.setEjbName(newEjbName);
	    changeName(newEjbName);
	    this.ejbName.setText(newEjbName);
	    this.fieldsNoFilled.remove(Consts.LABEL_EJB_NAME);
	}
    }

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization	

	// DISPLAY NAME
	tempString = displayName.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setDisplayName(tempString);
	} else {
	    xmlEntity.setDisplayName(null);
	}
	
	// DESCRIPTION
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setDescription(tempString);
	} else { 
	    xmlEntity.setDescription(null);
	}

	// EJB Name
	tempString = ejbName.getText();
	if (tempString.length() != 0) {
	    this.setEjbName(tempString);
	}      
	else {
	    Trace.outln(trace,
			"PEntity.setValues(): ejb-name is an obligatory field, it must be filled.");
	    xmlEntity.setEjbName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}

	// Small-icon
	tempString = smallIcon.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setSmallIcon(tempString);
	} else { 
	    xmlEntity.setSmallIcon(null);
	}

	// Large-icon
	tempString = largeIcon.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setLargeIcon(tempString);
	} else { 
	    xmlEntity.setLargeIcon(null);
	}	
	
	// Home class
	tempString = home.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setHome(tempString);
	} else { 
	    Trace.outln(trace,
			"PEntity.setValues(): home is an obligatory field, it must be filled.");
	    xmlEntity.setHome(null);
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);
	}

	// Remote Class
	tempString = remote.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setRemote(tempString);
	} else { 
	    Trace.outln(trace,
			"PEntity.setValues(): remote is an obligatory field, it must be filled.");
	    xmlEntity.setRemote(null);
	    this.fieldsNoFilled.add(Consts.LABEL_REMOTE);
	}

	// ejb class
	tempString = ejbClass.getText(); 
	if (tempString.length() != 0) {
	    xmlEntity.setEjbClass(tempString);
	} else {
	    Trace.outln(trace,
			"PEntity.setValues(): ejb-class is an obligatory field, it must be filled.");
	    xmlEntity.setEjbClass(null);
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);
	}

       	// Persistence type
	tempString = persistenceType.getText();
	if (tempString != null){
	    if (tempString.length() != 0) {
		xmlEntity.setPersistenceType(tempString);
	    } else {
		Trace.outln(trace,
			"PEntity.setValues(): persitence type is an obligatory field, it must be filled.");
		xmlEntity.setPersistenceType(null);
		this.fieldsNoFilled.add(Consts.LABEL_PERSISTENCE_TYPE);
	    }
	}

	// Prim-key-class
	tempString = primKeyClass.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setPrimKeyClass(tempString);
	} else { 
	    Trace.outln(trace,
			"PEntity.setValues(): Prim-key-class is an obligatory field, it must be filled.");
	    xmlEntity.setPrimKeyClass(null);
	    this.fieldsNoFilled.add(Consts.LABEL_PRIM_KEY_CLASS);
	}

	// Prim-Key-Field
	tempString = primKeyField.getText();
	if (tempString.length() != 0) {
	    xmlEntity.setPrimkeyField(tempString);
	} else { 
	    xmlEntity.setPrimkeyField(null);
	}

	if ( reentrant.isSelectedAt(0)) {
	    xmlEntity.setReentrant(Consts.TRUE);
	} else {
	    xmlEntity.setReentrant(Consts.FALSE);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( xmlEntity.getDisplayName() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getDisplayName() = "+
			xmlEntity.getDisplayName());
	}
	if ( xmlEntity.getDescription() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getDescription() = "+
			xmlEntity.getDescription());
	}
	if ( xmlEntity.getEjbName() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getEjbName() = "+
			xmlEntity.getEjbName());
	}
	if ( xmlEntity.getSmallIcon() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getSmallIcon() = "+
			xmlEntity.getSmallIcon());
	}
	if ( xmlEntity.getLargeIcon() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getLargeIcon() = "+
			xmlEntity.getLargeIcon());
	}
	if ( xmlEntity.getHome() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getHome() = "+
			xmlEntity.getHome());
	}
	if ( xmlEntity.getRemote() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getRemote() = "+
			xmlEntity.getRemote());
	}
	if ( xmlEntity.getEjbClass() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getEjbClass() = "+
			xmlEntity.getEjbClass());
	}
	if ( xmlEntity.getPersistenceType() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getPersistenceType() = "+
			xmlEntity.getPersistenceType());
	}
	if ( xmlEntity.getPrimKeyClass() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getPrimKeyClass() = "+
			xmlEntity.getPrimKeyClass());
	}
	if ( xmlEntity.getPrimkeyField() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getPrimKeyField() = "+
			xmlEntity.getPrimkeyField());
	}
	if ( xmlEntity.getReentrant() != null ) {
	    Trace.outln(trace,
			"PEntity.getValues(): xmlEntity.getReentrant() = "+
			xmlEntity.getReentrant());
	}
    }
    
    /**
     * Get the value of the ejb-name field.
     * @return the ejb-name.
     */
    public String getEjbName(){
	return ejbName.getText();
    }
   
}
