/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   EnvEntry Panel                   */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.common.Trace;

import javax.swing.JPanel;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import org.objectweb.jonas_ejb.deployment.xml.EnvEntry;

import java.util.Vector;

/**
 * Panel EnvEntry.
 * There are :
 * <li> one TextFieldDesc
 * <li> one ComboBoxList
 * <li> one TextFieldElement
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> enterprise-beans -> entity or session -> ENV
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PEnvEntry extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private EnvEntry xmlEnvEntry;

    private TextFieldElement    envEntryName;
    private TextAreaDescription description;
    private ComboBoxList        envEntryType;
    private TextFieldElement    envEntryValue;

    private Vector fieldsNoFilled;

    /**
     * Build a new EnvEntry panel.
     */
    public PEnvEntry() {
	super();
	this.init();
	this.xmlEnvEntry = new EnvEntry();
	this.xmlCreation();
    }

    /**
     * Build an existing EnvEntry panel.
     * @param xmlEnvEntry the existing env-entry Castor object.
     */
    public PEnvEntry(EnvEntry xmlEnvEntry) {
	super();
	this.init();
	this.xmlEnvEntry = xmlEnvEntry;
	this.xmlCreation();
	this.xmlRead(xmlEnvEntry);
    }

    /**
     * Initialize the layout and its constraints.
     * Add a TextFieldDesc, a ComboBoxList and a TextFieldElement
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }

    /**
     * Add a TextFieldDesc, a ComboBoxList and a TextFieldElement.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add env-entry-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	envEntryName = new TextFieldElement(Consts.LABEL_ENV_ENTRY_NAME,true);
	panelLayout.setConstraints(envEntryName,constraints);
	this.add(envEntryName);
	this.fieldsNoFilled.add(Consts.LABEL_ENV_ENTRY_NAME);

	// Add env-entry-type : ComboBoxList - obligatory field.
	Vector envEntryTypeList = new Vector();
	envEntryTypeList.add(Consts.TYPE_BOOLEAN);
	envEntryTypeList.add(Consts.TYPE_STRING);
	envEntryTypeList.add(Consts.TYPE_INTEGER);
	envEntryTypeList.add(Consts.TYPE_DOUBLE);
	envEntryTypeList.add(Consts.TYPE_BYTE);
	envEntryTypeList.add(Consts.TYPE_SHORT);
	envEntryTypeList.add(Consts.TYPE_LONG);
	envEntryTypeList.add(Consts.TYPE_FLOAT);

	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	envEntryType = new ComboBoxList(Consts.LABEL_ENV_ENTRY_TYPE,
					envEntryTypeList,true);
	panelLayout.setConstraints(envEntryType,constraints);
	this.add(envEntryType);
	this.fieldsNoFilled.add(Consts.LABEL_ENV_ENTRY_TYPE);

	// Add env-entry-value : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	envEntryValue = new TextFieldElement(Consts.LABEL_ENV_ENTRY_VALUE,false);
	panelLayout.setConstraints(envEntryValue,constraints);
	this.add(envEntryValue);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * reading xml values.
     * @param xmlEnvEntry the existing Castor object
     *        associated to this env-entry's panel.
     */
    private void xmlRead(EnvEntry xmlEnvEntry) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlEnvEntry.getEnvEntryName() != null ) {
	    String envEntryNameString = xmlEnvEntry.getEnvEntryName();
	    envEntryName.setText(envEntryNameString);
	    changeName(envEntryNameString);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_ENV_ENTRY_NAME);
	}
	
	if ( xmlEnvEntry.getDescription() != null ) {
	    description.setText(xmlEnvEntry.getDescription());
	}
    
	if ( xmlEnvEntry.getEnvEntryType() != null ) {
	    envEntryType.setText(xmlEnvEntry.getEnvEntryType());
	} else {
	    this.fieldsNoFilled.remove(Consts.LABEL_ENV_ENTRY_TYPE);
	}
	
	if ( xmlEnvEntry.getEnvEntryValue() != null ) {
	    envEntryValue.setText(xmlEnvEntry.getEnvEntryValue());
	}

	if (fieldsNoFilled.size() != 0) {
	    obligatoryTable.put(this,fieldsNoFilled);
	} else {
	    obligatoryTable.remove(this);
	}
    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_ENV;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlEnvEntry;
    }

    /**
     * This method is not used because EnvEntry is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because EnvEntry is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }    

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// ENV-ENTRY-NAME
	tempString = envEntryName.getText();
	if (tempString.length() != 0) {
	    xmlEnvEntry.setEnvEntryName(tempString);
	    changeName(tempString);
	} else {
	    Trace.outln(trace,
			"PEnvEntry.setValues(): env-entry-name is an obligatory field, it must be filled.");
	    xmlEnvEntry.setEnvEntryName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_ENV_ENTRY_NAME);
	}
	
	// DESCRIPTION
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlEnvEntry.setDescription(tempString);
	} else { 
	    xmlEnvEntry.setDescription(null);
	}

	// ENV-ENTRY-TYPE
	tempString = envEntryType.getText();
	if (tempString.length() != 0) {
	    xmlEnvEntry.setEnvEntryType(tempString);
	} else {
	    Trace.outln(trace,
			"PEnvENtry.setValues(): env-entry-type is an obligatory field, it must be filled.");
	    xmlEnvEntry.setEnvEntryType(null);
	    this.fieldsNoFilled.add(Consts.LABEL_ENV_ENTRY_TYPE);
	}

	// ENV-ENTRY-VALUE
	tempString = envEntryValue.getText();
	if (tempString.length() != 0) {
	    xmlEnvEntry.setEnvEntryValue(tempString);
	} else {
	    xmlEnvEntry.setEnvEntryValue(null);
	}

	if (fieldsNoFilled.size() != 0) {
	    obligatoryTable.put(this,fieldsNoFilled);
	} else {
	    obligatoryTable.remove(this);
	}

	return result;
    }

    /**
     * Get values in the associated Castor class.
     */
    public void getValues() {
	if ( xmlEnvEntry.getEnvEntryName() != null ) {
	    Trace.outln(trace,
			"PEnvEntry.getValues(): xmlEnvEntry.getEnvEntryName() = "+
			xmlEnvEntry.getEnvEntryName());
	}
	if ( xmlEnvEntry.getDescription() != null ) {
	    Trace.outln(trace,
			"PEnvEntry.getValues(): xmlEnvEntry.getDescription() = "+
			xmlEnvEntry.getDescription());
	}
	if ( xmlEnvEntry.getEnvEntryType() != null ) {
	    Trace.outln(trace,
			"PEnvEntry.getValues(): xmlEnvEntry.getEnvEntryType() = "+
			xmlEnvEntry.getEnvEntryType());
	}
	if ( xmlEnvEntry.getEnvEntryValue() != null ) {
	    Trace.outln(trace,
			"PEnvEntry.getValues(): xmlEnvEntry.getEnvEntryValue() = "+
			xmlEnvEntry.getEnvEntryValue());
	}
    }

}
