/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*         finder method jdbc mapping Panel           */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.FinderMethodJdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.JonasMethod;
import org.objectweb.jonas_ejb.deployment.xml.MethodParams;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JSeparator;

/**
 * Panel finder mapping jdbc Mapping.
 * There are :
 * <li> one ComboBoxList
 * <li> one TableComponent,
 * <li> one TextAreaDescription
 * <p>
 * The associated node is Ejb Jar -> META-INF/jonas-ejb-jar.xml -> entity -> jdbc mapping
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PFinderMethodJdbcMapping extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private Vector methodNameList;  // For ComboBoxList
    private Vector methodParamsList;// For TableComponent

    String[] columnNames = {Consts.LABEL_METHOD_PARAM};

    private JLabel           jonasMethod;
    private ComboBoxList     methodName;
    private TableComponent   methodParams;
    private TextFieldElement jdbcWhereClause;

    private FinderMethodJdbcMapping xmlFinderMethodJdbcMapping;
    // Temporary Castor objects
    private JonasMethod             xmlJonasMethod;
    private MethodParams            xmlMethodParams;

    private Vector fieldsNoFilled;

    /**
     * Build Finder Method Jdbc Mapping panel
     **/
    public PFinderMethodJdbcMapping() {
	super();
	this.xmlCreation();
	this.xmlFinderMethodJdbcMapping = new FinderMethodJdbcMapping();
	this.init();
    }  

    /**
     * Build an existing Finder Method Jdbc mapping panel.
     * @param xmlFinderMethodJdbcMapping the existing finder-method-jdbc-mapping.
     */
    public PFinderMethodJdbcMapping(FinderMethodJdbcMapping xmlFinderMethodJdbcMapping) {
	super();
	this.xmlCreation();
	this.xmlFinderMethodJdbcMapping = xmlFinderMethodJdbcMapping;
	this.init();
	this.xmlRead(xmlFinderMethodJdbcMapping);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add jonas-method : label to indicate an xml tag.
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jonasMethod = new JLabel(Consts.LABEL_JONAS_METHOD+Consts.OBLIGATORY_STAR);
	panelLayout.setConstraints(jonasMethod,constraints);
	this.add(jonasMethod);

	// Add method-name : ComboBoxList - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	methodNameList = new Vector();
	methodName = new ComboBoxList(Consts.LABEL_METHOD_NAME,
				      methodNameList,true);
	methodName.setEditable(true);
	panelLayout.setConstraints(methodName,constraints);
	this.add(methodName);
	this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);
	
	// Add method-params : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	methodParamsList = new Vector();
	String[] temp = {Consts.EMPTY_STRING};
	methodParamsList.add(temp);
	methodParams = new TableComponent(Consts.LABEL_METHOD_PARAMS,
					  columnNames,methodParamsList,false);
	panelLayout.setConstraints(methodParams,constraints);
	this.add(methodParams);

	// Add a JToolBarSeparator
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.insets = new Insets(Consts.TOP_INSETS_SEPARATOR,
					Consts.LEFT_INSETS_SEPARATOR,
					Consts.BOTTOM_INSETS_SEPARATOR,
					Consts.RIGHT_INSETS_SEPARATOR);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	JSeparator separator = new JSeparator();
	panelLayout.setConstraints(separator,constraints);
	this.add(separator);

	// add jdbc-where-clause : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+4;
	constraints.insets = new Insets(Consts.TOP_INSETS_DEFAULT,
					Consts.LEFT_INSETS_DEFAULT,
					Consts.BOTTOM_INSETS_DEFAULT,
					Consts.RIGHT_INSETS_DEFAULT);
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jdbcWhereClause = new TextFieldElement(Consts.LABEL_JDBC_WHERE_CLAUSE,true);
	panelLayout.setConstraints(jdbcWhereClause,constraints);
	this.add(jdbcWhereClause);

	obligatoryTable.put(this,fieldsNoFilled);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	xmlJonasMethod = new JonasMethod();
	xmlMethodParams = new MethodParams();
    }

    /**
     * Reading xml values.
     * @param xmlFinderMethodJdbcMapping the existing Castor object
     *        associated to this finder-method-jdbc-mapping's panel.
     */   
    private void xmlRead(FinderMethodJdbcMapping xmlFinderMethodJdbcMapping) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlFinderMethodJdbcMapping.getJonasMethod() != null) {
	    xmlJonasMethod = xmlFinderMethodJdbcMapping.getJonasMethod();
	    if (xmlJonasMethod.getMethodName() != null) {
		String methodNameString = xmlJonasMethod.getMethodName();
		methodName.setText(methodNameString);
		changeName(methodNameString);
	    } else {
		this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);
	    }
	    if (xmlJonasMethod.getMethodParams() != null) {
		Enumeration e = xmlJonasMethod.getMethodParams().enumerateMethodParam();
		boolean modif=false;
		while ( e.hasMoreElements() ){
		    modif=true;
		    String methodParam = (String)e.nextElement();
		    methodParamsList.add(methodParam);
		    xmlMethodParams.addMethodParam(methodParam);
		}
		if (modif) {
		    methodParamsList.removeElementAt(0); // vector contains empty string at element 0
		    methodParams.setValue(methodParamsList);
		}
	    }
	}

	if ( xmlFinderMethodJdbcMapping.getJdbcWhereClause() != null )
	    jdbcWhereClause.setText(xmlFinderMethodJdbcMapping.getJdbcWhereClause());

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else
	    obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_FINDER_METHOD_JM;
    }
    
    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlFinderMethodJdbcMapping;
    }

    /**
     * This method is not used because EjbRef is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because EjbRef is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// method name	
	tempString = this.methodName.getText();
	if (tempString.length() != 0) {
	    xmlJonasMethod.setMethodName(tempString);
	    changeName(tempString);
	} else {
	    Trace.outln(trace,
			"PFinderMethodJdbcMapping.setValues(): method-name is an obligatory field, it must be filled.");
	    xmlJonasMethod.setMethodName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);
	} 
	
	// method-params
	methodParams.tableReading();
	if (methodParams.getRowsCount() > 1) {
	    for (int i = 0 ; i < methodParams.getRowsCount() ; i++) {
		if ( ((String)methodParams.getData(i,0)).length() != 0) {
		    xmlMethodParams.addMethodParam((String)methodParams.getData(i,0));
		}
	    }
	    xmlJonasMethod.setMethodParams(xmlMethodParams);
	}
	xmlFinderMethodJdbcMapping.setJonasMethod(xmlJonasMethod);
	
	//jdbc where clause
	tempString = this.jdbcWhereClause.getText();
	if (tempString.length() != 0) {
	    xmlFinderMethodJdbcMapping.setJdbcWhereClause(tempString);	
	} else {
	    xmlFinderMethodJdbcMapping.setJdbcWhereClause(Consts.EMPTY_STRING);
	}
	
	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get Values of the associated castor Class.
     **/
    public void getValues(){
	if ( xmlFinderMethodJdbcMapping.getJonasMethod() != null) {
	    JonasMethod jmethod =  xmlFinderMethodJdbcMapping.getJonasMethod();
	    if ( jmethod.getMethodName() != null) {
		Trace.outln(trace,
			    "PFinderMethodJdbcMapping.getValues(): jmethod.getMethodName() = "+
			    jmethod.getMethodName());
	    }
	    if ( jmethod.getMethodParams() != null) {
		Enumeration e = jmethod.getMethodParams().enumerateMethodParam();
		while ( e.hasMoreElements() ){
		    String param = (String)e.nextElement();
		    Trace.outln(trace,
			    "PFinderMethodJdbcMapping.getValues():j method.getParams() = "+
				param);
		}
	    }
	}

	if ( xmlFinderMethodJdbcMapping.getJdbcWhereClause() != null ) {
	    Trace.outln(trace,
			    "PFinderMethodJdbcMapping.getValues(): xmlFinderMethodJdbcMapping.getJdbcWhereClause() = "+
			xmlFinderMethodJdbcMapping.getJdbcWhereClause());
	}
    }

}
