/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 jdbc Mapping Panel                 */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.CmpFieldJdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.FinderMethodJdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.JdbcMapping;

import java.util.Vector;
import java.util.Enumeration;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Panel Jonas jdbc Mapping.
 * There are :
 * <li> two TextFieldElement
 * <li> one TableComponent,
 * <p>
 * The associated node is Ejb Jar -> META-INF/jonas-ejb-jar.xml -> entity -> jdbc mapping
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PJdbcMapping extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement jndiName;
    private TextFieldElement jdbcTableName;
    private TableComponent   cmpFieldJdbcMapping;

    private Vector           cmpFieldJdbcMappingList;

    private JdbcMapping         xmlJdbcMapping;

    String[] columnNames = {Consts.LABEL_FIELD_NAME,Consts.LABEL_JDBC_FIELD_NAME};

    private Vector fieldsNoFilled;

    /**
     * Build a new Jdbc mapping panel.
     */
    public PJdbcMapping() {
	super();
	this.init();
	this.xmlJdbcMapping = new JdbcMapping();
	this.xmlCreation();
    }  

    /**
     * Build an existing Jdbc mapping panel.
     * @param xmlJdbcMapping the existing jdbc-mapping Castor object.
     */
    public PJdbcMapping(JdbcMapping xmlJdbcMapping) {
	super();
	this.init();
	this.xmlJdbcMapping = xmlJdbcMapping;
	this.xmlCreation();
	this.xmlRead(xmlJdbcMapping);
    }  
   /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add jndi-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jndiName = new TextFieldElement(Consts.LABEL_JNDI_NAME, true);
	panelLayout.setConstraints(jndiName,constraints);
	this.add(jndiName);
	this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);

	// Add jdbc-table-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jdbcTableName = new TextFieldElement(Consts.LABEL_JDBC_TABLE_NAME, true);
	panelLayout.setConstraints(jdbcTableName,constraints);
	this.add(jdbcTableName);
	this.fieldsNoFilled.add(Consts.LABEL_JDBC_TABLE_NAME);

	// Add cmp-field-jbdc-mapping : TableComponent - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	cmpFieldJdbcMappingList = new Vector();
	String[] temp={Consts.EMPTY_STRING, Consts.EMPTY_STRING};
	cmpFieldJdbcMappingList.add(temp);	
	cmpFieldJdbcMapping = new TableComponent(Consts.LABEL_CMP_FIELD_JDBC_MAPPING,
						 columnNames,cmpFieldJdbcMappingList,true);
	panelLayout.setConstraints(cmpFieldJdbcMapping,constraints);
	this.add(cmpFieldJdbcMapping);
	this.fieldsNoFilled.add(Consts.LABEL_CMP_FIELD_JDBC_MAPPING);

	obligatoryTable.put(this,fieldsNoFilled);
    }  
    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlJdbcMapping the existing Castor object
     *        associated to this jdbc-mappping's panel.
     */   
    private void xmlRead(JdbcMapping xmlJdbcMapping) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlJdbcMapping.getJndiName() != null ) {
	    jndiName.setText(xmlJdbcMapping.getJndiName());
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);
	}

	if ( xmlJdbcMapping.getJdbcTableName() != null ) {
	    jdbcTableName.setText(xmlJdbcMapping.getJdbcTableName());
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_JDBC_TABLE_NAME);
	}

	Enumeration e = xmlJdbcMapping.enumerateCmpFieldJdbcMapping();
	boolean modif = false;
	while ( e.hasMoreElements() ) {
	    modif = true;
	    CmpFieldJdbcMapping cmpField = (CmpFieldJdbcMapping)e.nextElement();
	    String fieldName = cmpField.getFieldName();
	    String jdbcFieldName = cmpField.getJdbcFieldName();
	    String[] temp = {fieldName,jdbcFieldName};
	    cmpFieldJdbcMappingList.add(temp);
	    Trace.outln(trace,
			"PJdbcMapping.xmlRead(): "+
			cmpField.getFieldName()+" " +
			cmpField.getJdbcFieldName());
	}
	if (modif) {
	    cmpFieldJdbcMappingList.removeElementAt(0); // vector contains empty string at element 0
	    cmpFieldJdbcMapping.setValue(cmpFieldJdbcMappingList);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_CMP_FIELD_JDBC_MAPPING);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }
    
    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_JDBC_MAPPING;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlJdbcMapping;
    }

    /**
     * Add one of following Castor ojbect :
     * <li> finder-method-jdbc-mapping
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof FinderMethodJdbcMapping)
	    this.addFinderMethodJdbcMapping((FinderMethodJdbcMapping)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor ojbect :
     * <li> finder-method-jdbc-mapping
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof FinderMethodJdbcMapping)
	    this.removeFinderMethodJdbcMapping((FinderMethodJdbcMapping)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add the FinderMethodJdbcMapping.
     * @param xmlFinderMethodJdbcMapping the finder-method-jdbc-mapping to set.
     */
    public void addFinderMethodJdbcMapping(FinderMethodJdbcMapping xmlFinderMethodJdbcMapping){
	this.xmlJdbcMapping.addFinderMethodJdbcMapping(xmlFinderMethodJdbcMapping);
    }    

    /**
     * Remove the FinderMethodJdbcMapping
     * @param xmlFinderMethodJdbcMapping the finder-method-jdbc-mapping to remove.
     */
    public void removeFinderMethodJdbcMapping(FinderMethodJdbcMapping xmlFinderMethodJdbcMapping){
	FinderMethodJdbcMapping[] fMethodTable = this.xmlJdbcMapping.getFinderMethodJdbcMapping();
	int i = 0;
	boolean found = false;
	while (i <= fMethodTable.length && !found) {
	    if (fMethodTable[i].equals(xmlFinderMethodJdbcMapping))
		found = true;
	    i++;
	}
	this.xmlJdbcMapping.removeFinderMethodJdbcMapping(i-1);
    }


    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization
	
	// jndi-name
	tempString = jndiName.getText();
	if (tempString.length() != 0) {
	    xmlJdbcMapping.setJndiName(tempString);
	} else {
	    Trace.outln(trace,
			"PJdbcMapping.setValues(): Jdni-resource is an obligatory field, it must be filled.");
	    xmlJdbcMapping.setJndiName(null);
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);
	}
	
	// Jdbc-Table-Name
	tempString = jdbcTableName.getText();
	if (tempString.length() != 0) {
	    xmlJdbcMapping.setJdbcTableName(tempString);
	} else {
	    Trace.outln(trace,
			"PJdbcMapping.setValues(): jdbc-table-name is an obligatory field, it must be filled.");
	    xmlJdbcMapping.setJdbcTableName(null);
	    this.fieldsNoFilled.add(Consts.LABEL_JDBC_TABLE_NAME);
	}

	cmpFieldJdbcMapping.tableReading();
	xmlJdbcMapping.removeAllCmpFieldJdbcMapping();
	// cmp-field-jbdc-mapping
	boolean existingCmpFieldJM = false;
	for (int i = 0 ; i < cmpFieldJdbcMapping.getRowsCount() ; i++) {
	     if ( ((String)cmpFieldJdbcMapping.getData(i,0)).length() != 0) {
		 CmpFieldJdbcMapping xmlCmpFieldJdbcMappingTemp = new CmpFieldJdbcMapping();
		 xmlCmpFieldJdbcMappingTemp.setFieldName((String)cmpFieldJdbcMapping.getData(i,0));
		 xmlCmpFieldJdbcMappingTemp.setJdbcFieldName((String)cmpFieldJdbcMapping.getData(i,1));
		 xmlJdbcMapping.addCmpFieldJdbcMapping(xmlCmpFieldJdbcMappingTemp);
		 existingCmpFieldJM = true;
	     }
	}

	if (!existingCmpFieldJM) {
	    this.fieldsNoFilled.add(Consts.LABEL_CMP_FIELD_JDBC_MAPPING);
	}

	if (fieldsNoFilled.size() != 0) {
	    obligatoryTable.put(this,fieldsNoFilled);
	} else {
	    obligatoryTable.remove(this);
	}

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if (xmlJdbcMapping.getJndiName() != null)
	    Trace.outln(trace,
			"PJdbcMapping.getValues: xmlJdbcMapping.getJndiName() = "+
			xmlJdbcMapping.getJndiName());
	if (xmlJdbcMapping.getJdbcTableName() != null)
	    Trace.outln(trace,
			"PJdbcMapping.getValues: xmlJdbcMapping.getJdbcTableName() = "+
			xmlJdbcMapping.getJdbcTableName());

        Enumeration e = xmlJdbcMapping.enumerateCmpFieldJdbcMapping();
	while ( e.hasMoreElements() ){
	    CmpFieldJdbcMapping cmpField = (CmpFieldJdbcMapping)e.nextElement();

	    Trace.outln(trace,
			"CmpFieldJdbcMapping.getValues(): xmlCmpFieldJdbcMapping.getFieldName() = "+
			cmpField.getFieldName());
	    Trace.outln(trace,
			"CmpFieldJdbcMapping.getValues(): xmlCmpFieldJdbcMapping.getJdbcFieldName() = "+
			cmpField.getJdbcFieldName());
	}	    
    }

}
