/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*               JonasEjbJar Panel                    */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;

import org.objectweb.jonas_ejb.deployment.xml.JonasEjbJar;
import org.objectweb.jonas_ejb.deployment.xml.JonasSession;
import org.objectweb.jonas_ejb.deployment.xml.JonasEntity;

import javax.swing.JPanel;

/**
 * Empty panel.
 * Contains a list of session reference and entity reference.
 * <p>
 * The associated node is Ejb Jar -> META-INF/jonas-ejb-jar.xml
 * @author Magali MORAEL and Christophe GERMAIN.
 **/
public class PJonasEjbJar extends DeployerPanel {
    private JonasEjbJar xmlJonasEjbJar;

    /**
     * Build a new JOnAS Ejb Jar.
     */
    public PJonasEjbJar() {
	super();
	this.xmlJonasEjbJar = new JonasEjbJar();	
    }
      
    /**
     * Build an existing JOnAS Ejb Jar.
     * @param xmlJonasEjbJar the existing jonas-ejb-jar Castor object.
     */   
    public PJonasEjbJar(JonasEjbJar xmlJonasEjbJar) {
	super();
	this.xmlJonasEjbJar = xmlJonasEjbJar ;	
	this.xmlRead(xmlJonasEjbJar);
    }

    /**
     * Reading xml values.
     * @param xmlJonasEjbJar is the existing Castor object
     *        associated to this META-INF/jonas-ejb-jar.xml's panel.
     */   
    private void xmlRead(JonasEjbJar xmlJonasEjbJar) {}   

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_META_INF_JONAS_EJB_JAR_XML;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlJonasEjbJar;
    }

    /**
     * Add one of following Castor Object :
     * <li> jonas-entity
     * <li> jonas-session
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof JonasEntity)
	    addJonasEntity((JonasEntity)xmlObject);
	else if (xmlObject instanceof JonasSession)
	    addJonasSession((JonasSession)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor Object :
     * <li> jonas-entity
     * <li> jonas-session
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof JonasEntity)
	    removeJonasEntity((JonasEntity)xmlObject);
	else if (xmlObject instanceof JonasSession)
	    removeJonasSession((JonasSession)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add a JOnAS Entity
     * @param jEntity the jonas-entity to add.
    **/
    public void addJonasEntity(JonasEntity jEntity) {
        xmlJonasEjbJar.addJonasEntity(jEntity);
    }

    /**
     * Remove a JOnAS Entity
     * @param jEntity the jonas-entity to remove.
    **/
    public void removeJonasEntity(JonasEntity jEntity) {
	JonasEntity[] jonasEntityTable = this.xmlJonasEjbJar.getJonasEntity();
	int i = 0;
	boolean found = false;
	while (i <= jonasEntityTable.length && !found) {
	    if (jonasEntityTable[i].equals(jEntity))
		found = true;
	    i++;
	}
	this.xmlJonasEjbJar.removeJonasEntity(i-1);
    }
   
    /**
     * Add a JOnAS Session.
     * @param jSession the jonas-session to add.
    **/
    public void addJonasSession(JonasSession jSession) {
        xmlJonasEjbJar.addJonasSession(jSession);
    }

    /**
     * Remove a JOnAS Session.
     * @param jSession the jonas-session to remove.
    **/
    public void removeJonasSession(JonasSession jSession) {
	JonasSession[] jonasSessionTable = this.xmlJonasEjbJar.getJonasSession();
	int i = 0;
	boolean found = false;
	while (i <= jonasSessionTable.length && !found) {
	    if (jonasSessionTable[i].equals(jSession))
		found = true;
	    i++;
	}	
	this.xmlJonasEjbJar.removeJonasSession(i-1);
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public boolean setValues() {
	return true;
    }

    /**
     * This method is used but there are no information at this level,
     * so there is no action associated.
     */
    public void getValues() {}
}
