/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 Jonas entity Panel                 */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.JPanel;
import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.common.Trace;

import org.objectweb.jonas_ejb.deployment.xml.JdbcMapping;
import org.objectweb.jonas_ejb.deployment.xml.JonasEntity;
import org.objectweb.jonas_ejb.deployment.xml.JonasResource;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRef;


/**
 * Panel Jonas Session.
 * There are :
 * <li> two TextFieldElement
 * <li> one TableComponent,
 * <p>
 * The associated node is Ejb Jar -> META-INF/jonas-ejb-jar.xml -> entity
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PJonasEntity extends DeployerPanelWithEjbName {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement ejbName;
    private TextFieldElement jndiName;
    private TableComponent   jonasResource;
    private Vector           jonasResourceList;
    private TableComponent   jonasEjbRef;
    private Vector           jonasEjbRefList;
    private TextFieldElement isModifiedMethodName;
    private TextFieldElement passivationTimeout;
    private ComboBoxList     methodName;

    private JonasEntity          xmlJonasEntity;

    private Vector methodList;
    private String[] columnNamesResource = {Consts.LABEL_RES_REF_NAME, Consts.LABEL_JNDI_NAME};
    private String[] columnNamesEjbRef   = {Consts.LABEL_EJB_REF_NAME, Consts.LABEL_JNDI_NAME};

    private Vector fieldsNoFilled;

    /**
     * Build a new Jonas Entity panel.
     */
    public PJonasEntity() {
	super();
	this.init();
	this.xmlJonasEntity = new JonasEntity();
	this.xmlCreation();
	this.initVectors();
    }  

    /**
     * Build an existing Jonas Entity panel.
     * @param xmlJonasEntity the existing jonas-entity Castor object.
     */
    public PJonasEntity(JonasEntity xmlJonasEntity) {
	super();
	this.init();
	this.xmlJonasEntity = xmlJonasEntity;
	this.initVectors();
	this.xmlCreation();
	this.xmlRead(xmlJonasEntity);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add ejb-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	ejbName = new TextFieldElement(Consts.LABEL_EJB_NAME, true);
	panelLayout.setConstraints(ejbName, constraints);
	this.add(ejbName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	// Add jndi-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jndiName = new TextFieldElement(Consts.LABEL_JNDI_NAME, true);
	panelLayout.setConstraints(jndiName, constraints);
	this.add(jndiName);
	this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);

	// Add jonas-resource : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jonasResourceList = new Vector();
	String[] temp = {Consts.EMPTY_STRING, Consts.EMPTY_STRING};
	jonasResourceList.add(temp);	
	jonasResource = new TableComponent(Consts.LABEL_JONAS_RESOURCE,
					   columnNamesResource, jonasResourceList, false);
	panelLayout.setConstraints(jonasResource, constraints);
	this.add(jonasResource);

	// Add jonas-ejb-ref : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jonasEjbRefList = new Vector();
	String[] temp1 = {Consts.EMPTY_STRING, Consts.EMPTY_STRING};
	jonasEjbRefList.add(temp1);	
	jonasEjbRef = new TableComponent(Consts.LABEL_JONAS_EJB_REF,
					 columnNamesEjbRef, jonasEjbRefList, false);
	panelLayout.setConstraints(jonasEjbRef, constraints);
	this.add(jonasEjbRef);

	// add is-modified-method-name : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+4;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	isModifiedMethodName = new TextFieldElement(Consts.LABEL_IS_MODIFIED_METHOD_NAME, false);
	panelLayout.setConstraints(isModifiedMethodName, constraints);
	this.add(isModifiedMethodName);

	// add passivation-timeout : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+5;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	passivationTimeout = new TextFieldElement(Consts.LABEL_PASSIVATION_TIMEOUT, false);
	panelLayout.setConstraints(passivationTimeout, constraints);
	this.add(passivationTimeout);

	obligatoryTable.put(this, fieldsNoFilled);
    } 

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Initialize the list of JOnAS resource and JOnAS ejb-ref
     */
    private void initVectors() {
	jonasResourceList = new Vector();
	jonasEjbRefList = new Vector();
    }

    /**
     * Reading xml values.
     * @param xmlJonasEntity is the existing Castor object
     *        associated to this jonas-entity's panel.
     */   
    private void xmlRead(JonasEntity xmlJonasEntity) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if (xmlJonasEntity.getEjbName() != null) {
	    String ejbNameString = xmlJonasEntity.getEjbName();
	    ejbName.setText(ejbNameString);
	    changeName(ejbNameString);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}

	if (xmlJonasEntity.getJndiName() != null) {
	    jndiName.setText(xmlJonasEntity.getJndiName());
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);
	}

	Enumeration e = xmlJonasEntity.enumerateJonasResource();
	boolean modif = false;
	while (e.hasMoreElements()) {
	    modif=true;
	    JonasResource jonres = (JonasResource)e.nextElement();
	    String resRefName = jonres.getResRefName();
	    String jndiName = jonres.getJndiName();
	    String[] temp = {resRefName, jndiName};
	    jonasResourceList.add(temp);
	}
	if (modif)
	    jonasResource.setValue(jonasResourceList);

	e = xmlJonasEntity.enumerateJonasEjbRef();
        modif = false;
	while (e.hasMoreElements()) {
	    modif=true;
	    JonasEjbRef joner = (JonasEjbRef)e.nextElement();
	    String ejbRefName = joner.getEjbRefName();
	    String jndiName = joner.getJndiName();
	    String[] temp = {ejbRefName, jndiName};
	    jonasEjbRefList.add(temp);
	}
	if (modif)
	    jonasEjbRef.setValue(jonasEjbRefList);

	if (xmlJonasEntity.getIsModifiedMethodName() != null) {
	    isModifiedMethodName.setText(xmlJonasEntity.getIsModifiedMethodName());
	}
	if (xmlJonasEntity.getPassivationTimeout() != null) {
	    passivationTimeout.setText(xmlJonasEntity.getPassivationTimeout());
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this, fieldsNoFilled);
	else 
	    obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_JONAS_ENTITY;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return this.xmlJonasEntity;
    }

    /**
     * Add one of following Castor object :
     * <li> jdbc-mapping
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof JdbcMapping)
	    this.setJdbcMapping((JdbcMapping)xmlObject);
	else 
	    throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor object :
     * <li> jdbc-mapping
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof JdbcMapping)
	    this.setJdbcMapping(null);
	else 
	    throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Set the JdbcMapping
     * @param xmlJdbcMapping the jdbc-mapping to set.
     */
    public void setJdbcMapping(JdbcMapping xmlJdbcMapping){
	this.xmlJonasEntity.setJdbcMapping(xmlJdbcMapping);
    }

    /**
     * Set ejb-name in the panel.
     * This method is used in this class and invoked
     * by DeployerMenuBar when user create a new bean.
     * @param newEjbName the new ejb name.
     */
    public void setEjbName(String newEjbName) {
	if (newEjbName.length() != 0) {
	    xmlJonasEntity.setEjbName(newEjbName);
	    changeName(newEjbName);
	    this.ejbName.setText(newEjbName);
	    this.fieldsNoFilled.remove(Consts.LABEL_EJB_NAME);
	}
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// ejb-name
	tempString = ejbName.getText();
	if (tempString.length() != 0) {
	    this.setEjbName(tempString);
	} else { 
	    Trace.outln(trace,
			"PJonasEntity.setValues(): ejb-name is an obligatory field, it must be filled.");
	    xmlJonasEntity.setEjbName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}
	
	// jndi-name
	tempString = jndiName.getText();
	if (tempString.length() != 0) {
	    xmlJonasEntity.setJndiName(tempString);
	} else { 
	    Trace.outln(trace,
			"PJonasEntity.setValues(): Jndi-name is an obligatory field, it must be filled.");
	    xmlJonasEntity.setJndiName(null);
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);
	}

	// jonas-resource
	jonasResource.tableReading();
	xmlJonasEntity.removeAllJonasResource();
	for (int i = 0 ; i < jonasResource.getRowsCount() ; i++) {
	    if (((String)jonasResource.getData(i, 0)).length() != 0) {
		JonasResource xmlJonasResourceTemp = new JonasResource();
		xmlJonasResourceTemp.setResRefName((String)jonasResource.getData(i, 0));
		xmlJonasResourceTemp.setJndiName((String)jonasResource.getData(i, 1));
		xmlJonasEntity.addJonasResource(xmlJonasResourceTemp);
	    }
	}

	// jonas-ejb-ref
	jonasEjbRef.tableReading();
	xmlJonasEntity.removeAllJonasEjbRef();
	for (int i = 0 ; i < jonasEjbRef.getRowsCount() ; i++) {
	    if (((String)jonasEjbRef.getData(i, 0)).length() != 0) {
		JonasEjbRef xmlJonasEjbRefTemp = new JonasEjbRef();
		xmlJonasEjbRefTemp.setEjbRefName((String)jonasEjbRef.getData(i, 0));
		xmlJonasEjbRefTemp.setJndiName((String)jonasEjbRef.getData(i, 1));
		xmlJonasEntity.addJonasEjbRef(xmlJonasEjbRefTemp);
	    }
	}

	// is-modified-method-name
	tempString = isModifiedMethodName.getText();
	Trace.outln(trace,
		    "PJonasEntity.setValues(): isModifiedMethodName.getText() = "+
		    tempString);
	if (tempString.length() != 0) {
	    xmlJonasEntity.setIsModifiedMethodName(tempString);
	} else { 
	    xmlJonasEntity.setIsModifiedMethodName(null);
	}

	// passivation-timeout
	tempString = passivationTimeout.getText();
	Trace.outln(trace,
		    "PJonasEntity.setValues(): passivationTimeout.getText() = "+
		    tempString);
	if (tempString.length() != 0) {
	    xmlJonasEntity.setPassivationTimeout(tempString);
	} else { 
	    xmlJonasEntity.setPassivationTimeout(null);
	}

	this.getValues();

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this, fieldsNoFilled);
	else 
	    obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {

	// ejb-name
	if (xmlJonasEntity.getEjbName() != null) {
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getEjbName() = "+
			 xmlJonasEntity.getEjbName());
	}

	// jndi-name
	if (xmlJonasEntity.getJndiName() != null) {
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getJndiName() = "+
			xmlJonasEntity.getJndiName());
	}

	// jonas-resource
        Enumeration e = xmlJonasEntity.enumerateJonasResource();
	while (e.hasMoreElements()) {
	    JonasResource  jr = (JonasResource)e.nextElement();
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getResRefName() = "+
			jr.getResRefName());
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getJndiNameforResRefName() = "+
			jr.getJndiName());
	}

	// jonas-ejb-ref
        e = xmlJonasEntity.enumerateJonasEjbRef();
	while (e.hasMoreElements()) {
	    JonasEjbRef  jr = (JonasEjbRef)e.nextElement();
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getEjbRefName() = "+
			jr.getEjbRefName());
	    Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getJndiNameforEjbRefName() = "+
			jr.getJndiName());
	}

	// is-modified-method-name
	if (xmlJonasEntity.getIsModifiedMethodName() != null) {
	   Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getIsModifiedMethodName() = "+
		       xmlJonasEntity.getIsModifiedMethodName());  
	}

	// passivation-timeout
	if (xmlJonasEntity.getPassivationTimeout() != null) {
	   Trace.outln(trace,
			"JonasEntity.getValue(): xmlJonasEntity.getPassivationTimeout() = "+
		       xmlJonasEntity.getPassivationTimeout());  
	}
    }

    /**
     * Get the value of the ejb-name field of the panel.
     * @return the ejb-name.
     */
    public String getEjbName(){
	return ejbName.getText();
    }

}
