/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 Jonas  Session Panel               */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.common.Trace;

import org.objectweb.jonas_ejb.deployment.xml.JonasSession;
import org.objectweb.jonas_ejb.deployment.xml.JonasResource;
import org.objectweb.jonas_ejb.deployment.xml.JonasEjbRef;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Panel Jonas Session.
 * There are :
 * <li> two TextFieldElement
 * <li> one TableComponent,
 * <p>
 * The associated node is Ejb Jar -> META-INF/jonas-ejb-jar.xml -> session
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PJonasSession extends DeployerPanelWithEjbName {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement ejbName;
    private TextFieldElement jndiName;
    private TableComponent   jonasResource;
    private Vector           jonasResourceList;
    private TableComponent   jonasEjbRef;
    private Vector           jonasEjbRefList;
    private TextFieldElement sessionTimeout;

    private JonasSession  xmlJonasSession;
    // Temporary Castor objects


    private JonasResource xmlJonasResource; 
    private JonasEjbRef xmlJonasEjbRef; 

    private String[] columnNamesResource = {Consts.LABEL_RES_REF_NAME,Consts.LABEL_JNDI_NAME};
    private String[] columnNamesEjbRef   = {Consts.LABEL_EJB_REF_NAME,Consts.LABEL_JNDI_NAME};

    private Vector fieldsNoFilled;

    /**
     * Build a new Jonas Session panel.
     */
    public PJonasSession() {
	super();
	this.init();
	this.xmlJonasSession = new JonasSession();
	this.initVectors();
	this.xmlCreation();

    }  

    /**
     * Build an existing Jonas Session panel.
     * @param xmlJonasSession the existing jonas-session Castor object.
     */
    public PJonasSession(JonasSession xmlJonasSession) {
	super();
	this.init();
	this.xmlJonasSession = xmlJonasSession;
	this.initVectors();
	this.xmlCreation();
	this.xmlRead(xmlJonasSession);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add ejb-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	ejbName = new TextFieldElement(Consts.LABEL_EJB_NAME, true);
	panelLayout.setConstraints(ejbName,constraints);
	this.add(ejbName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	// Add jndi-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jndiName = new TextFieldElement(Consts.LABEL_JNDI_NAME, true);
	panelLayout.setConstraints(jndiName,constraints);
	this.add(jndiName);
	this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);

	// Add jonas-ressoure : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jonasResourceList = new Vector();
	String[] temp={"", ""};
	jonasResourceList.add(temp);	
	jonasResource = new TableComponent(Consts.LABEL_JONAS_RESOURCE,
					   columnNamesResource,jonasResourceList,false);
	panelLayout.setConstraints(jonasResource,constraints);
	this.add(jonasResource);

	// Add jonas-ejb-ref : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	jonasEjbRefList = new Vector();
	String[] temp1={"", ""};
	jonasEjbRefList.add(temp1);	
	jonasEjbRef = new TableComponent(Consts.LABEL_JONAS_EJB_REF,
					 columnNamesEjbRef,jonasEjbRefList,false);
	panelLayout.setConstraints(jonasEjbRef,constraints);
	this.add(jonasEjbRef);

	// add session-timeout : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+4;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	sessionTimeout = new TextFieldElement(Consts.LABEL_SESSION_TIMEOUT, false);
	panelLayout.setConstraints(sessionTimeout, constraints);
	this.add(sessionTimeout);

	obligatoryTable.put(this,fieldsNoFilled);
    }  
    
    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	xmlJonasResource = new JonasResource();
	xmlJonasEjbRef = new JonasEjbRef();
    }

    /**
     * Inialize the list of JOnAS resource and JOnAS ejb-ref.
     */
    private void initVectors() {
	jonasResourceList = new Vector();
	jonasEjbRefList = new Vector();
    }

    /**
     * Reading xml values.
     * @param xmlJonasSession the existing Castor object
     *        associated to this jonas-session's panel.
     */   
    private void xmlRead(JonasSession xmlJonasSession) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlJonasSession.getEjbName() != null ) {
	    String ejbNameString = xmlJonasSession.getEjbName();
	    ejbName.setText(ejbNameString);
	    changeName(ejbNameString);
	} else
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	if ( xmlJonasSession.getJndiName() != null )
	    jndiName.setText(xmlJonasSession.getJndiName());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);

	Enumeration e=xmlJonasSession.enumerateJonasResource();
	boolean modif=false;
	while ( e.hasMoreElements() ) {
	    modif=true;
	    JonasResource jonres = (JonasResource)e.nextElement();
	    String resRefName = jonres.getResRefName();
	    String jndiName = jonres.getJndiName();
	    String[] temp={resRefName,jndiName};
	    jonasResourceList.add(temp);
	}
	if ( modif )
	    jonasResource.setValue(jonasResourceList);

	e=xmlJonasSession.enumerateJonasEjbRef();
	modif=false;
	while ( e.hasMoreElements() ) {
	    modif=true;
	    JonasEjbRef joner = (JonasEjbRef)e.nextElement();
	    String ejbRefName = joner.getEjbRefName();
	    String jndiName = joner.getJndiName();
	    String[] temp={ejbRefName,jndiName};
	    jonasEjbRefList.add(temp);
	}
	if ( modif )
	    jonasEjbRef.setValue(jonasEjbRefList);

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
 	else obligatoryTable.remove(this);
    }
	
    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_JONAS_SESSION;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlJonasSession;
    }

    /**
     * This method is not used because JonasSession is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because JonasSession is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Set ejb-name in the panel.
     * This method is used in this class and invoked
     * by DeployerMenuBar when user create a new bean.
     * @param newEjbName is the new ejb name.
     */
    public void setEjbName(String newEjbName) {
	if (newEjbName.length() != 0) {
	    xmlJonasSession.setEjbName(newEjbName);
	    changeName(newEjbName);
	    this.ejbName.setText(newEjbName);
	    this.fieldsNoFilled.remove(Consts.LABEL_EJB_NAME);
	}
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// ejb-name
	tempString = ejbName.getText();
	if (tempString.length() != 0) {
	    this.setEjbName(tempString);
	}
	else {
	    Trace.outln(trace,
			"PJonasSession.setValues(): ejb-name is an obligatory field, it must be filled.");
	    xmlJonasSession.setEjbName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}
	
	// jndi-name
	tempString = jndiName.getText();
	if (tempString.length() != 0) {
	    xmlJonasSession.setJndiName(tempString);
	} else { 
	    Trace.outln(trace,
			"PJonasSession.setValues(): jndi-name is an obligatory field, it must be filled.");
	    xmlJonasSession.setJndiName(null);
	    this.fieldsNoFilled.add(Consts.LABEL_JNDI_NAME);
	}

	// resource
	jonasResource.tableReading();
	Trace.outln(trace,
		    "PJonasSession.setValues(): jonasResource.getRowsCount() = "+
		    jonasResource.getRowsCount());
	xmlJonasSession.removeAllJonasResource();
	for (int i = 0 ; i < jonasResource.getRowsCount() ; i++) {
	    Trace.outln(trace,
			"PJonasSession.setValues(): rows i ="+ i);
	    if ( ((String)jonasResource.getData(i,0)).length() != 0 ) {
		JonasResource xmlJonasResourceTemp = new JonasResource();
		xmlJonasResourceTemp.setResRefName((String)jonasResource.getData(i,0));
		xmlJonasResourceTemp.setJndiName((String)jonasResource.getData(i,1));
		xmlJonasSession.addJonasResource(xmlJonasResourceTemp);
	    }
	}	

	// ejb-ref
	jonasEjbRef.tableReading();
	Trace.outln(trace,
		    "PJonasSession.setValues(): jonasEjbRef.getRowsCount() = "+
		    jonasEjbRef.getRowsCount());
	xmlJonasSession.removeAllJonasEjbRef();
	for (int i = 0 ; i < jonasEjbRef.getRowsCount() ; i++) {
	    Trace.outln(trace,
			"PJonasSession.setValues(): rows i ="+ i);
	    if ( ((String)jonasEjbRef.getData(i,0)).length() != 0) {
		JonasEjbRef xmlJonasEjbRefTemp = new JonasEjbRef();
		xmlJonasEjbRefTemp.setEjbRefName((String)jonasEjbRef.getData(i,0));
		xmlJonasEjbRefTemp.setJndiName((String)jonasEjbRef.getData(i,1));
		xmlJonasSession.addJonasEjbRef(xmlJonasEjbRefTemp);
	    }
	}	

	// session-timeout
	tempString = sessionTimeout.getText();
	Trace.outln(trace,
		    "PJonasSession.setValues(): sessionTimeout.getText() = "+
		    tempString);
	if (tempString.length() != 0) {
	    xmlJonasSession.setSessionTimeout(tempString);
	} else { 
	    xmlJonasSession.setSessionTimeout(null);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
 	else
	    obligatoryTable.remove(this);

	return result;
   }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if (xmlJonasSession.getEjbName() != null) {
	    Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getEjbName() = "+
			xmlJonasSession.getEjbName());
	}
	if (xmlJonasSession.getJndiName() != null) {
	     Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getJndiName() = "+
			 xmlJonasSession.getJndiName());
	}
        Enumeration e = xmlJonasSession.enumerateJonasResource();
	while ( e.hasMoreElements() ){
	    JonasResource  jr = (JonasResource)e.nextElement();
	    Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getResRefName() = "
			+jr.getResRefName());
	    Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getJndiName()) = "
			+jr.getJndiName());
	}
        e = xmlJonasSession.enumerateJonasEjbRef();
	while ( e.hasMoreElements() ){
	    JonasEjbRef  jr = (JonasEjbRef)e.nextElement();
	    Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getEjbRefName() = "
			+jr.getEjbRefName());
	    Trace.outln(trace,
			"PJonasSession.getValue(): xmlJonasSession.getJndiName() = "
			+jr.getJndiName());
	}
	// session-timeout
	if (xmlJonasSession.getSessionTimeout() != null) {
	   Trace.outln(trace,
		       "JonasSession.getValue(): xmlJonasSession.getSessionTimeout() = "+
		       xmlJonasSession.getSessionTimeout());  
	}
    }

    /**
     * Get the value of the ejb-name field of the panel.
     * @return the ejb-name.
     */
    public String getEjbName(){
	return ejbName.getText();
    }

}
