/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                      method  Panel                 */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;

import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.Method;
import org.objectweb.jonas_ejb.deployment.xml.MethodParams;

import java.util.Enumeration;
import java.util.Vector;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Panel Method .
 * There are :
 * <li> one TextAreaDescription;
 * <li> one comboboxList
 * <li> one TableComponent;
 * <li> one ComboBoxList
 * <li> one RadioButtonList
 * <p>
 * The associated node is 
 * <li> Ejb Jar -> META-INF/ejb-jar.xml -> assembly-descriptor -> TX -> Method
 * or
 * <li> Ejb Jar -> META-INF/ejb-jar.xml -> assembly-descriptor -> method-permission -> Method
 * @author Magali MORAEL and Christophe GERMAIN.
 **/
public class PMethod extends DeployerPanelWithEjbName {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private Vector ejbNameList;     // For ComboBoxList
    private Vector methodNameList;  // For ComboBoxList
    private Vector methodParamsList;// For TableComponent
    String[] columnNames = {Consts.LABEL_METHOD_PARAM};

    private TextAreaDescription description;
    private ComboBoxList        ejbName;
    private RadioButtonList     methodIntf;
    private ComboBoxList        methodName;
    private TableComponent      methodParams;

    private Method       xmlMethod;
    // Temporary Castor objects
    private MethodParams xmlMethodParams;

    private boolean replacedEjbName    = false;
    private boolean replacedMethodName = false;
    private String  temporaryEjbName;
    private String  temporaryMethodName;

    private Vector fieldsNoFilled;

    /**
     * Build a new method panel.
     */
    public PMethod() {
	super();
	this.init();
	this.xmlMethod = new Method();
	this.xmlCreation();
    }

    /**
     * Build an existing Method panel.
     * @param xmlMethod the associated Castor object.
     */
    public PMethod(Method xmlMethod) {
	super();
	this.init();
	this.xmlCreation();
	this.xmlMethod = xmlMethod;
	this.xmlRead(xmlMethod);
   }

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }

    /**
     * Add 
     * - a TextAreaDescription, 
     * - a ComboBoxList, 
     * - a RadioButtonList, 
     * - a ComboBoxList and 
     * - a TableComponent.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add ejb-name : ComboBoxList - obligatory field.
	ejbNameList = new Vector();
	// tips to not have a line with empty string because of bad display on Solaris and Windows
	ejbNameList.add(Consts.DEFAULT_STRING); 
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	ejbName = new ComboBoxList(Consts.LABEL_EJB_NAME,
				   ejbNameList,true);
	ejbName.setEditable(true);
	panelLayout.setConstraints(ejbName,constraints);
	this.add(ejbName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	// add method-intf : RadioButtonList
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector classeList = new Vector();
	classeList.add(Consts.TYPE_HOME);
	classeList.add(Consts.TYPE_REMOTE);
		
	methodIntf = new RadioButtonList(Consts.LABEL_METHOD_INTF,classeList,false);
	panelLayout.setConstraints(methodIntf,constraints);
	this.add(methodIntf);

	// Add methodName :  ComboBoxList - obligatory field.
	methodNameList = new Vector();
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	methodName = new ComboBoxList(Consts.LABEL_METHOD_NAME,
				      methodNameList,true);
	methodName.setEditable(true);
	panelLayout.setConstraints(methodName,constraints);
	this.add(methodName);
	this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);

	// Add method-params : TableComponent
	constraints.gridy = Consts.GRIDY_FIRST+4;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	methodParamsList = new Vector();
	String[] temp={""};
	methodParamsList.add(temp);	
	methodParams= new TableComponent(Consts.LABEL_METHOD_PARAMS,columnNames,methodParamsList,false);
	panelLayout.setConstraints(methodParams,constraints);
	this.add(methodParams);

	obligatoryTable.put(this,fieldsNoFilled);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	xmlMethodParams = new MethodParams();
    }  
 
    /**
     * Reading xml values.
     * @param xmlMethod is the existing Castor object
     *        associated to this method's panel.
     */   
    private void xmlRead(Method xmlMethod) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlMethod.getDescription() != null )
	    description.setText(xmlMethod.getDescription());

	if ( xmlMethod.getEjbName() != null) {
	    String ejbNameString = xmlMethod.getEjbName();
	    ejbName.setText(ejbNameString);
	    this.temporaryEjbName = ejbNameString;
	    this.replacedEjbName = true;
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}

	if ( xmlMethod.getMethodIntf() != null) {
	    if (!(methodIntf.setTextAndCheck(xmlMethod.getMethodIntf()))) {
		xmlMethod.setMethodIntf(null);
	    }
	}

	if (  xmlMethod.getMethodName() != null) {
	    String methodNameString = xmlMethod.getMethodName();
	    methodName.setText(methodNameString);
	    this.temporaryMethodName = methodNameString;
	    this.replacedMethodName = true;
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);
	}

	if (xmlMethod.getMethodParams() != null) {
	    Enumeration e = xmlMethod.getMethodParams().enumerateMethodParam();
	    boolean modif = false;
	    while ( e.hasMoreElements() ){
		modif=true;
		String methodParamTemp = (String)e.nextElement();
		String [] temp = {methodParamTemp};
		methodParamsList.add(temp);
		this.addMethodParam(methodParamTemp);
	    }
	    if (modif) {
		methodParamsList.removeElementAt(0); // vector contains empty string at element 0
		methodParams.setValue(methodParamsList);
	    }
	}

	if (replacedEjbName && replacedMethodName)
	    this.panelName = this.temporaryEjbName+Consts.SEPARATOR+this.temporaryMethodName;

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the type of the node.
     */
    public String getType() {
	return Consts.STRING_METHOD;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlMethod;
    }

    /**
     * Add one MethodParam.
     * @param xmlObject is the Castor object or a String object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof String) {
	    addMethodParam((String)xmlObject);
	} else {
	    throw new Error("The parameter is not a Castor object.");
	}
    }

    /**
     * Update the ejb-name list.
     * Method invoked when user enter a new ejb-name which is not in the list.
     * @param newList the new list of choices in the ejb-name combo box.
     */
    public void udpateListEjbName(Vector newList){
	this.ejbName.updateList(newList);
    }

    /**
     * This method is not used.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Add a MethodParam
     * @param methodParam is a param to add.
     */
    public void addMethodParam(String methodParam) {
	xmlMethodParams.addMethodParam(methodParam);
    }

    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// DESCRIPTION
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlMethod.setDescription(tempString);
	} else { 
	    xmlMethod.setDescription(null);
	}

	// EJB-NAME
	tempString = ejbName.getText();
	if (tempString !=null ){
	    if (tempString.length() != 0) {
		xmlMethod.setEjbName(tempString);
		this.temporaryEjbName = tempString;
		this.replacedEjbName = true;
	    }
	} else {
	    Trace.outln(trace,
			"PMethod.setValues(): ejb-name is an obligatory field, it must be filled.");
	    xmlMethod.setEjbName(null);
	    if (replacedEjbName) {
		this.panelName = this.getType();
		this.replacedEjbName = false;
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}

	//method-intf
	tempString = methodIntf.getText();
	if (tempString != null){
	    if (tempString.length() != 0) {
		xmlMethod.setMethodIntf(tempString);
	    } else {
		xmlMethod.setMethodIntf(null);
	    }
	}
	
	// method-name
	tempString = methodName.getText();
	if (tempString.length() != 0) {
	    xmlMethod.setMethodName(tempString);
	    this.temporaryMethodName = tempString;
	    this.replacedMethodName = true;
	} else {
	    Trace.outln(trace,
			"PMethod.setValues(): Method-name is an obligatory field, it must be filled.");
	    xmlMethod.setMethodName(null);
	    if (replacedMethodName) {
		this.panelName = this.getType();
		this.replacedMethodName = false;
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_METHOD_NAME);
	}

	methodParams.tableReading();
	if (methodParams.getRowsCount() > 1) {
	    xmlMethodParams.removeAllMethodParam();
	    for (int i = 0 ; i < methodParams.getRowsCount() ; i++) {
		String methodParamTemp = (String)(methodParams.getData(i,0));
		if (methodParamTemp.length() != 0) {
		    Trace.outln(trace,
				"PMethod.setValues(): method param = "+
				methodParamTemp);
		    this.addMethodParam(methodParamTemp);
		}
	    }
	    xmlMethod.setMethodParams(xmlMethodParams);
	}
	
	if (replacedEjbName && replacedMethodName)
		this.panelName = this.temporaryEjbName+Consts.SEPARATOR+this.temporaryMethodName;

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( xmlMethod.getDescription() != null)
	    Trace.outln(trace,
			"PMethod.getValues(): xmlMethod.getDescription() = "+
			xmlMethod.getDescription());

       	if ( xmlMethod.getEjbName() != null)
	    Trace.outln(trace,
			"PMethod.getValues(): xmlMethod.getEjbName() = "+
			xmlMethod.getEjbName());

       	if ( xmlMethod.getMethodIntf() != null)
	    Trace.outln(trace,
			"PMethod.getValues(): xmlMethod.getMethodIntf() = "+
			xmlMethod.getMethodIntf()); 

       	if ( xmlMethod.getMethodName() != null)
	    Trace.outln(trace,
			"PMethod.getValues(): xmlMethod.getMethodName().getContent() = "+
			xmlMethod.getMethodName());
	
	if ( xmlMethod.getMethodParams() != null ) {
	    Enumeration e = xmlMethod.getMethodParams().enumerateMethodParam();
	    while (e.hasMoreElements()) {
		String methodParam = (String)e.nextElement();
		Trace.outln(trace,
			    "PMethod.getValues(): xmlMethodParam = "+
			    methodParam);	
	    }
	}
   }
    
    public String getEjbName(){
	return ejbName.getText();
    }

}
