/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 method permission Panel            */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.TableComponent;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.MethodPermission;
import org.objectweb.jonas_ejb.deployment.xml.Method;

import java.util.Enumeration;
import java.util.Vector;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Panel Method Permission.
 * There are :
 * <li> one TextAreaDescription;
 * <li> one TableComponent;
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> assembly-descriptor -> method-permission
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class  PMethodPermission extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextAreaDescription description;
    private TableComponent      roleNames;

    private MethodPermission xmlMethodPermission;
    
    private Vector roleNamesList;
    String[] columnNames = {Consts.LABEL_ROLE_COLUMNS};
  
    private Vector  fieldsNoFilled;
    private boolean oneMethod = false;

    /**
     * Build a new Method Permission panel.
     */
    public PMethodPermission() {
	super();
	this.init();
	this.xmlMethodPermission = new MethodPermission();
	this.xmlCreation();
    }  

    /**
     * Build an existing Method Permission panel.
     * @param xmlMethodPermission the method-permission Castor object.
     */
    public PMethodPermission(MethodPermission xmlMethodPermission) {
	super();
	this.init();
	this.xmlMethodPermission = xmlMethodPermission;
	this.xmlCreation();
	this.xmlRead(xmlMethodPermission);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }

    /**
     * Add a TextAreaDescription and a TableComponent.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	Trace.outln(trace, "PMethodPermission.addGraphicElements():");
	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add role-name : TableComponent - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	roleNamesList = new Vector();
	String[] temp={""};
	roleNamesList.add(temp);	
	roleNames= new TableComponent(Consts.LABEL_ROLE_NAME,columnNames,roleNamesList,true);
	panelLayout.setConstraints(roleNames,constraints);
	this.add(roleNames);	
	this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);

	// There is one method created in the same time
	oneMethod = true;

	obligatoryTable.put(this,fieldsNoFilled);
    }  

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlMethodPermission the existing Castor object
     *        associated to this method-permission's panel.
     */
    private void xmlRead(MethodPermission xmlMethodPermission) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlMethodPermission.getDescription() != null )
	    description.setText(xmlMethodPermission.getDescription());
	
	Enumeration e = xmlMethodPermission.enumerateRoleName();
	Trace.outln(trace,
		    "PMethodPermission.xmlRead(): xmlMethodPermission.getRoleNameCount() = "+
		    xmlMethodPermission.getRoleNameCount());
	boolean modif=false;
	while ( e.hasMoreElements() ) {
	    modif=true;
	    String role = (String)e.nextElement();
	    String[] temp={role};
	    roleNamesList.add(temp);
	}

	if ( modif ){
	    roleNamesList.removeElementAt(0); // vector contains empty string at element 0
	    roleNames.setValue(roleNamesList);
	}
	else this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);

	if (!oneMethod)
	    this.fieldsNoFilled.add(Consts.STRING_METHOD);

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_METHOD_PERMISSION;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlMethodPermission;
    }

    /**
     * Add one of the following Castor object :
     * <li> method
     * <li> role-name
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Method)
	    this.addMethod((Method)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of the following Castor object :
     * <li> method
     * <li> role-name
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof Method)
	    this.removeMethod((Method)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add Method.
     * @param xmlMethod is the method to add.
     */
    public void addMethod(Method xmlMethod) {
	xmlMethodPermission.addMethod(xmlMethod);
	if (!oneMethod) {
	    oneMethod = true;
	    this.fieldsNoFilled.remove(Consts.STRING_METHOD);
 	    if (fieldsNoFilled.size() != 0)
		obligatoryTable.put(this,fieldsNoFilled);
	    else obligatoryTable.remove(this);
	}
    }

    /**
     * Remove Method.
     * @param xmlMethod is the method to remove.
     */
    public void removeMethod(Method xmlMethod) {
	Method[] methodTable = this.xmlMethodPermission.getMethod();
	int i = 0;
	boolean found = false;
	while (i <= methodTable.length && !found) {
	    if (methodTable[i].equals(xmlMethod))
		found = true;
	    i++;
	}
	xmlMethodPermission.removeMethod(i-1);
	// There are n methods in the list before the removing, there are n-1 method after.
	if (methodTable.length-1 == 0) {
	    oneMethod = false;
	    this.fieldsNoFilled.add(Consts.STRING_METHOD);
	    obligatoryTable.put(this,fieldsNoFilled);
	}
    }

    /**
     * Add a role-name.
     * @param roleName is the role-name to add.
     */
    public void addRoleName(String roleName) {
	xmlMethodPermission.addRoleName(roleName);
    }
    
    /**
     * Set values of the panel.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// DESCRIPTION
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlMethodPermission.setDescription(tempString);
	} else { 
	    xmlMethodPermission.setDescription(null);
	}

	roleNames.tableReading();
	boolean existingRoleNames = false;
	if (roleNames.getRowsCount() > 1) {
	    xmlMethodPermission.removeAllRoleName();
	    for (int i = 0 ; i < roleNames.getRowsCount() ; i++) {
		String roleName = (String)(roleNames.getData(i,0));
		if (roleName.length() != 0) {
		    this.addRoleName(roleName);
		    existingRoleNames = true;
		}
	    }
	}

	if (!existingRoleNames)
	    this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);
	
	if (!oneMethod)
	    this.fieldsNoFilled.add(Consts.STRING_METHOD);
	
	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
	
	return result;
    }

    /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( xmlMethodPermission.getDescription() != null ) {
	    Trace.outln(trace,
			"MethodPermission.getValue(): xmlMethodPermission.getDescription() = "+
			xmlMethodPermission.getDescription());
	}
	Enumeration e = xmlMethodPermission.enumerateRoleName();
	while ( e.hasMoreElements() ){
	    String roleName=(String)e.nextElement();
	     Trace.outln(trace,
			"MethodPermission.getValue(): roleName = "+
			 roleName);
	}
    }

}
