/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                Resource Ref Panel                  */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import java.util.Vector; 

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.ComboBoxList;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.ResourceRef;

import javax.swing.JPanel;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

/**
 * Panel Resource-Ref.
 * There are :
 * <li> one TextFieldElement
 * <li> one TextAreaDescription
 * <li> one ComboBoxList
 * <li> one RadioButtonList
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> entity or session -> RES-REF
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PResourceRef extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    resRefName;
    private TextAreaDescription description;
    private ComboBoxList        resType;
    private RadioButtonList     resAuth;

    private Vector resTypeList;
    private Vector resAuthList;

    private ResourceRef    xmlResourceRef;

    private Vector fieldsNoFilled;

    /**
     * Build a new Resouce Ref panel.
     */
    public PResourceRef() {
	super();
	this.init();
	this.xmlResourceRef = new ResourceRef();
	this.xmlCreation();
    }

    /**
     * Build an existing PResourceRef panel.
     * @param xmlResourceRef the associated Castor class.
     */
    public PResourceRef(ResourceRef xmlResourceRef) {
	super();
	this.init();
	this.xmlResourceRef = xmlResourceRef;
	this.xmlCreation();
	this.xmlRead(xmlResourceRef);
    }

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }

    /**
     * Add a TextFieldDesc, a TextAreaDescription, a ComboBoxList and a RadioButtonList.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add res-ref-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	resRefName = new TextFieldElement(Consts.LABEL_RES_REF_NAME,true);
	panelLayout.setConstraints(resRefName,constraints);
	this.add(resRefName);
	this.fieldsNoFilled.add(Consts.LABEL_RES_REF_NAME);

	// Add res-type : ComboBoxList - obligatory field.
	resTypeList = new Vector();
	resTypeList.add(Consts.TYPE_JDBC);
	resTypeList.add(Consts.TYPE_JAVAMAIL);
	resTypeList.add(Consts.TYPE_URL);
	resTypeList.add(Consts.TYPE_QJMS);
	resTypeList.add(Consts.TYPE_TJMS);

	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	resType = new ComboBoxList(Consts.LABEL_RES_TYPE,
					resTypeList,true);
	panelLayout.setConstraints(resType,constraints);
	this.add(resType);
	this.fieldsNoFilled.add(Consts.LABEL_RES_TYPE);
	
	// Add res-auth : RadioButtonList
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	
        resAuthList = new Vector();
	resAuthList.add(Consts.STRING_APPLICATION);
	resAuthList.add(Consts.STRING_CONTAINER);
	resAuth = new RadioButtonList(Consts.LABEL_RES_AUTH,resAuthList,true);
	resAuth.setText(Consts.STRING_CONTAINER);
	panelLayout.setConstraints(resAuth,constraints);
	this.add(resAuth);

	obligatoryTable.put(this,fieldsNoFilled);
    } 

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlResourceRef is the existing Castor object
     *        associated to this resource-ref's panel.
     */   
    private void xmlRead(ResourceRef xmlResourceRef) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlResourceRef.getDescription() != null )
	    description.setText(xmlResourceRef.getDescription());

	if ( xmlResourceRef.getResRefName() != null ) {
	    String resRefNameString = xmlResourceRef.getResRefName();
	    resRefName.setText(resRefNameString);
	    changeName(resRefNameString);
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_RES_REF_NAME);
	}
	
	if ( xmlResourceRef.getResType() != null ) {
	    resType.setText(xmlResourceRef.getResType());
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_RES_TYPE);
	}

	if ( xmlResourceRef.getResAuth() != null ) {
	    if (!(resAuth.setTextAndCheck(xmlResourceRef.getResAuth()))) {
		xmlResourceRef.setResAuth(null);
	    }
	}

	if (fieldsNoFilled.size() != 0) {
	    obligatoryTable.put(this,fieldsNoFilled);
	} else {
	    obligatoryTable.remove(this);
	}
    }

    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_RES_REF;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlResourceRef;
    }

    /**
     * This method is not used because ResourceRef is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because ResourceRef is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// res-ref-name
	tempString = resRefName.getText();
	if (tempString.length() != 0) {
	    xmlResourceRef.setResRefName(tempString);
	    changeName(tempString);
	} else {
	    Trace.outln(trace,
			"PResourceRef.setValues(): res-ref-name is an obligatory field, it must be filled.");
	    xmlResourceRef.setResRefName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_RES_REF_NAME);
	}

	// description
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlResourceRef.setDescription(tempString);
	} else {
	    xmlResourceRef.setDescription(null);
	}
	
	// res-type
	tempString = resType.getText();
	if (tempString.length() != 0) {
	    xmlResourceRef.setResType(tempString);
	} else {
	    Trace.outln(trace,
			"PResourceRef.setValues(): res-type is an obligatory field, it must be filled.");
	    xmlResourceRef.setResType(null);
	    this.fieldsNoFilled.add(Consts.LABEL_RES_TYPE);
	}

	// res-auth
	tempString = resAuth.getText();
	if (tempString != null){
	    if (tempString.length() != 0) {
		xmlResourceRef.setResAuth(tempString);
	    } else {
		xmlResourceRef.setResAuth(null);
	    }
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

     /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( xmlResourceRef.getResRefName() != null) {
	    Trace.outln(trace,
			"PResourceRef.getValues(): xmlResourceRef.getResRefName() = "+
			xmlResourceRef.getResRefName());
	}
	if ( xmlResourceRef.getDescription() != null) {
	    Trace.outln(trace,
			"PResourceRef.getValues(): xmlResourceRef.getDescription() = "+
			xmlResourceRef.getDescription());
	}
	if ( xmlResourceRef.getResType() != null) {
	    Trace.outln(trace,
			"PResourceRef.getValues(): xmlResourceRef.getResType() = "+
			xmlResourceRef.getResType());
	}
 	if ( xmlResourceRef.getResAuth() != null) {
	    Trace.outln(trace,
			"PResourceRef.getValues(): xmlResourceRef.getResAuth()) = "+
			xmlResourceRef.getResAuth());
	}
    }

}
