/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                      root ejb jar                  */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import java.util.Vector;

/**
 * Associated object of the node <i>Ejb Jar</i> in the tree.
 * A list of ejb names is associated to an Ejb Jar.
 * @author Christophe GERMAIN and Magali MORAEL.
 */
public class PRootEjbJar{
    private Vector ejbNameList;

    /**
     * Build the object.
     */
    public PRootEjbJar() {
	ejbNameList=new Vector();
    }

    /**
     * Add an ejb-name. This method is invoked when a PEntity or a PSession is update.
     * @param ejbName the name of ejb-name to insert.
     */
    public void addEjbName(String ejbName) {
	if ( ejbName != "" ) {
	    if ( ejbNameList.indexOf(ejbName) == -1)
		ejbNameList.add(ejbName);
	}
    }

    /**
     * Remove an ejb-name.
     * @param ejbName the name to remove.
     */
    public void removeEjbName(String ejbName) {
	ejbNameList.remove(ejbName);
    }

    /**
     * Get the list of ejb names.
     * @return the list.
     */
    public Vector getVectorEjbName(){
	return ejbNameList;
    }

    /**
     * Method necessary for an object associated to a node to have a name.
     * @return the name of the panel.
     */
    public String toString() {
	return Consts.STRING_EJB_JAR;
    }
}
