/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                 Security ref Panel                 */
/*----------------------------------------------------*/
package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRoleRef;

import javax.swing.JPanel;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import java.util.Vector;

/**
 * Panel SecurityRoleRef.
 * There are :
 * <li> one TextFieldElement
 * <li> one TextAreaDescription
 * <li> one TextFieldElement
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml 
 *                        -> enterprise-beans -> entity or session -> Role-Ref
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PSecurityRoleRef extends DeployerPanel {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    roleName;
    private TextAreaDescription description;
    private TextFieldElement    roleLink;

    private SecurityRoleRef xmlSecurityRoleRef;

    private Vector fieldsNoFilled;

    /**
     * Build a new SecurityRoleRef panel.
     */
    public PSecurityRoleRef() {
	super();
	this.init();
	this.xmlSecurityRoleRef = new SecurityRoleRef();
	this.xmlCreation();
    }

    /**
     * Build an existing SecurityRoleRef panel.
     * @param xmlSecurityRoleRef the associated Castor object.
     */
    public PSecurityRoleRef(SecurityRoleRef xmlSecurityRoleRef) {
	super();
	this.init();
	this.xmlSecurityRoleRef = xmlSecurityRoleRef;
	this.xmlCreation();
	this.xmlRead(xmlSecurityRoleRef);
    }

    /**
     * Initialize the layout and its constraints.
     * Add a TextFieldDesc, a TextAreaDescription and a TextFieldElement
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	this.setLayout(panelLayout);
	constraints = new GridBagConstraints();	

	this.addGraphicElements();
    }
 
    /**
     * Add a TextFieldDesc, a TextAreaDescription and a TextFieldElement.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add role-name : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	roleName = new TextFieldElement(Consts.LABEL_ROLE_NAME,true);
	panelLayout.setConstraints(roleName,constraints);
	this.add(roleName);
	this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);

	// Add role-link : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	roleLink = new TextFieldElement(Consts.LABEL_ROLE_LINK,false);
	panelLayout.setConstraints(roleLink,constraints);
	this.add(roleLink);

	obligatoryTable.put(this,fieldsNoFilled);
    }

    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * reading xml values.
     * @param xmlSecurityRoleRef the existing Castor object
     *        associated to this security-role-ref's panel.
     */
    private void xmlRead(SecurityRoleRef xmlSecurityRoleRef) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlSecurityRoleRef.getRoleName() != null ) {
	    String roleNameString = xmlSecurityRoleRef.getRoleName();
	    roleName.setText(roleNameString);
	    changeName(roleNameString);
	}
	else this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);

	if ( xmlSecurityRoleRef.getDescription() != null )
	    description.setText(xmlSecurityRoleRef.getDescription());
    
	if ( xmlSecurityRoleRef.getRoleLink() != null )
	    roleLink.setText(xmlSecurityRoleRef.getRoleLink());

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_ROLE_REF;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlSecurityRoleRef;
    }

    /**
     * This method is not used because SecurityRoleRef is a leaf of the xml file.
     */
    public void addInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * This method is not used because SecurityRoleRef is a leaf of the xml file.
     */
    public void removeInSpecificList(Object xmlObject) {
	throw new Error("This method does not exist in this class.");
    }

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.
	
	this.fieldsNoFilled.removeAllElements(); // re-initialization

	// ROLE NAME
	tempString = roleName.getText();
	if (tempString.length() != 0) {
	    xmlSecurityRoleRef.setRoleName(tempString);
	    changeName(tempString);
	} else {
	    Trace.outln(trace,
			"PSecurityRoleRef.setValues(): role-name is an obligatory field, it must be filled.");
	    xmlSecurityRoleRef.setRoleName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_ROLE_NAME);
	}

	// DESCRIPTION
	tempString = description.getText();
	Trace.outln(trace,
		    "PSecurityRoleRef.setValues(): "+
		    "description="+tempString+
		    " and tempString.length()="+tempString.length());
	if (tempString.length() != 0) {
	    xmlSecurityRoleRef.setDescription(tempString);
	} else { 
	    xmlSecurityRoleRef.setDescription(null);
	}

	// ROLE LINK
	tempString = roleLink.getText();
	if (tempString.length() != 0) {
	    xmlSecurityRoleRef.setRoleLink(tempString);
	} else {
	    xmlSecurityRoleRef.setRoleLink(null);
	}

	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

    /**
     * Get values in the associated Castor class.
     */
    public void getValues() {
	if ( xmlSecurityRoleRef.getRoleName() != null )
	    Trace.outln(trace,
			"PSecurityRoleRef.getValues(): xmlSecurityRoleRef.getRoleName() = "+
			xmlSecurityRoleRef.getRoleName());
	
	if ( xmlSecurityRoleRef.getDescription() != null )
	    Trace.outln(trace,
			"PSecurityRoleRef.getValues(): xmlSecurityRoleRef.getDescription()) = "+
			xmlSecurityRoleRef.getDescription());

	if ( xmlSecurityRoleRef.getRoleLink() != null )
	    Trace.outln(trace,
			"PSecurityRoleRef.getValues(): xmlSecurityRoleRef.getRoleLink() = "+
			xmlSecurityRoleRef.getRoleLink());
    }

}
