/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


/*----------------------------------------------------*/
/*                   Session Panel                    */
/*----------------------------------------------------*/

package org.objectweb.jonas.beandeployer.panels;

import org.objectweb.jonas.beandeployer.Consts;
import org.objectweb.jonas.beandeployer.components.CheckBoxList;
import org.objectweb.jonas.beandeployer.components.FileChooserElement;
import org.objectweb.jonas.beandeployer.components.TextFieldElement;
import org.objectweb.jonas.beandeployer.components.TextAreaDescription;
import org.objectweb.jonas.beandeployer.components.RadioButtonList;
import org.objectweb.jonas.common.Trace;

import org.objectweb.jonas_ejb.deployment.xml.EjbRef;
import org.objectweb.jonas_ejb.deployment.xml.EnvEntry;
import org.objectweb.jonas_ejb.deployment.xml.ResourceRef;
import org.objectweb.jonas_ejb.deployment.xml.SecurityRoleRef;
import org.objectweb.jonas_ejb.deployment.xml.Session;

import java.util.Enumeration;
import java.util.Vector;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import javax.swing.JPanel;

/**
 * Panel Session.
 * There are :
 * <li> one TextFieldDesc.
 * <li> three TextFieldElement,
 * <li> one FileChooserMultiple,
 * <li> one TextFieldMultiple,
 * <li> one RadioButtonList.
 * <p>
 * The associated node is Ejb Jar -> META-INF/ejb-jar.xml -> enterprise-beans -> session
 * @author Magali MORAEL and Christophe GERMAIN.
 */
public class PSession extends DeployerPanelWithEjbName {

    static private final int trace = Trace.DB_19;

    private GridBagLayout      panelLayout;
    private GridBagConstraints constraints;

    private TextFieldElement    displayName;
    private TextAreaDescription description;
    private TextFieldElement    ejbName;
    private FileChooserElement  smallIcon, largeIcon;
    private TextFieldElement    home, remote, ejbClass;
    private RadioButtonList     sessionType;
    private RadioButtonList     transactionType;

    private Session         xmlSession;
   
    private Vector fieldsNoFilled;

    /**
     * Build a new Entity panel.
     */
    public PSession() {
	super();
	this.init();
	this.xmlSession = new Session();
	this.xmlCreation();
    }  

    /**
     * Build an existing Entity panel.
     * @param xmlSession the existing Castor object.
     */
    public PSession(Session xmlSession) {
	super();
	this.init();
	this.xmlSession = xmlSession;
	this.xmlCreation();
	this.xmlRead(xmlSession);
    }  

    /**
     * Initialize the layout and its constraints.
     */
    private void init() {
	this.fieldsNoFilled = new Vector();

	panelLayout = new GridBagLayout();
	setLayout(panelLayout);
	constraints = new GridBagConstraints();
	
	this.addGraphicElements();
    }

    /**
     * Add all graphic element.
     */
    private void addGraphicElements() {
	// All components are in the first column.
	constraints.gridx = Consts.GRIDX_FIRST;

	// Add description : TextAreaDescription
	constraints.gridy = Consts.GRIDY_FIRST;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	description = new TextAreaDescription();
	panelLayout.setConstraints(description,constraints);
	this.add(description);

	// Add display-name : TextFieldElement
	constraints.gridy = Consts.GRIDY_FIRST+1;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	displayName = new TextFieldElement(Consts.LABEL_DISPLAY_NAME, false);
	panelLayout.setConstraints(displayName,constraints);
	this.add(displayName);

	// Add smallIcon : FileChooserElement
	constraints.gridy = Consts.GRIDY_FIRST+2;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	smallIcon = new FileChooserElement(Consts.LABEL_SMALL_ICON,false);
	this.smallIcon.setEnabled(false);
	panelLayout.setConstraints(smallIcon,constraints);
	this.add(smallIcon);

	// Add largeIcon : FileChooserElement
	constraints.gridy = Consts.GRIDY_FIRST+3;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	largeIcon = new FileChooserElement(Consts.LABEL_LARGE_ICON,false);
	this.largeIcon.setEnabled(false);
	panelLayout.setConstraints(largeIcon,constraints);
	this.add(largeIcon);

	// Add ejbName : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+4;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	ejbName = new TextFieldElement(Consts.LABEL_EJB_NAME, true);
	panelLayout.setConstraints(ejbName,constraints);
	this.add(ejbName);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	// Add home : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+5;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	home = new TextFieldElement(Consts.LABEL_HOME,true);
	panelLayout.setConstraints(home,constraints);
	this.add(home);
	this.fieldsNoFilled.add(Consts.LABEL_HOME);

	// Add remote : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+6;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	remote = new TextFieldElement(Consts.LABEL_REMOTE,true);
	panelLayout.setConstraints(remote,constraints);
	this.add(remote);
	this.fieldsNoFilled.add(Consts.LABEL_REMOTE);

	// Add ejb-class : TextFieldElement - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+7;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	ejbClass = new TextFieldElement(Consts.LABEL_EJB_CLASS,true);
	panelLayout.setConstraints(ejbClass,constraints);
	this.add(ejbClass);
	this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);

	// Add session-type : RadioButtonList - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+8;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector sessionList = new Vector();
	sessionList.add(Consts.LABEL_STATEFUL);
	sessionList.add(Consts.LABEL_STATELESS);
	sessionType = new RadioButtonList(Consts.LABEL_SESSION_TYPE,sessionList,true);
	panelLayout.setConstraints(sessionType,constraints);
	this.add(sessionType);
	this.fieldsNoFilled.add(Consts.LABEL_SESSION_TYPE);

	// Add transaction-type : RadioButtonList - obligatory field.
	constraints.gridy = Consts.GRIDY_FIRST+9;
	constraints.fill = GridBagConstraints.HORIZONTAL;
	Vector transactionList = new Vector();
	transactionList.add(Consts.LABEL_BEAN);
	transactionList.add(Consts.LABEL_CONTAINER);
	transactionType = new RadioButtonList(Consts.LABEL_TRANSACTION_TYPE,transactionList,true);
	panelLayout.setConstraints(transactionType,constraints);
	this.add(transactionType);
	this.fieldsNoFilled.add(Consts.LABEL_TRANSACTION_TYPE);

	obligatoryTable.put(this,fieldsNoFilled);
    }
    
    /**
     * Creation of xml values.
     */
    private void xmlCreation() {
	// Nothing to do
    }

    /**
     * Reading xml values.
     * @param xmlSession the existing Castor object
     *        associated to this session's panel.
     */
    private void xmlRead(Session xmlSession) {
	fieldsNoFilled.removeAllElements(); // re-initialization

	if ( xmlSession.getDisplayName() != null )
	    displayName.setText(xmlSession.getDisplayName());
    
	if ( xmlSession.getDescription() != null )
	    description.setText(xmlSession.getDescription());
	
	if ( xmlSession.getEjbName() != null ) {
	    String ejbNameString = xmlSession.getEjbName();
	    ejbName.setText(ejbNameString);
	    changeName(ejbNameString);
	}
	else
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);

	if ( xmlSession.getSmallIcon() != null )
	    smallIcon.setText(xmlSession.getSmallIcon());

	if ( xmlSession.getLargeIcon() != null )
	    largeIcon.setText(xmlSession.getLargeIcon());

	if ( xmlSession.getHome() != null )
	    home.setText(xmlSession.getHome());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);

	if ( xmlSession.getRemote() != null )
	    remote.setText(xmlSession.getRemote());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_REMOTE);

	if ( xmlSession.getEjbClass() != null )
	    ejbClass.setText(xmlSession.getEjbClass());
	else
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);

	if ( xmlSession.getSessionType() != null ) {
	    if (!(sessionType.setTextAndCheck(xmlSession.getSessionType()))) {
		xmlSession.setSessionType(null);
		this.fieldsNoFilled.add(Consts.LABEL_SESSION_TYPE);
	    }
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_SESSION_TYPE);
	}

	if ( xmlSession.getTransactionType() != null ) {
	    if (!(transactionType.setTextAndCheck(xmlSession.getTransactionType()))) {
		xmlSession.setTransactionType(null);
		this.fieldsNoFilled.add(Consts.LABEL_TRANSACTION_TYPE);
	    }
	} else {
	    this.fieldsNoFilled.add(Consts.LABEL_TRANSACTION_TYPE);
	}
    
	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else 
	    obligatoryTable.remove(this);
    }

    /**
     * Get the type of the node.
     * @return the node's type.
     */
    public String getType() {
	return Consts.STRING_SESSION;
    }

    /**
     * Get the Castor object instanciate by this panel.
     * @return the Castor object.
     */
    public Object getCastorObject() {
	return xmlSession;
    }

    /**
     * Add one of following Castor Object :
     * <li> ejb-ref
     * <li> env-entry
     * <li> resource-ref
     * <li> security-role-ref
     * @param xmlObject the Castor object to add.
     */
    public void addInSpecificList(Object xmlObject) {
	if (xmlObject instanceof EjbRef)
	    this.addEjbRef((EjbRef)xmlObject);
	else if (xmlObject instanceof EnvEntry)
	    this.addEnvEntry((EnvEntry)xmlObject);
	else if (xmlObject instanceof ResourceRef)
	    this.addResourceRef((ResourceRef)xmlObject);
	else if (xmlObject instanceof SecurityRoleRef)
	    this.addSecurityRoleRef((SecurityRoleRef)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Remove one of following Castor Object :
     * <li> ejb-ref
     * <li> env-entry
     * <li> resource-ref
     * <li> security-role-ref
     * @param xmlObject the Castor object to remove.
     */
    public void removeInSpecificList(Object xmlObject) {
	if (xmlObject instanceof EjbRef)
	    this.removeEjbRef((EjbRef)xmlObject);
	else if (xmlObject instanceof EnvEntry)
	    this.removeEnvEntry((EnvEntry)xmlObject);
	else if (xmlObject instanceof ResourceRef)
	    this.removeResourceRef((ResourceRef)xmlObject);
	else if (xmlObject instanceof SecurityRoleRef)
	    this.removeSecurityRoleRef((SecurityRoleRef)xmlObject);
	else throw new Error("The parameter is not a Castor object.");
    }

    /**
     * Add an EjbRef.
     * @param xmlEjbRef the ejb-ref to add.
     */
    public void addEjbRef(EjbRef xmlEjbRef) {
	xmlSession.addEjbRef(xmlEjbRef);
    }

    /**
     * Remove an EjbRef.
     * @param xmlEjbRef the ejb-ref to remove.
     */
    public void removeEjbRef(EjbRef xmlEjbRef) {
	EjbRef[] ejbRefTable = this.xmlSession.getEjbRef();
	int i = 0;
	boolean found = false;
	while (i <= ejbRefTable.length && !found) {
	    if (ejbRefTable[i].equals(xmlEjbRef))
		found = true;
	    i++;
	}
	this.xmlSession.removeEjbRef(i-1);
    }
    
    /**
     * Add a EnvEntry.
     * @param xmlEnvEntry the env-entry to add.
     */
    public void addEnvEntry(EnvEntry xmlEnvEntry) {
	xmlSession.addEnvEntry(xmlEnvEntry);
    }

    /**
     * Remove a EnvEntry.
     * @param xmlEnvEntry the env-entry to remove.
     */
    public void removeEnvEntry(EnvEntry xmlEnvEntry) {
	EnvEntry[] envEntryTable = this.xmlSession.getEnvEntry();
	int i = 0;
	boolean found = false;
	while (i <= envEntryTable.length && !found) {
	    if (envEntryTable[i].equals(xmlEnvEntry))
		found = true;
	    i++;
	}
	this.xmlSession.removeEnvEntry(i-1);
    }

    /**
     * Add a ResourceRef.
     * @param xmlResourceRef the resource-ref to add.
     */
    public void addResourceRef(ResourceRef xmlResourceRef) {
	xmlSession.addResourceRef(xmlResourceRef);
    }

    /**
     * Remove a ResourceRef.
     * @param xmlResourceRef the resource-ref to remove.
     */
    public void removeResourceRef(ResourceRef xmlResourceRef) {
	ResourceRef[] resourceRefTable = this.xmlSession.getResourceRef();
	int i = 0;
	boolean found = false;
	while (i <= resourceRefTable.length && !found) {
	    if (resourceRefTable[i].equals(xmlResourceRef))
		found = true;
	    i++;
	}
	this.xmlSession.removeResourceRef(i-1);
    }

    /**
     * Add a SecurityRoleRef.
     * @param xmlSecurityRoleRef the security-role-ref to add.
     */
    public void addSecurityRoleRef(SecurityRoleRef xmlSecurityRoleRef) {
	xmlSession.addSecurityRoleRef(xmlSecurityRoleRef);
    }

    /**
     * Remove a SecurityRoleRef.
     * @param xmlSecurityRoleRef the security-role-ref to remove.
     */
    public void removeSecurityRoleRef(SecurityRoleRef xmlSecurityRoleRef) {
	SecurityRoleRef[] securityRoleRefTable = this.xmlSession.getSecurityRoleRef();
	int i = 0;
	boolean found = false;
	while (i <= securityRoleRefTable.length && !found) {
	    if (securityRoleRefTable[i].equals(xmlSecurityRoleRef))
		found = true;
	    i++;
	}
	this.xmlSession.removeSecurityRoleRef(i-1);
    }

    /**
     * Set ejb-name in the panel.
     * This method is used in this class and invoked
     * by DeployerMenuBar when user create a new bean.
     * @param newEjbName the new ejb name.
     */
    public void setEjbName(String newEjbName) {
	if (newEjbName.length() != 0) {
	    xmlSession.setEjbName(newEjbName);
	    changeName(newEjbName);
	    this.ejbName.setText(newEjbName);
	    this.fieldsNoFilled.remove(Consts.LABEL_EJB_NAME);
	}
    }

    /**
     * Setting of xml values in the associated Castor class.
     * @return true if there is a change between this taking of account and values in the Castor object.
     */
    public boolean setValues() {
	String tempString = new String();
	boolean result = true; // result is true if there is a change.

	this.fieldsNoFilled.removeAllElements(); // re-initialization
	
	// DISPLAY NAME
	tempString = displayName.getText();
	if (tempString.length() != 0) {
	    xmlSession.setDisplayName(tempString);
	} else {
	    xmlSession.setDisplayName(null);
	}
	
	// DESCRIPTION
	tempString = description.getText();
	if (tempString.length() != 0) {
	    xmlSession.setDescription(tempString);
	} else { 
	    xmlSession.setDescription(null);
	}

	// EJB Name
	tempString = ejbName.getText();
	if (tempString.length() != 0) {
	    this.setEjbName(tempString);		
	}      
	else {
	    Trace.outln(trace,
			"PSession.setValues(): ejb-name is an obligatory field, it must be filled.");
	    xmlSession.setEjbName(null);
	    if (replacedName) {
		initializeName();
	    }
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_NAME);
	}

	// Small-icon
	tempString = smallIcon.getText();
	if (tempString.length() != 0) {
	    xmlSession.setSmallIcon(tempString);
	} else { 
	    xmlSession.setSmallIcon(null);
	}

	// Large-icon
	tempString = largeIcon.getText();
	if (tempString.length() != 0) {
	    xmlSession.setLargeIcon(tempString);
	} else { 
	   xmlSession.setLargeIcon(null);
	}	
	
	// Home class
	tempString = home.getText();
	if (tempString.length() != 0) {
	    xmlSession.setHome(tempString);
	} else { 
	    Trace.outln(trace,
			"PSession.setValues(): home is an obligatory field, it must be filled.");
	    xmlSession.setHome(null);
	    this.fieldsNoFilled.add(Consts.LABEL_HOME);
	}

	// Remote Class
	tempString = remote.getText();
	if (tempString.length() != 0) {
	    xmlSession.setRemote(tempString);
	} else { 
	    Trace.outln(trace,
			"PSession.setValues(): remote is an obligatory field, it must be filled.");
	    xmlSession.setRemote(null);
	    this.fieldsNoFilled.add(Consts.LABEL_REMOTE);
	}

	// ejb class
	tempString = ejbClass.getText(); 
	if (tempString.length() != 0) {
	    xmlSession.setEjbClass(tempString);
	} else {
 	    Trace.outln(trace,
			"PSession.setValues(): ejb-class is an obligatory field, it must be filled.");
	    this.fieldsNoFilled.add(Consts.LABEL_EJB_CLASS);
	}
       	
	// Session type
	tempString = sessionType.getText();
	if (tempString != null){
	    if (tempString.length() != 0) {
		xmlSession.setSessionType(tempString);
	    } else { 
		Trace.outln(trace,
			"PSession.setValues(): session type is an obligatory field, it must be filled.");
		xmlSession.setSessionType(null);
		this.fieldsNoFilled.add(Consts.LABEL_SESSION_TYPE);
	    }
	}
	    
	// transaction type
	tempString = transactionType.getText();
	if (tempString != null){
	    if (tempString.length() != 0) {
		xmlSession.setTransactionType(tempString);
	    } else { 
		Trace.outln(trace,
			"PSession.setValues(): session type is an obligatory field, it must be filled.");
		xmlSession.setTransactionType(null);
		this.fieldsNoFilled.add(Consts.LABEL_TRANSACTION_TYPE);
	    }
	}
	
	if (fieldsNoFilled.size() != 0)
	    obligatoryTable.put(this,fieldsNoFilled);
	else obligatoryTable.remove(this);

	return result;
    }

     /**
     * Get values of the associated Castor class.
     */
    public void getValues() {
	if ( xmlSession.getDisplayName() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getDisplayName() = "+
			xmlSession.getDisplayName());
	}
	if ( xmlSession.getDescription() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getDescription() = "+
			xmlSession.getDescription());
	}
	if ( xmlSession.getEjbName() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getEjbName() = "+
			xmlSession.getEjbName());
	}
	if ( xmlSession.getSmallIcon() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getSmallIcon() = "+
			xmlSession.getSmallIcon());
	}
	if ( xmlSession.getLargeIcon() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getLargeIcon() = "+
			xmlSession.getLargeIcon());
	}
	if ( xmlSession.getHome() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getHome() = "+
			xmlSession.getHome());
	}
	if ( xmlSession.getRemote() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getRemote() = "+
			xmlSession.getRemote());
	}
	if ( xmlSession.getEjbClass() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getEjbClass() = "+
			xmlSession.getEjbClass());
	}
	if ( xmlSession.getSessionType() != null ) {
	    Trace.outln(trace,
			"PSession.getValues(): xmlSession.getSessionType() = "+
			xmlSession.getSessionType());
	}
	if ( xmlSession.getTransactionType() != null ) {
	   Trace.outln(trace,
		       "PSession.getValues():  xmlSession.getTransactionType() = "+
		       xmlSession.getTransactionType());
	}
    }

    /**
     * Get the value of the ejb-name field.
     * @return the ejb-name.
     */
    public String getEjbName(){
	return ejbName.getText();
    }

}
