/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.common;

import java.io.OutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Date;

public class LogWriter extends PrintWriter {
    
    public boolean	isHeader = false;	// if the header have to be inserted in the log
    public String	prefix = "";		// prefix to insert
    public int		mTempo = 0;		// tempo after the log

    /**
     * Constructors
     */
    public LogWriter(PrintWriter printWriter) {
	super(printWriter, true);
    }
    
    public LogWriter(OutputStream out) {
	super(out, true);
    } 
    
    public LogWriter(PrintStream out) {
	super(out, true);
    }
    
    /**
     * set the header
     */
    public void setHeader(boolean isHeader) {
	this.isHeader = isHeader;
    }
    
    /**
     * set the prefix
     */
    public void setPrefix(String prefix) {
	this.prefix = prefix;
    }
    
    /**
     * set the tempo
     */
    public void setMTempo(int mTempo) {
	this.mTempo = mTempo;
    }
    
    /**
     * Returns an header message for trace messages
     * @returns     header message
     */
    public String header() {
	if (isHeader) {
	    return(prefix +
		   Thread.currentThread().getName() + 
		   " : " + 
		   hour());
	} else {
	    return("");
	}
    }
    
    public synchronized String hour() {
        GregorianCalendar cal = new GregorianCalendar();
	Date             date = new Date();
	cal.setTime(date);
	
	return 
	    cal.get(Calendar.HOUR)+":"+
	    cal.get(Calendar.MINUTE)+":"+
	    cal.get(Calendar.SECOND)+":"+
	    cal.get(Calendar.MILLISECOND)+" ";
    }
    
    /**
     * Delegation of println(String)
     */
    public synchronized void println(String msg) {
	super.println(header()+" "+msg);
	flush();
	tempo();
    }
    
    /**
     * To make a tempo after message
     */
    public synchronized void tempo() {
	if (mTempo > 0) {
	    try {
		Thread.currentThread().sleep(mTempo);
	    } catch (InterruptedException e) {
	    }
	}
    }

}
