/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import java.rmi.Remote;
import java.rmi.RemoteException;
import javax.management.*;

/**
 * interface for JMX Remote Call
 * a simple listing of JMX Server method
 * see the jmx api for further informations
 */
public interface RMIConnector extends Remote {

    public java.lang.Object instantiate(java.lang.String className)
	throws ReflectionException, 
	       MBeanException, 
	       RemoteException;
    
    public java.lang.Object instantiate(java.lang.String className,
					ObjectName loaderName)
	throws ReflectionException, 
	       MBeanException, 
	       InstanceNotFoundException, 
	       RemoteException;
    
    public java.lang.Object instantiate(java.lang.String className,
					java.lang.Object[] params,
					java.lang.String[] signature)
	throws ReflectionException, 
	       MBeanException, 
	       RemoteException;

    public java.lang.Object instantiate(java.lang.String className,
					ObjectName loaderName,
					java.lang.Object[] params,
					java.lang.String[] signature)
	throws ReflectionException, 
	       MBeanException, 
	       InstanceNotFoundException, 
	       RemoteException;

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       RemoteException;

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      ObjectName loaderName)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       InstanceNotFoundException,
	       RemoteException;

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      java.lang.Object[] params,
				      java.lang.String[] signature)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       RemoteException;

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      ObjectName loaderName,
				      java.lang.Object[] params,
				      java.lang.String[] signature)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       InstanceNotFoundException, 
	       RemoteException;

    public ObjectInstance registerMBean(java.lang.Object object,
					ObjectName name)
	throws InstanceAlreadyExistsException,
	       MBeanRegistrationException,
	       NotCompliantMBeanException,
	       RemoteException;

    public void unregisterMBean(ObjectName name)
	throws InstanceNotFoundException,
	       MBeanRegistrationException,
	       RemoteException;

    public ObjectInstance getObjectInstance(ObjectName name)
	throws InstanceNotFoundException,
	       RemoteException;

    public java.util.Set queryMBeans(ObjectName name,
				     QueryExp query)
	throws RemoteException;

    public java.util.Set queryNames(ObjectName name,
				    QueryExp query)
	throws RemoteException;

    public boolean isRegistered(ObjectName name)
	throws RemoteException;

    public java.lang.Integer getMBeanCount()
	throws RemoteException;

    public java.lang.Object getAttribute(ObjectName name,
					 java.lang.String attribute)
	throws MBeanException,
	       AttributeNotFoundException,
	       InstanceNotFoundException,
	       ReflectionException,
	       RemoteException;

    public AttributeList getAttributes(ObjectName name,
				       java.lang.String[] attributes)
	throws InstanceNotFoundException,
	       ReflectionException,
	       RemoteException;

    public void setAttribute(ObjectName name,
			     Attribute attribute)
	throws InstanceNotFoundException,
	       AttributeNotFoundException,
	       InvalidAttributeValueException,
	       MBeanException,
	       ReflectionException,
	       RemoteException;

    public AttributeList setAttributes(ObjectName name,
				       AttributeList attributes)
	throws InstanceNotFoundException,
	       ReflectionException,
	       RemoteException;

    public java.lang.Object invoke(ObjectName name,
				   java.lang.String operationName,
				   java.lang.Object[] params,
				   java.lang.String[] signature)
	throws InstanceNotFoundException,
	       MBeanException,
	       ReflectionException,
	       RemoteException;

    public java.lang.String getDefaultDomain()
	throws RemoteException;

    public void addNotificationListener(ObjectName name,
					NotificationListener listener,
					NotificationFilter filter,
					java.lang.Object handback)
	throws InstanceNotFoundException,
	       RemoteException;

    public void addNotificationListener(ObjectName name,
					ObjectName listener,
					NotificationFilter filter,
					java.lang.Object handback)
	throws InstanceNotFoundException,
	       RemoteException;

    public void removeNotificationListener(ObjectName name,
					   NotificationListener listener)
	throws InstanceNotFoundException,
	       ListenerNotFoundException,
	       RemoteException;

    public void removeNotificationListener(ObjectName name,
					   ObjectName listener)
	throws InstanceNotFoundException,
	       ListenerNotFoundException,
	       RemoteException;

    public MBeanInfo getMBeanInfo(ObjectName name)
	throws InstanceNotFoundException,
	       IntrospectionException,
	       ReflectionException,
	       RemoteException;

    public boolean isInstanceOf(ObjectName name,
				java.lang.String className)
	throws InstanceNotFoundException,
	       RemoteException;


    // Problem with david/jrmic compilation 
//     public java.io.ObjectInputStream deserialize(ObjectName name,
// 						 byte[] data)
// 	throws InstanceNotFoundException,
// 	       OperationsException,
// 	       RemoteException;

//     public java.io.ObjectInputStream deserialize(java.lang.String className,
// 						 byte[] data)
// 	throws OperationsException,
// 	       ReflectionException,
// 	       RemoteException;

//     public java.io.ObjectInputStream deserialize(java.lang.String className,
// 						 ObjectName loaderName,
// 						 byte[] data)
// 	throws InstanceNotFoundException,
// 	       OperationsException,
// 	       ReflectionException,
// 	       RemoteException;
}
