/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Michel Bruno and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.jmx;

import org.objectweb.common.RemoteObject;
import java.rmi.RemoteException;
import javax.management.*;

/**
 * JMX Remote Class
 * a simple remote delegation of JMX Server method
 * see the jmx api for further informations
 */
public class RMIConnectorImpl extends RemoteObject implements RMIConnector {
    protected MBeanServer server;

    public RMIConnectorImpl(MBeanServer server)
	throws RemoteException 
    {
	super();
	this.server = server;
    }

    public java.lang.Object instantiate(java.lang.String className)
	throws ReflectionException, 
	       MBeanException, 
	       RemoteException 
    {
	return this.server.instantiate(className);
    }
    
    public java.lang.Object instantiate(java.lang.String className,
					ObjectName loaderName)
	throws ReflectionException, 
	       MBeanException, 
	       InstanceNotFoundException, 
	       RemoteException 
    {
	return this.server.instantiate(className, loaderName);
    }
    
    public java.lang.Object instantiate(java.lang.String className,
					java.lang.Object[] params,
					java.lang.String[] signature)
	throws ReflectionException, 
	       MBeanException, 
	       RemoteException 
    {
	return this.server.instantiate(className, params, signature);
    }

    public java.lang.Object instantiate(java.lang.String className,
					ObjectName loaderName,
					java.lang.Object[] params,
					java.lang.String[] signature)
	throws ReflectionException, 
	       MBeanException, 
	       InstanceNotFoundException, 
	       RemoteException 
    {
	return this.server.instantiate(className, loaderName, params, signature);
    }

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       RemoteException 
    {
	return this.server.createMBean(className, name);
    }

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      ObjectName loaderName)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       InstanceNotFoundException,
	       RemoteException 
    {
	return this.server.createMBean(className, name, loaderName);
    }

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      java.lang.Object[] params,
				      java.lang.String[] signature)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       RemoteException 
    {
	return this.server.createMBean(className, name, params, signature);
    }

    public ObjectInstance createMBean(java.lang.String className,
				      ObjectName name,
				      ObjectName loaderName,
				      java.lang.Object[] params,
				      java.lang.String[] signature)
	throws ReflectionException, 
	       InstanceAlreadyExistsException, 
	       MBeanRegistrationException, 
	       MBeanException, 
	       NotCompliantMBeanException, 
	       InstanceNotFoundException, 
	       RemoteException 
    {
	return this.server.createMBean(className, name, params, signature);
    }

    public ObjectInstance registerMBean(java.lang.Object object,
					ObjectName name)
	throws InstanceAlreadyExistsException,
	       MBeanRegistrationException,
	       NotCompliantMBeanException,
	       RemoteException 
    {
	return this.server.registerMBean(object, name);
    }

    public void unregisterMBean(ObjectName name)
	throws InstanceNotFoundException,
	       MBeanRegistrationException,
	       RemoteException 
    {
	this.server.unregisterMBean(name);
    }

    public ObjectInstance getObjectInstance(ObjectName name)
	throws InstanceNotFoundException,
	       RemoteException
    {
	return this.server.getObjectInstance(name);
    }

    public java.util.Set queryMBeans(ObjectName name,
				     QueryExp query)
	throws RemoteException 
    {
	return this.server.queryMBeans(name, query);
    }

    public java.util.Set queryNames(ObjectName name,
				    QueryExp query)
	throws RemoteException 
    {
	return this.server.queryNames(name, query);
    }

    public boolean isRegistered(ObjectName name)
	throws RemoteException 
    {
	return this.server.isRegistered(name);
    }

    public java.lang.Integer getMBeanCount()
	throws RemoteException 
    {
	return this.server.getMBeanCount();
    }

    public java.lang.Object getAttribute(ObjectName name,
					 java.lang.String attribute)
	throws MBeanException,
	       AttributeNotFoundException,
	       InstanceNotFoundException,
	       ReflectionException,
	       RemoteException 
    {
	return this.server.getAttribute(name, attribute);
    }

    public AttributeList getAttributes(ObjectName name,
				       java.lang.String[] attributes)
	throws InstanceNotFoundException,
	       ReflectionException,
	       RemoteException 
    {
	return this.server.getAttributes(name, attributes);
    }

    public void setAttribute(ObjectName name,
			     Attribute attribute)
	throws InstanceNotFoundException,
	       AttributeNotFoundException,
	       InvalidAttributeValueException,
	       MBeanException,
	       ReflectionException,
	       RemoteException 
    {
	this.server.setAttribute(name, attribute);
    }

    public AttributeList setAttributes(ObjectName name,
				       AttributeList attributes)
	throws InstanceNotFoundException,
	       ReflectionException,
	       RemoteException 
    {
	return this.server.setAttributes(name, attributes);
    }

    public java.lang.Object invoke(ObjectName name,
				   java.lang.String operationName,
				   java.lang.Object[] params,
				   java.lang.String[] signature)
	throws InstanceNotFoundException,
	       MBeanException,
	       ReflectionException,
	       RemoteException 
    {
	return this.server.invoke(name, operationName, params, signature);
    }

    public java.lang.String getDefaultDomain()
	throws RemoteException 
    {
	return this.server.getDefaultDomain();
    }

    public void addNotificationListener(ObjectName name,
					NotificationListener listener,
					NotificationFilter filter,
					java.lang.Object handback)
	throws InstanceNotFoundException,
	       RemoteException 
    {
	this.server.addNotificationListener(name, listener, filter, handback);
    }

    public void addNotificationListener(ObjectName name,
					ObjectName listener,
					NotificationFilter filter,
					java.lang.Object handback)
	throws InstanceNotFoundException,
	       RemoteException 
    {
       this.server.addNotificationListener(name, listener, filter, handback);
    }

    public void removeNotificationListener(ObjectName name,
					   NotificationListener listener)
	throws InstanceNotFoundException,
	       ListenerNotFoundException,
	       RemoteException 
    {
	this.server.removeNotificationListener(name, listener);
    }

    public void removeNotificationListener(ObjectName name,
					   ObjectName listener)
	throws InstanceNotFoundException,
	       ListenerNotFoundException,
	       RemoteException 
    {
	this.server.removeNotificationListener(name, listener);
    }

    public MBeanInfo getMBeanInfo(ObjectName name)
	throws InstanceNotFoundException,
	       IntrospectionException,
	       ReflectionException,
	       RemoteException 
    {
	return this.server.getMBeanInfo(name);
    }

    public boolean isInstanceOf(ObjectName name,
				java.lang.String className)
	throws InstanceNotFoundException,
	       RemoteException 
    {
	return this.server.isInstanceOf(name, className);
    }


    // Problem with david/jrmic compilation 
//     public java.io.ObjectInputStream deserialize(ObjectName name,
// 						 byte[] data)
// 	throws InstanceNotFoundException,
// 	       OperationsException,
// 	       RemoteException 
//     {
// 	return this.server.deserialize(name, data);
//     }

//     public java.io.ObjectInputStream deserialize(java.lang.String className,
// 						 byte[] data)
// 	throws OperationsException,
// 	       ReflectionException,
// 	       RemoteException 
//     {
// 	return this.server.deserialize(className, data);
//     }

//     public java.io.ObjectInputStream deserialize(java.lang.String className,
// 						 ObjectName loaderName,
// 						 byte[] data)
// 	throws InstanceNotFoundException,
// 	       OperationsException,
// 	       ReflectionException,
// 	       RemoteException 
//     {
// 	return this.server.deserialize(className, loaderName, data);
//     }
}
