/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import org.objectweb.jonas.jmx.RMIConnector;
import javax.management.*;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class AllMBeansServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("AllMBeans"));
	out.println(openMain());
	
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	
	try {
	    // Getting all ObjectNames:
	    java.util.Set allObjectNames = ConnectorFactory.getRMIConnector().queryNames(null, null);
	    
	    // Traversing all ObjectNames:
	    for (Iterator i=allObjectNames.iterator() ; i.hasNext() ; ) {
		ObjectName currentObjectName = (ObjectName)i.next();
		MBeanInfo currentMBeanInfo = ConnectorFactory.getRMIConnector().getMBeanInfo(currentObjectName);
		
		out.println(title(currentMBeanInfo.getClassName(), 2));
		out.println("  <tr>");
		out.println("    <td colspan=\"2\">"+currentObjectName+"</td>");
		out.println("  </tr>");
		out.println(openArea());

		// Getting all MBeanAttributeInfo:
		MBeanAttributeInfo[] currentMBeanAttributeInfo = currentMBeanInfo.getAttributes();

		// Traversing all MBeanAttributeInfo:
		for (int j=0 ; j<currentMBeanAttributeInfo.length ; j++) {
		    try {
			String type = currentMBeanAttributeInfo[j].getType();
			if (currentMBeanAttributeInfo[j].isWritable()) type = "<b>write</b> "+type;
			if (currentMBeanAttributeInfo[j].isReadable()) type = "<b>read</b> "+type;
			out.println(attribute(type, 
					      "<b>"+currentMBeanAttributeInfo[j].getName()+"</b> == "
					      +ConnectorFactory.getRMIConnector().getAttribute(currentObjectName, 
											       currentMBeanAttributeInfo[j].getName())));
		    } catch (ManagementException e) {
			out.println(error("Error while getting attribute:", e.getMessage(), 2));
		    }
		}

		// Getting all MBeanOperationInfo:
		MBeanOperationInfo[] currentMBeanOperationInfo = currentMBeanInfo.getOperations();
		for (int j=0 ; j<currentMBeanOperationInfo.length ; j++) {
		    try {
			// Getting operation's parameters:
			MBeanParameterInfo[] currentMBeanParameterInfo = currentMBeanOperationInfo[j].getSignature();
			StringBuffer params = new StringBuffer();
			for (int k=0 ; k<currentMBeanParameterInfo.length ; k++) {
			    params.append(currentMBeanParameterInfo[k].getType());
			    if (k < currentMBeanParameterInfo.length-1) params.append(", ");
			}
			out.println(attribute(currentMBeanOperationInfo[j].getReturnType(), 
					      "<b>"+currentMBeanOperationInfo[j].getName()
					      +"</b>("+new String(params)+")"));
		    } catch (ManagementException e) {
			out.println(error("Error while getting information:", e.getMessage(), 2));
		    }
		}
		out.println(closeArea());
	    }

	} catch (Exception e) {
	    out.println(error("Error while looking for MBeans:", e.getMessage()));
	}
	out.println("</table>");
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(ALLMBEANS);
    }
}
