/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class CMPServlet extends EntityBeanServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	String fileName = null;
	String ejbName = null;
	// Retrieving request parameters
	for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
	    String paramName = (String)e.nextElement();
	    if (paramName.equals("fileName")) 
		fileName = req.getParameter("fileName");
	    else if (paramName.equals("ejbName")) 
		ejbName = req.getParameter("ejbName");
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("CMP?fileName="+fileName+"&#38;ejbName="+ejbName));
	out.println(openMain());
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	out.println(path(fileName));
	
	// Display the CMP info:
	out.println(title("Container Managed Persistence: "+ejbName));
	    out.println(openArea());
	out.println(ejbClass("ejbcmp", fileName, ejbName));
	out.println(entityBeanInfo("ejbcmp", fileName, ejbName));
	out.println(closeArea());
	

	out.println(ejbDataSource("ejbcmp", fileName, ejbName));
	out.println(ejbJMSConnectionFactory("ejbcmp", fileName, ejbName));
	out.println(ejbJMSDestination("ejbcmp", fileName, ejbName));

	
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }
}
