/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.ObjectName;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class ContainerServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("remove")) {
		    Object[] param = { req.getParameter("remove") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
					       "removeContainer", param, signature);
		} else if (paramName.equals("create")) {
		    Object[] param = { req.getParameter("create") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
					       "createContainer", param, signature);
		} else if (paramName.equals("run")) {
		    Object[] param = new Object[0];
		    String[] signature = new String[0];
		    JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
					       req.getParameter("run"), param, signature);
		}
	    }
	} catch (ManagementException e) {
	    exception = e;
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("ContainerService"));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	if (JonasManagementRepr.isRegistered(JonasObjectName.ejbService())) {
	    // Display the container service management information:

	    out.println(title("Container service"));
	    out.println(openArea());
	    out.println("        <tr>");
	    out.println("          <td width=\"50%\">");
	    out.println("            <table>");
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
								      "CurrentNumberOfContainer")
				     +" container(s):", 2));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    }
	    
	    java.util.Set allContainers = JonasManagementRepr.queryNames(JonasObjectName.allContainers());
	    for (Iterator i = allContainers.iterator(); i.hasNext() ;) {
		try {
		    ObjectName current = (ObjectName)i.next();
		    String fileName = (String)JonasManagementRepr.getAttribute(current, "FileName");
		    Object containerName = JonasManagementRepr.getAttribute(current, "ContainerName");
		    out.println(attribute("<a title=\"View this container\" "
					  +"href=\"Container?fileName="+fileName.replace(' ', '+')
					  +"\">"+containerName+"</a>", 
					  "<a title=\"Remove this container\" "
					  +"href=\"ContainerService?remove="+fileName.replace(' ', '+')
					  +"\"><b>Remove</b></a>"));
		} catch (ManagementException e) {
		    out.println(error("Error while getting attribute:", e.getMessage(), 2));
		}
	    }
	    out.println("            </table>");
	    out.println("          </td>");
	    
	    out.println("          <td width=\"50%\">");
	    out.println("            <form method=\"get\" action=\"ContainerService\">");
	    out.println("              <table>");
	    out.println(subtitle("ejb-jar deployment descriptor or jar file:", 2));
	    out.println(attribute("<input type=\"text\" name=\"create\">", "<input type=\"submit\" value=\"Create\">"));
	    out.println("              </table>");
	    out.println("            </form>");
	    out.println("          </td>");
	    out.println("        </tr>");
	    out.println(attribute("&nbsp;", "&nbsp;"));
	    out.println(attribute("<b><a href=\"ContainerService?run=syncAllEntities\">"
				  +"Synchronize all Entity Beans</a></b>", 
				  "&nbsp;"));
	    out.println(closeArea());

	    out.println(title("EJB in all containers"));
	    out.println(openArea());
	    try {
		out.println(attribute("Total:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfBeanType")+"</b>"));
		out.println(attribute("Entity Bean-managed persistence:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfBMPType")+"</b>"));
		out.println(attribute("Entity Container-managed persistence:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfCMPType")+"</b>"));
		out.println(attribute("Session Bean Stateful:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfSBFType")+"</b>"));
		out.println(attribute("Session Bean Stateless:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfSBLType")+"</b>"));
		out.println(attribute("Message Driven Bean:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbService(), 
									     "TotalCurrentNumberOfMDBType")+"</b>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());

	} else {
	    out.println(title("Service not available."));
	}

	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(CONTAINER);
    }
}
