/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.ObjectName;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class ContainerServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	String fileName = null;
	// Retrieving request parameters
	for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
	    String paramName = (String)e.nextElement();
	    if (paramName.equals("fileName")) 
		fileName = req.getParameter("fileName");
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("Container?fileName="+fileName));
	out.println(openMain());
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	// Display a link to the 'ContainerService' view:
	out.println("  <tr>");
	out.println("    <td>"
		    +"<a title=\"Return to the container service view\" href=\"ContainerService\">"
		    +"Container service</a><p></td>");
	out.println("  </tr>");
	
	if (fileName == null) {
	    out.println(title("No container selected"));
	} else {
	    // Display the container info:
	    try {
		out.println(title((String)JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
									   "ContainerName")));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(openArea());
	    try {
		out.println(attribute("File name:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
									     "FileName")+"</b>"));
		out.println(attribute("Current number of EJB:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
									     "CurrentNumberOfBeanType")+"</b>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(closeArea());
	    // Display Beans in this container:
	    out.println(title("EJB currently in this container"));
	    out.println(openArea());
	    try {
		out.println(bean(JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
								  "CurrentNumberOfBMPType")
				 +" Entity Bean-managed persistence:", "BMP", fileName, 
				 JonasObjectName.allEjbBmp(fileName)));
		out.println(bean(JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
								  "CurrentNumberOfCMPType")
				 +" Entity Container-managed persistence:", "CMP", fileName, 
				 JonasObjectName.allEjbCmp(fileName)));
		out.println(bean(JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
								  "CurrentNumberOfSBFType")
				 +" Session Bean Stateful:", "SBF", fileName, 
				 JonasObjectName.allEjbSbf(fileName)));
		out.println(bean(JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
								  "CurrentNumberOfSBLType")
				 +" Session Bean Stateless:", "SBL", fileName, 
				 JonasObjectName.allEjbSbl(fileName)));
		out.println(bean(JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), 
								  "CurrentNumberOfMDBType")
				 +" Message Driven Bean:", "MDB", fileName, 
				 JonasObjectName.allEjbMdb(fileName)));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(closeArea());
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    private String bean(String bean, String type, String fileName, ObjectName pattern) {
	java.util.Set allBeans = (java.util.Set)JonasManagementRepr.queryNames(pattern);
	StringBuffer sb = new StringBuffer("        <tr>\n");
	sb.append("          <td>\n");
	sb.append("            <table>\n");
	sb.append("              <tr>\n");
	sb.append("                <td width=\"240\" align=\"right\">"+bean+"</td>\n");
	sb.append("                <td>\n");
	if (allBeans.size() > 0) {
	    for (Iterator i = allBeans.iterator(); i.hasNext() ;) {
		ObjectName current = (ObjectName)i.next();
		String beanName = (String)JonasManagementRepr.getAttribute(current, "EJBName");
		sb.append("                  <a title=\"View this EJB\" href=\""+type+"?fileName="
			  +fileName.replace(' ', '+')+"&#38;ejbName="
			  +beanName.replace(' ', '+')
			  +"\"><b>");
		sb.append(beanName); sb.append("</b><br></a>\n");
	    }
	sb.append("                </td>\n");
	sb.append("              </tr>\n");
	sb.append("            </table>\n");
	sb.append("          </td>\n");
	sb.append("        </tr>");
	} else {
	    sb = new StringBuffer("");
	}
	return sb.toString();
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(CONTAINER);
    }
}
