/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class DatabaseServiceServlet extends JonasManagementServlet {
	    
    private Exception exception = null;

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("unload")) {
		    Object[] param = { req.getParameter("unload") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
					       "unloadDataSource", param, signature);
		}
	    }
	} catch (ManagementException e) {
	    exception = e;
	}

	// display the servlet content:
	content(req, res);
    }

    /**
     * Respond to a POST request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doPost(HttpServletRequest req, 
		       HttpServletResponse res) throws IOException, ServletException {

	try {
	    Properties p = new Properties();
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (! req.getParameter(paramName).equals("")) 
		    p.put(paramName, req.getParameter(paramName));
		
	    }
	    Object[] param = { p };
	    String[] signature = { "java.util.Properties" };
	    JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
				       "loadDataSource", param, signature);
	    // display the servlet content:
	    content(req, res);
	} catch (ManagementException e) {
	    res.setContentType("text/html");
	    PrintWriter out = res.getWriter();
	    
	    out.println(doctype());
	    out.println(openHtml());
	    out.println(head());
	    out.println(openBody());
	    out.println(links());
	    out.println(openTable());
	    out.println(index());
	    out.println(banner("DatabaseService"));
	    out.println(openMain());
	    
	    // Content of the main cell of the table:
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	    out.println("  <tr>");
	    out.println("    <td>"
			+"<a title=\"Return to the database service view\" href=\"DatabaseService\">"
			+"Database service</a><p></td>");
	    out.println("  </tr>");
	    out.println(error("Error while loading datasource:", e.getMessage()));
	    out.println("</table>");
	    // End of the content of the main cell of the table
	    
	    out.println(closeMain());
	    out.println(closeTable());
	    out.println(foot());
	    out.println(closeBody());
	    out.println(closeHtml());
	}
    }

    protected void content(HttpServletRequest req, 
			   HttpServletResponse res) throws IOException, ServletException {

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("DatabaseService"));
	out.println(openMain());

	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	if (JonasManagementRepr.isRegistered(JonasObjectName.databaseService())) {
	    // Display the database service management information:
	    out.println(title("Database service"));
	    out.println(openArea());
	    out.println("        <tr>");
	    // First column: display the datasources available:
	    out.println("          <td width=\"50%\">");
	    out.println("            <table>");
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.databaseService(), 
								      "CurrentNumberOfDataSource")
				     +" datasource(s):", 2));
		java.util.Set allDatasources = JonasManagementRepr.queryNames(JonasObjectName.allDatasources());
		for (Iterator i = allDatasources.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    String datasourceName = (String)JonasManagementRepr.getAttribute(current, 
										     "DSName");
		    String op;
		    Object[] param = { datasourceName };
		    String[] signature = { "java.lang.String" };
		    if (( !JonasManagementRepr.isRegistered(JonasObjectName.ejbService())) 
			|| (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
								       "getDataSourceDependence", 
								       param, signature)).isEmpty()))
			op = "<a title=\"Unload this datasource\" "
			    +"href=\"DatabaseService?unload="
			    +datasourceName.replace(' ', '+')+"\"><b>Unload</b></a>";
		    else 
			op = "&nbsp;";
		    out.println(attribute("<a title=\"View this datasource\" "
					  +"href=\"Database?dsName="
					  +datasourceName.replace(' ', '+')+"\">"
					  +datasourceName+"</a>", op));
		}
		out.println(attribute("&nbsp;", "&nbsp;"));
		out.println(attribute("Number of JDBC open connection(s):", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.databaseService(), 
									     "TotalCurrentNumberOfJDBCConnectionOpen")+"</b>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println("            </table>");
	    out.println("          </td>");
	    // Second column: display the load datasources form:
	    out.println("          <td width=\"50%\">");
	    out.println("            <form method=\"get\" action=\"LoadDataSource\">");
	    out.println("              <table>");
	    out.println(subtitle("datasource file:", 2));
	    out.println(attribute("<input type=\"text\" name=\"loadFromFile\">", 
				  "<input type=\"submit\" value=\"Read\">"));
	    out.println("              </table>");
	    out.println("            </form>");
	    out.println("Alternatively, you can <a href=\"LoadDataSource\">fill a empty form</a> "
			+"to configure a new datasource.");
	    out.println("          </td>");

	    out.println("        </tr>");
	    out.println(closeArea());
	} else {
	    out.println(title("Service not available."));
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(DATABASE);
    }
}
