/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class DatabaseServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	String dsName = null;
	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("dsName")) 
		    dsName = req.getParameter("dsName");
		else if (paramName.equals("JDBCConnMaxAge")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCConnMaxAge", 
						     new Integer(req.getParameter("JDBCConnMaxAge")));
		else if (paramName.equals("JDBCConnCheckLevel")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCConnCheckLevel", 
						     new Integer(req.getParameter("JDBCConnCheckLevel")));
		else if (paramName.equals("JDBCTestStatement")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCTestStatement", 
						     new String(req.getParameter("JDBCTestStatement")));
		else if (paramName.equals("JDBCMinConnPool")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCMinConnPool", 
						     new Integer(req.getParameter("JDBCMinConnPool")));
		else if (paramName.equals("JDBCMaxConnPool")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCMaxConnPool", 
						     new Integer(req.getParameter("JDBCMaxConnPool")));
		else if (paramName.equals("JDBCSleepTimeOut")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCSleepTimeOut", 
						     new Integer(req.getParameter("JDBCSleepTimeOut")));
		else if (paramName.equals("JDBCDeadLockPool")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCDeadLockPool", 
						     new Integer(req.getParameter("JDBCDeadLockPool")));
		else if (paramName.equals("JDBCLoopWaitPool")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.datasource(req.getParameter("dsName")), 
						     "JDBCLoopWaitPool", 
						     new Integer(req.getParameter("JDBCLoopWaitPool")));
	    }
	} catch (Exception e) {
	    exception = e;
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("Database?dsName="+dsName));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	// Display a link to the 'DatabaseService' view:
	out.println("  <tr>");
	out.println("    <td>"
		    +"<a title=\"Return to the database service view\" href=\"DatabaseService\">"
		    +"Database service</a><p></td>");
	out.println("  </tr>");
	
	if (dsName == null) {
	    out.println(title("No datasource selected"));
	} else {
	    // Display the datasource info:
	    out.println(title(""+dsName+": information"));
	    out.println(openArea());
	    try {
		out.println(attribute("Datasource type:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "DataSourceType")+"</b>"));
		out.println(attribute("Number of JDBC connection open:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "CurrentNumberOfJDBCConnectionOpen")+"</b>"));
		out.println(attribute("&nbsp;", "&nbsp;"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }

	    try {
		Object[] param = { dsName };
		String[] signature = { "java.lang.String" };
		java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
										 "getDataSourceDependence", 
										 param, signature);
		if (allDep.isEmpty()) {
		    out.println(subtitle("Currently unused.", 2));
		} else {
		    out.println(subtitle("Currently used by:", 2));
		    for (Iterator i = allDep.iterator(); i.hasNext() ;) {
			ObjectName current = (ObjectName)i.next();
			String type = (String)JonasObjectName.getType(current);
			String fileName = (String)JonasManagementRepr.getAttribute(current, "EJBFileName");
			String ejbName = (String)JonasManagementRepr.getAttribute(current, "EJBName");
			if (type.equals("ejbbmp")) type  = "BMP";
			else if (type.equals("ejbcmp")) type  = "CMP";
			else if (type.equals("ejbsbf")) type  = "SBF";
			else if (type.equals("ejbsbl")) type  = "SBL";
			else if (type.equals("ejbmdb")) type  = "MDB";
			out.println("  <tr>");
			out.println("    <td width=\"240\" align=\"right\">");
			out.println("      <a title=\"View this EJB\" "
				    +"href=\""+type+"?fileName="+fileName.replace(' ', '+')
				    +"&#38;ejbName="+ejbName.replace(' ', '+')+"\">"
				    +ejbName+"</a>");
			out.println("    </td>");
			out.println("    <td>in container <a title=\"View this container\" "
				    +"href=\"Container?fileName="+fileName+"\">"
				    +JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName")+"</a></td>");
			out.println("  </tr>");
		    }
		}
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(closeArea());
	    
	    out.println(title(dsName+": configuration"));
	    out.println(openArea());
	    try {
		out.println(attribute("Datasource name:", "<b>"+dsName+"</b>"));
		out.println(attribute("Datasource description:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "DataSourceDescription")+"</b>"));
		out.println(attribute("Datasource URL:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "Url")+"</b>"));
		out.println(attribute("Datasource driver class name:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "ClassName")+"</b>"));
		out.println(attribute("Datasource user name:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "UserName")+"</b>"));
		out.println(attribute("Datasource user password:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "Password")+"</b>"));
		out.println(attribute("Datasource factory:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "DataSourceFactory")+"</b>"));
		out.println(attribute("JNDI name for the XADatasource:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									     "DataSourceXADataName")+"</b>"));
		out.println(attribute("JDBC connection maximum age:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCConnMaxAge\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCConnMaxAge")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in minutes</form>"));
		out.println(attribute("JDBC connection checking level:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCConnCheckLevel\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCConnCheckLevel")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"0 = no special checking<br>"
				      +"1 = check before reuse if physical connection is still open<br>"
				      +"2 = try every connection before reuse</form>"));
		out.println(attribute("SQL query for test statement:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCTestStatement\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCTestStatement")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"></form>"));
		out.println(attribute("Minimum number of connections used by the pool:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCMinConnPool\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCMinConnPool")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"></form>"));
		out.println(attribute("Maximum number of connections used by the pool:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCMaxConnPool\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCMaxConnPool")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"></form>"));
		out.println(attribute("Duration between 2 clean-up of unused connection:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCSleepTimeOut\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCSleepTimeOut")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in milliseconds</form>"));
		out.println(attribute("Global duration of retry before throwing "
				      +"an exception if no connection is available in the pool:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCDeadLockPool\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCDeadLockPool")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in milliseconds</form>"));
		out.println(attribute("Unit duration of retry before throwing "
				      +"an exception if no connection is available in the pool:", 
				      "<form action=\"Database\" method=\"get\">"
				      +"<input type=\"hidden\" name=\"dsName\" value=\""+dsName+"\">"
				      +"<input type=\"text\" name=\"JDBCLoopWaitPool\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.datasource(dsName), 
									"JDBCLoopWaitPool")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in milliseconds</form>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(closeArea());
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(DATABASE);
    }
}
