/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class EJBServerServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("run")) {
		    Object[] param = new Object[0];
		    String[] signature = new String[0];
		    JonasManagementRepr.invoke(JonasObjectName.ejbServer(), 
					       req.getParameter("run"), param, signature);
		} else if (paramName.equals("Activated")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "Activated", 
						     new Boolean(req.getParameter("Activated")));
		} else if (paramName.equals("NumberOfMeasures")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "NumberOfMeasures", 
						     new Integer(req.getParameter("NumberOfMeasures")));
		} else if (paramName.equals("Range")) {
		    JonasManagementRepr.setAttribute(JonasObjectName.ejbServer(), 
						     "Range", 
						     new Integer(req.getParameter("Range")));
		}
	    }
	} catch (ManagementException e) {
	    exception = e;
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();

	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());

	int range = 60; int nbOfMeasures = 1;
	if (JonasManagementRepr.isRegistered(JonasObjectName.ejbServer())) {
	    range = ((Integer)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "Range")).intValue();
	    nbOfMeasures = ((Integer)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), 
								      "NumberOfMeasures")).intValue();
	    // insert javascript code for the monitoring image auto-refresh:
	    out.println("<script type=\"text/javascript\">");
	    out.println("<!--");
	    out.println("  function refresh(){");
	    out.println("    rfsh = new Date() ; rfsh = \"?\"+rfsh.getTime()");
	    out.println("    document.images[\"memory\"].src = \"MemoryGraph\"+rfsh");
	    out.println("    document.images[\"memory\"].width = \""+MemoryGraphServlet.getWidth(nbOfMeasures)+"\"");
	    out.println("    document.images[\"memory\"].height = \""+MemoryGraphServlet.getHeight()+"\"");
	    out.println("    setTimeout(\"refresh()\", "
			+Math.max(range*1000, 60000)+")");
	    out.println("  }");
	    out.println("//-->");
	    out.println("</script>");
	}

	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("EJBServer"));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	if (JonasManagementRepr.isRegistered(JonasObjectName.ejbServer())) {
	    out.println(title("JOnAS Server"));
	    out.println(openArea());
	    try {
		out.println(attribute("Server Name:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasName")+"</b>"));
		out.println(attribute("Implementation version:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasVersion")+"</b>"));
		out.println(attribute("Protocol:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "JonasORB")+"</b>"));
		out.println(attribute("&nbsp;", "&nbsp;"));
		out.println(attribute("<b><a href=\"EJBServer?run=runGC\">Run Garbage Collector</a></b>", "&nbsp;"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	    
	    out.println(title("Monitoring"));
	    out.println(openArea());
	    if (((Boolean)JonasManagementRepr.getAttribute(JonasObjectName.ejbServer(), "Activated")).booleanValue()) {
		out.println("  <tr>");
		out.println("    <td colspan=\"2\">");
		out.println("      <a href=\"EJBServer?Activated=false\">Desactivate monitoring</a>");
		out.println("    </td>");
		out.println("  </tr>");
		out.println("  <tr>");
		out.println("    <td align=\"center\" colspan=\"2\">");
		//out.println("      <img src=\"../img/memory.gif\" name=\"memory\" alt=\"Memory\">");
		out.println("      <img src=\"MemoryGraph?"+System.currentTimeMillis()
			    +"\" width=\""+MemoryGraphServlet.getWidth(nbOfMeasures)
			    +"\" height=\""+MemoryGraphServlet.getHeight()
			    +"\" name=\"memory\" alt=\"Memory\">");
		out.println("      <script type=\"text/javaScript\">");
		out.println("      <!--");
		out.println("        if( document.images )");
		//out.println("          refresh();");
		out.println("    setTimeout(\"refresh()\", "
			    +Math.max(range*1000, 60000)+")");
		out.println("      //-->");
		out.println("      </script>");
		out.println("    </td>");
		out.println("  </tr>");
		
		out.println(attribute("Number of measures:", 
				      "<form action=\"EJBServer\" method=\"get\">"
				      +"<input type=\"text\" name=\"NumberOfMeasures\" value=\""
				      +nbOfMeasures
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"></form>"));
		out.println(attribute("Periodicity:", 
				      "<form action=\"EJBServer\" method=\"get\">"
				      +"<input type=\"text\" name=\"Range\" value=\""
				      +range
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in seconds, (must greater or equals to 10)</form>"));
	    } else {
		out.println("<tr><td><a href=\"EJBServer?Activated=true\">Activate monitoring</a></td></tr>");
	    }

	    out.println(closeArea());

	} else {
	    out.println(title("Service not available."));
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }


    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(EJBSERVER);
    }
}
