/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public abstract class EJBServlet extends JonasManagementServlet {

    protected String index() {
	return index(CONTAINER);
    }

    protected String path(String fileName) {
	// Display a link to the 'ContainerService' view and a link to the 'Container' view:
	try {
	    return new String("  <tr>\n"
			      +"    <td>"
			      +"<a title=\"Return to the container service view\" href=\"ContainerService\">"
			      +"Container service</a>&nbsp;&gt;&nbsp;"
			      +"<a title=\"Return to the container view\" href=\"Container?fileName="
			      +fileName.replace(' ', '+')+"\">"
			      +JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName")
			      +"</a><p></td>\n"
			      +"  </tr>");
	} catch (Exception e) {
	    return error("Error while getting attribute:", e.getMessage());
	}
    }

    protected String ejbClass(String ejbType, 
			      String fileName, 
			      String ejbName) {
	try {
	    return attribute("EJB class:", 
			     "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.ejb(ejbType, 
											fileName, 
											ejbName), 
								    "EJBClass")+"</b>");
	} catch (Exception e) {
	    return error("Error while getting attribute:", e.getMessage(), 2);
	}
    }

    protected String ejbDataSource(String ejbType,
				   String fileName, 
				   String ejbName) {
	try {
	    java.util.Set allDataSources 
		= (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.ejb(ejbType, 
										      fileName, 
										      ejbName), 
								  "AllDataSourceName");
	    StringBuffer sb = new StringBuffer();
	    if (!allDataSources.isEmpty()) {
		sb.append(title("Data source"));
		sb.append(openArea());
		sb.append(subtitle(allDataSources.size()+" data source(s):\n", 2));
		for (Iterator i = allDataSources.iterator(); i.hasNext() ;) {
		    String dataSourceName = (String)i.next();
		    sb.append(attribute("<a title=\"View this data source\" "
					+"href=\"Database?dsName="
					+dataSourceName.replace(' ', '+')+"\">"
					+dataSourceName+"</a>", "&nbsp;"));
		}
		sb.append(closeArea());
	    }
	    return new String(sb);
	} catch (Exception e) {
	    return error("Error while getting attribute:", e.getMessage(), 2);
	}
    }

    protected String ejbJMSConnectionFactory(String ejbType,
					     String fileName, 
					     String ejbName) {
	try {
	    java.util.Set allJMSConnectionFactories 
		= (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.ejb(ejbType, 
										      fileName, 
										      ejbName), 
								  "AllJMSConnectionFactoryName");
	    StringBuffer sb = new StringBuffer();
	    if (!allJMSConnectionFactories.isEmpty()) {
		sb.append(title("JMS connection factories"));
		sb.append(openArea());
		sb.append(subtitle(allJMSConnectionFactories.size()+" JMS connection factory(ies):", 2));
		for (Iterator i = allJMSConnectionFactories.iterator(); i.hasNext() ;) {
		    String jmsConnectionFactoryName = (String)i.next();
		    sb.append(attribute("<a title=\"View this JMS connection factory\" "
					+"href=\"JMSConnectionFactory?name="
					+jmsConnectionFactoryName.replace(' ', '+')+"\">"
					+jmsConnectionFactoryName+"</a>", "&nbsp;"));
		}	    
		sb.append(closeArea());
	    }
	    return new String(sb);
	} catch (Exception e) {
	    return error("Error while getting attribute:", e.getMessage());
	}
    }

    protected String ejbJMSDestination(String ejbType,
				       String fileName, 
				       String ejbName) {
	try {
	    java.util.Set allJMSDestinations 
		= (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.ejb(ejbType, 
										      fileName, 
										      ejbName), 
								  "AllJMSDestinationName");
	    StringBuffer sb = new StringBuffer();
	    if (!allJMSDestinations.isEmpty()) {
		sb.append(title("JMS destinations"));
		sb.append(openArea());
		sb.append(subtitle(allJMSDestinations.size()+" JMS destination(s):", 2));
		for (Iterator i = allJMSDestinations.iterator(); i.hasNext() ;) {
		    String jmsDestinationName = (String)i.next();
		    sb.append(attribute("<a title=\"View this JMS destination\" "
					+"href=\"JMSDestination?name="
					+jmsDestinationName.replace(' ', '+')+"\">"
					+jmsDestinationName+"</a>", "&nbsp;"));
		}
		sb.append(closeArea());
	    }
	    return new String(sb);
	} catch (Exception e) {
	    return error("Error while getting attribute:", e.getMessage(), 2);
	}
    }
}
