/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class JMSConnectionFactoryServlet extends JMSServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	String name = null;
	// Retrieving request parameters
	for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
	    String paramName = (String)e.nextElement();
	    if (paramName.equals("name")) 
		name = req.getParameter("name");
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("JMSConnectionFactory?name="+name));
	out.println(openMain());
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	out.println(path());
	// Display the container service management information:
	out.println(title("JMS Connector Factory: "+name));
	try {
	    Object[] param = { name };
	    String[] signature = { "java.lang.String" };
	    java.util.Set allDep = (java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
									     "getJmsConnectionFactoryDependence", 
									     param, signature);
	    if (allDep.isEmpty()) {
		out.println(subtitle("Currently unused.", 2));
	    } else {
		out.println(subtitle("Currently used by:", 2));
		for (Iterator i = allDep.iterator(); i.hasNext() ;) {
		    ObjectName current = (ObjectName)i.next();
		    String type = (String)JonasObjectName.getType(current);
		    String fileName = (String)JonasManagementRepr.getAttribute(current, "EJBFileName");
		    String ejbName = (String)JonasManagementRepr.getAttribute(current, "EJBName");
		    if (type.equals("ejbbmp")) type  = "BMP";
		    else if (type.equals("ejbcmp")) type  = "CMP";
		    else if (type.equals("ejbsbf")) type  = "SBF";
		    else if (type.equals("ejbsbl")) type  = "SBL";
		    else if (type.equals("ejbmdb")) type  = "MDB";
		    out.println(attribute("<a title=\"View this EJB\" "
					  +"href=\""+type+"?fileName="+fileName.replace(' ', '+')
					  +"&#38;ejbName="+ejbName.replace(' ', '+')+"\">"
					  +ejbName+"</a>", 
					  "in container <a title=\"View this container\" "
					  +"href=\"Container?fileName="+fileName+"\">"
					  +JonasManagementRepr.getAttribute(JonasObjectName.container(fileName), "ContainerName")+"</a>"));
		}
	    }
	} catch (ManagementException e) {
	    out.println(error("Error while getting attribute:", e.getMessage()));
	} catch (javax.management.MalformedObjectNameException e) {
	    out.println(error("Incompatible name:", e.getMessage()));
	}
	out.println(closeArea());

	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }
}
