/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class JMSServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("removeFactory")) {
		    Object[] param = { req.getParameter("removeFactory") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.jmsService(), 
					       "removeJmsConnectionFactory", param, signature);

		} if (paramName.equals("removeDestination")) {
		    Object[] param = { req.getParameter("removeDestination") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.jmsService(), 
					       "removeJmsDestination", param, signature);

		} if ((paramName.equals("createDestination")) 
		      && (req.getParameter("type").equals("topic"))) {
		    Object[] param = { req.getParameter("createDestination") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.jmsService(), 
					       "createJmsTopicDestination", param, signature);

		} if ((paramName.equals("createDestination")) 
		      && (req.getParameter("type").equals("queue"))) {
		    Object[] param = { req.getParameter("createDestination") };
		    String[] signature = { "java.lang.String" };
		    JonasManagementRepr.invoke(JonasObjectName.jmsService(), 
					       "createJmsQueueDestination", param, signature);
		}
	    }
	} catch (Exception e) {
	    exception = e;
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("JMSService"));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	if (JonasManagementRepr.isRegistered(JonasObjectName.jmsService())) {
	    // Display the container service management information:
	    out.println(title("JMS service"));
	    out.println(openArea());
	    
	    // Topic connection factory
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
								      "CurrentNumberOfJmsTopicConnectionFactory")
				     +" JMS topic connection factory(ies):", 2));
		java.util.Set allJms = (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
										       "AllJmsTopicConnectionFactoryNames");
		for (Iterator i = allJms.iterator(); i.hasNext() ;) {
		    String current = (String)i.next();
		    String action;
		    Object[] param = { current };
		    String[] signature = { "java.lang.String" };
		    if (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
								   "getJmsConnectionFactoryDependence", 
								   param, signature)).isEmpty()) 
			action = "<a title=\"Remove this connection factory\" "
			    +"href=\"JMSService?removeFactory="
			    +current.replace(' ', '+')
			    +"\"><b>Remove</b></a>";
		    else 
			action = "&nbsp;";
		    out.println(attribute("<a title=\"View this JMS topic connection factory\" "
					  +"href=\"JMSConnectionFactory?name="
					  +current.replace(' ', '+')+"\">"
					  +current+"</a>", 
					  action));
		}
		String defaultCF = (String)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), "DefaultTopicConnectionFactoryName");
		out.println(attribute("<a title=\"View this JMS topic connection factory\" "
				      +"href=\"JMSConnectionFactory?name="
				      +defaultCF.replace(' ', '+')+"\">"
				      +defaultCF+"</a>", 
				      "(default)"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }

	    // Queue connection factory
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
								      "CurrentNumberOfJmsQueueConnectionFactory")
				     +" JMS queue connection factory(ies):", 2));
		java.util.Set allJms = (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
									 "AllJmsQueueConnectionFactoryNames");
		for (Iterator i = allJms.iterator(); i.hasNext() ;) {
		    String current = (String)i.next();
		    String action;
		    Object[] param = { current };
		    String[] signature = { "java.lang.String" };
		    if (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
								   "getJmsConnectionFactoryDependence", 
								   param, signature)).isEmpty()) 
			action = "<a title=\"Remove this connection factory\" "
			    +"href=\"JMSService?removeFactory="
			    +current.replace(' ', '+')
			    +"\"><b>Remove</b></a>";
		    else 
			action = "&nbsp;";
		    out.println(attribute("<a title=\"View this JMS queue connection factory\" "
					  +"href=\"JMSConnectionFactory?name="
					  +current.replace(' ', '+')+"\">"
					  +current+"</a>", 
					  action));
		}
		String defaultCF = (String)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
								     "DefaultQueueConnectionFactoryName");
		out.println(attribute("<a title=\"View this JMS queue connection factory\" "
				      +"href=\"JMSConnectionFactory?name="
				      +defaultCF.replace(' ', '+')+"\">"
				      +defaultCF+"</a>", 
				      "(default)"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    
	    // Topic destination
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
								      "CurrentNumberOfJmsTopicDestination")
				     +" JMS topic destination(s):", 2));
		java.util.Set allJms = (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
									 "AllJmsTopicDestinationNames");
		for (Iterator i = allJms.iterator(); i.hasNext() ;) {
		    String current = (String)i.next();
		    String action;
		    Object[] param = { current };
		    String[] signature = { "java.lang.String" };
		    if (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
								   "getJmsDestinationDependence", 
								   param, signature)).isEmpty()) 
			action = "<a title=\"Remove this destination\" "
			    +"href=\"JMSService?removeDestination="
			    +current.replace(' ', '+')
			    +"\"><b>Remove</b></a>";
		    else 
			action = "&nbsp;";
		    out.println(attribute("<a title=\"View this JMS topic destination\" "
					  +"href=\"JMSDestination?name="
					  +current.replace(' ', '+')+"\">"
					  +current+"</a>", 
					  action));
		}
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    
	    // Queue destination
	    try {
		out.println(subtitle(JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
								      "CurrentNumberOfJmsQueueDestination")
				     +" JMS queue destination(s):", 2));
		java.util.Set allJms = (java.util.Set)JonasManagementRepr.getAttribute(JonasObjectName.jmsService(), 
									 "AllJmsQueueDestinationNames");
		for (Iterator i = allJms.iterator(); i.hasNext() ;) {
		    String current = (String)i.next();
		    String action;
		    Object[] param = { current };
		    String[] signature = { "java.lang.String" };
		    if (((java.util.Set)JonasManagementRepr.invoke(JonasObjectName.ejbService(), 
								   "getJmsDestinationDependence", 
								   param, signature)).isEmpty()) 
			action = "<a title=\"Remove this destination\" "
			    +"href=\"JMSService?removeDestination="
			    +current.replace(' ', '+')
			    +"\"><b>Remove</b></a>";
		    else 
			action = "&nbsp;";
		    out.println(attribute("<a title=\"View this JMS queue destination\" "
					  +"href=\"JMSDestination?name="
					  +current.replace(' ', '+')+"\">"
					  +current+"</a>", 
					  action));
		}
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	    
	    out.println(title("Connection factory"));
	    out.println(openArea());
	    out.println("  <tr>");
	    out.println("    <td>");
	    out.println("      <form method=\"get\" action=\"JMSService\">");
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	    out.println(attribute("JNDI name:", 
				  "<input type=\"text\" name=\"createConnectionFactory\"><input type=\"submit\" value=\"Create\">"));
	    out.println("        <tr>");
	    out.println("          <td align=\"center\" colspan=\"2\">");
	    out.println("<input type=\"radio\" name=\"type\" value=\"topic\" checked>"
			+"Topic connection factory ");
	    out.println("<input type=\"radio\" name=\"type\" value=\"queue\">"
			+"Queue connection factory ");
	    out.println("          </td>");
	    out.println("        </tr>");
	    out.println("</table>");
	    out.println("      </form>");
	    out.println("    </td>");
	    out.println("  </tr>");
	    out.println(closeArea());
	    
	    out.println(title("Destination"));
	    out.println(openArea());
	    out.println("  <tr>");
	    out.println("    <td>");
	    out.println("      <form method=\"get\" action=\"JMSService\">");
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	    out.println(attribute("JNDI name:", 
				  "<input type=\"text\" name=\"createDestination\"><input type=\"submit\" value=\"Create\">"));
	    out.println("        <tr>");
	    out.println("          <td align=\"center\" colspan=\"2\">");
	    out.println("<input type=\"radio\" name=\"type\" value=\"topic\" checked>"
			+"Topic destination ");
	    out.println("<input type=\"radio\" name=\"type\" value=\"queue\">"
			+"Queue destination ");
	    out.println("          </td>");
	    out.println("        </tr>");
	    out.println("</table>");
	    out.println("      </form>");
	    out.println("    </td>");
	    out.println("  </tr>");
	    out.println(closeArea());
	} else {
	    out.println(title("Service not available."));
	}

	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(JMS);
    }
}
