/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// servlet imports
import javax.servlet.http.HttpServlet;

// naming imports
import java.util.StringTokenizer;

// date imports
import java.text.DateFormat;
import java.util.Date;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

/**
 * All management servlet must extends this class.
 */
public abstract class JonasManagementServlet extends HttpServlet {

    protected static final int EJBSERVER = 0;
    protected static final int CONTAINER = 1;
    protected static final int DATABASE = 2;
    protected static final int RESOURCE = 3;
    protected static final int TRANSACTION = 4;
    protected static final int TRACE = 5;
    protected static final int JMS = 6;
    protected static final int SECURITY = 7;
    protected static final int MANAGEMENT = 8;
    protected static final int ALLMBEANS = 9; // this index MUST be the last one

    protected static final String BG_COLOR = "#dae6e6";
    protected static final String BORDER_COLOR = "#8080ff";
    protected static final String TD_COLOR = "#0000C0";
    protected static final String LIGHT_TD_COLOR = "#8080ff";
    protected static final String LINK_COLOR = "#e0e0e0";
    protected static final String VLINK_COLOR = "#e0e0e0";
    protected static final String ALINK_COLOR = "#ffffff";

    /**
     * @return The document doctype tag: html 4.01 transitional.
     */
    protected String doctype() {
	return new String("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
    }

    /**
     * @return The open html tag.
     */
    protected String openHtml() {
	return new String("<html>");
    }

    /**
     * @return The close html tag.
     */
    protected String closeHtml() {
	return new String("</html>");
    }

    /**
     * @return The headers tag of the document.
     */
    protected String head() {
	return new String("<head>\n"
			  +"  <title>Jadmin</title>\n"
			  +"  <link rel=\"stylesheet\" href=\"../style.css\" type=\"text/css\">\n"
			  +"</head>\n");
    }

    /**
     * @return The body tag and the beginning of the document.
     */
    protected String openBody() {
	return new String("<body bgcolor=\""+BG_COLOR+"\" text=\"#000000\" "
			  +"link=\""+LINK_COLOR+"\" vlink=\""+VLINK_COLOR+"\" alink=\""+ALINK_COLOR+"\">\n"
			  +"<center>");
    }

    /**
     * @return A string which closes all tags openned in the openBody method.
     */
    protected String closeBody() {
	return new String("</center>\n"
			  +"</body>");
    }

    /**
     * @return A link bar.
     */
    protected String links() {
	return new String("<p align=\"center\">\n"
			  +"<a class=\"linkbar\" href=\"http://www.evidian.com/jonas/\">JOnAS Home Page</a>"
			  +" - <a class=\"linkbar\" href=\"http://www.objectweb.org/\">Objectweb</a>\n"
			  +"</p>\n");
    }

    /**
     * @return the table open tag with cellpadding=2 cellspacing=1 width=750.
     */
    protected String openTable() {
	return new String("<table align=\"center\" bgcolor=\""+BG_COLOR+"\" "
			  +"border=\"0\" cellspacing=\"1\" cellpadding=\"1\" width=\"750\">");
    }

    /**
     * @return the table close tag.
     */
    protected String closeTable() {
	return new String("</table>");
    }

    /**
     * @param index The selected thumb index (EJBSERVER | CONTAINER | ...).
     * @return A string for the thumb index, includes the tr tag.
     */
    protected String index(int index) {
	StringBuffer sb = new StringBuffer("  <!-- The thumb index -->\n");
	sb.append("  <tr>\n");
	if (index == EJBSERVER) sb.append(tdBlue("EJB&nbsp;Server"));
	else sb.append(tdLink("EJBServer", "EJB&nbsp;Server"));
	if (index == CONTAINER) sb.append(tdBlue("Container"));
	else sb.append(tdLink("ContainerService", "Container"));
	if (index == DATABASE) sb.append(tdBlue("Database"));
	else sb.append(tdLink("DatabaseService", "Database"));
	if (index == RESOURCE) sb.append(tdBlue("Resource"));
	else sb.append(tdLink("ResourceService", "Resource"));
	if (index == TRANSACTION) sb.append(tdBlue("Transaction"));
	else sb.append(tdLink("TransactionService", "Transaction"));
	if (index == TRACE) sb.append(tdBlue("Trace"));
	else sb.append(tdLink("TraceService", "Trace"));
	if (index == JMS) sb.append(tdBlue("JMS"));
	else sb.append(tdLink("JMSService", "JMS"));
	if (index == SECURITY) sb.append(tdBlue("Security"));
	else sb.append(tdLink("SecurityService", "Security"));
	if (index == MANAGEMENT) sb.append(tdBlue("Management"));
	else sb.append(tdLink("ManagementService", "Management"));
	if (index == ALLMBEANS) sb.append(tdBlue("All MBeans"));
	else sb.append(tdLink("AllMBeans", "All MBeans"));
	sb.append("  </tr>\n");
	return sb.toString();
    }

    /**
     * @param link The link for the reload link.
     * @return A string for the banner, includes the tr tag.
     */
    protected String banner(String link) {
	return new String("  <tr>\n"
			  +"    <td bgcolor=\""+TD_COLOR+"\" colspan=\""+(ALLMBEANS+1)+"\">\n"
			  +"      <table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"100%\">\n"
			  +"        <tr>\n"
			  +"          <td align=\"left\">&nbsp;<a title=\"Reload this page\" href=\""
			  +link+"\">Refresh</a></td>\n"
			  +"          <td align=\"right\">"
			  +DateFormat.getInstance().format(new Date())+"&nbsp;</td>\n"
			  +"        </tr>\n"
			  +"      </table>\n"
			  +"    </td>\n"
			  +"  </tr>\n");
    }

    /**
     * @return Open tags for the main cell of the table (i.e.: tr and td tags).
     */
    protected String openMain() {
	return new String("  <!-- The main TD -->\n"
			  +"  <tr>\n"
			  +"    <td bgcolor=\""+TD_COLOR+"\" colspan=\""+(ALLMBEANS+1)+"\">\n"
			  +"      <table border=\"0\" cellspacing=\"0\" cellpadding=\"10\" width=\"100%\">\n"
			  +"        <tr>\n"
			  +"          <td>");
    }

    /**
     * @return Close tags for the main cell of the table (i.e.: td and tr tags).
     */
    protected String closeMain() {
	return new String("          </td>\n"
			  +"        </tr>\n"
			  +"      </table>\n"
			  +"    </td>\n"
			  +"  </tr>");
/*	return new String("    </td>\n"
			  +"  </tr>");*/
    }

    /**
     * @return The row (with tr tags) containing a table header cell (th) in order to display a title.
     */
    protected String title(String title) {
	return title(title, 1);
    }

    /**
     * @param colspan The cell colspan.
     * @return The a row (with tr tags) containing a table header cell (th) in order to display a title.
     */
    protected String title(String title, int colspan) {
	return new String("  <tr>\n"
			  +"    <th colspan=\""+colspan+"\">"+title+"</th>\n"
			  +"  </tr>");
    }

    /**
     * @return The row (with tr tags) containing a table cell (td) in order to display a subtitle.
     */
    protected String subtitle(String title, int colspan) {
	return new String("  <tr>\n"
			  +"    <td colspan=\""+colspan+"\"><b>"+title+"</b></td>\n"
			  +"  </tr>");
    }

    /**
     * @return Open tags for an area, i.e. with a border
     */
    protected String openArea() {
	return new String("<tr><td><table bgcolor=\""+BORDER_COLOR+"\" "
			  +"border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">\n"
			  +"  <tr>\n"
			  +"    <td>\n"
			  +"      <table bgcolor=\""+TD_COLOR+"\" "
			  +"border=\"0\" cellspacing=\"0\" cellpadding=\"3\" width=\"100%\">");
    }

    /**
     * @return Close tags for an area
     */
    protected String closeArea() {
	return new String("      </table>\n"
			  +"    </td>\n"
			  +"  </tr>\n"
			  +"</table><br></td></tr>\n");
    }

    /**
     * @param name The attribute name.
     * @param value The attribute value.
     * @return The a row (with tr tags) containing two cells: the attribute name and its value.
     */
    protected String attribute(String name, Object value) {
	return new String("<tr>\n"
			  +"  <td width=\"240\" align=\"right\" nowrap>"+name+"</td>\n"
			  +"  <td width=\"750\" align=\"left\">"+value+"</td>\n"
			  +"</tr>");
    }

    /**
     * @return The foot of the table (tr tag containing a cell with the 'valid HTML 4.01' icon.
     */
    protected String foot() {
	return new String("<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" width=\"750\">"
			  +"  <tr>\n"
			  +"    <td>\n"
			  +"      <a href=\"http://validator.w3.org/check/referer\">\n"
			  +"        <img border=\"0\" src=\"../img/valid-html401.gif\" alt=\"Valid HTML 4.01!\" "
			  +"height=\"31\" width=\"88\">\n"
			  +"      </a>\n"
			  +"    </td>\n"
			  +"  </tr>\n"
			  +"</table>");
    }
    
    /**
     * @param label The label for this thumb index.
     * @return the cell (td tag) displaying the specified selected label.
     */
    protected String tdBlue(String label) {
	return new String("<td align=\"center\" bgcolor=\""+TD_COLOR+"\">"+label+"</td>\n");
    }

    /**
     * @param link The link url.
     * @param label The label for this thumb index.
     * @return the cell (td tag) displaying the specified clicable label.
     */
    protected String tdLink(String link, String label) {
	return new String("<td align=\"center\" bgcolor=\""+LIGHT_TD_COLOR+"\">"+"<a href=\""+link+"\">"+label+"</a></td>\n");
    }

    /**
     * @param msg The error title.
     * @param msg The error message.
     * @return a string displaying the error msg in tr tags.
     */
    protected String error(String title, String msg) {
	return error(title, msg, 1);
    }

    /**
     * @param msg The error title.
     * @param msg The error message.
     * @param colspan The cell colspan.
     * @return a string displaying the error msg in tr tags.
     */
    protected String error(String title, String msg, int colspan) {
	return new String("  <tr>\n"
			  +"    <td colspan=\""+colspan+"\"><b>"+title+"</b><br>"+msg+"</td>\n"
			  +"  </tr>");
    }

    /**
     * @param msg The error title.
     * @param msg The error message.
     * @return a string displaying the error msg in tr tags.
     */
    protected String alert(String msg) {
	StringBuffer sb = new StringBuffer("<script>alert(\"");
	StringTokenizer st = new StringTokenizer(msg, "\n");
	while (st.hasMoreTokens()) 
	    sb.append(st.nextToken()+"\\n");
	sb.append("\")</script>");
	return new String(sb);
    }
}
