/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class LoadDataSourceServlet extends JonasManagementServlet {
	    
    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	
	Properties p = new Properties();
	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("loadFromFile")) {
		    Object[] param = { req.getParameter("loadFromFile") };
		    String[] signature = { "java.lang.String" };
		    p = (Properties)JonasManagementRepr.invoke(JonasObjectName.databaseService(), 
							       "getDataSourcePropertiesFile", 
							       param, signature);
		}
	    }
	} catch (Exception e) {
	    exception = e;
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("LoadDataSource"));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	out.println("  <tr>");
	out.println("    <td>"
		    +"<a title=\"Return to the database service view\" href=\"DatabaseService\">"
		    +"Database service</a><p></td>");
	out.println("  </tr>");
	// Display the datasource load and configuration screen:
	out.println(title("Load datasource configuration"));
	out.println(openArea());
	out.println("      <form method=\"post\" action=\"DatabaseService\">");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.name</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.name\" value=\""
		    +p.getProperty("datasource.name", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource name</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.description</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.description\" value=\""
		    +p.getProperty("datasource.description", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource description</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.url</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.url\" value=\""
		    +p.getProperty("datasource.url", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource URL</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.classname</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.classname\" value=\""
		    +p.getProperty("datasource.classname", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource driver class name</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.username</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.username\" value=\""
		    +p.getProperty("datasource.username", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource user name</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.password</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"password\" name=\"datasource.password\" value=\""
		    +p.getProperty("datasource.password", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">DataSource password</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.factory</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.factory\" value=\""
		    +p.getProperty("datasource.factory", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Name of the class implementing the XADataSource</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.xadataname</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.xadataname\" value=\""
		    +p.getProperty("datasource.xadataname", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">JNDI name use to bind the XADataSource</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.mincon</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.mincon\" value=\""
		    +p.getProperty("datasource.mincon", "5")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Minimal number of connection for the DataSource</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.maxcon</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.maxcon\" value=\""
		    +p.getProperty("datasource.maxcon", "10")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Maximal number of connection for the DataSource</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">datasource.isolationlevel</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"datasource.isolationlevel\" value=\""
		    +p.getProperty("datasource.isolationlevel", "")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Isolation level for the DataSource</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.connmaxage</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.connmaxage\" value=\""
		    +p.getProperty("jdbc.connmaxage", "30")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Maximal age for JDBC connection (in minute)</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.connchecklevel</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.connchecklevel\" value=\""
		    +p.getProperty("jdbc.connchecklevel", "1")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">JDBC connection checking level:<br>"
		    +"0 = no special checking<br>"
		    +"1 = check before reuse if physical connection is still open<br>"
		    +"2 = try every connection before reuse</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.connteststmt</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.connteststmt\" value=\""
		    +p.getProperty("jdbc.connteststmt", "SELECT 1")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Statement test</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.minconpool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.minconpool\" value=\""
		    +p.getProperty("datasource.minconpool", "10")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Minimum number of XAConnection used by the pool</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.maxconpool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.maxconpool\" value=\""
		    +p.getProperty("datasource.maxconpool", "20")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Maximum number of XAConnection used by the pool</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.sleeptimepool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.sleeptimepool\" value=\""
		    +p.getProperty("datasource.sleeptimepool", "300000")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Clean-up interval for unused connection (in millisecond)</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.gcpool</td>");
	out.print("          <td width=\"22%\">"
		  +"<input type=\"radio\" name=\"jdbc.gcpool\" value=\"true\"");
	if (p.getProperty("jdbc.gcpool", "").equals("true")) out.print(" checked");
	out.print(">true <input type=\"radio\" name=\"jdbc.gcpool\" value=\"false\"");
	if ( ! p.getProperty("jdbc.gcpool", "").equals("true")) out.print(" checked");
	out.println(">false</td>");
	out.println("          <td align=\"left\" width=\"48%\">If true, force the GC to be launched when clean-up</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.generationpool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.generationpool\" value=\""
		    +p.getProperty("datasource.generationpool", "1")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">&nbsp;</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.deadlockpool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.deadlockpool\" value=\""
		    +p.getProperty("datasource.deadlockpool", "300000")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Deadlock pool is the global time to re-try before throwing an exception if no connection is available in the pool (in millisecond)</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td align=\"right\" width=\"30%\">jdbc.loopwaitpool</td>");
	out.println("          <td width=\"22%\">"
		    +"<input type=\"text\" name=\"jdbc.loopwaitpool\" value=\""
		    +p.getProperty("datasource.loopwaitpool", "10000")+"\"></td>");
	out.println("          <td align=\"left\" width=\"48%\">Loopwait pool is the unit time to re-try if no connection is available in the pool (in millisecond)</td>");
	out.println("        </tr>");

	out.println("        <tr>");
	out.println("          <td colspan=\"3\" align=\"center\">");
	out.println("            <input type=\"submit\" value=\"Load\">");
	out.println("            <input type=\"reset\" value=\"Reset\">");
	out.println("          </td>");
	out.println("        </tr>");

	out.println("      </form>");
	out.println(closeArea());
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(DATABASE);
    }
}
