/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Set;

// naming imports
import java.util.Properties;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.rmi.PortableRemoteObject;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class ManagementServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	boolean list = false;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("providerUrl")) {
		    ConnectorFactory.setJonasNamingServiceURL(req.getParameter("providerUrl"));
		} else if (paramName.equals("ejbServer")) {
		    ConnectorFactory.setCurrentRMIConnectorName(req.getParameter("ejbServer"));
		} else if (paramName.equals("stop")) {
		    // We save the current RMI connector name in order to restore it later:
		    String rmiConnectorNameBackup = ConnectorFactory.getCurrentRMIConnectorName();
		    // Select the server to stop:
		    ConnectorFactory.setCurrentRMIConnectorName(req.getParameter("stop"));
		    try {
			// Stop the server:
			JonasManagementRepr.invoke(JonasObjectName.ejbServer(), "stop", new Object[0], new String[0]);
		    } catch (ManagementException me) {
			// a java.rmi.UnmarshalException is thrown because the server is stopped!
			if ( (!(me.getCause() instanceof java.rmi.UnmarshalException))
			     && (!(me.getCause() instanceof java.rmi.MarshalException)) )
			    throw me;
		    }
		    // If the saved RMI connector name is the stopped server, then currentRMIConnectorName
		    // is set to null, if it is not the same, we restore the saved value:
		    if (rmiConnectorNameBackup.equals(req.getParameter("stop"))) 
			ConnectorFactory.resetCurrentRMIConnectorName();
		    else 
			ConnectorFactory.setCurrentRMIConnectorName(rmiConnectorNameBackup);
		} else if (paramName.equals("list") && req.getParameter("list").equals("true")) 
		    list = true;
	    }
	} catch (Exception e) {
	    exception = e;
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("ManagementService?list="+list));
	out.println(openMain());

	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	out.println(title("Registry URL"));
	out.println(openArea());
	out.println("        <tr>");
	out.println("          <td>");
	out.println("            <form name=\"registryURL\" method=\"get\" action=\"ManagementService\">");
	out.println("<input type=\"text\" name=\"providerUrl\" size=\"50\" value=\""
		    +ConnectorFactory.getJonasNamingServiceURL()+"\">");
	out.println("<input type=\"submit\" value=\"Submit\">");
	out.println("<input type=\"reset\" value=\"Reset\">");
	out.println("<input type=\"hidden\" name=\"list\" value=\""+list+"\">");
	out.println("            </form>");
	if (list) {
	    out.println("            <a href=\"ManagementService\">Hide all names bound in this registry</a>");
	    try {
		Context ctx = ConnectorFactory.getContext();
		out.println("<ul type=\"square\">");
		boolean isEmpty = true;
		for (NamingEnumeration e = ctx.list(""); e.hasMoreElements(); ) {
		    out.println("  <li>"+((NameClassPair)e.nextElement()).getName());
		    isEmpty = false;
		}
		if (isEmpty) out.println("  <li><i>none</i>");
		out.println("</ul>");
	    } catch (javax.naming.NamingException ne) {
		out.println("Cannot enumerates the names bound in the named context: '"
			    +ConnectorFactory.getJonasNamingServiceURL()
			    +"' (registry probably not launched).");
	    }
	} else {
	    out.println("            <a href=\"ManagementService?list=true\">Show all names bound in this registry</a>");
	}
	out.println("          </td>");
	out.println("        </tr>");
	out.println(closeArea());

	out.println(title("JOnAS server(s) available in this registry"));
	out.println(openArea());
	out.println("        <tr>");
	out.println("          <td>");
	try {
	    Set allRMIC = ConnectorFactory.getRMIConnectorsNames();
	    if (allRMIC.isEmpty()) {
		out.println("No JOnAS server available.");
	    } else {
		out.println("            <form name=\"ejbServer\" method=\"get\" action=\"ManagementService\">");
		for (Iterator i = allRMIC.iterator() ; i.hasNext() ;) {
		    String current = (String)i.next();
		    out.print("<input type=\"radio\" name=\"ejbServer\" onClick=\"this.form.submit();\"");
		    if (current.equals(ConnectorFactory.getCurrentRMIConnectorName())) 
			out.print("checked ");
		    out.println("value=\""+current+"\">"+current.substring(current.indexOf("_")+1)
				+" <a href=\"ManagementService?list="+list+"&#38;stop="+current
				+"\"><b>Stop this JOnAS server.</b></a><br>");
		}
		out.println("<p align=\"center\">");
		//		out.println("<input type=\"submit\" value=\"Submit\">");
		//out.println("<input type=\"reset\" value=\"Reset\">");
		out.println("<input type=\"hidden\" name=\"list\" value=\""+list+"\">");
		out.println("</p>");
		out.println("            </form>");
	    }
	} catch (javax.naming.NamingException ne) {
	    out.println(ne.getMessage());
	}
	out.println("          </td>");
	out.println("        </tr>");
	out.println(closeArea());

	if (JonasManagementRepr.isRegistered(JonasObjectName.jmxService())) {
	    out.println(title("JMX Server running"));
	    out.println(openArea());
	    try {
		out.println(attribute("MBean server ID:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
								       "MBeanServerId")+"</b>"));
		out.println(attribute("Specification:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "SpecificationName")+"</b>"));
		out.println(attribute("Specification version:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "SpecificationVersion")+"</b>"));
		out.println(attribute("Specification vendor:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "SpecificationVendor")+"</b>"));
		out.println(attribute("Implementation:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "ImplementationName")+"</b>"));
		out.println(attribute("Implementation version:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "ImplementationVersion")+"</b>"));
		out.println(attribute("Implementation vendor:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.jmxService(), 
									     "ImplementationVendor")+"</b>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	} else {
	    out.println(title("Service not available."));
	}
	
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(MANAGEMENT);
    }
}
