/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id:
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Properties;
import java.util.StringTokenizer;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

//JMX imports
import javax.management.*;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class ResourceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	String jndiName = null;
	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("JndiName")) 
		    jndiName = req.getParameter("JndiName");
	    }
	} catch (Exception e) {
	    exception = e;
	}
	
	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("Resource?JndiName="+jndiName));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	// Display a link to the 'ResourceService' view:
	out.println("  <tr>");
	out.println("    <td>"
		    +"<a title=\"Return to the resource service view\" href=\"ResourceService\">"
		    +"Resource service</a><p></td>");
	out.println("  </tr>");
	
	if (jndiName == null) {
	    out.println(title("No resource selected"));
	} else {
	    // Display the datasource properties:
	    out.println(title(jndiName+": properties"));
	    out.println(openArea());
	    try {
		Properties p = (Properties)JonasManagementRepr.getAttribute(JonasObjectName.resourceAdapter(jndiName), 
									    "Properties");
		for (Enumeration e = p.propertyNames() ; e.hasMoreElements() ;) {
		    String key = (String)e.nextElement();
		    out.println(attribute(key+":", "<b>"+p.getProperty(key)+"</b>"));
		}
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage()));
	    } catch (javax.management.MalformedObjectNameException e) {
		out.println(error("Incompatible name:", e.getMessage()));
	    }
	    out.println(closeArea());
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(RESOURCE);
    }
}
