/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class SecurityServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("SecurityService"));
	out.println(openMain());
	
	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	if (JonasManagementRepr.isRegistered(JonasObjectName.securityService())) {
	    out.println(title("Security service"));
	    out.println(openArea());
	    out.println(attribute("<b>Principal</b>", "<b>Role</b>"));
	    try {
		Hashtable mapping = (Hashtable)JonasManagementRepr.getAttribute(JonasObjectName.securityService(), 
										"PrincipalDeclarativeRoleMapping");
		if ((mapping != null) && (!mapping.isEmpty())) {
		    for (Enumeration e = mapping.keys() ; e.hasMoreElements() ; ) {
			String currentKey = (String)e.nextElement();
			String[] currentRole = (String[])mapping.get(currentKey);
			// The currentRole array cannot be empty, so no exception should be thrown...
			String roles = currentRole[0];
			for (int i=1 ; i<currentRole.length ; i++) 
			    roles = roles+", "+currentRole[i];
			out.println(attribute(currentKey, roles));
		    }
		} else {
		    out.println("<tr><td><i>No mapping between principal and role.</i></td></tr>");
		}
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	} else {
	    out.println(title("Service not available."));
	}
	out.println("</table>");
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(SECURITY);
    }
}
