/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// JOnAS JMX imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class TraceServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	try {
	    // Retrieving request parameters
	    if ((req.getParameter("header") != null)) {
		JonasManagementRepr.setAttribute(JonasObjectName.traceService(), 
						 "CurrentTraceHeader", 
						 new Boolean(req.getParameter("header")));

	    } else if ((req.getParameter("level") != null)) {
		Boolean[] newTraceLevel = new Boolean[32];
		for (int i=0 ; i<newTraceLevel.length ; i++) newTraceLevel[i] = new Boolean(false);
		
		for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		    String paramName = (String)e.nextElement();
		    try {
			// Try to parse the paramName string as an int...
			Integer.parseInt(paramName);
			if ((req.getParameter(paramName).equals("on"))) {
			    newTraceLevel[Integer.parseInt(paramName)] = new Boolean(true);
			}
		    } catch (NumberFormatException nfe) {
			// A NumberFormatException means that the 
			// current parameter (paramName) is not an Integer
		    }
		}
		JonasManagementRepr.setAttribute(JonasObjectName.traceService(), 
						 "CurrentTraceLevel", 
						 newTraceLevel);

	    } else if ((req.getParameter("output") != null)) {
		JonasManagementRepr.setAttribute(JonasObjectName.traceService(), 
						 "CurrentTraceOut", 
						 req.getParameter("output"));
	    }
	} catch (Exception e) {
	    exception = e;
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();
	
	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	
	// insert javascript code for the 'select all' and the 'deselect all' buttons:
	out.println();
	out.println("<script type=\"text/javascript\">");
	out.println("<!--");
	out.println("function ToggleAll(checked) {");
	out.println("  len = document.tracelevel.elements.length;");
	out.println("  var i = 0;");
	out.println("  for(i = 0; i < len; i++) {");
	out.println("    document.tracelevel.elements[i].checked = checked;");
	out.println("  }");
	out.println("}");
	out.println("//-->");
	out.println("</script>");
	out.println();
	
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("TraceService"));
	out.println(openMain());
	
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	
	if (JonasManagementRepr.isRegistered(JonasObjectName.traceService())) {
	    out.println(title("Trace level"));
	    out.println(openArea());
	    out.println("  <tr>");
	    out.println("    <td>");
	    out.println("      <form name=\"tracelevel\" method=\"get\" action=\"TraceService\">");
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	    // The select all / deselect all buttons:
	    out.println("        <tr>");
	    out.println("          <td colspan=\"6\" align=\"center\">");	
	    out.println("<a href=\"javascript:ToggleAll(true)\" "
			+"onMouseOver=\"status='select all'; return true;\" "
			+"onMouseOut=\"status='';\">"
			+"select all</a> |");
	    out.println("<a href=\"javascript:ToggleAll(false)\" "
			+"onMouseOver=\"status='deselect all'; return true;\" "
			+"onMouseOut=\"status='';\">"
			+"deselect all</a>");
	    out.println("          </td>");
	    out.println("        </tr>");
	    
	    try {
		String[] levelName = (String[])JonasManagementRepr.getAttribute(JonasObjectName.traceService(), 
										"TraceLevelName");
		Boolean[] level = (Boolean[])JonasManagementRepr.getAttribute(JonasObjectName.traceService(), 
									      "CurrentTraceLevel");
	    
		// Write data in columns order:
		//    0 3
		//    1 4
		//    2 5
		if (levelName != null) {
		    for (int i=0 ; i<(levelName.length/2) ; i++) {
			out.println("        <tr>");
			for (int j=0 ; j<levelName.length ; j+=(levelName.length/2)) {
			    out.println("          <td width=\"240\" align=\"right\">"+levelName[i+j]+"</td>");
			    out.println("          <td align=\"left\">(Trace.DB_"+(i+j)+")</td>");
			    out.print("          <td>");
			    if (level[i+j].booleanValue())
				out.print("<input type=\"checkbox\" checked name=\""+(i+j)+"\">");
			    else
				out.print("<input type=\"checkbox\" name=\""+(i+j)+"\">");
			    out.println("</td>");
			}
			out.println("        </tr>");
		    }
		}
	    
		// The submit/reset buttons:
		out.println("        <tr>");
		out.println("          <td colspan=\"6\" align=\"center\">");
		out.println("            <input type=\"hidden\" name=\"level\" value=\"change\">");
		out.println("            <input type=\"submit\" value=\"Submit\">");
		out.println("            <input type=\"reset\" value=\"Reset\">");
		out.println("          </td>");
		out.println("        </tr>");
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println("</table>");
	    out.println("      </form>");
	    out.println("    </td>");
	    out.println("  </tr>");
	    out.println(closeArea());
	    
	    out.println(title("Trace header"));
	    out.println(openArea());
	    out.println("  <tr>");
	    out.println("    <td>");
	    out.println("      <form method=\"get\" action=\"TraceService\">");
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	    try {
		boolean traceHeader = ((Boolean)JonasManagementRepr.getAttribute(JonasObjectName.traceService(), 
										 "CurrentTraceHeader")
				       ).booleanValue();
		String value;
		if (traceHeader) 
		    value = "<input type=\"radio\" checked name=\"header\" value=\"true\"> Yes<br>"
			+"<input type=\"radio\" name=\"header\" value=\"false\"> No";
		else 
		    value = "<input type=\"radio\" name=\"header\" value=\"true\"> Yes<br>"
			+"<input type=\"radio\" checked name=\"header\" value=\"false\"> No";
		out.println(attribute("Show trace header:", value));
		
		// The submit/reset buttons:
		out.println("        <tr>");
		out.println("          <td colspan=\"2\" align=\"center\">");
		out.println("            <input type=\"submit\" value=\"Submit\">");
		out.println("            <input type=\"reset\" value=\"Reset\">");
		out.println("          </td>");
		out.println("        </tr>");
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println("</table>");
	    out.println("      </form>");
	    out.println("    </td>");
	    out.println("  </tr>");
	    out.println(closeArea());

	    out.println(title("Trace output"));
	    out.println(openArea());
	    out.println("  <tr>");
	    out.println("    <td>");
	    out.println("      <form method=\"get\" action=\"TraceService\">");
	    out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");
	    try {
		out.println(attribute("Trace standard output:", 
				      "<input type=\"text\" name=\"output\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.traceService(), 
									"CurrentTraceOut")
				      +"\">"));
		// The submit/reset buttons:
		out.println("        <tr>");
		out.println("          <td colspan=\"2\" align=\"center\">");
		out.println("            <input type=\"submit\" value=\"Submit\">");
		out.println("            <input type=\"reset\" value=\"Reset\">");
		out.println("          </td>");
		out.println("        </tr>");
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println("</table>");
	    out.println("      </form>");
	    out.println("    </td>");
	    out.println("  </tr>");
	    out.println(closeArea());

	} else {
	    out.println(title("Service not available."));
	}
	
	out.println("</table>");
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }

    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(TRACE);
    }
}
