/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Bruno Michel and Guillaume Riviere
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jmx.jadmin.servlet;

// standard imports
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

// servlet imports
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

// JOnAS jmx imports
import org.objectweb.jonas.jmx.JonasObjectName;

// Jadmin imports
import org.objectweb.jonas.jmx.jadmin.*;

public class TransactionServiceServlet extends JonasManagementServlet {

    /**
     * Respond to a GET request for the content produced by
     * this servlet.
     *
     * @param req The servlet request we are processing
     * @param res The servlet response we are producing
     *
     * @exception IOException if an input/output error occurs
     * @exception ServletException if a servlet error occurs
     */
    public void doGet(HttpServletRequest req, 
		      HttpServletResponse res) throws IOException, ServletException {

	Exception exception = null;
	try {
	    // Retrieving request parameters
	    for (Enumeration e = req.getParameterNames() ; e.hasMoreElements() ;) {
		String paramName = (String)e.nextElement();
		if (paramName.equals("reset") 
		    && req.getParameter("reset").equals("AllTxTotalCounters")) 
		    JonasManagementRepr.invoke(JonasObjectName.transactionService(), 
					       "resetAllTxTotalCounters", 
					       new Object[0], new String[0]);
		else if (paramName.equals("CurrentTimeOut")) 
		    JonasManagementRepr.setAttribute(JonasObjectName.transactionService(), 
						     "CurrentTimeOut", 
						     new Integer(req.getParameter("CurrentTimeOut")));
	    }
	} catch (Exception e) {
	    exception = e;
	}

	res.setContentType("text/html");
	PrintWriter out = res.getWriter();

	out.println(doctype());
	out.println(openHtml());
	out.println(head());
	out.println(openBody());
	out.println(links());
	out.println(openTable());
	out.println(index());
	out.println(banner("TransactionService"));
	out.println(openMain());
		
	if (exception != null) out.println(alert(exception+":\n"+exception.getMessage()));

	// Content of the main cell of the table:
	out.println("<table border=\"0\" cellspacing=\"0\" cellpadding=\"1\" width=\"100%\">");

	if (JonasManagementRepr.isRegistered(JonasObjectName.transactionService())) {
	    out.println(title("Transaction service"));
	    out.println(openArea());
	    try {
		out.println(attribute("Current transaction time out:", 
				      "<form action=\"TransactionService\" method=\"get\">"
				      +"<input type=\"text\" name=\"CurrentTimeOut\" value=\""
				      +JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
									"CurrentTimeOut")
				      +"\"><input type=\"submit\" value=\"Submit\">"
				      +"<input type=\"reset\" value=\"Reset\"><br>"
				      +"value in seconds</form>"));
		String jtm = "other JVM";
		if (((Boolean)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
							       "JTMLocal")).booleanValue()) jtm = "collocated";
 
		out.println(attribute("Java Transaction Manager:", 
				      "<b>"+jtm+"</b>"));
		out.println(attribute("Java Transaction Manager host name:", 
				      "<b>"+JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
									     "JTMHostName")+"</b>"));
		Integer portNumber = (Integer)JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
									       "JTMPortNumber");
		String value;
		if (portNumber.equals(new Integer(0))) 
		    value = "undefined";
		else 
		    value = portNumber.toString();
		out.println(attribute("Java Transaction Manager port number:", "<b>"+value+"</b>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	    
	    out.println(title("Transaction status"));
	    out.println(openArea());
	    try {
		Integer [] allTxCounters = (Integer [])JonasManagementRepr.getAttribute(JonasObjectName.transactionService(), 
								       "TxCounters");
		out.println(attribute("Current number of transaction(s):", 
				      "<b>"+allTxCounters[0]+"</b>"));
		out.println(attribute("&nbsp;", "&nbsp;"));
		out.println(attribute("Number of begun transaction(s):", 
				      "<b>"+allTxCounters[1]+"</b>"));
		out.println(attribute("Number of committed transaction(s):", 
				      "<b>"+allTxCounters[2]+"</b>"));
		out.println(attribute("Number of rollbacked transaction(s):", 
				      "<b>"+allTxCounters[3]+"</b>"));
		out.println(attribute("Number of expired time out:", 
				      "<b>"+allTxCounters[4]+"</b>"));
		out.println(attribute("&nbsp;",
				      "<a title=\"Reset counters\" "
				      +"href=\"TransactionService?reset=AllTxTotalCounters\"><b>Reset</b></a>"));
	    } catch (ManagementException e) {
		out.println(error("Error while getting attribute:", e.getMessage(), 2));
	    }
	    out.println(closeArea());
	} else {
	    out.println(title("Service not available."));
	}
	out.println("</table>");
	// End of the content of the main cell of the table
	
	out.println(closeMain());
	out.println(closeTable());
	out.println(foot());
	out.println(closeBody());
	out.println(closeHtml());
    }


    /**
     * @return A string for the thumb index, includes the tr tag
     */
    protected String index() {
	return index(TRANSACTION);
    }
}
