/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Jeff Mesnil
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 * 
 */

package org.objectweb.jonas.rmifilters ;

import org.objectweb.security.propagation.SecuritySender ;
import org.objectweb.security.context.SecurityCurrent ;
import org.objectweb.security.context.SecurityContext ;

import org.objectweb.jonas.common.Trace ;

/**
 * This class represents the implementation of a SecuritySender in JOnAS.
 *
 * @author Jeff Mesnil
 */
public class JonasSecuritySender implements SecuritySender {

    private static final int trace = Trace.DB_25 ;

     /**
     * Empty constructor
     */
    public JonasSecuritySender () {	
	Trace.outln (trace, "JonasSecuritySender()") ;
    }

    /**
     * we do not use the received security context to set it (EJB, v1.1, chapter 15.5)
     */
    public void received_reply (int request_id, SecurityContext ctx) {
	Trace.outln (trace, "JonasSecuritySender.received_reply("+ctx+")") ;
    }
    
    public SecurityContext sending_request (int request_id) {

	// Gets Current object (always existing in EJBServer)
	SecurityCurrent current = SecurityCurrent.getCurrent();
        if (current == null) {
	    return null;
	}
	// Get the Security Context
	SecurityContext ctx = current.getSecurityContext();
	Trace.outln (trace, "JonasSecuritySender.sending_request() : "+ctx+"") ;
	return ctx ;
    }
}
