/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Jeff Mesnil
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.security;

import java.util.Hashtable;
import java.security.Guard;
import java.security.Principal;
import org.objectweb.security.context.SecurityContext;
import org.objectweb.security.context.SecurityCurrent;
import org.objectweb.jonas.common.Trace;

public class MethodGuard implements Guard, java.io.Serializable {

    private static final int trace = Trace.DB_25;
    
    // list of method signatures identifying the methods to guard.
    private Hashtable methods = new Hashtable();

     private String containerName = null;

     public MethodGuard (String containerName) {
 	this.containerName = containerName;
     }

    public void add(String methodSignature, String[] roles) {
	Trace.outln(trace, "MethodGuard.add "+methodSignature);
	methods.put(methodSignature, roles);
    }
    
    public void checkGuard(Object methodSignature) throws SecurityException {
	Trace.outln(trace, "MethodGuard.checkGuard()");
	boolean isAuthorized = false;
	String[] roles = (String[])methods.get(methodSignature);

	//if no roles, everyone can access the method
	if (roles.length == 0) {
	    Trace.outln(trace, "access to method not protected");
	    return;
	}

	// verify if principal has at least one of the roles
	SecurityCurrent current = SecurityCurrent.getCurrent();
	SecurityContext ctx = current.getSecurityContext();
	if (ctx == null) {
	     Trace.outln(trace, "no security context exists");
	    throw new SecurityException("no security context exists");
	}
	Principal principal = ctx.getCallerPrincipal();
	RoleMechanism roleMechanism =  RoleMechanism.getInstance();
	isAuthorized = roleMechanism.isCallerInRoles(principal, roles);
	if (!isAuthorized) {
	    Trace.outln(trace, "not authorized to access Method");
	    throw new SecurityException("caller not authorized to access the method");
	}
	Trace.outln(trace, "authorized to access Method");
	return;
    }

     public String toString() {
 	return "MethodGuard of container : "+ containerName;
     }
}
