/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Jeff Mesnil
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.security;

import java.security.Principal;
import java.security.Guard;
import java.util.Hashtable;
import org.objectweb.security.context.SecurityCurrent;
import org.objectweb.security.context.SecurityContext;
import org.objectweb.jonas.common.Trace;

public class RoleGuard implements Guard {

    private static final int trace = Trace.DB_25;
    
    private String containerName = null;

    private Hashtable roleRefs = new Hashtable();

     public RoleGuard(String containerName) {
 	this.containerName = containerName;
     }

    public void add(String roleRef, String role) {
	Trace.outln(trace, "RoleGuard.add "+role);
	roleRefs.put(roleRef, role) ;
    }

    public void checkGuard(Object roleString) throws SecurityException {
	Trace.outln(trace,"checkGuard asked for ",(String)roleString);
	String roleRef = (String) roleString;
	String role = (String)roleRefs.get(roleRef);
	if (role == null) {
	    Trace.outln(trace, roleRef," has not been defined");
	    throw new SecurityException("the logical role associated with "+roleRef+" has not been defined");
	}
	RoleMechanism rm = RoleMechanism.getInstance();
	SecurityCurrent current = SecurityCurrent.getCurrent();
	SecurityContext ctx = current.getSecurityContext();
	if (ctx == null) {
	    Trace.outln(trace, "no security context exists");
	    throw new SecurityException("no security context exists");
	}
	Principal principal = ctx.getCallerPrincipal();
	boolean isCallerInRole = rm.isCallerInRole(principal, role);
	if (isCallerInRole) {
	    Trace.outln(trace, "Caller is in role ",roleString);
	    return;
	} else {
	    Trace.outln(trace, "Caller is not in role ",roleString);
	    throw new SecurityException("the caller is not in the role "+role);
	}
    }

     public String toString() {
 	return "RoleGuard of container : "+ containerName;
     }
}
