/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import java.io.Serializable;

/**
 * This class implements the javax.ejb.EJBMetaData interface.
 *
 */
public class JBeanMetaData implements EJBMetaData, Serializable {

    /**
     * @serial
     */
    private EJBHome home;

    /**
     * @serial
     */
    private Class   homeInterfaceClass;

    /**
     * @serial
     */
    private Class   remoteInterfaceClass;

    /**
     * @serial
     */
    private Class   primaryKeyClass;

    /**
     * @serial
     */
    private boolean isSession;

    /**
     * @serial
     */
    private boolean isStatelessSession;

    public JBeanMetaData(JBeanHome ejbHome,
			 Class  homeInterfaceClass,
			 Class  remoteInterfaceClass,
			 Class  primaryKeyClass,
			 boolean isSession,
			 boolean isStatelessSession) {
	if (TraceEjb.isDebug) {
	    String pkName = (primaryKeyClass==null) ? "null" : primaryKeyClass.getName();
	    TraceEjb.debug("JBeanMetaData.JBeanMetaData(ejbHome," 
			   +homeInterfaceClass.getName()+", "+remoteInterfaceClass.getName()
			   +", "+pkName+", "+isSession+", "+isStatelessSession+")");
	}
	this.home = ejbHome;
	this.homeInterfaceClass   = homeInterfaceClass;
	this.remoteInterfaceClass = remoteInterfaceClass;
	this.primaryKeyClass      = primaryKeyClass;
	this.isSession = isSession;
	this.isStatelessSession = isStatelessSession;
    }

    /**
     * Obtains the home interface of the enterprise Bean. 
     */
    public EJBHome getEJBHome() {
	TraceEjb.debug("JBeanMetaData.getEJBHome() ");
	TraceEjb.error("EJBMetaData.getEJBHome(): Not yet implemented");
	return home;
    }    

    /**
     * Obtains the Class object for the enterprise Bean's home interface. 
     */
    public Class getHomeInterfaceClass() {
	TraceEjb.debug("JBeanMetaData.getHomeInterface() ");
	return homeInterfaceClass;
    }

    /**
     * Obtains the Class object for the enterprise Bean's primary key class. 
     */
    public Class getPrimaryKeyClass() {
	TraceEjb.debug("JBeanMetaData.getPrimaryKeyClass() ");
	if (isSession) {
	    throw new RuntimeException("No PrimaryKey Class for session Bean");
	}
	return primaryKeyClass;
    }

    /**
     * Obtains the Class object for the enterprise Bean's remote interface. 
     */
    public Class getRemoteInterfaceClass() {
	TraceEjb.debug("JBeanMetaData.getRemoteInterfaceClass() ");
	return remoteInterfaceClass;
    }
    
    /**
     * Tests if the enterprise Bean's type is "session".
     */
    public boolean isSession() {
	if (TraceEjb.isDebug) {
	    TraceEjb.debug("JBeanMetaData.isSession() = " + isSession);
	}
	return isSession;
    }

    /**
     * Test if the enterprise Bean's type is "stateless session".
     * SINCE EJB 1.1
     *
     * @return True if the type of the enterprise Bean is stateless session.
     */
    public boolean isStatelessSession() {
	if (TraceEjb.isDebug) {
	    TraceEjb.debug("JBeanMetaData.isStatelessSession() = " + isStatelessSession);
	}
	return isStatelessSession;
    }

}


 
