/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): Jeff Mesnil, Xavier Spengler
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.container;

import java.security.Guard;
import org.enhydra.naming.ContainerNaming;

/**
 * This class represents an JOnAS EJB container.
 *
 * A container is where an enterprise Bean object lives. 
 * Several EJB can be installed in a single container.
 * In a  JOnAS container are installed all beans from the same ejb-jar file.
 * Several JOnAS container may be instantiated in the same JOnAS Server.
 * For each EJB installed, the container provides a home interface and
 * makes this interface available in the JNDI name space.
 */
public interface JContainer {
    public String [] listBeanNames();
    public String getName();
    public String getFileName();
    public void setMethodGuard(Guard methodGuard);
    public void setRoleGuard(Guard roleGuard);
    public Guard getMethodGuard();
    public Guard getRoleGuard();
    public void setContainerNaming(ContainerNaming naming);
    public ContainerNaming getContainerNaming();
    public ClassLoader getClassLoader();
    public void remove();
    public void setPrincipalFactory(PrincipalFactory pf);
    public PrincipalFactory getPrincipalFactory();
}
