/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s):
 * Dean Jennings : optimization
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Vector;
import java.util.List;
import javax.ejb.SessionBean;
import javax.ejb.SessionSynchronization;
import javax.ejb.EJBObject;
import javax.ejb.RemoveException;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.transaction.Transaction;
import javax.transaction.Synchronization;
import javax.transaction.Status;
import javax.transaction.SystemException;

/**
 * This class implements javax.ejb.SessionContext interface.
 * it should be implemented by JContextSessionSf and JContextSessionSl
 * depending if the beans is stateful or stateless.
 */
public abstract class JContextSession extends JContext implements SessionContext, Serializable {

    // Keep EJBObject in memory, because no POA yet!
    transient protected JBeanSession ejbObject = null; // the remote object

    protected SessionBean sessionBean = null; // the bean instance

    // ------------------------------------------------------------------
    // constructors
    // ------------------------------------------------------------------

    /**
     * Constructs a SessionContext 
     */
    public JContextSession(JBeanHome home, SessionBean sb) {
	super(home);
	sessionBean = sb;
    }
 

    // ------------------------------------------------------------------
    // SessionContext implementation
    // ------------------------------------------------------------------

    /**
     * Obtains a reference to the EJB object that is currently associated with the instance.
     * @return    The EJB object currently associated with the instance. 
     * @exception IllegalStateException: Thrown if the instance invokes this method while 
     *            the instance is in a state that does not allow the instance to invoke 
     *            this method.
     */
    public EJBObject getEJBObject() throws IllegalStateException {
	return ejbObject;
    }

    // ------------------------------------------------------------------
    // Methods used by GenIC generated code.
    // - Don't forget to check GenIC before changing interface.
    // ------------------------------------------------------------------

    /**
     * Return the bean instance associated to this Context.
     * @return the bean instance
     */
    public SessionBean getInstance() {
	if (sessionBean == null) {
	    TraceEjb.error("JContextSession.getInstance -> null!!!");
	}
	return sessionBean;
    }

    // ------------------------------------------------------------------
    // Other methods
    // ------------------------------------------------------------------

    /**
     * Call ejbRemove on instance.
     * Stateless Sessions: when container is removed.
     * Stateful  Sessions: when session is removed.
     */
    public void removeMyInstance() throws RemoveException {
	TraceEjb.debugCtx("JContextSession.removeMyInstance");
	if (sessionBean == null) {
	    TraceEjb.error("removeMyInstance: null instance!");
	    return;
	}
	try {
	    sessionBean.ejbRemove();
	} catch(RemoteException e) {
	    TraceEjb.error("JContextSession.removeMyInstance : ", e);
	    throw new RemoveException();
	}
    }

    public void initSessionContext(JBeanSession remote) {
	// reinit Context for reuse
	ejbObject = remote;
    }

    public void setConnectionList(List list) {
	ejbObject.setMyConnectionList(list);
    }

}
