/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *   Dean Jennings : optimization
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.util.List;
import javax.ejb.EJBContext;
import javax.naming.Context;
import javax.transaction.Transaction;

/**
 * Context associated to a request to a bean.
 * mandatory for finder methods that return collections.
 * used for all business methods.
 */
public class RequestCtx { 

    private Transaction clientTransaction = null; // Tx to be resumed at postinvoke
    private Transaction currTx = null; // transaction in which the request will execute
    private boolean mustBeCommitted = false; // must commit current Tx at postinvoke
    private Throwable sysExc = null; // system exception or error raised in business method
    private Context oldJNDIctx = null; // JNDICtx to be resumed at postinvoke
    private List oldConnectionList = null;
    private EJBContext ejbContext = null;
    private int txattr = 0;

    /**
     * constructors. Called at preinvoke.
     */
    public RequestCtx(Transaction cltx, boolean commit) {
	clientTransaction = cltx;
	mustBeCommitted = commit;
    }
    public RequestCtx(int txattr) {
	this.txattr = txattr;
    }

    /**
     * set transaction parameters
     */
    public void setTransactions(boolean mc, Transaction cltx, Transaction curr) {
	mustBeCommitted = mc;
	clientTransaction = cltx;
	currTx = curr;
    }
	
    /**
     * set transaction parameters
     */
    public void setClientTx(Transaction cltx) {
	clientTransaction = cltx;
    }
	
    /**
     * Obtains the client Transaction to be resumed at postinvoke
     */
    public Transaction getClientTx() {
	return clientTransaction;
    }

    /**
     * Obtains the current Transaction
     */
    public Transaction getCurrentTx() {
	return currTx;
    }

    /**
     * return true if must commit the current transaction at postinvoke
     */
    public boolean mustCommit() {
	return mustBeCommitted;
    }    

    /**
     * set system exception in context
     */
    public void setSysExc(Throwable e) {
	this.sysExc = e;
    }

    /**
     * get system exception in context
     */
    public Throwable getSysExc() {
	return this.sysExc;
    }

    /**
     * set JNDICtx
     */
    public void setOldJNDICtx(Context c) {
	this.oldJNDIctx = c;
    }

    /**
     * get JNDICtx
     */
    public Context getOldJNDICtx() {
	return this.oldJNDIctx;
    }

    /**
     * save Connection List
     */
    public void setOldConnectionList(List list) {
	this.oldConnectionList = list;
    }

    /**
     * get saved Connection List
     */
    public List getOldConnectionList() {
	return this.oldConnectionList;
    }

    /**
     * set ejbContext
     */
    public void setEJBContext(EJBContext c) {
	this.ejbContext = c;
    }

    /**
     * get ejbContext
     */
    public EJBContext getEJBContext() {
	return ejbContext;
    }
   
    /**
     * set TxAttr
     */
    public void setTxAttr(int a) {
	this.txattr = a;
    }

    /**
     * get TxAttr
     */
    public int getTxAttr() {
	return this.txattr;
    }
   
}
