/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_ejb.container;

import java.io.PrintWriter;

public class TraceEjb {

    static public boolean isDebugCtx    = false; // if the container-context is logged
    static public boolean isDebugServer = false; // if the container-server is logged
    static public boolean isDebug       = false; // if the container is logged
    static public boolean isDebugInvoke = false; // if the container-invoke is logged
    static public boolean isDebugPool   = false; // if the container-pool is logged
    static public boolean isDebugSecurity = false; // if the container-security is logged
    static public boolean isDebugGenIC = false;	// if the GenIC is logged
    static public boolean isDebugJms = false; // if we print traces relatives to jms 
    static public boolean isDebugThread = false; // ThreadPool
    static public boolean isVerbose = false; // if we print the verbose message

    
    static public PrintWriter logWriter = null;	// our log writer
    
    /**
     * set the ThreadPool logging
     */
    static public void setDebugThread(boolean set) {
	isDebugThread = set;
    }
    
    /**
     * set the debug container-context
     */
    static public void setDebugCtx(boolean set) {
	isDebugCtx = set;
    }
    
    /**
     * set the debug container-server
     */
    static public void setDebugServer(boolean set) {
	isDebugServer = set;
    }
    
    /**
     * set the debug container
     */
    static public void setDebug(boolean set) {
	isDebug = set;
    }
    
    /**
     * set the debug container-invoke
     */
    static public void setDebugInvoke(boolean set) {
	isDebugInvoke = set;
    }

    /**
     * set the debug container-pool
     */
    static public void setDebugPool(boolean set) {
	isDebugPool = set;
    }
    
    /**
     * set the debug container-security
     */
    static public void setDebugSecurity(boolean set) {
	isDebugSecurity = set;
    }

   /**
     * set the debug GenIC
     */
    static public void setDebugGenIC(boolean set) {
	isDebugGenIC = set;
    }

   /**
     * set the debug for JMS 
     */
    static public void setDebugJms(boolean set) {
	isDebugJms = set;
    }

    /**
     * set the verbose flag
     */
    static public void setVerbose(boolean set) {
	isVerbose = set;
    }
    
    /**
     * set the log writer
     */
    static public void setLogWriter(PrintWriter log) {
	logWriter = log;
    }
    
    /**
     * print the verbose message if the logger is not null
     */
    static public void verbose(String msg) {
	if ((isVerbose) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug container-ctx message if the logger is not null
     */
    static public void debugCtx(String msg) {
	if ((isDebugCtx) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug ThreadPool message if the logger is not null
     */
    static public void debugThread(String msg) {
	if (isDebugThread && logWriter != null)
	    logWriter.println(msg);
    }

    /**
     * print the debug container-server message if the logger is not null
     */
    static public void debugServer(String msg) {
	if ((isDebugServer) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * concatenation and print the debug container-server message if the logger is not null
     */
    static public void debugServer(String msg1, String msg2) {
	if ((isDebugServer) && (logWriter!=null))
	    logWriter.println(msg1 + msg2);
    }
    
    /**
     * print the debug container message if the logger is not null
     */
    static public void debug(String msg) {
	if ((isDebug) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug container-invoke message if the logger is not null
     */
    static public void debugInvoke(String msg) {
	if ((isDebugInvoke) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug container-pool message if the logger is not null
     */
    static public void debugPool(String msg) {
	if ((isDebugPool) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug container-security message if the logger is not null
     */
    static public void debugSecurity(String msg) {
	if ((isDebugSecurity) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the debug GenIC message if the logger is not null
     */
    static public void debugGenIC(String msg) {
	if ((isDebugGenIC) && (logWriter!=null))
	    logWriter.println(msg);
    }


    /**
     * print the debug message for JMS  if the logger is not null
     */
    static public void debugJms(String msg) {
	if ((isDebugJms) && (logWriter!=null))
	    logWriter.println(msg);
    }

    /**
     * print the error message if the logger is not null
     */
    static public void error(String msg) {
	if (logWriter!=null)
	    logWriter.println(msg);
    }

    /**
     * print the message and the throwable and its backtrace, if the logger is not null
     */
    static public void error(String msg, Throwable th) {
	if (logWriter!=null) {
	    logWriter.println(msg);
	    th.printStackTrace(logWriter);
	}
    }

}
