/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.io.IOException;
import java.lang.String;
import java.lang.reflect.Method;

import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionDesc;
import org.objectweb.jonas_ejb.lib.BeanNaming;
import org.objectweb.jonas_ejb.lib.JavaType;


/**
 * This class allows to generate the source of the derived bean class of 
 * a given Enterprise Java Bean.
 */
abstract class GenICBean extends GenICCommon {

    /**
     * GenICBean Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICBean(BeanDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Generate the source of the class which implements the Home interface
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Declared methods in the Enterprise bean's home interface;
	Method methods[];

	// Comments
	src.println("//");
	src.println("// "+wrpBeanBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool of JOnAS");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(wrpBeanName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpBeanBaseName+
		    " extends "+ejbBeanName+" {");
	src.println();
	src.indentPlus();

	// setEntityContext
	genSetEntityContext();
	src.println();

	// isModifiedData()
	genIsModifiedData();
	src.println();

	// create() and find...() Methodes defined in the home interface
	methods = ejbHomeClass.getMethods();
	for (int m=0; m < methods.length; m++) {
	    // Skip the methods defined in the 'javax.ejb.EJBHome' interface
	    Method method = methods[m];
	    if (!method.getDeclaringClass().getName().equals("javax.ejb.EJBHome")) {

		boolean isCreate    = method.getName().equals("create");
		boolean isFindByPK  = method.getName().equals("findByPrimaryKey");
		boolean isFindOther = ((!isFindByPK) && (method.getName().indexOf("find") == 0));

		if (isCreate) {
		    genEjbCreate(method);
		} else if (isFindByPK) {
		    genEjbFindByPK(method);
		} else if (isFindOther) {
		    genEjbFindOther(method);
		} else {
		    System.err.println("GenICBean() WARNING: Method '" + method.getName() +
				       "' in the Enterprise bean's home interface IGNORED");
		}

		src.println();
	    }
	}

	// ejbLoad()
	genEjbLoad();
	src.println();

	// ejbStore()
	genEjbStore();
	src.println();

	// ejbRemove()
	genEjbRemove();
	src.println();

	// Class End
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }


    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

   /**
     * Source generation of the setEntityContext() method
     * @exception GenICException in error case
     */
    abstract void   genSetEntityContext() throws GenICException;

   /**
     * Source generation of the isModifiedData() method
     * @exception GenICException in error case
     */
    abstract void   genIsModifiedData() throws GenICException;

    /**
     * Source generation of the ejbCreate() method associated to the given create() method
     * described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genEjbCreate(Method method) throws GenICException;

    /**
     * Source generation of the ejbFindByPrimaryKey() method associated to the findByPrimaryKey()
     * method described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genEjbFindByPK(Method method) throws GenICException;

    /**
     * Source generation of the ejbFindXXX() method associated to thefindXXX() method
     * described in the Home interface
     * @exception GenICException in error case
     */
    abstract void   genEjbFindOther(Method method) throws GenICException;

    /**
     * Source generation of the ejbLoad() method
     * @exception GenICException in error case
     */
    abstract void   genEjbLoad() throws GenICException;

    /**
     * Source generation of the ejbStore() method
     * @exception GenICException in error case
     */
    abstract void   genEjbStore() throws GenICException;

    /**
     * Source generation of the ejbRemove() method
     * @exception GenICException in error case
     */
    abstract void   genEjbRemove() throws GenICException;

}

