/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;

import java.lang.Class;
import java.lang.String;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Vector;
import org.objectweb.jonas_ejb.deployment.api.EntityCmpDesc;
import org.objectweb.jonas_ejb.lib.BeanNaming;
import org.objectweb.jonas_ejb.lib.JavaType;


/**
 *  This class allows to generate the source of the derived class of a given 
 *  Entity Enterprise Java Bean with container managed persistence.
 *  Its extends GenICBean class.
 */
abstract class GenICEntityCMBean extends GenICBean {

    /**
     * GenICEntityCMBean Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     *                 with container managed persistence
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityCMBean(EntityCmpDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Source generation of the setEntityContext() method
     * @exception GenICException in error case
     */
    void genSetEntityContext() throws GenICException {

	Method method = null;
	Class params[] = new Class[1];
	try {
	    params[0] = javax.ejb.EntityContext.class;
	    method = ejbBeanClass.getMethod("setEntityContext", params);
	} catch (Exception e) {
	    throw new GenICException("Cannot get the "+ejbBeanName+".setEntityContext() method by reflection ("+e.getMessage()+")");
	}

	try {
	    src.println("private EntityContext ejbContext = null;");
	    src.println("private JEntityHome ejbHome = null;");
	    src.println();
	    genHeaderMethod(method, wrpBeanBaseName);
	    src.println("this.ejbContext = p1;");
	    src.println("this.ejbHome = (JEntityHome)p1.getEJBHome();");
	    src.println("super.setEntityContext(p1);");
	    src.indentMinus();
	    src.println("}");
	    src.println();
	} catch (Exception e) {
	    throw new GenICException(e.getMessage());
	}
    }

   /**
     * Source generation of the isModifiedData() method
     * @exception GenICException in error case
     */
    void genIsModifiedData() throws GenICException {
	src.println("private boolean isModifiedData() {");
	src.indentPlus();
	if (((EntityCmpDesc)ddesc).hasIsModifiedMethod()) {
	    String isModifiedMethodName = ((EntityCmpDesc)ddesc).getIsModifiedMethod().getName();
	    src.println("boolean b = super."+isModifiedMethodName+"();");
	} else {
	    src.println("boolean b = true;");
	}
	src.println("TraceEjb.debugGenIC(\"" + wrpBeanBaseName + ".isModifiedData() --> \" + b);");
	src.println("return(b);");
	src.indentMinus();
	src.println("}");
	src.println();
    }

}


