/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.lang.String;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;


/**
 *  This class allows to generate the source of the class that implements
 *  the Handle of a given Entity Enterprise Java Bean.
 *  Its extends GenICHandle class.
 */

class GenICEntityHandle extends GenICHandle {

    // Full names of the interfaces and the classes
    String ejbPrimaryKeyName = null;

    /**
     * GenICEntityHandle Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityHandle (EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
	ejbPrimaryKeyName = ((EntityDesc)ddesc).getPrimaryKeyClass().getName();
    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return("EntityHandleImpl");
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.String;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.util.Properties;");
	src.println("import javax.ejb.EJBObject;");
	src.println("import javax.ejb.HomeHandle;");
	src.println();
	src.println("import org.objectweb.jonas_ejb.container.JEntityHome;");
	src.println("import org.objectweb.jonas_ejb.container.EntityHandleImpl;");
    }

    /**
     * Source generation of the attributes part
     * @exception GenICException in error case
     */
    void genAttributes() throws GenICException {
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {

	src.println("public " + wrpHandleBaseName + "("+ wrpRemoteName +
		    " ejbobject) {");

	src.indentPlus();
	src.println("super(ejbobject);");
	src.println("try {");
	src.indentPlus();
	src.println("pk = ("+ejbPrimaryKeyName+")ejbobject.getPrimaryKey();");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("pk = null;");
	src.indentMinus();
	src.println("}");  // try-catch
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getEJBObject() method
     * @exception GenICException in error case
     */
    void genGetEJBObject() throws GenICException {
	src.println("public EJBObject getEJBObject() throws RemoteException {");
	src.indentPlus();
	src.println(ejbRemoteName + " ejbobj = null;");
	src.println("try {");
	src.indentPlus();
	src.println(ejbHomeName + " h = ("+ejbHomeName+")javax.rmi.PortableRemoteObject.narrow(homehandle.getEJBHome(),"+ejbHomeName+".class);");
	src.println("ejbobj = h.findByPrimaryKey(("+ejbPrimaryKeyName+")pk);");
	src.indentMinus();
	src.println("} catch (Exception e) {");
	src.indentPlus();
	src.println("throw new RemoteException(\""+wrpHandleBaseName+".getEJBObject(): \"+e, e);");
	src.indentMinus();
	src.println("}");  // try-catch
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }

}

