/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.lang.String;
import java.lang.reflect.Method;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.deployment.api.MethodDesc ;
import org.objectweb.jonas_ejb.lib.BeanNaming;
import org.objectweb.jonas_ejb.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's Home interface of a given Entity Enterprise Java Bean.
 *  Its extends GenICHome class.
 */
class GenICEntityHome extends GenICHome {

    // Full names of the interfaces and the classes
    String ejbPrimaryKeyName = null;
    String wrpBeanName       = null;

    /**
     * GenICEntityHome Constructor
     *
     * @param dd       deployment descriptor of the Entity Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityHome (EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	ejbPrimaryKeyName = dd.getPrimaryKeyClass().getName();
	BeanNaming bn     = new BeanNaming(ddesc);
	wrpBeanName       = bn.getFullDerivedBeanName();
	isSynchronized = false;	// do not synchronize methods in Home
    }

    /**
     * Name of the superclass of the generated class
     */
    String getSuperClass() {
	return(new String("JEntityHome"));
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.Object;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import java.util.Enumeration;");
	src.println("import javax.ejb.EJBObject;");
	src.println("import javax.ejb.EntityBean;");
	src.println("import javax.ejb.FinderException;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.ObjectNotFoundException;");
	src.println("import org.objectweb.jonas_ejb.container.JContainerImpl;");
	src.println("import org.objectweb.jonas_ejb.container.JContextEntity;");
	src.println("import org.objectweb.jonas_ejb.container.JEntityHome;");
	src.println("import org.objectweb.jonas_ejb.container.EntityHandleImpl;");
	src.println("import org.objectweb.jonas_ejb.container.RequestCtx;");
	src.println("import org.objectweb.jonas_ejb.container.TraceEjb;");
	src.println("import org.objectweb.jonas_ejb.deployment.api.EntityDesc;");
	src.println("import org.objectweb.jonas_ejb.lib.CollectionEnum;");
	src.println("import "+wrpRemoteName+";");
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {

	src.println("public " + wrpHomeBaseName +
		    "(EntityDesc desc, String BeanName, JContainerImpl cont, Object ds) throws RemoteException {");
	src.indentPlus();
	src.println("super(desc, BeanName, cont, ds);");
	src.println("TraceEjb.debugGenIC(\"" + wrpHomeBaseName + "(\"+BeanName+\")\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given create() method described in the Home interface
     * @exception GenICException in error case
     */
    void genCreate(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();

	genHeaderMethod(method, wrpHomeBaseName);
	src.println(wrpRemoteBaseName + " ejbobj = null;");
	src.println(wrpBeanName + " eb = null;");
	genSecuritySignature(method);
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+", methodSignature) ;");
	src.println("JContextEntity ctx = null;");
	src.println(ejbPrimaryKeyName + " pk;");
	src.println("try {");
	src.indentPlus();
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println("ctx = ejbobj.getNewEntityContext();");
	src.println("rctx.setEJBContext(ctx);");
	src.println("eb = ("+wrpBeanName +")ctx.getInstance();");
	// Cast useful in case of the parameter is a java.lang.Object (CMP)
	src.print  ("pk = ("+ejbPrimaryKeyName+")eb.ejbCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	src.println("ejbobj.setPrimaryKey(pk);");
	src.println("synchronized(this) {");
	src.indentPlus();
	src.println("putEJBObject(ejbobj);");
	src.indentMinus();
	src.println("}");
	src.print  ("eb.ejbPostCreate(");
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBean(method, true);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	src.println("return(ejbobj);");
	src.indentMinus();
	src.println("}");
    }


    /**
     * Source generation for the remove methods
     * @exception GenICException in error case
     */
    void genRemove(Method method) throws GenICException {

	Class params[] = method.getParameterTypes();

	boolean isHandle = JavaType.getName(params[0]).equals("javax.ejb.Handle");

	genHeaderMethod(method, wrpHomeBaseName);
	src.println(wrpRemoteBaseName + " ejbobj = null;");
	MethodDesc md = ddesc.getMethodDesc(method) ;
	genSecuritySignature(method) ;
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+", methodSignature) ;");
	src.println("try {");
	src.indentPlus();
	if (isHandle) {
	    src.println("EntityHandleImpl h = (EntityHandleImpl)p1;");
	    src.println("java.lang.Object pk = h.getPK();");
	} else {
	    src.println("java.lang.Object pk = p1;");
	}
	src.println("synchronized(this) {");
	src.indentPlus();
	src.println("ejbobj = ("+wrpRemoteBaseName+")getEJBObject(pk);");
	src.println("if (ejbobj == null) {");
	src.indentPlus();
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println("ejbobj.setPrimaryKey(pk);");
	src.println("putEJBObject(ejbobj);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	src.println("JContextEntity ctx = ejbobj.getEntityContext();");
	src.println("rctx.setEJBContext(ctx);");
	src.println("ctx.remove();");
	genCatchBean(method, true);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for the given find method described in the Home interface
     * @exception GenICException in error case
     */
    void genFind(Method method) throws GenICException {
	Class params[] = method.getParameterTypes();
	boolean isWithCollection = ! method.getReturnType().equals(ejbRemoteClass);
	boolean isEnumerationReturnType = method.getReturnType().equals(java.util.Enumeration.class);

	genHeaderMethod(method, wrpHomeBaseName);
	if (isWithCollection) {
	    if (isEnumerationReturnType) {
		src.println("CollectionEnum ejbobjV = new CollectionEnum();");
	    } else {
		src.println("java.util.Vector ejbobjV = new java.util.Vector();");
	    }
	}
	src.println(wrpRemoteBaseName + " ejbobj = null;");
	genSecuritySignature(method);
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+", methodSignature) ;");
	src.println("try {");
	src.indentPlus();
	src.println("JContextEntity ctx = getAvailable();");
	src.println(wrpBeanName + " eb = ("+wrpBeanName +")ctx.getInstance();");
	src.println(ejbPrimaryKeyName + " pk;");
	if (isWithCollection) {
	    if (isEnumerationReturnType) {
		src.print("Enumeration");
	    } else {
		src.print("java.util.Collection");
	    }
	    src.print(" pkC = eb.ejb" + BeanNaming.firstToUpperCase(method.getName()) + "(");
	    for (int p = 1; p <= params.length; p++) {
		src.print("p" + p, false);
		if (p < params.length) {
		    src.print(", ", false);
		}
	    }
	    src.println(");", false);	    
	    if (isEnumerationReturnType) {
		src.println("while (pkC.hasMoreElements()) {");
		src.indentPlus();
		src.println("pk = ("+ejbPrimaryKeyName+")pkC.nextElement();");
	    } else {
		src.println("java.util.Iterator pkIC = pkC.iterator();");
		src.println("while (pkIC.hasNext()) {");
		src.indentPlus();
		src.println("pk = ("+ejbPrimaryKeyName+")pkIC.next();");
	    }
	} else {
	    if (method.getName().equals("findByPrimaryKey")) {
		// Cast useful in case of the parameter is a java.lang.Object (CMP)
		src.println("pk = eb.ejbFindByPrimaryKey(("+ejbPrimaryKeyName+") p1);");
	    } else {
		src.print("pk = eb.ejb" + BeanNaming.firstToUpperCase(method.getName()) + "(");
		for (int p = 1; p <= params.length; p++) {
		    src.print("p" + p, false);
		    if (p < params.length) {
			src.print(", ", false);
		    }
		}
		src.println(");", false);
	    }
	}
	src.println("synchronized(this) {");
	src.indentPlus();
	src.println("ejbobj = (" + wrpRemoteBaseName + ") getEJBObject(pk);");
	src.println("if (ejbobj == null) {");
	src.indentPlus();
	src.println("ejbobj = new " + wrpRemoteBaseName + "(this);");
	src.println("ejbobj.setPrimaryKey(pk);");
	src.println("putEJBObject(ejbobj);");
	src.indentMinus();
	src.println("}");
	src.indentMinus();
	src.println("}");
	if (isWithCollection) {
	    src.println("ejbobjV.addElement((Object)ejbobj);");
	    src.indentMinus();
	    src.println("}");
	}
	src.println("freeEJBCtx(ctx);");
	genCatchBean(method, true);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	if (isWithCollection) {
	    src.println("return (ejbobjV);");
	} else {
	    src.println("return (ejbobj);");
	}
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation for callejbCreate method
     * @exception GenICException in error case
     */
    void genCallejbCreate() {
	// Nothing to do
	// No CallejbCreate method in case of Entity	
    }

    /**
     * Source generation for createNewInstance method
     */
    void genCreateNewInstance() {
	src.println("protected EntityBean createNewInstance() throws Exception {");
	src.indentPlus();
	src.println("return (new " + wrpBeanName + "());");
	src.indentMinus();
	src.println("}");
    }
}

