/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.lang.String;
import org.objectweb.jonas_ejb.deployment.api.EntityCmpDesc;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.lib.BeanNaming;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Entity Enterprise Java Bean.
 *  Its extends GenICRemote class.
 */

class GenICEntityRemote extends GenICRemote {

    // Full name of the Handle class
    String wrpHandleName = null;

    /**
     * GenICEntityRemote Constructor
     *
     * @param dd       deployment descriptor of the Entity Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICEntityRemote(EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	BeanNaming bn = new BeanNaming(ddesc);
	wrpHandleName = bn.getFullWrpHandleName();
	isSynchronized = false;	// Reentrance is managed in generic classes
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanEntity in case of Entity Bean)
     */
    String getSuperClass() {
	return(new String("JBeanEntity"));
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import javax.ejb.EnterpriseBean;");
	src.println("import javax.ejb.EJBException;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.EntityBean;");
	src.println("import org.objectweb.jonas_ejb.container.JBeanEntity;");
	src.println("import org.objectweb.jonas_ejb.container.JContextEntity;");
	src.println("import org.objectweb.jonas_ejb.container.JEntityHome;");
	src.println("import org.objectweb.jonas_ejb.container.RequestCtx;");
	src.println("import org.objectweb.jonas_ejb.container.TraceEjb;");
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {
	src.println("public " + wrpRemoteBaseName +
		    "(JEntityHome home) throws RemoteException {");
	src.indentPlus();
	src.println("super(home,"+(ddesc instanceof EntityCmpDesc)+");");
	src.println("TraceEjb.debugGenIC(\"" + wrpRemoteBaseName + "()\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getHandle() method
     * @exception GenICException in error case
     */
    void genGetHandle() throws GenICException {
	src.println("public javax.ejb.Handle getHandle() throws RemoteException {");
	src.indentPlus();
	src.println("TraceEjb.debugGenIC(\"" + wrpRemoteBaseName + ".getHandle()\");");
	src.println("if (handleEJBObject == null) {");
	src.indentPlus();
	src.println("handleEJBObject = new "+wrpHandleName+"(this);");
	src.indentMinus();
	src.println("}");
	src.println("return(handleEJBObject);");
	src.indentMinus();
	src.println("}");
    }

}

