/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.io.IOException;
import java.lang.String;
import org.objectweb.jonas_ejb.deployment.api.EntityDesc;
import org.objectweb.jonas_ejb.lib.BeanNaming;


/**
 *  This class allows to generate the source of the class that implements
 *  the Handle of a given Enterprise Java Bean.
 *  (Only useful for Entity Bean)
 */

abstract class GenICHandle extends GenICCommon {

    /**
     * GenICHandle Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICHandle(EntityDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Generate the source of the class which implements the handle
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Comments
	src.println("//");
	src.println("// "+wrpHandleBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool of JOnAS");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(wrpHandleName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpHandleBaseName+
		    " extends " + getSuperClass() + " {");
	src.println();
	src.indentPlus();

	// Attributes declaration
	genAttributes();
	src.println();

	// Constructor()
	genConstructor();
	src.println();

	// getEJBObject() Methode
	genGetEJBObject();
	src.println();

	// Class End
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }

    /**
     * Name of the superclass of the generated class
     */
    abstract String getSuperClass();

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation of the attributes part
     * @exception GenICException in error case
     */
    abstract void   genAttributes() throws GenICException;

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    abstract void   genConstructor() throws GenICException;

    /**
     * Source generation of the getEJBObject() method
     * @exception GenICException in error case
     */
    abstract void   genGetEJBObject() throws GenICException;

}

