/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */



package org.objectweb.jonas_ejb.tools;


import java.io.IOException;
import java.lang.String;
import java.lang.reflect.Method;
import java.util.Vector;
import org.objectweb.jonas_ejb.deployment.api.BeanDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionDesc;
import org.objectweb.jonas_ejb.deployment.api.MethodDesc ;
import org.objectweb.jonas_ejb.lib.BeanNaming;
import org.objectweb.jonas_ejb.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Enterprise Java Bean.
 */

abstract class GenICRemote extends GenICCommon {

    /**
     * GenICRemote Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICRemote(BeanDesc dd, String fileName) throws GenICException {
	super(dd, fileName);
    }

    /**
     * Generate the source of the class which implements the Remote interface
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Declared methods in the Enterprise bean's remote interface;
	Method methods[];

	// Comments
	src.println("//");
	src.println("// "+wrpRemoteBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool of JOnAS");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(ejbRemoteName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpRemoteBaseName+
		    " extends "+getSuperClass()+
		    " implements "+ejbRemoteName+" {");
	src.println();
	src.indentPlus();

	// Constructor()
	genConstructor();
	src.println();

	// getHandle() Method
	genGetHandle();
	src.println();

	// Business Methods defined in the remote interface
	methods = ejbRemoteClass.getMethods();
	Vector methAlreadyGen = new Vector();
	genBussinessMethodsOfInt(methods, ejbRemoteClass, methAlreadyGen);

	// End of the Class
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }

    /**
     * Source generation for the given bussiness methods.
     * First, methods declared in the given interface are generated,
     * Then, methods declared in the super-interfaces are generated,
     * (if there are not already generated).
     * Don't generate several time an overriding method.
     * @param methods        list of methods to generate
     * @param intrf          interface
     * @param methAlreadyGen list of methods already generated
     * @exception GenICException in error case
     */
    void genBussinessMethodsOfInt(Method[] methods, Class intrf, Vector methAlreadyGen)
	throws GenICException {
	for (int m=0; m < methods.length; m++) {
	    Method method = methods[m];
	    String methString = convertMethod(method);
	    if (method.getDeclaringClass().equals(intrf) && !methAlreadyGen.contains(methString)) {
		// pure business methods
		if (!method.getDeclaringClass().equals(javax.ejb.EJBObject.class)) {
		    genBussinessMethod(method);
		    src.println();
		    methAlreadyGen.add(methString);
		}
		// remove (the only EJBObject method to consider)
		if (method.getName().equals("remove")
		    && method.getDeclaringClass().equals(javax.ejb.EJBObject.class)) {
		    genRemove(method);
		    src.println();
		    methAlreadyGen.add(methString);
		}
	    }
	}
	Class[] superIntrfs = intrf.getInterfaces();
	for (int i=0; i < superIntrfs.length; i++) {
	    genBussinessMethodsOfInt(methods, superIntrfs[i], methAlreadyGen);
	}
    }

    /**
     * Source generation for the given bussiness method described in the Remote interface
     * @exception GenICException in error case
     */
    void genBussinessMethod(Method method) throws GenICException {

	int transAttribute;
	Class params[] = method.getParameterTypes();
	boolean isReturnValue = ! JavaType.isVoid(method.getReturnType());

	genHeaderMethod(method, wrpRemoteBaseName);

	transAttribute = ddesc.getMethodDesc(method).getTxAttribute();
	genSecuritySignature(method);
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+", methodSignature) ;");
	if (ddesc instanceof SessionDesc) {
	    src.println("JContextSession ctx = (JContextSession) rctx.getEJBContext();");
	} else {
	    src.println("JContextEntity ctx = (JContextEntity) rctx.getEJBContext();");
	}
	src.println(ejbBeanName + " eb = ("+ejbBeanName +")ctx.getInstance();");
	if (isReturnValue) {
	    src.println(JavaType.getName(method.getReturnType()) + " result;");
	}
	src.println("try {");
	src.indentPlus();
	if (isReturnValue) {
	    src.print("result = ");
	}
	src.print("eb." + method.getName() + "(", !isReturnValue);
	for (int p=1; p <= params.length; p++) {
	    src.print("p" + p, false);
	    if (p < params.length) {
		src.print(", ", false);
	    }
	}
	src.println(");", false);
	genCatchBean(method, false);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	if (isReturnValue) {
	    src.println("return(result);");
	}
	src.indentMinus(); // end of core
	src.println("}");
    }

    /**
     * Source generation for the remove method
     * @exception GenICException in error case
     */
    void genRemove(Method method) throws GenICException {

	genHeaderMethod(method, wrpRemoteBaseName);

	int transAttribute = ddesc.getMethodDesc(method).getTxAttribute();
	genSecuritySignature(method);
	src.println("RequestCtx rctx = preinvoke("+ddesc.getMethodDesc(method).getTxAttribute()+", methodSignature) ;");
	if (!(ddesc instanceof SessionDesc)) {
	    src.println("JContextEntity ctx = (JContextEntity) rctx.getEJBContext();");
	}
	src.println("try {");
	src.indentPlus();
	if (ddesc instanceof SessionDesc) {
	    src.println("removeEJB();");
	} else {
	    src.println("ctx.remove();");
	}
	genCatchBean(method, false);
	src.println("} finally {");
	src.indentPlus();
	src.println("postinvoke(rctx);");
	src.indentMinus();
	src.println("}");
	src.indentMinus(); // end of core
	src.println("}");
    }

    /*
     * Convert a method to a string depending on its name ans its parameters list.
     */
    private static String convertMethod(Method meth) {
	String value = new String(meth.getName());
	Class[] params = meth.getParameterTypes();
	value = value + "_" + params.length;
	for (int p=0; p<params.length; p++) {
	    value = value + "_" + JavaType.getName(params[p]);
	}
	return(value);
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanSession or JBeanEntity, depending on the Bean type)
     */
    abstract String getSuperClass();

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    abstract void   genConstructor() throws GenICException;

    /**
     * Source generation of the getHandle() method
     * (In some case, no source is generated; see sub-classes)
     * @exception GenICException in error case
     */
    abstract void   genGetHandle() throws GenICException;

}

