/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.lang.String;
import org.objectweb.jonas_ejb.deployment.api.SessionDesc;
import org.objectweb.jonas_ejb.deployment.api.SessionStatefulDesc;
import org.objectweb.jonas_ejb.lib.JavaType;


/**
 *  This class allows to generate the source of the class that implements
 *  the Enterprise bean's remote interface of a given Session Enterprise Java Bean.
 *  Its extends GenICRemote class.
 */

class GenICSessionRemote extends GenICRemote {

    boolean isSynchro = false;
    boolean isStateful;

    /**
     * GenICSessionRemote Constructor
     *
     * @param dd       deployment descriptor of the Session Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICSessionRemote(SessionDesc dd, String fileName) throws GenICException {
	super(dd, fileName);

	isSynchro = JavaType.implementsInterface(ejbBeanClass, javax.ejb.SessionSynchronization.class);
	isStateful = (dd instanceof SessionStatefulDesc);
	isSynchronized = true;
    }

    /**
     * Name of the superclass of the generated class
     * (JBeanSession in case of Session Bean)
     */
    String getSuperClass() {
	if (isStateful) {
	    return(new String("JBeanSessionSf"));
	} else {
	    return(new String("JBeanSessionSl"));
	}
    }

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    void genImport() throws GenICException {
	src.println("import java.lang.Error;");
	src.println("import java.lang.RuntimeException;");
	src.println("import java.rmi.RemoteException;");
	src.println("import javax.ejb.EnterpriseBean;");
	src.println("import javax.ejb.RemoveException;");
	src.println("import javax.ejb.SessionBean;");
	if (isStateful) {
	    src.println("import org.objectweb.jonas_ejb.container.JBeanSessionSf;");
	} else {
	    src.println("import org.objectweb.jonas_ejb.container.JBeanSessionSl;");
	}
	src.println("import org.objectweb.jonas_ejb.container.JContextSession;");
	src.println("import org.objectweb.jonas_ejb.container.RequestCtx;");
	src.println("import org.objectweb.jonas_ejb.container.JSessionHome;");
	src.println("import org.objectweb.jonas_ejb.container.TraceEjb;");
    }

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    void genConstructor() throws GenICException {
	src.println("public " + wrpRemoteBaseName +
		    "(JSessionHome home) throws RemoteException {");
	src.indentPlus();
	if (isStateful) {
	    src.println("super(home, "+isSynchro+");");
	} else {
	    src.println("super(home);");
	}
	src.println("TraceEjb.debugGenIC(\"" + wrpRemoteBaseName + " constructor\");");
	src.indentMinus();
	src.println("}");
    }

    /**
     * Source generation of the getHandle() method
     * In case of Session Bean, there is no source to generate.
     * @exception GenICException in error case
     */
    void genGetHandle() throws GenICException {
	// Nothing to do !!!
	// No source to generate for Session (Generic source in SessionHandleImpl)
    }

}

