/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas_ejb.tools;


import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.String;


/**
 * The IndentedFile Class allows to create an ASCCI file with indented lines.
 * This class is used for the creation of the implementation classes sources
 * of the Bean's interfaces via GenIC.
 */

public class IndentedFile {

    private int         curIndentation;
    private PrintWriter fOut;
    private String      fName;

    /**
     * Cerates a new IndentedFile with the given name.
     *  @param name is the name of the IndentedFile to create
     */
    public IndentedFile(String name) throws IOException {
	curIndentation = 0;
	fName = new String(name);
	int li = fName.lastIndexOf(File.separatorChar);
	if (li != -1) {
	    String dirName = fName.substring(0, li);
	    File dir = new File(dirName);
	    if (!dir.exists()) {
		if (!dir.mkdirs()) {
		    throw new IOException("Cannot create the directory '"+dirName+"'");
		}
	    }
	}
	fOut = new PrintWriter(new FileWriter(name));
    }

    /**
     * Increment the indentation value.
     */
    public void indentPlus() {
	curIndentation++;
    }

    /**
     * Decrement the indentation value.
     */
    public void indentMinus() {
	curIndentation--;
    }

    /**
     * Print the given message with or without indentation.
     */
    public void print(String msg, boolean isIndent) {
	if (isIndent == true) {
	    String blank = new String();
	    for (int i=0; i < curIndentation; i++) {
		blank = blank.concat("    ");
	    }
	    fOut.print(blank);
	}
	fOut.print(msg);
	fOut.flush();
    }

    /**
     * Print the given message with indentation.
     */
    public void print(String msg) {
	print(msg, true);
    }

    /**
     * Finish the line.
     */
    public void println() {
	fOut.println();
    }

    /**
     * Print the given message with indentation, and then finish the line.
     */
    public void println(String msg) {
	print(msg, true);
	fOut.println();
    }

    /**
     * Print the given message with or without indentation, and then finish the line.
     */
    public void println(String msg, boolean isIndent) {
	print(msg, isIndent);
	fOut.println();
    }

    /**
     * Close the file
     */
    public void close() {
	fOut.close();
    }

    /**
     * Close and delete the file
     */
    public void delete() {
	close();
	File f = new File(fName);
	f.delete();
    }
}
