/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): ____________________________________.
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas_timer;

import java.io.PrintWriter;

public class TraceTimer {
    
    static public boolean isDebug = false;	// if the timer is logged
    static public boolean isVerbose = false;	// if we print the verbose message
    static public PrintWriter logWriter = null;	// our log writer
    
    /**
     * set the debug timer
     */
    static public void setDebug(boolean set) {
	isDebug = set;
    }

    /**
     * set the verbose flag
     */
    static public void setVerbose(boolean set) {
	isVerbose = set;
    }
    
    /**
     * set the log writer
     */
    static public void setLogWriter(PrintWriter log) {
	logWriter = log;
    }
    
    /**
     * print the verbose message if the logger is not null
     */
    static public void verbose(String msg) {
	if ((isVerbose) && (logWriter != null))
	    logWriter.println(msg);
    }

    /**
     * print the debug timer message if the logger is not null
     */
    static public void debug(String msg) {
	if ((isDebug) && (logWriter != null))
	    logWriter.println(msg);
    }

    /**
     * print the error message if the logger is not null
     */
    static public void error(String msg) {
	if (logWriter != null)
	    logWriter.println(msg);
    }

    /**
     * print the throwing message if the logger is not null
     */
    static public void error(String msg, Throwable th) {
	if (logWriter != null) {
	    logWriter.println(msg);
	    th.printStackTrace(logWriter);
	}
    }

}
